package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;
import java.util.Random;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Animated Image
 * @author R.Bacher, DESY
 */
public class Web2cAnimatedImage extends Web2cComponent {

	private Random randomGenerator = null;
	
	/**
	 * Animated Image
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cAnimatedImage(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 1;
		registerMonitors(false);
		initSoundAndBlinking(1);
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {		
			if (propertyLinksEstablished == true) {
				componentHTML.add("			<img id=\"" + id + "\" src=\"" + highIcon + "\" title=\"" + toolTipText + "\" alt=\"Image temporarily unavailable\" onclick=\"localHistory('" + property + "')\"  style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "\">");
			}
			else {
				componentHTML.add("");
			}
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {		
			if (propertyLinksEstablished == true) {
				componentJavaScript.add("	//AnimatedIcon: " + id);
			}
			else {
				componentJavaScript.add("");
			}
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		double number = 0.0;
		String soundAndBlinking = "";
		String style = "";

		alarmThreshold = "5.0";	
		number = (double)randomGenerator.nextInt(10);
		
		identifier = "<id>" + id + "</id>";
		if (number > Double.valueOf(alarmThreshold )) {
			value = "<value>" + highIcon + "</value>";
			soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);
		}
		else {
			value = "<value>" + lowIcon + "</value>";
			soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);
		}
		style = getStyleUpdate();	
		str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";

		return str;
	}

	/**
	 * Returns real data from common device access plug or from executing the extension code; performs remote actions (if any)
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		String[] ret = new String[componentDataSize];
		String soundAndBlinking = "";
		String style = "";
		String[] act = null;
		
		if (!extensionManifest.equals("")) {
			if (extensionPropertyLinksEstablished == true) {			
				extensionExecutor.executeExtensionCode("live");
				ret[0] = ((String)(extensionExecutor.getData()[0])).split(">")[1].split("<")[0];
			}
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		else {
			if (propertyLinksEstablished == true) {
				ret[0] = (commonDeviceAccess.getMonitorData(property.get(0))).split(">")[1].split("<")[0];
			}
		}
		
		if (extensionPropertyLinksEstablished == true || propertyLinksEstablished == true) {
			identifier = "<id>" + id + "</id>";	
			if (Double.valueOf(characterEscaper.decodeString(ret[0])) > Double.valueOf(alarmThreshold)) {
				value = "<value>" + highIcon + "</value>";
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);
			}
			else {
				value = "<value>" + lowIcon + "</value>";
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);
			}
			style = getStyleUpdate();	
			str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";
		}
		
		return str;
	}
}
