package de.desy.web2cToolkit.web2c.web2cComponents;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Random;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.LogarithmicAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYAreaRenderer;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.data.time.Millisecond;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Histogram-type Plot
 * @author R.Bacher, DESY
 */
public class Web2cHisto extends Web2cComponent {
	
	private Random randomGenerator = null;
	private String web2cSession = null;
	private float[][] dataValues = null;
	private double[][] dataIndices = null;
	private int numberOfDataPoints = 0;
	private int numberOfDataTraces = 0;
	private String[] dataAddressName = null;
	private String title = null;
	private int dataArrayCompletionLength = 0;
	
	/**
	 * Histogram-type Plot
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 * @param session Web2c session id
	 */
	public Web2cHisto(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch, String session) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);

		if (extensionManifest.equals("")) componentDataSize = property.size();
		registerMonitors(false);
		initSoundAndBlinking(1);

		web2cSession = session;
		randomGenerator = new Random();		
		
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		int i = 0;
		int j = 0;
		int from = 0;
		int to = 0;
		String parameters = null;
		
		parameters = ((String)property.toArray()[0]).substring(((String)property.toArray()[0]).indexOf("(") + 1, ((String)property.toArray()[0]).indexOf(")"));
		from = Integer.valueOf(parameters.split(";")[0]);
		to = Integer.valueOf(parameters.split(";")[1]);
		numberOfDataPoints = to - from + 1;		
		dataArrayCompletionLength = numberOfDataPoints;
		
		if (plotType.equals(linePlot) || plotType.equals(areaPlot) || plotType.equals(barPlot)) {
			if (xmin.equals(xmax)) {
				xmin = "0";
				xmax = String.valueOf(dataArrayCompletionLength - 1);
			}	
		}
		else {	
			if (xmin.equals(xmax)) {
				Calendar calendar = Calendar.getInstance();
				xmax = String.valueOf(calendar.getTimeInMillis());
				xmin = String.valueOf(Long.valueOf(xmax) - 86400000);
			}	
			else {
				Calendar calendar = Calendar.getInstance();
				int year = Integer.valueOf(xmax.split(" ")[0].split("-")[0]);
				int month = Integer.valueOf(xmax.split(" ")[0].split("-")[1]) - 1;
				int day = Integer.valueOf(xmax.split(" ")[0].split("-")[2]);
				int hour = Integer.valueOf(xmax.split(" ")[1].split(":")[0]);
				int minute = Integer.valueOf(xmax.split(" ")[1].split(":")[1]);
				int second = Integer.valueOf(xmax.split(" ")[1].split(":")[2]);
				calendar.set(year, month, day, hour, minute, second);
				xmax = String.valueOf(calendar.getTimeInMillis());
				year = Integer.valueOf(xmin.split(" ")[0].split("-")[0]);
				month = Integer.valueOf(xmin.split(" ")[0].split("-")[1]) - 1;
				day = Integer.valueOf(xmin.split(" ")[0].split("-")[2]);
				hour = Integer.valueOf(xmin.split(" ")[1].split(":")[0]);
				minute = Integer.valueOf(xmin.split(" ")[1].split(":")[1]);
				second = Integer.valueOf(xmin.split(" ")[1].split(":")[2]);
				calendar.set(year, month, day, hour, minute, second);
				xmin = String.valueOf(calendar.getTimeInMillis());
			}
		}
		
		numberOfDataTraces = property.size();
		dataAddressName = new String[numberOfDataTraces];
		dataIndices = new double[numberOfDataTraces][dataArrayCompletionLength];
		dataValues = new float[numberOfDataTraces][dataArrayCompletionLength];
		for (j = 0; j < numberOfDataTraces; j++) {
			for (i = 0; i < dataArrayCompletionLength; i++) {
				dataIndices[j][i] = Double.valueOf(xmin) + ((i + 1) * (Double.valueOf(xmax) - Double.valueOf(xmin)) / dataArrayCompletionLength);
				dataValues[j][i] = 0;
			}
			dataAddressName[j] = "";
		}

		title = "";
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		Calendar calendar = null;
		String nowTime = "";
		int now = 0;
		
		calendar = Calendar.getInstance();
		now = (int)(calendar.getTimeInMillis() / 1000.0);
		nowTime = String.valueOf(now);	

		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentHTML.add("			<img id=\"" + id + "\" src=\"Web2c?param=<plot><sessionId>" + web2cSession + "</sessionId><id>" + id + "</id><timeStamp>" + nowTime + "</timeStamp></plot>\" title=\"" + toolTipText + "\" alt=\"Image temporarily unavailable\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "\">");
			}
			else {
				componentHTML.add("");
			}
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentJavaScript.add("	//Histo: " + id);
			}
			else {
				componentJavaScript.add("");
			}
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		String identifier = "";
		String value = "";
		String plot = "";
		Calendar calendar = null;
		String nowTime = "";
		int now = 0;
		int i = 0;
		int j = 0;	
		
		calendar = Calendar.getInstance();
		now = (int)(calendar.getTimeInMillis() / 1000.0);
		nowTime = String.valueOf(now);	
		
		title = caption.get(0);
		plot = "<plot><sessionId>" + web2cSession + "</sessionId><id>" + id + "</id><timeStamp>" + nowTime + "</timeStamp></plot>";
		
		dataArrayCompletionLength = numberOfDataPoints;
		for (j = 0; j < numberOfDataTraces; j++) {
			for (i = 0; i < dataArrayCompletionLength; i++) {	
				if (plotType.equals(linePlot) || plotType.equals(areaPlot) || plotType.equals(barPlot)) {
					dataIndices[j][i] = Double.valueOf(xmin) + ((i + 1) * (Double.valueOf(xmax) - Double.valueOf(xmin)) / dataArrayCompletionLength);
				}
				else {
					dataIndices[j][i] = ((now * 1000.0) - Double.valueOf(xmax)) + Double.valueOf(xmin) + ((i + 1) * (Double.valueOf(xmax) - Double.valueOf(xmin)) / dataArrayCompletionLength);				
				}
				dataValues[j][i] = randomGenerator.nextInt(100);
			}
			dataAddressName[j] = "Simulated Data " + String.valueOf(j);
		}
		
		identifier = "<id>" + id + "</id>";
		value = "<value>" + plot + "</value>";

		str = "<update>" + identifier + value + "</update>";

		return str;
	}
	
	/**
	 * Returns real data from common device access plug or from executing the extension code
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String identifier = "";
		String value = "";
		String plot = "";
		String style = "";
		String[] ret = new String[componentDataSize];
		String[] retData = null;
		String[] act = null;
		Calendar calendar = null;
		String nowTime = "";
		int now = 0;
		int i = 0;
		int j = 0;	
		
		calendar = Calendar.getInstance();
		now = (int)(calendar.getTimeInMillis() / 1000.0);
		nowTime = String.valueOf(now);	
		
		title = caption.get(0);
		plot = "<plot><sessionId>" + web2cSession + "</sessionId><id>" + id + "</id><timeStamp>" + nowTime + "</timeStamp></plot>";

		if (!extensionManifest.equals("")) {
			if (extensionPropertyLinksEstablished == true) {
				extensionExecutor.executeExtensionCode("live");
				for (i = 0; i < ret.length; i++) {
					ret[i] = (String)(extensionExecutor.getData()[i]);
					ret[i] = ret[i].split(">")[1].split("<")[0];	
					dataAddressName[i] = "Trace" + i;
				}
			}
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		else {
			if (propertyLinksEstablished == true) {
				for (i = 0; i < ret.length; i++) {
					String address = (property.get(i));
					ret[i] = (commonDeviceAccess.getMonitorData(address)).split(">")[1].split("<")[0];
					dataAddressName[i] = address;
				}
			}
		}
		
		if (propertyLinksEstablished == true || extensionPropertyLinksEstablished == true) {
			dataArrayCompletionLength = numberOfDataPoints;	
			for (j = 0; j < numberOfDataTraces; j++) {
				retData = ret[j].split(",");
				for (i = 0; i < dataArrayCompletionLength; i++) {		
					if (plotType.equals(linePlot) || plotType.equals(areaPlot) || plotType.equals(barPlot)) {
						dataIndices[j][i] = Double.valueOf(xmin) + ((i + 1) * (Double.valueOf(xmax) - Double.valueOf(xmin)) / dataArrayCompletionLength);
					}
					else {
						dataIndices[j][i] = ((now * 1000.0) - Double.valueOf(xmax)) + Double.valueOf(xmin) + ((i + 1) * (Double.valueOf(xmax) - Double.valueOf(xmin)) / dataArrayCompletionLength);				
					}
					dataValues[j][i] = Float.valueOf(characterEscaper.decodeString(retData[i]));
					if (functionType.contains("power")) {
						if (functionParameter.equals("") || functionParameter.equals(" ")) {
							functionParameter = "1.0";
						}
						dataValues[j][i] = (float)(Math.pow((double)dataValues[j][i], Double.valueOf(functionParameter)));
					}
					if (functionType.contains("factor")) {
						if (functionParameter.equals("") || functionParameter.equals(" ")) {
							functionParameter = "1.0";
						}
						dataValues[j][i] =  dataValues[j][i] * Float.valueOf(functionParameter);
					}
					if (functionType.contains("offset")) {
						if (functionParameter.equals("") || functionParameter.equals(" ")) {
							functionParameter = "0.0";
						}
						dataValues[j][i] =  dataValues[j][i] + Float.valueOf(functionParameter);
					}
				}
			}
			if (functionType.contains("norm")) {
				if (functionParameter.equals("") || functionParameter.equals(" ")) {
					functionParameter = "1";
				}
				for (j = 0; j < numberOfDataTraces; j++) {
					if (j != (Integer.valueOf(functionParameter) - 1)) {
						for (i = 0; i < dataArrayCompletionLength; i++) {
							dataValues[j][i] =  (dataValues[j][i] / dataValues[Integer.valueOf(functionParameter) - 1][i]) * 100.0f;
						}
					}
				}
				for (i = 0; i < dataArrayCompletionLength; i++) {
					dataValues[Integer.valueOf(functionParameter) - 1][i] = 100.0f;
				}
			}
			if (functionType.contains("ref")) {
				if (functionParameter.equals("") || functionParameter.equals(" ")) {
					functionParameter = "1";
				}
				for (j = 0; j < numberOfDataTraces; j++) {
					if (j != (Integer.valueOf(functionParameter) - 1)) {
						for (i = 0; i < dataArrayCompletionLength; i++) {
							dataValues[j][i] =  dataValues[j][i] - dataValues[Integer.valueOf(functionParameter) - 1][i];
						}
					}
				}
				for (i = 0; i < dataArrayCompletionLength; i++) {
					dataValues[Integer.valueOf(functionParameter) - 1][i] = 0.0f;
				}
			}
			if (functionType.contains("log")) {
				if (functionParameter.equals("") || functionParameter.equals(" ")) {
					functionParameter = "1.0";
				}
				for (j = 0; j < numberOfDataTraces; j++) {
					for (i = 0; i < dataArrayCompletionLength; i++) {
						if (dataValues[j][i] <= 0.0f) {
							dataValues[j][i] = 1.0f;
						}
						dataValues[j][i] = dataValues[j][i] * Float.valueOf(functionParameter);
					}
				}
			}
	
			identifier = "<id>" + id + "</id>";
			value = "<value>" + plot + "</value>";
			style = getStyleUpdate();

			str = "<update>" + identifier + value + style + "</update>";
		}

		return str;
	}  
	
	/**
	 * Returns chart width [px]
	 * @return width of chart
	 */
	public int getChartWidth() {
		
		return Integer.valueOf(width.substring(0,width.length() - 2));		
	}
	
	/**
	 * Returns chart height [px]
	 * @return height of chart
	 */
	public int getChartHeight() {
		
		return Integer.valueOf(height.substring(0,height.length() - 2));		
	}
	
	/**
	 * Returns chart
	 * @return chart
	 */
	public JFreeChart getChart() {
		
		int i = 0;
		JFreeChart chart = null;
		
		try {
			if (plotType.equals(linePlot)) {
				chart = ChartFactory.createXYLineChart(title , xLabel, "", null, PlotOrientation.VERTICAL, true, true, false);
			}
			else if (plotType.equals(areaPlot)) {
				chart = ChartFactory.createXYAreaChart(title , xLabel, "", null, PlotOrientation.VERTICAL, true, true, false);
			}
			else if (plotType.equals(barPlot)) {
				chart = ChartFactory.createXYAreaChart(title , xLabel, "", null, PlotOrientation.VERTICAL, true, true, false);
			}
			else {
				chart = ChartFactory.createTimeSeriesChart(title , xLabel, "", null, true, true, false);
			}
			XYPlot plot = chart.getXYPlot();
			plot.setForegroundAlpha(1.0f);	
			ValueAxis domainAxis = plot.getDomainAxis();		
			if (plotType.equals(linePlot) || plotType.equals(areaPlot) || plotType.equals(barPlot)) {
				domainAxis.setRange(Double.valueOf(xmin), Double.valueOf(xmax));
			}
			domainAxis.setLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(fontSize.substring(0, fontSize.length() - 2))));
			domainAxis.setTickLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(fontSize.substring(0, fontSize.length() - 2))));
			domainAxis.setLowerMargin(0);
			domainAxis.setUpperMargin(0);
			plot.setDomainAxis(domainAxis);
			if (plotAxis.equals(multi)) {
				for (i = 0; (i < numberOfDataTraces) && (i < maxTraces); i++) {
					if (functionType.contains("log")) {
						LogarithmicAxis axis = new LogarithmicAxis((String)(legend.toArray()[i]));
						axis.setLog10TickLabelsFlag(true);
						axis.setAllowNegativesFlag(false);
						if (!((Double)(yMin.toArray()[0])).equals((Double)(yMax.toArray()[0]))) {
							axis.setAutoRange(false);
							axis.setRange((Double)(yMin.toArray()[i]), (Double)(yMax.toArray()[i]));
						}
						else {
							axis.setAutoRange(true);
						}
						axis.setLabelPaint(plotColor[i]);
						axis.setTickLabelPaint(plotColor[i]);	
						axis.setLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(fontSize.substring(0, fontSize.length() - 2))));	
						axis.setTickLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(fontSize.substring(0, fontSize.length() - 2))));
						plot.setRangeAxis(i,axis);
						plot.setRangeAxisLocation(i, AxisLocation.BOTTOM_OR_LEFT);
					}
					else {
						NumberAxis axis = new NumberAxis((String)(legend.toArray()[i]));
						if (!((Double)(yMin.toArray()[0])).equals((Double)(yMax.toArray()[0]))) {
							axis.setAutoRange(false);
							axis.setRange((Double)(yMin.toArray()[i]), (Double)(yMax.toArray()[i]));
						}
						else {
							axis.setAutoRange(true);
						}
						axis.setLabelPaint(plotColor[i]);
						axis.setTickLabelPaint(plotColor[i]);	
						axis.setLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(fontSize.substring(0, fontSize.length() - 2))));	
						axis.setTickLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(fontSize.substring(0, fontSize.length() - 2))));
						plot.setRangeAxis(i,axis);
						plot.setRangeAxisLocation(i, AxisLocation.BOTTOM_OR_LEFT);
					}
					if (plotType.equals(linePlot)) {
						XYDataset dataset = createXYLineDataset(i);
						plot.setDataset(i, dataset);
						plot.mapDatasetToRangeAxis(i, i);
						StandardXYItemRenderer renderer = new StandardXYItemRenderer();
						renderer.setSeriesPaint(0, plotColor[i]);
						plot.setRenderer(i, renderer);
					}
					else if (plotType.equals(areaPlot)) {
						XYDataset dataset = createXYLineDataset(i);
						plot.setDataset(i, dataset);
						plot.mapDatasetToRangeAxis(i, i);
						XYAreaRenderer renderer = new XYAreaRenderer();
						renderer.setSeriesPaint(0, plotColor[i]);
						plot.setRenderer(i, renderer);
					}
					else if (plotType.equals(barPlot)) {
						XYDataset dataset = createXYLineDataset(i);
						plot.setDataset(i, dataset);
						plot.mapDatasetToRangeAxis(i, i);
						XYBarRenderer renderer = new XYBarRenderer();
						renderer.setSeriesPaint(0, plotColor[i]);
						plot.setRenderer(i, renderer);
					}
					else {
						XYDataset dataset = createXYTimeDataset(i);
						plot.setDataset(i, dataset);
						plot.mapDatasetToRangeAxis(i, i);
						if (plotType.equals(lineTimePlot)) {
							StandardXYItemRenderer renderer = new StandardXYItemRenderer();
							renderer.setSeriesPaint(0, plotColor[i]);
							plot.setRenderer(i, renderer);
						}
						else if (plotType.equals(areaTimePlot)) {
							XYAreaRenderer renderer = new XYAreaRenderer();
							renderer.setSeriesPaint(0, plotColor[i]);
							plot.setRenderer(i, renderer);
						}
						else if (plotType.equals(barTimePlot)) {
							XYBarRenderer renderer = new XYBarRenderer();
							renderer.setSeriesPaint(0, plotColor[i]);
							plot.setRenderer(i, renderer);
						}
					}
				}
			}
			else if (plotAxis.equals(single)){
				if (functionType.contains("log")) {
					LogarithmicAxis axis = new LogarithmicAxis("");
					axis.setLog10TickLabelsFlag(true);
					axis.setAllowNegativesFlag(false);
					if (!((Double)(yMin.toArray()[0])).equals((Double)(yMax.toArray()[0]))) {
						axis.setAutoRange(false);
						axis.setRange((Double)(yMin.toArray()[0]), (Double)(yMax.toArray()[0]));
					}
					else {
						axis.setAutoRange(true);
					}
					axis.setLabelPaint(Color.BLACK);
					axis.setTickLabelPaint(Color.BLACK);	
					axis.setLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(fontSize.substring(0, fontSize.length() - 2))));	
					axis.setTickLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(fontSize.substring(0, fontSize.length() - 2))));
					plot.setRangeAxis(0,axis);
					plot.setRangeAxisLocation(0, AxisLocation.BOTTOM_OR_LEFT);
				}
				else {
					NumberAxis axis = new NumberAxis("");
					if (!((Double)(yMin.toArray()[0])).equals((Double)(yMax.toArray()[0]))) {
						axis.setAutoRange(false);
						axis.setRange((Double)(yMin.toArray()[0]), (Double)(yMax.toArray()[0]));
					}
					else {
						axis.setAutoRange(true);
					}
					axis.setLabelPaint(Color.BLACK);
					axis.setTickLabelPaint(Color.BLACK);	
					axis.setLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(fontSize.substring(0, fontSize.length() - 2))));	
					axis.setTickLabelFont(new Font("SansSerif", Font.BOLD, Integer.valueOf(fontSize.substring(0, fontSize.length() - 2))));
					plot.setRangeAxis(0,axis);
					plot.setRangeAxisLocation(0, AxisLocation.BOTTOM_OR_LEFT);
				}
				for (i = 0; (i < numberOfDataTraces) && (i < maxTraces); i++) {
					if (plotType.equals(linePlot)) {
						XYDataset dataset = createXYLineDataset(i);
						plot.setDataset(i, dataset);
						StandardXYItemRenderer renderer = new StandardXYItemRenderer();
						renderer.setSeriesPaint(0, plotColor[i]);
						plot.setRenderer(i, renderer);
					}
					else if (plotType.equals(areaPlot)) {
						XYDataset dataset = createXYLineDataset(i);
						plot.setDataset(i, dataset);
						XYAreaRenderer renderer = new XYAreaRenderer();
						renderer.setSeriesPaint(0, plotColor[i]);
						plot.setRenderer(i, renderer);
					}
					else if (plotType.equals(barPlot)) {
						XYDataset dataset = createXYLineDataset(i);
						plot.setDataset(i, dataset);
						XYBarRenderer renderer = new XYBarRenderer();
						renderer.setSeriesPaint(0, plotColor[i]);
						plot.setRenderer(i, renderer);
					}
					else {
						XYDataset dataset = createXYTimeDataset(i);
						plot.setDataset(i, dataset);
						if (plotType.equals(lineTimePlot)) {
							StandardXYItemRenderer renderer = new StandardXYItemRenderer();
							renderer.setSeriesPaint(0, plotColor[i]);
							plot.setRenderer(i, renderer);
						}
						if (plotType.equals(areaTimePlot)) {
							XYAreaRenderer renderer = new XYAreaRenderer();
							renderer.setSeriesPaint(0, plotColor[i]);
							plot.setRenderer(i, renderer);
						}
						if (plotType.equals(barTimePlot)) {
							XYBarRenderer renderer = new XYBarRenderer();
							renderer.setSeriesPaint(0, plotColor[i]);
							plot.setRenderer(i, renderer);
						}
					}
				}
			}
		}
		catch (Exception e) {
			System.out.println(e.toString());
			e.printStackTrace();
			System.out.println("Error generating chart!");
		}
		
		return chart;
	}
	
	/**
	 * Returns line data set
	 * @param i trace index
	 * @return data set
	 */
	private XYDataset createXYLineDataset(int i) {
		
		int j = 0;
		
			XYSeries series = new XYSeries((String)(legend.toArray()[i]));					
			if (plotOrder.equals(normal)) {
				for (j = 0; j < dataArrayCompletionLength; j++) {
					if (functionType.contains("log") && (dataValues[i][j] <= 0)) {
						dataValues[i][j] = 1;
					}
					series.addOrUpdate(dataIndices[i][j],(double)(dataValues[i][j]));
				}		
			}
			else if (plotOrder.equals(invert)) {
				for (j = 0; j < dataArrayCompletionLength; j++) {
					if (functionType.contains("log") && (dataValues[i][dataArrayCompletionLength - j - 1] <= 0)) dataValues[i][dataArrayCompletionLength - j - 1] = 1;
					series.addOrUpdate(dataIndices[i][j],(double)(dataValues[i][dataArrayCompletionLength - j - 1]));
				}		
			}			
			XYSeriesCollection dataset = new XYSeriesCollection();
			dataset.addSeries(series);
		
		return dataset;
	}
	
	/**
	 * Returns time data set
	 * @param i trace index
	 * @return data set
	 */
	private XYDataset createXYTimeDataset(int i) {
		
		int j = 0;
		Calendar calendar = null;
		
		TimeSeries series = new TimeSeries((String)(legend.toArray()[i]), Millisecond.class);	
		calendar = Calendar.getInstance();
		
		if (plotOrder.equals(normal)) {
			for (j = 0; j < dataArrayCompletionLength; j++) {
				calendar.setTimeInMillis((long)dataIndices[i][j]);
				if (functionType.contains("log") && (dataValues[i][j] <= 0)) dataValues[i][j] = 1;
				series.addOrUpdate(new Millisecond(calendar.getTime()),(double)(dataValues[i][j]));
			}		
		}
		else if (plotOrder.equals(invert)) {
			for (j = 0; j < dataArrayCompletionLength; j++) {
				calendar.setTimeInMillis((long)dataIndices[i][j]);
				if (functionType.contains("log") && (dataValues[i][dataArrayCompletionLength - j - 1] <= 0)) dataValues[i][dataArrayCompletionLength - j - 1] = 1;
				series.addOrUpdate(new Millisecond(calendar.getTime()),(double)(dataValues[i][dataArrayCompletionLength - j - 1]));
			}		
		}		
		
		TimeSeriesCollection dataset = new TimeSeriesCollection();
		dataset.addSeries(series);
		
		return dataset;
	}
}
