package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;
import java.util.Random;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Indicator
 * @author R.Bacher, DESY
 */
public class Web2cIndicator extends Web2cComponent {
	
	private Random randomGenerator = null;
	
	/**
	 * Indicator
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cIndicator(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 1;
		registerMonitors(false);
		initSoundAndBlinking(1);

		randomGenerator = new Random();	
		
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentHTML.add("			<img id=\"" + id + "\" src=\"" + falseIcon + "\" title=\"" + toolTipText + "\" alt=\"Image temporarily unavailable\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; width:" + width + "; height:" + height + "\">");
			}
			else {
				componentHTML.add("");
			}
		}
		
		return componentHTML;
	}
	
	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentJavaScript.add("	//Indicator: " + id);
			}
			else {
				componentJavaScript.add("");
			}
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */	
	public String getSimulatedData() {
		
		String str = "";
		String identifier = "";
		String value = "";
		double number = 0.0;
		String soundAndBlinking = "";

		number = (double)randomGenerator.nextInt(10);	

		identifier = "<id>" + id + "</id>";
		value = "";
		for (int j = 0; j < trueValue.length; j++) {
			if (Double.valueOf(number).equals(Double.valueOf(trueValue[j]))) {
				value = "<value>" + trueIcon[j] + "</value>";
			}
		}
		if (value.equals("")) {
			value = "<value>" + falseIcon + "</value>";
			soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);
		}
		else {
			soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);
		}	
		str = "<update>" + identifier + value + soundAndBlinking + "</update>";

		return str;
	}
	
	/**
	 * Returns real data from common device access plug or from executing the extension code
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String identifier = "";
		String value = "";
		String ret[] = new String[componentDataSize];
		String soundAndBlinking = "";
		String style = "";
		String[] act = null;
		
		if (!extensionManifest.equals("")) {
			if (extensionPropertyLinksEstablished == true) {
				extensionExecutor.executeExtensionCode("live");
				ret[0] = ((String)(extensionExecutor.getData()[0])).split(">")[1].split("<")[0];
			}
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		else {
			if (propertyLinksEstablished == true) {
				ret[0] = (commonDeviceAccess.getMonitorData(property.get(0))).split(">")[1].split("<")[0];
			}
		}
		
		if (extensionPropertyLinksEstablished == true || propertyLinksEstablished == true) {			
			identifier = "<id>" + id + "</id>";		
			value = "";
			for (int j = 0; j < trueValue.length; j++) {
				if (Double.valueOf(characterEscaper.decodeString(ret[0])).equals(Double.valueOf(trueValue[j]))) {
					value = "<value>" + trueIcon[j] + "</value>";
				}
			}
			if (value.equals("")) {
				value = "<value>" + falseIcon + "</value>";
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, 0);
			}
			else {
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, 0);
			}	
			style = getStyleUpdate();
			str = "<update>" + identifier + value + style + soundAndBlinking + "</update>";
		}
		
		return str;
	}
}
