package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * List (Table) of Indicators
 * @author R.Bacher, DESY
 */
public class Web2cIndicatorList extends Web2cComponent {

	private Random randomGenerator = null;

	/**
	 * List (Table) of Indicators
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cIndicatorList(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		if (extensionManifest.equals("")) componentDataSize = property.size();
		registerMonitors(false);
		initSoundAndBlinking(componentDataSize);

		randomGenerator = new Random();	
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
	
		int i = 0;
		int xPos = 0;
		int yPos = 0;
		int cellHeight = 0;
		int cellWidth = 0;
		int headerCellHeight = 0;
		int headerCellWidth = 0;
		int yStep = 0;
		int rows = 0;
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		width = String.valueOf(Integer.valueOf(width.split("p")[0]) - 1) + "px";
		height = String.valueOf(Integer.valueOf(height.split("p")[0]) - 1) + "px";
		
		rows = property.size();

		if (borderStyle.equals(solid)) {
			cellWidth = (Integer.valueOf(width.split("p")[0])) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
			if (!headerCaption.equals("") && !headerCaption.equals(" ")) {
				headerCellWidth = (Integer.valueOf(width.split("p")[0])) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
				headerCellHeight = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
				cellHeight = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows + 1));
			}
			else {
				cellHeight = (Integer.valueOf(height.split("p")[0]) / (rows)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows));
			}
		}
		else {	
			cellWidth = (Integer.valueOf(width.split("p")[0]));
			if (!headerCaption.equals("") && !headerCaption.equals(" ")) {
				cellHeight = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows + 1));
			}
			else {
				cellHeight = (Integer.valueOf(height.split("p")[0]) / (rows)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));	
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows));
			}
		}
		xPos = 0;
		yPos = 0;

		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentHTML.add("			<div id=\"" + id + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "\">");	
				if (!headerCaption.equals("") && !headerCaption.equals(" ")) {
					componentHTML.add("			<div id=\"" + id + "_header" + "\" title=\"" + toolTipText + "\" style=\"position:absolute; left:" + ((Integer)(xPos)).toString() + "px" + "; top:" + ((Integer)(yPos)).toString() + "px" + "; width:" + ((Integer)(headerCellWidth)).toString() + "px" + "; height:" + ((Integer)headerCellHeight).toString() + "px" + "; color:" + headerColor + "; background-color:" + headerBackgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + bold + "; text-align:" + center + "\">" + headerCaption + "</div>");
					yPos = yPos + yStep;
				}
				for (i = 0; i < rows; i++) {
					componentHTML.add("			<img id=\"" + id + "_cell" + ((Integer)i).toString() + "\" src=\"" + falseIcon + "\" title=\"" + toolTipText + "\" alt=\"Image temporarily unavailable\" style=\"position:absolute; left:" + ((Integer)(xPos)).toString() + "px" + "; top:" + ((Integer)yPos).toString() + "px" + "; width:" + ((Integer)(cellWidth)).toString() + "px" + "; height:" + ((Integer)cellHeight).toString() + "px" + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "\">");
					yPos = yPos + yStep;
				}		
				componentHTML.add("			</div>");
			}
			else {
				componentHTML.add("");
			}
		}
		
		return componentHTML;
	}
	
	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentJavaScript.add("	//IndicatorList: " + id);
			}
			else {
				componentJavaScript.add("");
			}
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		String identifer = "";
		String value = "";
		double number = 0.0;
		int i = -1;
		String soundAndBlinking = "";

		for (Iterator<String> it = property.iterator(); it.hasNext(); it.next()) {
			number = (double)randomGenerator.nextInt(10);
			i = i + 1;			
			value = "";
			identifer = "<id>" + id + "_cell" + ((Integer)i).toString() + "</id>";
			for (int j = 0; j < trueValue.length; j++) {
				if (((Double)number).equals(Double.valueOf(trueValue[j]))) {
					value = "<value>" + trueIcon[j] + "</value>";
				}
			}
			if (value.equals("")) {
				value = "<value>" + falseIcon + "</value>";
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, i);
			}
			else {
				soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, i);
			}	
			str = str + "<update>" + identifer + value + soundAndBlinking + "</update>";
		}
		
		return str;
	}
	/**
	 * Returns real data from common device access plug or from executing the extension code
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String identifer = "";
		String value = "";
		String[] ret = new String[componentDataSize];
		String soundAndBlinking = "";
		String style = "";
		String[] act = null;

		if (!extensionManifest.equals("")) {
			if (extensionPropertyLinksEstablished == true) {
				extensionExecutor.executeExtensionCode("live");
				for (int i = 0; i < ret.length; i++) {
					ret[i] = (String)(extensionExecutor.getData()[i]);
					ret[i] = ret[i].split(">")[1].split("<")[0];
				}
			}	
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		else {
			if (propertyLinksEstablished == true) {
				for (int i = 0; i < ret.length; i++) {
					String address = (property.get(i));
					ret[i] = (commonDeviceAccess.getMonitorData(address)).split(">")[1].split("<")[0];	
				}
			}
		}
		
		if (extensionPropertyLinksEstablished == true || propertyLinksEstablished == true) {
			for (int i = 0; i < ret.length; i++) {
				value = "";
				identifer = "<id>" + id + "_cell" + ((Integer)i).toString() + "</id>";				
				for (int j = 0; j < trueValue.length; j++) {
					if (Double.valueOf(characterEscaper.decodeString(ret[i])).equals(Double.valueOf(trueValue[j]))) {
						value = "<value>" + trueIcon[j] + "</value>";
					}
				}
				if (value.equals("")) {
					value = "<value>" + falseIcon + "</value>";
					soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, i);
				}
				else {
					soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, i);
				}	
				style = getStyleUpdate();
				str = str + "<update>" + identifer + value + style + soundAndBlinking + "</update>";
			}
		}

		return str;
	}
}
