package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;

import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Input
 * @author R.Bacher, DESY
 */
public class Web2cInput extends Web2cComponent {
	
	/**
	 * Input
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cInput(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 0;
		registerMonitors(true);
		initSoundAndBlinking(1);
		
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}
	
	/**
	 * Returns component's HTML tags
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		String html = null;
		
		if (userPrivilege.equals(layman)) {
			componentHTML.add("");
		}
		else if (userPrivilege.equals(expert) && superComponent.equals(yes)) {
			componentHTML.add("");
		}
		else if (userPrivilege.equals(supervisor) || (userPrivilege.equals(expert) && superComponent.equals(no))) {
			html = "			<div id=\"" + id + "\" title=\"" + toolTipText + "\"";
			html = html + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + left + "\">";		
			componentHTML.add(html);
			
			if (!caption.get(0).equals("") && !caption.get(0).equals(" ")) {
				String headerLeft = (0.05 * (Integer.valueOf(width.split("p")[0]))) + "px";
				String headerTop = (0.20 * (Integer.valueOf(height.split("p")[0]))) + "px";
				String headerWidth = String.valueOf((0.30 * (Integer.valueOf(width.split("p")[0])))) + "px";
				String headerHeight = String.valueOf((0.40 * (Integer.valueOf(height.split("p")[0])))) + "px";
				html = "				<div id=\"" + id + "_header" + "\" title=\"" + toolTipText + "\"";
				html = html + " style=\"position:absolute; left:" + headerLeft + "; top:" + headerTop + "; width:" + headerWidth + "; height:" + headerHeight + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + center + "\">" + caption.get(0) + "</div>";		
				componentHTML.add(html);    
			}
			
			if (!caption.get(0).equals("") && !caption.get(0).equals(" ")) {
				String inLeft = (0.40 * (Integer.valueOf(width.split("p")[0]))) + "px";
				String inTop = (0.1 * (Integer.valueOf(height.split("p")[0]))) + "px";
				String inWidth = String.valueOf((0.35 * (Integer.valueOf(width.split("p")[0])))) + "px";
				String inHeight = String.valueOf((0.80 * (Integer.valueOf(height.split("p")[0])))) + "px";
				html = "				<input type=\"text\" id=\"" + id + "_in" + "\" value=\"\" title=\"" + toolTipText + "\"";
				html = html + " style=\"position:absolute; left:" + inLeft + "; top:" + inTop + "; width:" + inWidth + "; height:" + inHeight + "; color:" + color + "; background-color:" + white + "; border-style:" + solid + "; border-width:" + thin + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + left + "\">";		
			}
			else {
				String inLeft = (0.05 * (Integer.valueOf(width.split("p")[0]))) + "px";
				String inTop = (0.1 * (Integer.valueOf(height.split("p")[0]))) + "px";
				String inWidth = String.valueOf((0.70 * (Integer.valueOf(width.split("p")[0])))) + "px";
				String inHeight = String.valueOf((0.80 * (Integer.valueOf(height.split("p")[0])))) + "px";
				html = "				<input type=\"text\" id=\"" + id + "_in" + "\" value=\"\" title=\"" + toolTipText + "\"";
				html = html + " style=\"position:absolute; left:" + inLeft + "; top:" + inTop + "; width:" + inWidth + "; height:" + inHeight + "; color:" + color + "; background-color:" + white + "; border-style:" + solid + "; border-width:" + thin + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + left + "\">";		
			}
			componentHTML.add(html);
			
			String buttonLeft = (0.80 * (Integer.valueOf(width.split("p")[0]))) + "px";
			String buttonTop = (0.20 * (Integer.valueOf(height.split("p")[0]))) + "px";
			String buttonWidth = String.valueOf((0.15 * (Integer.valueOf(width.split("p")[0])))) + "px";
			String buttonHeight = String.valueOf((0.60 * (Integer.valueOf(height.split("p")[0])))) + "px";
			html = "				<input type=\"button\" id=\"" + id + "_button" + "\" value=\"&#8658;\" title=\"" + toolTipText + "\"";
			html = html + " onclick=\"" + id + "_button_setValue()" + "\"";
			html = html + " style=\"position:absolute; left:" + buttonLeft + "; top:" + buttonTop + "; width:" + buttonWidth + "; height:" + buttonHeight + "; color:" + black + "; background-color:" + white + "; border-style:outset; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + center + "\">";		
			componentHTML.add(html);
			
			html = "			</div>";		
			componentHTML.add(html);
		}
		else {
			componentHTML.add("");
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman)) {
			componentJavaScript.add("");
		}
		else if (userPrivilege.equals(expert) && superComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else if (userPrivilege.equals(supervisor) || (userPrivilege.equals(expert) && superComponent.equals(no))) {
			componentJavaScript.add("	//Input: " + id);
			componentJavaScript.add("	function " + id + "_button_setValue()" + "{");
			componentJavaScript.add("		var ref;");
			componentJavaScript.add("		var inText;");
			componentJavaScript.add("		var xhraction;");
			componentJavaScript.add("		ref = document.getElementById(\"" +  id + "_in" + "\");");
			componentJavaScript.add("		inText = escape(ref.value);");
			componentJavaScript.add("		try {");                                                  
			componentJavaScript.add("			xhraction = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
			componentJavaScript.add("		} catch (event1) {");                             
			componentJavaScript.add("			try {");
			componentJavaScript.add("				xhraction = new ActiveXObject(\"Microsoft.XMLHTTP\");");
			componentJavaScript.add("			} catch (event2) {");
			componentJavaScript.add("				xhraction = false;");
			componentJavaScript.add("			}");
			componentJavaScript.add("		}");
			componentJavaScript.add("		if (!xhraction && typeof XMLHttpRequest !=\"undefined\") {");
			componentJavaScript.add("			xhraction = new XMLHttpRequest();");                 
			componentJavaScript.add("		}"); 	
			componentJavaScript.add("		var now = new Date();"); 
			componentJavaScript.add("		xhraction.open(\"post\", \"/web2cToolkit/Web2c?param=<action><sessionId>\" + sessionId + \"</sessionId><value>\" + inText + \"</value><id>\" + \"" + id + "\" + \"</id><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
			componentJavaScript.add("		xhraction.onreadystatechange = function() {};");   
			componentJavaScript.add("		xhraction.send(null);");  
			componentJavaScript.add("	}");
		}
		else {
			componentJavaScript.add("");
		}

		return componentJavaScript;
	}

	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {		
		
		String str = null;
		String identifier = "";
		String value = "";
		
		identifier = "<id>" + id + "</id>";
		value = "<value></value>";

		str = "<update>" + identifier + value + "</update>";
		
		if (!caption.get(0).equals("") && !caption.get(0).equals(" ")) {
			identifier = "<id>" + id + "_header" + "</id>";
			value = "<value>" + caption.get(0) + "</value>";

			str = str + "<update>" + identifier + value + "</update>";
		}
	
		return str;
	}
	
	/**
	 * Returns real data from common device access plug
	 * @return data (xml-type)
	 */
	public String getRealData() {		
		
		String str = null;
		String identifier = "";
		String value = "";
		String style = "";
		String blinking = "";
		String[] act = null;
		
		if (!extensionManifest.equals("")) {
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		
		identifier = "<id>" + id + "</id>";
		value = "<value></value>";
		style = getStyleUpdate();
		blinking = getBlinking(0);

		str = "<update>" + identifier + value + style + "</update>";
		
		if (!caption.get(0).equals("") && !caption.get(0).equals(" ")) {
			identifier = "<id>" + id + "_header" + "</id>";
			value = "<value>" + caption.get(0) + "</value>";

			str = str + "<update>" + identifier + value + style + blinking + "</update>";
		}
	
		return str;
	}
	
	/**
	 * Calls simulator
	 * @param action requested action
	 */	
	public void doSimulatedAction(String action) {
		
	}

	/**
	 * Sends client action requests to common device access plug
	 * @param action requested action
	 */	
	public void doRealAction(String action) {
		
		if (maxInputTextLength != 0) {
			if (maxInputTextLength > maxInputLength) maxInputTextLength = maxInputLength;
			if (action.length() > maxInputTextLength) {
				action = action.substring(0, maxInputTextLength);
			}
			else {
				for (int i = action.length() ; i < maxInputTextLength; i++) {
					action = action + " ";
				}
			}
		}
	
		action = characterEscaper.encodeString(action);
		commonDeviceAccess.execute(property.get(0).substring(0, property.get(0).length() - 2) + "(" + action + ")", true);
	}
}
