package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;

import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Static Label
 * @author R.Bacher, DESY
 */
public class Web2cLabel extends Web2cComponent {

	/**
	 * Static Label
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cLabel(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 1;
		registerMonitors(true);
		initSoundAndBlinking(1);
		
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			componentHTML.add("			<div id=\"" + id + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + ";  font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + "\">" + caption.get(0) + "</div>");
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			componentJavaScript.add("	//Label: " + id);
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		String value = "";
		String identifier = "";
	
		identifier = "<id>" + id + "</id>";		
		value = "<value>" + caption.get(0) + "</value>";
			
		str = "<update>" + identifier + value + "</update>";
		
		return str;
	}
	
	/**
	 * Returns real data from common device access plug
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		String style = "";
		String blinking = "";
		String[] act = null;
		
		if (!extensionManifest.equals("")) {
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
	
		identifier = "<id>" + id + "</id>";		
		value = "<value>" + caption.get(0) + "</value>";
		style = getStyleUpdate();
		blinking = getBlinking(0);
			
		str = "<update>" + identifier + value + style + blinking + "</update>";
		
		return str;
	}
}