package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;
import java.util.Iterator;

import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;


/**
 * List (Table) of Static Labels
 * @author R.Bacher, DESY
 */
public class Web2cLabelList extends Web2cComponent {

	/**
	 * List (Table) of Static Labels
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cLabelList(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		if (extensionManifest.equals("")) componentDataSize = caption.size();
		registerMonitors(true);
		initSoundAndBlinking(componentDataSize);
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {

		int i = 0;
		int xPos = 0;
		int yPos = 0;
		int cellHeight = 0;
		int cellWidth = 0;
		int headerCellHeight = 0;
		int headerCellWidth = 0;
		int yStep = 0;
		int rows = 0;
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		width = String.valueOf(Integer.valueOf(width.split("p")[0]) - 1) + "px";
		height = String.valueOf(Integer.valueOf(height.split("p")[0]) - 1) + "px";
		
		rows = caption.size();

		if (borderStyle.equals(solid)) {
			cellWidth = (Integer.valueOf(width.split("p")[0])) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
			if (!headerCaption.equals("") && !headerCaption.equals(" ")) {
				headerCellWidth = (Integer.valueOf(width.split("p")[0])) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
				headerCellHeight = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
				cellHeight = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) ;
			}
			else {
				cellHeight = (Integer.valueOf(height.split("p")[0]) / (rows)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows));
			}
		}
		else {	
			cellWidth = (Integer.valueOf(width.split("p")[0]));
			if (!headerCaption.equals("") && !headerCaption.equals(" ")) {
				cellHeight = (1 * Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));	
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows + 1));
			}
			else {
				cellHeight = (1 * Integer.valueOf(height.split("p")[0]) / (rows)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));	
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows));
			}
		}
		xPos = 0;
		yPos = 0;
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			componentHTML.add("			<div id=\"" + id + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + ";\">");	
			if (!headerCaption.equals("") && !headerCaption.equals(" ")) {
				componentHTML.add("			<div id=\"" + id + "_header" + "\" style=\"position:absolute; left:" + ((Integer)(xPos)).toString() + "px" + "; top:" + ((Integer)(yPos)).toString() + "px" + "; width:" + ((Integer)(headerCellWidth)).toString() + "px" + "; height:" + ((Integer)headerCellHeight).toString() + "px" + "; color:" + headerColor + "; background-color:" + headerBackgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + bold + "; text-align:" + center + "\">" + headerCaption + "</div>");
				yPos = yPos + yStep;
			}
			for (i = 0; i < rows; i++) {
				componentHTML.add("			<div id=\"" + id + "_cell" + ((Integer)i).toString() + "\" style=\"position:absolute; left:" + ((Integer)(xPos)).toString() + "px" + "; top:" + ((Integer)yPos).toString() + "px" + "; width:" + ((Integer)(cellWidth)).toString() + "px" + "; height:" + ((Integer)cellHeight).toString() + "px" + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + "\">" + caption.get(i) + "</div>");
				yPos = yPos + yStep;
			}		
			componentHTML.add("			</div>");
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			componentJavaScript.add("	//LabelList: " + id);
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		int i = -1;
		
		for (Iterator<String> it = caption.iterator(); it.hasNext();) {
			String item = it.next();
			i = i + 1;	
			identifier = "<id>" + id + "_cell" + ((Integer)i).toString() + "</id>";	
			value = "<value>" + item + "</value>";
		
			str = "<update>" + identifier + value + "</update>";
		}
		
		return str;
	}
	
	/**
	 * Returns real data from common device access plug
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String value = "";
		String identifier = "";
		int i = -1;
		String style = "";
		String blinking = "";
		String[] act = null; 

		if (!extensionManifest.equals("")) {
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int j = 0; j < act.length; j++) {
					commonDeviceAccess.execute(act[j], true);
				}
			}
		}

		for (Iterator<String> it = caption.iterator(); it.hasNext();) {
			String item = it.next();
			i = i + 1;	
			identifier = "<id>" + id + "_cell" + ((Integer)i).toString() + "</id>";	
			value = "<value>" + item + "</value>";
			style = getStyleUpdate();
			blinking = getBlinking(i);
		
			str = "<update>" + identifier + value + style + blinking + "</update>";
		}
		
		return str;
	}
}
