package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;
import java.util.Iterator;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cAnimatedImage;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cButton;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cHisto;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cImage;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cIndicator;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cIndicatorList;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cLabel;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cLabelList;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cSlider;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cText;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cTextList;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cTicker;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cTime;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cTrend;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cValue;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cValueList;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cWheelSwitch;

/**
 * Section
 * @author R.Bacher, DESY
 */
public class Web2cSection extends Web2cComponent {

	private ArrayList<Web2cComponent> sectionsElements = new ArrayList<Web2cComponent>();
	private ArrayList<ArrayList<String>> sectionsElementsHTML = new ArrayList<ArrayList<String>>();
	private ArrayList<ArrayList<String>> sectionsElementsJavaScript = new ArrayList<ArrayList<String>>();
	private String userName = null;
	private String referencePoint = null;
	private String web2cSession = null;
	private ArrayList<String> elementsComponentNames = null;
	private ArrayList<ArrayList<String>> elementsComponentAttributeNames = null;
	private ArrayList<ArrayList<String>> elementsComponentAttributeValues = null;
	private volatile Double[] constructionProgress = new Double[1];
	private String homeDir = null;

	/**
	 * Section
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param elementsNames list of names of components which belong to the actual section
	 * @param elementsAttributeNames list of valid attribute names of components which belong to the actual section
	 * @param elementsAttributeValues list of actual attribute values of components which belong to the actual section
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 * @param session Web2c session id
	 * @param refPoint coordinates (left upper corner) of reference component which component belongs to
	 * @param name actual user name
	 * @param access access mode
	 * @param progress construction progress
	 */
	public Web2cSection(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, ArrayList<String> elementsNames, ArrayList<ArrayList<String>> elementsAttributeNames, ArrayList<ArrayList<String>> elementsAttributeValues, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch, String session, String refPoint, String name, String access, Double[] progress) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 0;
		registerMonitors(true);
		initSoundAndBlinking(1);
		
		homeDir = home;
		web2cSession = session;
		referencePoint = refPoint;
		userName = name;
		elementsComponentNames = elementsNames;
		elementsComponentAttributeNames = elementsAttributeNames;
		elementsComponentAttributeValues = elementsAttributeValues;
		accessMode = access;
		constructionProgress = progress;
		
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		int xRef = 0;
		int yRef = 0;		
		int xLeft = 0;
		int yTop = 0;

		xRef = Integer.valueOf((referencePoint).split(",")[0]);
		yRef = Integer.valueOf((referencePoint).split(",")[1]);
		xLeft = Integer.valueOf(marginLeft.replace("px", ""));
		yTop = Integer.valueOf(marginTop.replace("px", ""));
		marginLeft = ((Integer)(xLeft + xRef)).toString() + "px";
		marginTop = ((Integer)(yTop + yRef)).toString() + "px";
		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}

		setSectionElements(elementsComponentNames, elementsComponentAttributeNames, elementsComponentAttributeValues);	
	}

	/**
	 * Returns a list of components which belong to the actual section
	 * @return list
	 */
	public ArrayList<Web2cComponent> getComponents() {
		
		ArrayList<Web2cComponent> list = new ArrayList<Web2cComponent>();
		for (Iterator<Web2cComponent> it = sectionsElements.iterator(); it.hasNext();) {
			list.add(it.next());
		}
		
		return list;
	}
	
	/**
	 * Returns component's HTML tags
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> sectionHTML = new ArrayList<String>();
		
		sectionHTML.add("		<div id=\"" + id + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "\">");
		for(Iterator<ArrayList<String>> it1 = sectionsElementsHTML.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				sectionHTML.add((String)it2.next());
			}
		}
		sectionHTML.add("		</div>");
		
		return sectionHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> sectionJavaScript = new ArrayList<String>();	
		
		sectionJavaScript.add("	//Section: " + id);
		for(Iterator<ArrayList<String>> it1 = sectionsElementsJavaScript.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				sectionJavaScript.add((String)it2.next());
			}
		}
		
		return sectionJavaScript;
	}
	
	private void setSectionElements(ArrayList<String> elementsNames, ArrayList<ArrayList<String>> elementsAttributeNames, ArrayList<ArrayList<String>> elementsAttributeValues) {

		double numberOfElements = 0;
		constructionProgress[0] = 0.0;
		Iterator<String> it3 = elementsNames.iterator();
		Iterator<ArrayList<String>> it2 = elementsAttributeValues.iterator();
		for (Iterator<ArrayList<String>> it1 = elementsAttributeNames.iterator(); it1.hasNext();) {
			String element = (String)it3.next();
			ArrayList<String> names = (ArrayList<String>)it1.next();
			ArrayList<String> values = (ArrayList<String>)it2.next();
			if (element == "Button") {
				Web2cButton value = new Web2cButton(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel, accessMode);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}			
			if (element == "Slider") {
				Web2cSlider value = new Web2cSlider(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}			
			if (element == "WheelSwitch") {
				Web2cWheelSwitch value = new Web2cWheelSwitch(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}			
			if (element == "Input") {
				Web2cInput value = new Web2cInput(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Label") {
				Web2cLabel value = new Web2cLabel(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "LabelList") {
				Web2cLabelList value = new Web2cLabelList(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Image") {
				Web2cImage value= new Web2cImage(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Text") {
				Web2cText value = new Web2cText(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "TextList") {
				Web2cTextList value = new Web2cTextList(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Value") {
				Web2cValue value = new Web2cValue(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "ValueList") {
				Web2cValueList value = new Web2cValueList(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "AnimatedImage") {
				Web2cAnimatedImage value = new Web2cAnimatedImage(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Indicator") {
				Web2cIndicator value = new Web2cIndicator(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "IndicatorList") {
				Web2cIndicatorList value = new Web2cIndicatorList(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Ticker") {
				Web2cTicker value = new Web2cTicker(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Trend") {
				Web2cTrend value = new Web2cTrend(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel, web2cSession);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Histo") {
				Web2cHisto value = new Web2cHisto(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel, web2cSession);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Video") {
				Web2cVideo value = new Web2cVideo(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, commonVideoConnector, chatChannel, web2cSession);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Time") {
				Web2cTime value = new Web2cTime(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			if (element == "Chat") {
				Web2cChat value = new Web2cChat(referenceId, homeDir, language, xScale, yScale, names, values, simulation, userPrivilege, commonDeviceAccess, null, chatChannel, userName);
				sectionsElements.add((Web2cComponent)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
				error = error + value.getError();
			}
			numberOfElements = numberOfElements + 1.0;
			constructionProgress[0] =  (numberOfElements / elementsAttributeNames.size()) * 100.0;
		}
	}

	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		String identifier = "";
		String value = "";

		identifier = "<id>" + id + "</id>";
		value = "<value></value>";	

		str = "<update>" + identifier + value + "</update>";
		
		return str;
	}

	/**
	 * Returns real data from common device access plug
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String identifier = "";
		String value = "";
		String style = "";
		String blinking = "";
		String[] act = null;

		if (!extensionManifest.equals("")) {
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}

		identifier = "<id>" + id + "</id>";
		value = "<value></value>";
		style = getStyleUpdate();
		blinking = getBlinking(0);

		str = "<update>" + identifier + value + style + blinking + "</update>";
		
		return str;
	}
}
