package de.desy.web2cToolkit.web2c.web2cComponents;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;

import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * @author R.Bacher, DESY
 */
public class Web2cSlider extends Web2cComponent {
	
	/**
	 * Slider
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cSlider(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		componentDataSize = 0;
		registerMonitors(true);
		initSoundAndBlinking(1);
		
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}
	
	/**
	 * Returns component's HTML tags
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {

		int componentWidth = 0;
		int componentHeight = 0;
		int componentLeft = 0;
		String subComponentLeft = "";
		String subComponentTop = "";
		String subComponentWidth= "";
		String subComponentHeight = "";
		int subComponentBorderWidth = 0;
		DecimalFormat formatter = null;

		formatter = new DecimalFormat("0.####E0", new DecimalFormatSymbols(localSettings));
		componentWidth = Integer.valueOf(width.substring(0, width.length() - 2));
		componentHeight = Integer.valueOf(height.substring(0, height.length() - 2));
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman)) {
			componentHTML.add("");
		}
		else if (userPrivilege.equals(expert) && superComponent.equals(yes)) {
			componentHTML.add("");
		}
		else if (userPrivilege.equals(supervisor) || (userPrivilege.equals(expert) && superComponent.equals(no))) {
			componentHTML.add("				<div id=\"" + id + "\" title=\"" + toolTipText + "\" onmouseup=\"" + id + "_mouseUp(event)\" onmousemove=\"" + id + "_mouseMove(event)\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "\">");	
				subComponentLeft = String.valueOf(componentLeft + (int)(0.05 * componentWidth)) + "px";
				componentLeft = Integer.valueOf(subComponentLeft.substring(0, subComponentLeft.length()-2));
				subComponentTop = String.valueOf((int)(0.40 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.20 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_buttonLeft" +  "\" title=\"\" value=\"<\" onclick=\"" + id + "_setSlider(-1)\" onmousemove=\"" + id + "_mouseMove(event)\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentBorderWidth = 2 * Integer.valueOf(thin.substring(0, thin.length() - 2));
				subComponentLeft = String.valueOf(componentLeft + (int)(0.05 * componentWidth) + 1) + "px";
				componentLeft = Integer.valueOf(subComponentLeft.substring(0, subComponentLeft.length()-2));
				subComponentTop = String.valueOf((int)(0.40 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.40 * componentWidth) - subComponentBorderWidth) + "px";
				subComponentHeight = String.valueOf((int)(0.20 * componentHeight) - subComponentBorderWidth) + "px";
			componentHTML.add("					<div id=\"" + id + "_sliderLeft" +  "\" title=\"\" onclick=\"" + id + "_setSlider(-10)\" onmousemove=\"" + id + "_mouseMove(event)\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; background-color:white; border-style:" + solid + "; border-width:" + thin + "; border-color:black\"></div>");	
				subComponentBorderWidth = 2 * Integer.valueOf(thin.substring(0, thin.length() - 2));
				subComponentLeft = String.valueOf(componentLeft + (int)(0.40 * componentWidth) + 1) + "px";
				componentLeft = Integer.valueOf(subComponentLeft.substring(0, subComponentLeft.length()-2));
				subComponentTop = String.valueOf((int)(0.35 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.02 * componentWidth) - subComponentBorderWidth) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight) - subComponentBorderWidth) + "px";
			componentHTML.add("					<div id=\"" + id + "_sliderCenter" +  "\" title=\"\" onmousedown=\"" + id + "_mouseDown(event)\" onmousemove=\"" + id + "_mouseMove(event)\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; background-color:gray; border-style:" + solid + "; border-width:" + thin + "; border-color:black\"></div>");
				subComponentBorderWidth = 2 * Integer.valueOf(thin.substring(0, thin.length() - 2));
				subComponentLeft = String.valueOf(componentLeft + (int)(0.02 * componentWidth) + 1) + "px";
				componentLeft = Integer.valueOf(subComponentLeft.substring(0, subComponentLeft.length()-2));
				subComponentTop = String.valueOf((int)(0.40 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.40 * componentWidth) - subComponentBorderWidth) + "px";
				subComponentHeight = String.valueOf((int)(0.20 * componentHeight) - subComponentBorderWidth) + "px";
			componentHTML.add("					<div id=\"" + id + "_sliderRight" +  "\" title=\"\" onclick=\"" + id + "_setSlider(10)\" onmousemove=\"" + id + "_mouseMove(event)\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; background-color:white; border-style:" + solid + "; border-width:" + thin + "; border-color:black\"></div>");
				subComponentLeft = String.valueOf(componentLeft + (int)(0.40 * componentWidth) + 1) + "px";
				componentLeft = Integer.valueOf(subComponentLeft.substring(0, subComponentLeft.length()-2));
				subComponentTop = String.valueOf((int)(0.40 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.20 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_buttonRight" +  "\" title=\"\" value=\">\" onclick=\"" + id + "_setSlider(1)\" onmousemove=\"" + id + "_mouseMove(event)\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.10 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.70 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.25 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.20 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_minimumValue" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:left\">" + formatter.format(Double.valueOf(minimumValue)) + "</div>");
				subComponentLeft = String.valueOf((int)(0.67 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.70 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.25 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.20 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_maximumValue" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:right\">" + formatter.format(Double.valueOf(maximumValue)) + "</div>");
				subComponentLeft = String.valueOf((int)(0.40 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.70 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.20 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.20 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_actualValue" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + formatter.format(((Double.valueOf(maximumValue) - Double.valueOf(minimumValue)) / 2.0)) + "</div>");
				subComponentLeft = String.valueOf((int)(0.10 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.10 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.35 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.20 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_caption" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + caption.get(0) + "</div>");
				subComponentLeft = String.valueOf((int)(0.60 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.10 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.20 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.20 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_setValue" +  "\" title=\"\" value=\"set Value\" onclick=\"" + id + "_setValue()\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
			componentHTML.add("				</div>");
		}
		else {
			componentHTML.add("");
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();

		if (userPrivilege.equals(layman)) {
			componentJavaScript.add("");
		}
		else if (userPrivilege.equals(expert) && superComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else if (userPrivilege.equals(supervisor) || (userPrivilege.equals(expert) && superComponent.equals(no))) {
			componentJavaScript.add("	//Slider: " + id); 
			
			componentJavaScript.add("	var " + id + "_sliderMove=false;"); 
			componentJavaScript.add("	var " + id + "_sliderCenterLeftStart;"); 
			componentJavaScript.add("	var " + id + "_sliderRightLeftStart;"); 
			componentJavaScript.add("	var " + id + "_sliderLeftLeftStart;"); 
			componentJavaScript.add("	var " + id + "_sliderCenterWidthStart;"); 
			componentJavaScript.add("	var " + id + "_sliderLeftWidthStart;"); 
			componentJavaScript.add("	var " + id + "_sliderRightWidthStart;"); 
			componentJavaScript.add("	var " + id + "_mouseStart;");  
			componentJavaScript.add("	var " + id + "_stepSize;"); 
			componentJavaScript.add("	var " + id + "_sliderLeft;");
			componentJavaScript.add("	var " + id + "_sliderRight;");
			componentJavaScript.add("	var " + id + "_sliderCenter;");
			componentJavaScript.add("	var " + id + "_actualValue;");
			componentJavaScript.add("	var " + id + "_minimumValue;");
			componentJavaScript.add("	var " + id + "_maximumValue;"); 
			componentJavaScript.add("	var " + id + "_buttonLeft;");
			componentJavaScript.add("	var " + id + "_buttonRight;");
			
			componentJavaScript.add("	function " + id + "_initSlider() {;");
			componentJavaScript.add("		" + id + "_sliderLeft=document.getElementById(\"" + id + "_sliderLeft\");");
			componentJavaScript.add("		" + id + "_sliderRight=document.getElementById(\"" + id + "_sliderRight\");");
			componentJavaScript.add("		" + id + "_sliderCenter=document.getElementById(\"" + id + "_sliderCenter\");");
			componentJavaScript.add("		" + id + "_actualValue=document.getElementById(\"" + id + "_actualValue\");");
			componentJavaScript.add("		" + id + "_minimumValue=document.getElementById(\"" + id + "_minimumValue\");");
			componentJavaScript.add("		" + id + "_maximumValue=document.getElementById(\"" + id + "_maximumValue\");");
			componentJavaScript.add("		" + id + "_buttonRight=document.getElementById(\"" + id + "_buttonRight\");");
			componentJavaScript.add("		" + id + "_buttonLeft=document.getElementById(\"" + id + "_buttonLeft\");");
			componentJavaScript.add("		" + id + "_sliderCenterLeftStart=parseInt(" + id + "_sliderCenter.style.left.substring(0," + id + "_sliderCenter.style.left.length-2));");
			componentJavaScript.add("		" + id + "_sliderRightLeftStart=parseInt(" + id + "_sliderRight.style.left.substring(0," + id + "_sliderRight.style.left.length-2));");
			componentJavaScript.add("		" + id + "_sliderLeftLeftStart=parseInt(" + id + "_sliderLeft.style.left.substring(0," + id + "_sliderLeft.style.left.length-2));");
			componentJavaScript.add("		" + id + "_sliderCenterWidthStart=parseInt(" + id + "_sliderCenter.style.width.substring(0," + id + "_sliderCenter.style.width.length-2));");
			componentJavaScript.add("		" + id + "_sliderRightWidthStart=parseInt(" + id + "_sliderRight.style.width.substring(0," + id + "_sliderRight.style.width.length-2));");
			componentJavaScript.add("		" + id + "_sliderLeftWidthStart=parseInt(" + id + "_sliderLeft.style.width.substring(0," + id + "_sliderLeft.style.width.length-2));");
			componentJavaScript.add("		" + id + "_minimumValue.innerHTML=String(parseFloat(parseFloat(" + id + "_minimumValue.innerHTML).toPrecision(5)).toExponential()).replace(\"e\", \"E\").replace(\"+\", \"\");");
			componentJavaScript.add("		" + id + "_maximumValue.innerHTML=String(parseFloat(parseFloat(" + id + "_maximumValue.innerHTML).toPrecision(5)).toExponential()).replace(\"e\", \"E\").replace(\"+\", \"\");");
			componentJavaScript.add("	}"); 
			
			componentJavaScript.add("	function " + id + "_setSlider(stepSize) {;");	
			componentJavaScript.add("		var browser = navigator.appName;");
			componentJavaScript.add("		" + id + "_initSlider();");
			componentJavaScript.add("		" + id + "_stepSize = stepSize;"); 
			componentJavaScript.add("		if (((" + id + "_sliderCenterLeftStart+" + id + "_stepSize) > (" + id + "_sliderLeftLeftStart + 2)) && ((" + id + "_sliderCenterLeftStart+" + id + "_stepSize) < (" + id + "_sliderRightLeftStart+" + id + "_sliderRightWidthStart +2-(" + id + "_sliderCenterWidthStart+2) -2))) {");
			componentJavaScript.add("			" + id + "_sliderCenter.style.left=String(" + id + "_sliderCenterLeftStart+" + id + "_stepSize) + \"px\";");
			componentJavaScript.add("			" + id + "_sliderLeft.style.width=String(" + id + "_sliderLeftWidthStart+" + id + "_stepSize) + \"px\";");
			componentJavaScript.add("			" + id + "_sliderRight.style.left=String(" + id + "_sliderRightLeftStart+" + id + "_stepSize) + \"px\";");
			componentJavaScript.add("			" + id + "_sliderRight.style.width=String(" + id + "_sliderRightWidthStart-" + id + "_stepSize) + \"px\";");
			componentJavaScript.add("			if (browser == \"Microsoft Internet Explorer\") {");
			componentJavaScript.add("				" + id + "_actualValue.innerHTML=String(parseFloat(parseFloat(String(parseFloat(\"" + Double.valueOf(minimumValue) + "\")+((parseFloat(\"" + Double.valueOf(maximumValue) + "\")-parseFloat(\"" + Double.valueOf(minimumValue) + "\"))/(" + id + "_sliderLeftWidthStart+" + id + "_sliderRightWidthStart-(4*" + thin.substring(0, thin.length()-2) + "))*(" + id + "_sliderLeftWidthStart-(2*" + thin.substring(0, thin.length()-2) + ")+" + id + "_stepSize)))).toPrecision(5)).toExponential()).replace(\"e\", \"E\").replace(\"+\", \"\");");
			componentJavaScript.add("			}");
			componentJavaScript.add("			if ((browser == \"Netscape\") || (browser == \"Opera\")) {");
			componentJavaScript.add("				" + id + "_actualValue.innerHTML=String(parseFloat(parseFloat(String(parseFloat(\"" + Double.valueOf(minimumValue) + "\")+((parseFloat(\"" + Double.valueOf(maximumValue) + "\")-parseFloat(\"" + Double.valueOf(minimumValue) + "\"))/(" + id + "_sliderLeftWidthStart+" + id + "_sliderRightWidthStart)*(" + id + "_sliderLeftWidthStart+" + id + "_stepSize)))).toPrecision(5)).toExponential()).replace(\"e\", \"E\").replace(\"+\", \"\");");
			componentJavaScript.add("			}");
			componentJavaScript.add("		}");
			componentJavaScript.add("	}"); 
			
			componentJavaScript.add("	function " + id + "_mouseDown(event) {"); 
			componentJavaScript.add("		if (!event) event=window.event;"); 
			componentJavaScript.add("		" + id + "_initSlider();");
			componentJavaScript.add("		" + id + "_mouseStart=event.clientX;");
			componentJavaScript.add("		" + id + "_sliderMove=true;");  
			componentJavaScript.add("	}");    
			
			componentJavaScript.add("	function " + id + "_mouseUp(event) {");
			componentJavaScript.add("		if (!event) event=window.event;");         
			componentJavaScript.add("		" + id + "_sliderMove=false;");
			componentJavaScript.add("	}");     
			
			componentJavaScript.add("	function " + id + "_mouseMove(event) {"); 	
			componentJavaScript.add("		var browser = navigator.appName;");
			componentJavaScript.add("		if (!event) event=window.event;");
			componentJavaScript.add("		if (" + id + "_sliderMove==true) {");
			componentJavaScript.add("			var " + id + "_mouseDiff=event.clientX - " + id + "_mouseStart;");
			componentJavaScript.add("			if (((" + id + "_sliderCenterLeftStart+" + id + "_mouseDiff) > (" + id + "_sliderLeftLeftStart + 2)) && ((" + id + "_sliderCenterLeftStart+" + id + "_mouseDiff) < (" + id + "_sliderRightLeftStart+" + id + "_sliderRightWidthStart+2-(" + id + "_sliderCenterWidthStart+2) - 2))) {");
			componentJavaScript.add("				" + id + "_sliderCenter.style.left=String(" + id + "_sliderCenterLeftStart+" + id + "_mouseDiff) + \"px\";");
			componentJavaScript.add("				" + id + "_sliderLeft.style.width=String(" + id + "_sliderLeftWidthStart+" + id + "_mouseDiff) + \"px\";");
			componentJavaScript.add("				" + id + "_sliderRight.style.left=String(" + id + "_sliderRightLeftStart+" + id + "_mouseDiff) + \"px\";");
			componentJavaScript.add("				" + id + "_sliderRight.style.width=String(" + id + "_sliderRightWidthStart-" + id + "_mouseDiff) + \"px\";");
			componentJavaScript.add("				if (browser == \"Microsoft Internet Explorer\") {");
			componentJavaScript.add("					" + id + "_actualValue.innerHTML=String(parseFloat(parseFloat(String(parseFloat(\"" + Double.valueOf(minimumValue) + "\")+((parseFloat(\"" + Double.valueOf(maximumValue) + "\")-parseFloat(\"" + Double.valueOf(minimumValue) + "\"))/(" + id + "_sliderLeftWidthStart+" + id + "_sliderRightWidthStart-(4*" + thin.substring(0, thin.length()-2) + "))*(" + id + "_sliderLeftWidthStart-(2*" + thin.substring(0, thin.length()-2) + ")+" + id + "_mouseDiff)))).toPrecision(5)).toExponential()).replace(\"e\", \"E\").replace(\"+\", \"\");");
			componentJavaScript.add("				}");
			componentJavaScript.add("				if ((browser == \"Netscape\") || (browser == \"Opera\")) {");
			componentJavaScript.add("					" + id + "_actualValue.innerHTML=String(parseFloat(parseFloat(String(parseFloat(\"" + Double.valueOf(minimumValue) + "\")+((parseFloat(\"" + Double.valueOf(maximumValue) + "\")-parseFloat(\"" + Double.valueOf(minimumValue) + "\"))/(" + id + "_sliderLeftWidthStart+" + id + "_sliderRightWidthStart)*(" + id + "_sliderLeftWidthStart+" + id + "_mouseDiff)))).toPrecision(5)).toExponential()).replace(\"e\", \"E\").replace(\"+\", \"\");");
			componentJavaScript.add("				}");
			componentJavaScript.add("			}");
			componentJavaScript.add("		}");
			componentJavaScript.add("	}");    
	
			componentJavaScript.add("	function " + id + "_setValue() {");
			componentJavaScript.add("		var xhraction;");
			componentJavaScript.add("		try {");                                                  
			componentJavaScript.add("			xhraction = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
			componentJavaScript.add("		} catch (event1) {");                             
			componentJavaScript.add("			try {");
			componentJavaScript.add("				xhraction = new ActiveXObject(\"Microsoft.XMLHTTP\");");
			componentJavaScript.add("			} catch (event2) {");
			componentJavaScript.add("				xhraction = false;");
			componentJavaScript.add("			}");
			componentJavaScript.add("		}");
			componentJavaScript.add("		if (!xhraction && typeof XMLHttpRequest !=\"undefined\") {");
			componentJavaScript.add("			xhraction = new XMLHttpRequest();");                 
			componentJavaScript.add("		}"); 	
			componentJavaScript.add("		var now = new Date();"); 
			componentJavaScript.add("		" + id + "_initSlider();");
			componentJavaScript.add("		xhraction.open(\"post\", \"/web2cToolkit/Web2c?param=<action><sessionId>\" + sessionId + \"</sessionId><value>\" + " + id + "_actualValue.innerHTML" + " + \"</value><id>\" + \"" + id + "\" + \"</id><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
			componentJavaScript.add("		xhraction.onreadystatechange = function() {};");   
			componentJavaScript.add("		xhraction.send(null);");  
			componentJavaScript.add("	}");
		}
		else {
			componentJavaScript.add("");
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {			
		
		String str = null;
		String identifier = "";
		String value = "";

		value = "<value></value>";
		identifier = "<id>" + id + "</id>";
		str = "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_minimumValue" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_maximumValue" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_actualValue" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_caption" + "</id>";
		value = "<value>" + caption.get(0) + "</value>";
		str = str + "<update>" + identifier + value + "</update>";

		value = "<value></value>";
		identifier = "<id>" + id + "_buttonLeft" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_buttonRight" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_setValue" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
	
		return str;
	}
	
	/**
	 * Returns real data from common device access plug
	 * @return data (xml-type)
	 */
	public String getRealData() {		
		
		String str = null;
		String identifier = "";
		String value = "";
		String style = "";
		String blinking = "";
		String[] act = null;

		if (!extensionManifest.equals("")) {
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}

		blinking = getBlinking(0);
		style = getStyleUpdate();
		value = "<value></value>";
		identifier = "<id>" + id + "</id>";
		str = "<update>" + identifier + value + style + blinking + "</update>";
		
		identifier = "<id>" + id + "_minimumValue" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_maximumValue" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_actualValue" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";

		value = "<value>" + caption.get(0) + "</value>";
		identifier = "<id>" + id + "_caption" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
				
		String[] styleArray = style.split("</style>");
		style = "";
		for (int i = 0; i < styleArray.length; i++) {
			if (!styleArray[i].contains("background-color")) style = style + styleArray[i] + "</style>";
		}
		value = "<value></value>";
		identifier = "<id>" + id + "_buttonLeft" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_buttonRight" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_setValue" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
	
		return str;
	}	
	
	/**
	 * Calls simulator
	 * @param action requested action
	 */	
	public void doSimulatedAction(String action) {
		
	}

	/**
	 * Sends client action requests to common device access plug
	 * @param action requested action
	 */	
	public void doRealAction(String action) {

		commonDeviceAccess.execute(property.get(0).substring(0, property.get(0).length() - 2) + "(" + characterEscaper.encodeString(action) + ")", true);
	}
}
