package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * List (Table) of Text Labels
 * @author R.Bacher, DESY
 */
public class Web2cTextList extends Web2cComponent{
	
	private Random randomGenerator = null;

	/**
	 * List (Table) of Text Labels
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cTextList(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);
		
		if (extensionManifest.equals("")) componentDataSize = property.size();
		registerMonitors(false);
		initSoundAndBlinking(componentDataSize);

		randomGenerator = new Random();		
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		int i = 0;
		int xPos = 0;
		int yPos = 0;
		int cellHeight = 0;
		int cellWidth = 0;
		int yStep = 0;
		int rows = 0;
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		width = String.valueOf(Integer.valueOf(width.split("p")[0]) - 1) + "px";
		height = String.valueOf(Integer.valueOf(height.split("p")[0]) - 1) + "px";
		
		rows = property.size();

		if (borderStyle.equals(solid)) {
			cellWidth = (Integer.valueOf(width.split("p")[0])) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
			if (headerCaption != null) {
				cellHeight = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
			}
			else {
				cellHeight = (Integer.valueOf(height.split("p")[0]) / (rows)) - (2 * Integer.valueOf(borderWidth.split("p")[0]));
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
			}
		}
		else {	
			cellWidth = (Integer.valueOf(width.split("p")[0]));
			if (headerCaption != null) {
				cellHeight = (1 * Integer.valueOf(height.split("p")[0]) / (rows + 1));	
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows + 1)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
			}
			else {
				cellHeight = (1 * Integer.valueOf(height.split("p")[0]) / (rows));	
				yStep = (Integer.valueOf(height.split("p")[0]) / (rows)) - (1 * Integer.valueOf(borderWidth.split("p")[0]));
			}
		}
		xPos = 0;
		yPos = 0;
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentHTML.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentHTML.add("			<div id=\"" + id + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "\">");	
				if (headerCaption != null) {
					componentHTML.add("			<div id=\"" + id + "_header" + "\" title=\"" + toolTipText + "\" style=\"position:absolute; left:" + ((Integer)(xPos)).toString() + "px" + "; top:" + ((Integer)(yPos)).toString() + "px" + "; width:" + ((Integer)(cellWidth)).toString() + "px" + "; height:" + ((Integer)cellHeight).toString() + "px" + "; color:" + headerColor + "; background-color:" + headerBackgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + bold + "; text-align:" + center + "\">" + headerCaption + "</div>");
					yPos = yPos + yStep;
				}
				for (i = 0; i < rows; i++) {
					componentHTML.add("			<div id=\"" + id + "_cell" + ((Integer)i).toString() + "\" title=\"" + toolTipText + "\" style=\"position:absolute; left:" + ((Integer)(xPos)).toString() + "px" + "; top:" + ((Integer)yPos).toString() + "px" + "; width:" + ((Integer)(cellWidth)).toString() + "px" + "; height:" + ((Integer)cellHeight).toString() + "px" + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + "\">" + "xxx" + "</div>");
					yPos = yPos + yStep;
				}		
				componentHTML.add("			</div>");
			}
			else {
				componentHTML.add("");
			}
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman) && expertComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else {	
			if (propertyLinksEstablished == true) {
				componentJavaScript.add("	//TextList: " + id);
			}
			else {
				componentJavaScript.add("");
			}
		}

		return componentJavaScript;
	}
	
	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {
		
		String str = "";
		double number = 0.0;
		int i = -1;
		String value = "";
		String identifier = "";
		String soundAndBlinking = "";
		String style = "";

		for (Iterator<String> it = property.iterator(); it.hasNext(); it.next()) {
			number = (double)randomGenerator.nextInt(10);
			i = i + 1;		
		    identifier = "<id>" + id + "_cell" + ((Integer)i).toString() + "</id>";
		    value = "<value>" + String.valueOf(number) + "</value>";
			if (!String.valueOf(number).equals(text[i])) soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, i);
			else soundAndBlinking = getConditionedSoundAndBlinkingUpdate(false, i);
			style = getStyleUpdate();				
			str = str + "<update>" + identifier + value + style + soundAndBlinking + "</update>";
			text[i] = String.valueOf(number);
		}

		return str;
	}

	/**
	 * Returns real data from common device access plug or from executing the extension code
	 * @return data (xml-type)
	 */
	public String getRealData() {
		
		String str = "";
		String[] ret = new String[componentDataSize];
		String value = "";
		String identifier = "";
		String soundAndBlinking = "";
		String style = "";
		String[] act = null;

		if (!extensionManifest.equals("")) {
			if (extensionPropertyLinksEstablished == true) {
				extensionExecutor.executeExtensionCode("live");
				for (int i = 0; i < ret.length; i++) {
					ret[i] = (String)(extensionExecutor.getData()[i]);
					if (ret[i].contains("<value></value>")) {
						ret[i] = ret[i].replace("<value></value>", "<value> </value>");
					}
					ret[i] = ret[i].split(">")[1].split("<")[0];
				}
			}
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}
		else {
			if (propertyLinksEstablished == true) {
				for (int i = 0; i < ret.length; i++) {
					String address = (property.get(i));
					ret[i] = (commonDeviceAccess.getMonitorData(address));	
					if (ret[i].contains("<value></value>")) {
						ret[i] = ret[i].replace("<value></value>", "<value> </value>");
					}
					ret[i] = ret[i].split(">")[1].split("<")[0];
				}
			}
		}
		
		if (extensionPropertyLinksEstablished == true || propertyLinksEstablished == true) {
			for (int i = 0; i < ret.length; i++) {
			    identifier = "<id>" + id + "_cell" + ((Integer)i).toString() + "</id>";
			    value = "<value>" + ret[i] + "</value>";
				if (!ret[i].equals(text[i])) soundAndBlinking = getConditionedSoundAndBlinkingUpdate(true, i);
				else soundAndBlinking =getConditionedSoundAndBlinkingUpdate(false, i);
				style = getStyleUpdate();				
				str = str + "<update>" + identifier + value + style + soundAndBlinking + "</update>";
				text[i] = ret[i];
			}
		}

		return str;
	}
}
