package de.desy.web2cToolkit.web2c.web2cComponents;

import java.util.ArrayList;

import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatChannel;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;

/**
 * Wheel Switch (Dial)
 * @author R.Bacher, DESY
 */
public class Web2cWheelSwitch extends Web2cComponent {
	
	/**
	 * Wheel Switch (Dial)
	 * @param refId ID of reference component which component belongs to
	 * @param home directory which contains user-specific files
	 * @param lang language
	 * @param x horizontal scaling factor
	 * @param y vertical scaling factor
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 * @param simulatedData flag for simulated or real data
	 * @param privilege actual user privilege
	 * @param cda actual common device access instance
	 * @param cvc actual common video connector instance
	 * @param ch actual chat channel instance
	 */
	public Web2cWheelSwitch(String refId, String home, int lang, double x, double y, ArrayList<String> names, ArrayList<String> values, boolean simulatedData, String privilege, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc, Web2cChatChannel ch) {

		super(refId, home, lang, x, y, names, values, simulatedData, privilege, cda, cvc, ch);	
		
		componentDataSize = 0;
		registerMonitors(true);
		initSoundAndBlinking(1);
		
		initComponent();
	}
	
	/**
	 * Initializes the component
	 */
	public void initComponent() {
		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}
	
	/**
	 * Returns component's HTML tags
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {

		int componentWidth = 0;
		int componentHeight = 0;
		String subComponentLeft = "";
		String subComponentTop = "";
		String subComponentWidth= "";
		String subComponentHeight = "";
		
		componentWidth = Integer.valueOf(width.substring(0, width.length() - 2));
		componentHeight = Integer.valueOf(height.substring(0, height.length() - 2));
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman)) {
			componentHTML.add("");
		}
		else if (userPrivilege.equals(expert) && superComponent.equals(yes)) {
			componentHTML.add("");
		}
		else if (userPrivilege.equals(supervisor) || (userPrivilege.equals(expert) && superComponent.equals(no))) {
			componentHTML.add("				<div id=\"" + id + "\" title=\"" + toolTipText + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "\">");	
				subComponentLeft = String.valueOf((int)(0.24 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_mantissaSign" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "+" + "</div>");
				subComponentLeft = String.valueOf((int)(0.31 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_mantissa0" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "0" + "</div>");
				subComponentLeft = String.valueOf((int)(0.38 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
				subComponentWidth = String.valueOf((int)(0.04 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_mantissaSeparator" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "." + "</div>");
				subComponentLeft = String.valueOf((int)(0.44 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_mantissa1" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "0" + "</div>");
			 	subComponentLeft = String.valueOf((int)(0.51 * componentWidth)) + "px";
			 	subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
			 	subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
			 	subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_mantissa2" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "0" + "</div>");
				subComponentLeft = String.valueOf((int)(0.58 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_mantissa3" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "0" + "</div>");			
				subComponentLeft = String.valueOf((int)(0.65 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_mantissa4" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "0" + "</div>");
				subComponentLeft = String.valueOf((int)(0.72 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_exponent" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "E" + "</div>");
				subComponentLeft = String.valueOf((int)(0.79 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_exponentSign" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "+" + "</div>");
			 	subComponentLeft = String.valueOf((int)(0.86 * componentWidth)) + "px";
			 	subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
			 	subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
			 	subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_exponent0" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "0" + "</div>");
				subComponentLeft = String.valueOf((int)(0.93 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.50 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_exponent1" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + "0" + "</div>");
				subComponentLeft = String.valueOf((int)(0.02 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.17 * componentHeight) - Integer.valueOf(fontSize.substring(0, fontSize.length() - 2)) / 2) + "px";
				subComponentWidth = String.valueOf((int)(0.20 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<div id=\"" + id + "_caption" +  "\" title=\"\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">" + caption.get(0) + "</div>");			
				subComponentLeft = String.valueOf((int)(0.02 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.68 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.20 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_setValue" +  "\" title=\"\" value=\"set Value\" onclick=\"" + id + "_setValue()\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");		
				subComponentLeft = String.valueOf((int)(0.24 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.02 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissaPlus" +  "\" title=\"\" value=\"+\" onclick=\"" + id + "_plus(" + id + "_mantissaSign" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.24 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.68 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissaMinus" +  "\" title=\"\" value=\"-\" onclick=\"" + id + "_minus(" + id + "_mantissaSign" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");	
				subComponentLeft = String.valueOf((int)(0.31 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.02 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissa0Up" +  "\" title=\"\" value=\">\" onclick=\"" + id + "_up(" + id + "_mantissa0" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.31 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.68 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissa0Down" +  "\" title=\"\" value=\"<\" onclick=\"" + id + "_down(" + id + "_mantissa0" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");	
				subComponentLeft = String.valueOf((int)(0.44 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.02 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissa1Up" +  "\" title=\"\" value=\">\" onclick=\"" + id + "_up(" + id + "_mantissa1" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.44 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.68 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissa1Down" +  "\" title=\"\" value=\"<\" onclick=\"" + id + "_down(" + id + "_mantissa1" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");	
				subComponentLeft = String.valueOf((int)(0.51 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.02 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissa2Up" +  "\" title=\"\" value=\">\" onclick=\"" + id + "_up(" + id + "_mantissa2" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.51 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.68 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissa2Down" +  "\" title=\"\" value=\"<\" onclick=\"" + id + "_down(" + id + "_mantissa2" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");	
				subComponentLeft = String.valueOf((int)(0.58 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.02 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissa3Up" +  "\" title=\"\" value=\">\" onclick=\"" + id + "_up(" + id + "_mantissa3" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.58 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.68 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissa3Down" +  "\" title=\"\" value=\"<\" onclick=\"" + id + "_down(" + id + "_mantissa3" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");	
				subComponentLeft = String.valueOf((int)(0.65 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.02 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissa4Up" +  "\" title=\"\" value=\">\" onclick=\"" + id + "_up(" + id + "_mantissa4" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.65 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.68 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_mantissa4Down" +  "\" title=\"\" value=\"<\" onclick=\"" + id + "_down(" + id + "_mantissa4" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.79 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.02 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_exponentPlus" +  "\" title=\"\" value=\"+\" onclick=\"" + id + "_plus(" + id + "_exponentSign" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.79 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.68 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_exponentMinus" +  "\" title=\"\" value=\"-\" onclick=\"" + id + "_minus(" + id + "_exponentSign" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");	
				subComponentLeft = String.valueOf((int)(0.86 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.02 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_exponent0Up" +  "\" title=\"\" value=\">\" onclick=\"" + id + "_up(" + id + "_exponent0" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.86 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.68 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_exponent0Down" +  "\" title=\"\" value=\"<\" onclick=\"" + id + "_down(" + id + "_exponent0" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");	
				subComponentLeft = String.valueOf((int)(0.93 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.02 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_exponent1Up" +  "\" title=\"\" value=\">\" onclick=\"" + id + "_up(" + id + "_exponent1" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");
				subComponentLeft = String.valueOf((int)(0.93 * componentWidth)) + "px";
				subComponentTop = String.valueOf((int)(0.68 * componentHeight)) + "px";
				subComponentWidth = String.valueOf((int)(0.05 * componentWidth)) + "px";
				subComponentHeight = String.valueOf((int)(0.30 * componentHeight)) + "px";
			componentHTML.add("					<input type=\"button\" id=\"" + id + "_exponent1Down" +  "\" title=\"\" value=\"<\" onclick=\"" + id + "_down(" + id + "_exponent1" + ")\" style=\"position:absolute; left:" + subComponentLeft + "; top:" + subComponentTop + "; width:" + subComponentWidth + "; height:" + subComponentHeight + "; border-style:outset; color:" + color + "; background-color:gray; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:center\">");	
			componentHTML.add("				</div>");
		}
		else {
			componentHTML.add("");
		}
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		if (userPrivilege.equals(layman)) {
			componentJavaScript.add("");
		}
		else if (userPrivilege.equals(expert) && superComponent.equals(yes)) {
			componentJavaScript.add("");
		}
		else if (userPrivilege.equals(supervisor) || (userPrivilege.equals(expert) && superComponent.equals(no))) {
			componentJavaScript.add("	//WheelSwitch: " + id); 
			
			componentJavaScript.add("	function " + id + "_up(digit) {");
			componentJavaScript.add("		if (digit.innerHTML==\"9\") {");
			componentJavaScript.add("			digit.innerHTML=\"8\";");
			componentJavaScript.add("		}");
			componentJavaScript.add("		digit.innerHTML=String(parseInt(digit.innerHTML) + 1 );");
			componentJavaScript.add("	}");
			
			componentJavaScript.add("	function " + id + "_down(digit) {");
			componentJavaScript.add("		if (digit.innerHTML==\"0\") {");
			componentJavaScript.add("			digit.innerHTML=\"1\";");
			componentJavaScript.add("		}");
			componentJavaScript.add("		digit.innerHTML=String(parseInt(digit.innerHTML) - 1 );");
			componentJavaScript.add("	}");
	
			componentJavaScript.add("	function " + id + "_plus(sign) {");
			componentJavaScript.add("		sign.innerHTML=\"+\";");
			componentJavaScript.add("	}");
			
			componentJavaScript.add("	function " + id + "_minus(sign) {");
			componentJavaScript.add("		sign.innerHTML=\"-\";");
			componentJavaScript.add("	}");
	
			componentJavaScript.add("	function " + id + "_setValue() {");
			componentJavaScript.add("		var xhraction;");
			componentJavaScript.add("		try {");                                                  
			componentJavaScript.add("			xhraction = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
			componentJavaScript.add("		} catch (event1) {");                             
			componentJavaScript.add("			try {");
			componentJavaScript.add("				xhraction = new ActiveXObject(\"Microsoft.XMLHTTP\");");
			componentJavaScript.add("			} catch (event2) {");
			componentJavaScript.add("				xhraction = false;");
			componentJavaScript.add("			}");
			componentJavaScript.add("		}");
			componentJavaScript.add("		if (!xhraction && typeof XMLHttpRequest !=\"undefined\") {");
			componentJavaScript.add("			xhraction = new XMLHttpRequest();");                 
			componentJavaScript.add("		}"); 	
			componentJavaScript.add("		var now = new Date();"); 
			componentJavaScript.add("		var " + id + "_mantissaPlus = document.getElementById(\"" + id + "_mantissaPlus\");");
			componentJavaScript.add("		var " + id + "_mantissa0 = document.getElementById(\"" + id + "_mantissa0\");");
			componentJavaScript.add("	 	var " + id + "_mantissa1 = document.getElementById(\"" + id + "_mantissa1\");");
			componentJavaScript.add("		var " + id + "_mantissa2 = document.getElementById(\"" + id + "_mantissa2\");");
			componentJavaScript.add("		var " + id + "_mantissa3 = document.getElementById(\"" + id + "_mantissa3\");");
			componentJavaScript.add("		var " + id + "_mantissa4 = document.getElementById(\"" + id + "_mantissa4\");");
			componentJavaScript.add("		var " + id + "_exponentSign = document.getElementById(\"" + id + "_exponentSign\");");
			componentJavaScript.add("		var " + id + "_exponent0 = document.getElementById(\"" + id + "_exponent0\");");
			componentJavaScript.add("		var " + id + "_exponent1= document.getElementById(\"" + id + "_exponent1\");");
			componentJavaScript.add("		var actualValue = parseFloat(" + id + "_mantissaPlus.innerHTML + " + id + "_mantissa0.innerHTML + \".\" + " + id + "_mantissa1.innerHTML + " + id + "_mantissa2.innerHTML + " + id + "_mantissa3.innerHTML + " + id + "_mantissa4.innerHTML + \"E\" + " + id + "_exponentSign.innerHTML + " + id + "_exponent0.innerHTML + " + id + "_exponent1.innerHTML);");
			componentJavaScript.add("		xhraction.open(\"post\", \"/web2cToolkit/Web2c?param=<action><sessionId>\" + sessionId + \"</sessionId><value>\" + String(actualValue) + \"</value><id>\" + \"" + id + "\" + \"</id><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
			componentJavaScript.add("		xhraction.onreadystatechange = function() {};");   
			componentJavaScript.add("		xhraction.send(null);");  
			componentJavaScript.add("	}");
		}
		else {
			componentJavaScript.add("");
		}

		return componentJavaScript;
	}

	/**
	 * Returns simulated data
	 * @return data (xml-type)
	 */
	public String getSimulatedData() {		
		
		String str = null;
		String identifier = "";
		String value = "";
		
		value = "<value></value>";		
		identifier = "<id>" + id + "</id>";	
		str = "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissaSign" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa0" + "</id>";		
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissaSeparator" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa1" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa2" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa3" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa4" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
			
		identifier = "<id>" + id + "_exponent" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_exponentSign" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_exponent0" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_exponent1" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		value = "<value>" + caption.get(0) + "</value>";	
		identifier = "<id>" + id + "_caption" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";

		value = "<value></value>";					
		identifier = "<id>" + id + "_setValue" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissaPlus" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissaMinus" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa0Up" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa0Down" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa1Up" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa1Down" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa2Up" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa2Down" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa3Up" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa3Down" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa4Up" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_mantissa4Down" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_exponentPlus" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_exponentMinus" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_exponent0Up" + "</id>";
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_exponent0Down" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_exponent1Up" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
		
		identifier = "<id>" + id + "_exponent1Down" + "</id>";	
		str = str + "<update>" + identifier + value + "</update>";
	
		return str;
	}
	
	/**
	 * Returns real data from common device access plug
	 * @return data (xml-type)
	 */
	public String getRealData() {		
		
		String str = null;
		String identifier = "";
		String value = "";
		String style = "";
		String blinking = "";
		String[] act = null;
		
		if (!extensionManifest.equals("")) {
			if (!userPrivilege.equals(layman) && extensionExecutor.getActions() != null) {
				act = extensionExecutor.getActions();
				for (int i = 0; i < act.length; i++) {
					commonDeviceAccess.execute(act[i], true);
				}
			}
		}

		blinking = getBlinking(0);
		style = getStyleUpdate();		
		value = "<value></value>";		
		identifier = "<id>" + id + "</id>";
		str = "<update>" + identifier + value + style + blinking + "</update>";
		
		identifier = "<id>" + id + "_mantissaSign" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa0" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissaSeparator" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa1" + "</id>";		
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa2" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa3" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa4" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
			
		identifier = "<id>" + id + "_exponent" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_exponentSign" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_exponent0" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_exponent1" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		value = "<value>" + caption.get(0) + "</value>";
		identifier = "<id>" + id + "_caption" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
			
		String[] styleArray = style.split("</style>");
		style = "";
		for (int i = 0; i < styleArray.length; i++) {
			if (!styleArray[i].contains("background-color")) style = style + styleArray[i] + "</style>";
		}		
		value = "<value></value>";		
		identifier = "<id>" + id + "_setValue" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissaPlus" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissaMinus" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa0Up" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa0Down" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa1Up" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa1Down" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa2Up" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa2Down" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa3Up" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa3Down" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa4Up" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_mantissa4Down" + "</id>";	
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_exponentPlus" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_exponentMinus" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_exponent0Up" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_exponent0Down" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_exponent1Up" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
		
		identifier = "<id>" + id + "_exponent1Down" + "</id>";
		str = str + "<update>" + identifier + value + style + "</update>";
	
		return str;
	}
	
	/**
	 * Calls simulator
	 * @param action requested action
	 */	
	public void doSimulatedAction(String action) {
	}

	/**
	 * Sends client action requests to common device access plug
	 * @param action requested action
	 */	
	public void doRealAction(String action) {

		commonDeviceAccess.execute(property.get(0).substring(0, property.get(0).length() - 2) + "(" + characterEscaper.encodeString(action) + ")", true);
	}
}

