package de.desy.web2cToolkit.web2c.web2cExtension;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cExtensionManifestReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * Extension code executor
 * @author R.Bacher, DESY
 */
public class Web2cExtensionExecutor implements IWeb2cConstants {

	private int dataSize = 0;
	private String componentType = "";
	private String className = "";
	private ArrayList<String> monitorURI = new ArrayList<String>();
	private String[] monitorURIArray = null;
	private String[] monitorData = null;
	private ArrayList<String> receiverURI = new ArrayList<String>();
	private String[] receiverURIArray = null;
	private ByteArrayOutputStream[] receiverData = null;
	private Object[] data = null;
	private ArrayList<String> remoteCommandURI = null;
	private String[] remoteCommandURIArray = null;
	private ArrayList<String> actionURI = null;
	private String[] actionURIArray = null;
	private String[] actionData = null;
	private Web2cCommonDeviceAccess commonDeviceAccess = null;
	private Web2cCommonVideoConnector commonVideoConnector = null;
	
	/** 
	 * Extension code executor
	 * @param manifest extension manifest
	 * @param cda common device access
	 * @param cvc common video connector
	 */
	public Web2cExtensionExecutor(File manifest, Web2cCommonDeviceAccess cda, Web2cCommonVideoConnector cvc) {
		
		commonDeviceAccess = cda;
		commonVideoConnector = cvc;
		
		Web2cExtensionManifestReader reader = new Web2cExtensionManifestReader(manifest);
		dataSize = reader.getComponentSize();
		componentType = reader.getComponentType();
		className = reader.getClassName();		
		remoteCommandURI = reader.getRemoteCommands();
		actionURI = reader.getActions();
		monitorURI = reader.getMonitors();
		receiverURI = reader.getReceivers();
		if (!remoteCommandURI.isEmpty()) {
			data = new Object[remoteCommandURI.size()];
			remoteCommandURIArray = new String[remoteCommandURI.size()];
			for (int i = 0; i < remoteCommandURI.size(); i++) {
				remoteCommandURIArray[i] = (String)(remoteCommandURI.toArray()[i]);
			}
		}
		else if (dataSize != 0) {
			data = new Object[dataSize];
			remoteCommandURIArray = new String[1];
		}
		else {
			data = new Object[1];
			remoteCommandURIArray = new String[1];
		}
		if (!actionURI.isEmpty()) {
			actionData = new String[actionURI.size()];
			actionURIArray = new String[actionURI.size()];
			for (int i = 0; i < actionURI.size(); i++) {
				actionURIArray[i] = (String)(actionURI.toArray()[i]);
			}
		}
		else {
			actionData = new String[1];
			actionURIArray = new String[1];
		}
		if (!monitorURI.isEmpty()) {
			monitorData = new String[monitorURI.size()];
			monitorURIArray = new String[monitorURI.size()];
			for (int i = 0; i < monitorURI.size(); i++) {
				monitorURIArray[i] = (String)(monitorURI.toArray()[i]);
			}
		}
		else {
			monitorData = new String[1];
			monitorURIArray = new String[1];
		}
		if (!receiverURI.isEmpty()) {
			receiverData = new ByteArrayOutputStream[receiverURI.size()];
			receiverURIArray = new String[receiverURI.size()];
			for (int i = 0; i < receiverURI.size(); i++) {
				receiverURIArray[i] = (String)(receiverURI.toArray()[i]);
			}
		}
		else {
			receiverData = new ByteArrayOutputStream[1];
			receiverURIArray = new String[1];
		}
	}
	
	/** 
	 * Returns array of data to be displayed or to be used to determine the data to be displayed 
	 * 	GUI data (out: String): 
	 * 	- Live: &#060;value&#062;data[startIndex],..,data[stopIndex]&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp&#060;&#047;timeStamp&#062;
	 * 	- History: &#060;value&#062;data1&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp1&#060;&#047;timeStamp&#062;&#060;value&#062;data2&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp2&#060;&#047;timeStamp&#062;.....)
	 * 	GUI data (out: ByteArrayOutputStream):: jpeg-encoded image
	 * 	Remote command data (out: String): remoteCommandURI(setValue)
	 * @return data array
	 */
	public Object[] getData() {
		
		Object[] componentData = null;
		
		System.out.println();
		
		if (dataSize != 0 || !remoteCommandURI.isEmpty()) componentData = data;
		
		return componentData;
	}
	
	/** 
	 * Returns array of actions to be performed: actionURI(setValue)
	 * @return action array
	 */
	public String[] getActions() {
		
		String[] componentActions = null;

		if (!actionURI.isEmpty()) componentActions = actionData;
		
		return componentActions;
	}
	
	/**
	 * Executes extension code	 
	 * @param dataFlag data flag, GUI String data only ("live" = live monitor/receiver/command data, "history" = local history monitor data)
	 * @return execution status
	 */
	public int executeExtensionCode(String dataFlag) {
		
		int status = 0;	
		
		Object[] parameterArray = new Object[9];
		Class<?>[] parameterTypeArray = new Class<?>[9];

		parameterArray[0] = componentType;
		parameterArray[1] = remoteCommandURIArray;
		parameterArray[2] = data;
		parameterArray[3] = monitorURIArray;
		parameterArray[4] = monitorData;
		parameterArray[5] = receiverURIArray;
		parameterArray[6] = receiverData;
		parameterArray[7] = actionURIArray;
		parameterArray[8] = actionData;
		parameterTypeArray[0] = componentType.getClass();
		parameterTypeArray[1] = remoteCommandURIArray.getClass();
		parameterTypeArray[2] = data.getClass();
		parameterTypeArray[3] = monitorURIArray.getClass();
		parameterTypeArray[4] = monitorData.getClass();
		parameterTypeArray[5] = receiverURIArray.getClass();
		parameterTypeArray[6] = receiverData.getClass();
		parameterTypeArray[7] = actionURIArray.getClass();
		parameterTypeArray[8] = actionData.getClass();

		if ((!monitorURI.isEmpty()) && (commonDeviceAccess != null)) {
			for (int j = 0; j < monitorURI.size(); j++) {
				if (dataFlag.equals("live")) {
					monitorData[j] = commonDeviceAccess.getMonitorData(monitorURI.get(j));
				}
				if (dataFlag.equals("history")) {
					monitorData[j] = commonDeviceAccess.getHistory(monitorURI.get(j));
				}
			}
		}
		if ((!receiverURI.isEmpty()) && (commonVideoConnector != null)) {
			for (int j = 0; j < receiverURI.size(); j++) {
				commonVideoConnector.getVideo(receiverData[j], receiverURI.get(j));
			}
		}	
		
		try {
			Class<?>  extensionClass =  Class.forName(className);
			Object extensionClassInstance = extensionClass.newInstance();
			Method executeExtensionCode = extensionClass.cast(extensionClassInstance).getClass().getMethod("executeExtensionCode", parameterTypeArray);
			if (dataFlag.equals("live")) {
				status = (Integer)(executeExtensionCode.invoke((Object)(extensionClass.cast(extensionClassInstance)), parameterArray));
			}
			if (dataFlag.equals("history")) {
				int numberOfPoints = Integer.valueOf(historyInterval);
				String[][] dataValues = new String[numberOfPoints][monitorURI.size()];
				String[][] timeStamp = new String[numberOfPoints][monitorURI.size()]; 
				String[][] datas = new String[numberOfPoints][dataSize];
				for (int j = 0; j < numberOfPoints; j++) {
					for (int i = 0; i < monitorURI.size(); i++) {
						dataValues[j][i] = monitorData[i].split("</timestamp>")[j].split(">")[1].split("<")[0];
						timeStamp[j][i] = monitorData[i].split("</timestamp>")[j].split(">")[3];
					}
					monitorData = dataValues[j];
					status = (Integer)(executeExtensionCode.invoke((Object)(extensionClass.cast(extensionClassInstance)), parameterArray));
					datas[j] = (String[])data;
				}
				for (int j = 0; j < dataSize; j++) {
					for (int i = 0; i < numberOfPoints; i++) {
						data[j] = data[j] + "<value>" + datas[i][j] + "</values><timeStamp>" + timeStamp[i][0] + "</timeStamp>";
					}
				}
			}
		} 
		catch (InstantiationException e) {System.out.println("Error (" + e.toString() + ") executing extension code " + className + "!");} 
		catch (IllegalAccessException e) {System.out.println("Error (" + e.toString() + ") executing extension code " + className + "!");} 
		catch (ClassNotFoundException e) {System.out.println("Error (" + e.toString() + ") executing extension code " + className + "!");} 
		catch (SecurityException e) {System.out.println("Error (" + e.toString() + ") executing extension code " + className + "");} 
		catch (IllegalArgumentException e) {System.out.println("Error (" + e.toString() + ") executing extension code " + className + "!");} 
		catch (InvocationTargetException e) {System.out.println("Error (" + e.toString() + ") executing extension code " + className + "!");} 
		catch (NoSuchMethodException e) {System.out.println("Error (" + e.toString() + ") executing extension code " + className + "!");}
		
		return status;
	}
}
