package de.desy.web2cToolkit.web2c.web2cVideoUtilities;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.Iterator;

import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cGarbageCollection;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cVideoClient;

public class Web2cCommonVideoConnector implements IWeb2cVideoClient, IWeb2cGarbageCollection {

	private ArrayList<String> urlList = null;	
	private Web2cTINEVideoConnector web2cTINE = null;	
	private Web2cRTPVideoConnector web2cRTP = null;	
	private ArrayList<Integer> numberVideoClientRequesters = new ArrayList<Integer>();

	public Web2cCommonVideoConnector() {
		
		web2cTINE = new Web2cTINEVideoConnector();
		web2cRTP = new Web2cRTPVideoConnector();
		urlList = new ArrayList<String>();
	}
	
	/**
	 * Checks whether the video client for the url specified already exists
	 * @param url url
	 */
	private synchronized boolean clientIsRegistered(String url) {
		
		boolean isRegistered = false;

		if (urlList.isEmpty()) {
			isRegistered = false;
		}
		else {
			isRegistered = false;
			for (Iterator<String> it = urlList.iterator();it.hasNext();) {
				String propertyString = (String)(it.next());
				if (url.equals(propertyString)) isRegistered = true;
			}
		}
		
		return isRegistered;
	}
	
	/**
    * Closes the receiver for the url requested
    * @param url url
    */
    public void closeReceiver(String url) {

		String protocol = url.split("://")[0];
		String address = url.split("://")[1];
		
		if (protocol.equals("tine")) {
			web2cTINE.closeReceiver(address);		
		}		
		if (protocol.equals("rtp")) {
			web2cRTP.closeReceiver(address);
		}	
    }
	
    /**
     * Connects to video source
     * @param url url
     */
    public synchronized void connectToVideoSource(String url) {
    	
		int numberRequesters = 0;
		
		if (!clientIsRegistered(url)) {	
			String protocol = url.split("://")[0];
			String address = url.split("://")[1];
			
			if (protocol.equals("tine")) {
				web2cTINE.connectToVideoSource(address);		
			}		
			if (protocol.equals("rtp")) {
				web2cRTP.connectToVideoSource(address);
			}	
			urlList.add(url);
			numberVideoClientRequesters.add(1);
		}
		else {
			int index = urlList.indexOf(url);
			numberRequesters = numberVideoClientRequesters.get(index) + 1;
			numberVideoClientRequesters.set(index, numberRequesters);
		}
    }
    
	/**
 	 * Assigns the frame of the url requested to the output stream
 	 * @param frame output stream
	 * @param url url
	 */
    public void getVideo(ByteArrayOutputStream frame, String url) {

		String protocol = url.split("://")[0];
		String address = url.split("://")[1];
		
		if (protocol.equals("tine")) {
			web2cTINE.getVideo(frame, address);		
		}		
		if (protocol.equals("rtp")) {
			web2cRTP.getVideo(frame, address);	
		}	
    }
    
	/**
	 * Returns error string of the url requested
	 * @param url url
	 * @return error string
	 */
    public String getError(String url) {
    	
    	String errorString = "";

		String protocol = url.split("://")[0];
		String address = url.split("://")[1];
		
		if (protocol.equals("tine")) {
			web2cTINE.getError(address);		
		}		
		if (protocol.equals("rtp")) {
			web2cRTP.getError(address);	
		}	
    	
    	return errorString;
    }
    
	/**
	 * Returns description for the url specified
	 * @param url url
	 * @return description
	 */
    public String getDescription(String url) {
    	
    	String description = "";

		String protocol = url.split("://")[0];
		String address = url.split("://")[1];
		
		if (protocol.equals("tine")) {
			description = web2cTINE.getDescription(address);		
		}		
		if (protocol.equals("rtp")) {
			description = web2cRTP.getDescription(address);	
		}	
    	
    	return description;
    }
	
	/**
    * Returns the number of requesters requesting the device access specified
    * @param id device access id
    * @return number of requesters
    */
	public synchronized int getNumberOfRequesters(String id) {
		
		int index = 0;
		int number = 0;

		index = urlList.indexOf(id);
		
		if (index >= 0) {
			number = numberVideoClientRequesters.get(urlList.indexOf(id));
		}
		else {
			number = -1;
		}
		
		return number;
	}
	
	/**
    * Decreases (-1) the number of requesters requesting the device access specified; removes the corresponding device access if number <= 0
    * @param id device access id
    */
	public synchronized void removeRequester(String id) {
		
		int index = 0;
		int number = 0;
		
		index = urlList.indexOf(id); 
		
		if (index >= 0) {
			number = getNumberOfRequesters(id);
			numberVideoClientRequesters.set(urlList.indexOf(id), number - 1);
//			System.out.println("Device access requester for " + id + " gone! Remaining " + getNumberOfRequesters(id) + " requester(s).");
		}
		
		if (number - 1 <= 0) removeResource(id);
	}
	
	/**
    * Removes the device access with the id specified and all associated resources
    * @param id device access id
    */
	public synchronized void removeResource(String id) {
		
		int index = 0;

		index = urlList.indexOf(id);
		
		if (index >= 0) {	
			closeReceiver(id);
			urlList.remove(index);
			numberVideoClientRequesters.remove(index);
//			System.out.println("Device access for " + id + " closed!");
		}
	}
}
