package de.desy.web2cToolkit.web2c.web2cVideoUtilities;

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import javax.imageio.ImageIO;
import de.desy.acop.video.AcopVideo;

/**
 * @author R.Bacher, DESY
 */
public class Web2cTINEVideoFrameGrabber {
	
	private AcopVideo acopVideo = null;
	private Image image = null;
	
	/**
 	 * Constructor
 	 * Grants access to individual frames of an TINE video stream
 	 * */
	public Web2cTINEVideoFrameGrabber(AcopVideo acop) {
		
		acopVideo = acop;
		image = acopVideo.getImageDisplayer().getSnapshotImage();
	}
	
	/**
 	 * Assigns the frame to the output stream
 	 * @param frame output stream
	 */
	public void assignFrame(ByteArrayOutputStream frame) {
		
		try {
			image = acopVideo.getImageDisplayer().getSnapshotImage();
			ImageIO.write((BufferedImage)image, "jpg", frame);
		} 
		catch (IOException e) {
			System.out.println("Error converting video frame buffer to byte stream !");
		} 
	}
	
	/**
 	 * Returns frame height
 	 * @return frame height
	 */
	public int getFrameHeight() {
		
		return image.getHeight(acopVideo);
	}
	
	/**
 	 * Returns frame width
 	 * @return frame width
	 */
	public int getFrameWidth() {
		
		return image.getWidth(acopVideo);
	}
}
