package de.desy.web2cToolkit.web2c.web2cVideoUtilities;

import java.beans.PropertyVetoException;
import java.io.ByteArrayOutputStream;
import com.cosylab.util.CommonException;
import de.desy.acop.transport.AccessMode;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.acop.video.AcopVideo;
import de.desy.tine.queryUtils.TPropertyQuery;
import de.desy.tine.queryUtils.TQuery;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cVideoClient;

/**
 * @author R.Bacher, DESY
 */
public class Web2cTINEVideoReceiver implements IWeb2cVideoClient {
	
	private AcopVideo acopVideo = null;
	private Web2cTINEVideoFrameGrabber frameGrabber = null;
    private String errorString = "";
    private String description = "";
    
    /**
     * Constructor
     * Receives video stream via rtp and grants access to individual frames
     */
    public Web2cTINEVideoReceiver() {
    }
    
    /**
     * Closes the TINE receiver for the address specified
     * @param address address
     */
    public void closeReceiver(String address) {
    	
    	acopVideo.stop();
    }
    
    /**
     * Connects to video source to the address specified
     * @param address address 
     */
    public void connectToVideoSource(String address) {
    	
    	int error = 0;

		String context = address.substring(1).split("/")[0];
		String server = address.substring(1).split("/")[1];
		String device = address.substring(context.length() + server.length() + 3, address.lastIndexOf("/"));
		String proper = address.substring(address.lastIndexOf("/") + 1);

    	try {    
    		TPropertyQuery[] propertyInformation = TQuery.getPropertyInformation(context, server, device, proper, 1000);
    		description = propertyInformation[0].prpDescription;
    		
			acopVideo = new AcopVideo();
			try {
				acopVideo.setConnectionParameters(new ConnectionParameters(address, AccessMode.POLL_CONNECT, 1000, -1));
			} 
			catch (CommonException e) {
				e.printStackTrace();
			} 
			catch (PropertyVetoException e) {
				e.printStackTrace();
			}
			acopVideo.start();
			frameGrabber = new Web2cTINEVideoFrameGrabber(acopVideo);
    	} 
    	catch (Exception e) {
    		error = error + 1;
    	}
    	
    	if (error == 0) {
    	   	System.out.println("Connected to video source tine://" + address + " (JPEG/TINE,  " + frameGrabber.getFrameWidth() + "x" + frameGrabber.getFrameHeight() + ").");
    	}
    	else {
    	   	System.out.println("Failed to connect to video source tine://" + address + "!");
			errorString = errorString + "[Video] Error establishing video connection tine://" + address + ";";
    	}
    }
    
	/**
 	 * Assigns the frame to the output stream
 	 * @param frame output stream
 	 * @param address address
	 */
    public void getVideo(ByteArrayOutputStream frame, String address) {
    	
    	frameGrabber.assignFrame(frame);
    }
    
	/**
	 * Returns error string
	 * @param address address
	 * @return error string
	 */
    public String getError(String address) {
    	
    	return errorString;
    }

	/**
	 * Returns description for the address specified
	 * @param address address
	 * @return description
	 */
    public String getDescription(String address) {
    	
    	return description;
    }
}
