package de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import javax.imageio.ImageIO;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.AxisSpace;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.LogarithmicAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.category.GanttRenderer;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYDotRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.renderer.xy.XYStepRenderer;
import org.jfree.data.category.IntervalCategoryDataset;
import org.jfree.data.gantt.Task;
import org.jfree.data.gantt.TaskSeries;
import org.jfree.data.gantt.TaskSeriesCollection;
import org.jfree.data.time.Millisecond;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveAccess.Web2cCommonArchiveAccess;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * @author R.Bacher, DESY
 */
public class Web2cArchiveViewerDataPlot implements IWeb2cConstants, Observer  {
	
	private String referenceId = null;
	private String id = null;
	private String width = null;
	private String height = null;
	private String marginLeft = null;
	private String marginTop  = null;
	private String backgroundColor = null;
	private String borderStyle = null;
	private String borderWidth = null;
	private String borderColor = null;
	private String toolTipText = null;
	private String axis = null;
	private String type = null;
	private String min = null;
	private String max = null;
	private String line = "";
	private String axisName = "";
	private boolean inline = false;
	private double[][] dataValues = null;
	private String[][] dataValuesAsText = null;
	private int[][] dataTimeStampsInSeconds = null;
	private int dataArrayCompletionLength = 0;	
	private int[] dataArrayLength = null;
	private int numberOfDataTraces = 0;
	private Web2cCommonArchiveAccess caa = null;
	private int tStart = 0;
	private int tWidth = 0;
	private int yStart = 0;
	private int yWidth = 0;
	private boolean archiveDataPlottable = false;
	private String plotDataStatus = "oldData";
	private ByteArrayOutputStream dataPlot = null;
	private String tableDataStatus = "oldData";
	private String[] dataTable = null;
	private PlotGenerator plotGenerator = null;
	private TableGenerator tableGenerator = null;
	private int tableReloadCounter = 0;
	private int tableMaxReloadCounter = 30;
	
	/**
	 * Data Plot
	 * @param refId ID of reference component which component belongs to
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 */
	public Web2cArchiveViewerDataPlot(String refId, ArrayList<String> names, ArrayList<String> values, Web2cCommonArchiveAccess commonArchiveAccess) {

		referenceId = refId;
		caa = commonArchiveAccess;
		decodeAttributes(names, values);
	}
	
	/**
	 * @return component id
	 */
	public String getId() {
		
		return id;
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	

		componentHTML.add("                <div id=\"" + id + "\" title=\"" + toolTipText + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "\">");
		if (inline == false) {
			componentHTML.add("					   <input type=\"button\" id=\"" + id + "_backwardButton\" value=\"backward\" onclick=\"" + id + "_backward()\" style=\"position:absolute; left:20px; top:155px; width:100px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_forwardButton\" value=\"forward\" onclick=\"" + id + "_forward()\" style=\"position:absolute; left:1130px; top:155px; width:100px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_zoomLeftButton\" value=\"left half\" onclick=\"" + id + "_zoomLeft()\" style=\"position:absolute; left:150px; top:330px; width:200px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_zoomResetButton\" value=\"reset zoom / shift\" onclick=\"" + id + "_resetZoom()\" style=\"position:absolute; left:515px; top:330px; width:200px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_zoomRightButton\" value=\"right half\" onclick=\"" + id + "_zoomRight()\" style=\"position:absolute; left:880px; top:330px; width:200px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <img id=\"" + id + "_plot\" src=\"\" title=\"To zoom, press mouse down and drag over area of interest (from left-upper to right-lower corner)\" alt=\"No data to plot\" style=\"position:absolute; left:140px; top:10px; width:" + horizontalArchivePlotSize + "; height:" + verticalArchivePlotSize + "; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <div id=\"" + referenceId + "_Status\" style=\"position:absolute; left:922px; top:14px; width:180px; height:20px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">" + "no data to display" + "</div>");
		}
		else {
			componentHTML.add("					   <input type=\"button\" id=\"" + id + "_backwardButton\" value=\"shift backward (24h)\" onclick=\"" + id + "_backward()\" style=\"position:absolute; left:1010px; top:120px; width:200px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_forwardButton\" value=\"shift forward (24h)\" onclick=\"" + id + "_forward()\" style=\"position:absolute; left:1010px; top:155px; width:200px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <input type=\"button\" id=\"" + id + "_zoomResetButton\" value=\"reset zoom / shift\" onclick=\"" + id + "_resetZoom()\" style=\"position:absolute; left:1010px; top:190px; width:200px; height:25px; color:" + black + "; background-color:" + "lightgrey" + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");	
			componentHTML.add("                    <img id=\"" + id + "_plot\" src=\"\" title=\"To zoom, press mouse down and drag over area of interest (from left-upper to right-lower corner)\" alt=\"No data to plot\" style=\"position:absolute; left:20px; top:10px; width:" + horizontalArchivePlotSize + "; height:" + verticalArchivePlotSize + "; color:" + black + "; background-color:" + white + "; font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">");
			componentHTML.add("                    <div id=\"" + referenceId + "_Status\" style=\"position:absolute; left:802px; top:14px; width:180px; height:20px; color:" + black + "; background-color:" + white + "; border-style:" + none + "; border-width:" + thin + "; border-color:" + borderColor + ";  font-size:" + medium + "; font-weight:" + normal + "; text-align:" + center + "\">" + "no data to display" + "</div>");	
		}
		componentHTML.add("                </div>");

		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();		
		
		componentJavaScript.add("            //ArchiveViewerDataPlot: " + id);
		
		componentJavaScript.add("                var " + referenceId + "_dataPlotId = \"" + id + "\";");		
		componentJavaScript.add("                var " + referenceId + "_dataStatusId = \"" + referenceId + "_Status\";");	
		componentJavaScript.add("                var " + id + "_shift;");
		componentJavaScript.add("	             var " + id + "_plotLeft;");                 
		componentJavaScript.add("	             var " + id + "_plotTop;"); 
		componentJavaScript.add("		         var " + id + "_plotWidth;");
		componentJavaScript.add("		         var " + id + "_plotHeight;");
		componentJavaScript.add("	             var " + id + "_dragStartTop;");  
		componentJavaScript.add("	             var " + id + "_dragStartLeft;");  
		componentJavaScript.add("	             var " + id + "_dragStopTop;");  
		componentJavaScript.add("	             var " + id + "_dragStopLeft;");   
		componentJavaScript.add("	             var " + id + "_dragging = false;"); 
	
		if (inline == false) {
			componentJavaScript.add("                function " + id + "_zoomLeft(){");
			componentJavaScript.add("		            var now = new Date();");
			componentJavaScript.add("                	" + referenceId + "_yMin = 0;");
			componentJavaScript.add("                	" + referenceId + "_yMax = 100;");
			componentJavaScript.add("                	" + id + "_shift = (" + referenceId + "_selectedToInMillis - " + referenceId + "_selectedFromInMillis) / 2;");
			componentJavaScript.add("        			" + referenceId + "_plotReloadCounter = 0;");
			componentJavaScript.add("			        " + referenceId + "_getPlotData();");
			componentJavaScript.add("	    			handleAll(" + referenceId + "_selectedFromInMillis, " + referenceId + "_selectedToInMillis, \"" + referenceId + "\");");
			componentJavaScript.add("                }");
			
			componentJavaScript.add("                function " + id + "_zoomRight(){");
			componentJavaScript.add("		            var now = new Date();");
			componentJavaScript.add("                	" + referenceId + "_yMin = 0;");
			componentJavaScript.add("                	" + referenceId + "_yMax = 100;");
			componentJavaScript.add("                	" + id + "_shift = (" + referenceId + "_selectedToInMillis - " + referenceId + "_selectedFromInMillis) / 2;");
			componentJavaScript.add("                	" + referenceId + "_selectedFromInMillis = " + referenceId + "_selectedFromInMillis + " + id + "_shift;");
			componentJavaScript.add("        			" + referenceId + "_plotReloadCounter = 0;");
			componentJavaScript.add("			        " + referenceId + "_getPlotData();");
			componentJavaScript.add("	    			handleAll(" + referenceId + "_selectedFromInMillis, " + referenceId + "_selectedToInMillis, \"" + referenceId + "\");");
			componentJavaScript.add("                }");
		}
			
		componentJavaScript.add("                function " + id + "_backward(){");
		componentJavaScript.add("		            var now = new Date();");
		componentJavaScript.add("                	" + referenceId + "_yMin = 0;");
		componentJavaScript.add("                	" + referenceId + "_yMax = 100;");
		componentJavaScript.add("                	" + id + "_shift = " + referenceId + "_selectedToInMillis - " + referenceId + "_selectedFromInMillis;");
		componentJavaScript.add("                	" + referenceId + "_selectedFromInMillis = " + referenceId + "_selectedFromInMillis - " + id + "_shift;");
		componentJavaScript.add("                	" + referenceId + "_selectedToInMillis = " + referenceId + "_selectedToInMillis - " + id + "_shift;");
		componentJavaScript.add("        			" + referenceId + "_plotReloadCounter = 0;");
		componentJavaScript.add("			        " + referenceId + "_getPlotData();");
		componentJavaScript.add("	    			handleAll(" + referenceId + "_selectedFromInMillis, " + referenceId + "_selectedToInMillis, \"" + referenceId + "\");");
		componentJavaScript.add("                }");
		
		componentJavaScript.add("                function " + id + "_forward(){");
		componentJavaScript.add("		            var now = new Date();");
		componentJavaScript.add("                	" + referenceId + "_yMin = 0;");
		componentJavaScript.add("                	" + referenceId + "_yMax = 100;");
		componentJavaScript.add("                	" + id + "_shift = " + referenceId + "_selectedToInMillis - " + referenceId + "_selectedFromInMillis;");
		componentJavaScript.add("                	" + referenceId + "_selectedFromInMillis = " + referenceId + "_selectedFromInMillis + " + id + "_shift;");
		componentJavaScript.add("                	" + referenceId + "_selectedToInMillis = " + referenceId + "_selectedToInMillis + " + id + "_shift;");
		componentJavaScript.add("        			" + referenceId + "_plotReloadCounter = 0;");
		componentJavaScript.add("			        " + referenceId + "_getPlotData();");
		componentJavaScript.add("	    			handleAll(" + referenceId + "_selectedFromInMillis, " + referenceId + "_selectedToInMillis, \"" + referenceId + "\");");
		componentJavaScript.add("                }");
		
		componentJavaScript.add("                function " + id + "_resetZoom(){");
		componentJavaScript.add("		            var now = new Date();");
		componentJavaScript.add("                	" + referenceId + "_yMin = 0;");
		componentJavaScript.add("                	" + referenceId + "_yMax = 100;");
		componentJavaScript.add("                	" + referenceId + "_selectedFromInMillis = " + referenceId + "_selectedFromInMillisOld;");
		componentJavaScript.add("                	" + referenceId + "_selectedToInMillis = " + referenceId + "_selectedToInMillisOld;");
		componentJavaScript.add("        			" + referenceId + "_plotReloadCounter = 0;");
		componentJavaScript.add("			        " + referenceId + "_getPlotData();");
		componentJavaScript.add("	    			handleAll(" + referenceId + "_selectedFromInMillis, " + referenceId + "_selectedToInMillis, \"" + referenceId + "\");");
		componentJavaScript.add("                }");
		                    
//		componentJavaScript.add("	             function " + referenceId + "_plotMouseClick(event) {");
//		componentJavaScript.add("		            var sectionLeft;");
//		componentJavaScript.add("		            var sectionTop;");
//		componentJavaScript.add("		            var dataPlotLeft;");
//		componentJavaScript.add("		            var dataPlotTop;");
//		componentJavaScript.add("		            var now = new Date();");
//		componentJavaScript.add("		            var millisPerPx = 0;");
//		componentJavaScript.add("		            var dyPerPx = 0;");
//		componentJavaScript.add("		            if (" + referenceId + "_plotZoomClick == false) {");
//		componentJavaScript.add("				        " + referenceId + "_plotZoomClick = true;");
//		componentJavaScript.add("		                " + referenceId + "_plot.style.cursor = \"crosshair\";");
//		componentJavaScript.add("		                sectionLeft = document.getElementById(" + referenceId + "_sectionId).style.left.replace(\"px\", \"\");"); 
//		componentJavaScript.add("		                sectionTop = document.getElementById(" + referenceId + "_sectionId).style.top.replace(\"px\", \"\");");
//		componentJavaScript.add("		                dataPlotLeft = document.getElementById(" + referenceId + "_dataPlotId).style.left.replace(\"px\", \"\");");
//		componentJavaScript.add("		                dataPlotTop = document.getElementById(" + referenceId + "_dataPlotId).style.top.replace(\"px\", \"\");");
//		componentJavaScript.add("		                " + id + "_plotLeft = document.getElementById(" + referenceId + "_dataPlotId + \"_plot\").style.left.replace(\"px\", \"\");");
//		componentJavaScript.add("		                " + id + "_plotWidth = " + referenceId + "_tWidth;");
//		componentJavaScript.add("		                " + id + "_plotTop = document.getElementById(" + referenceId + "_dataPlotId + \"_plot\").style.top.replace(\"px\", \"\");"); 
//		componentJavaScript.add("		                " + id + "_plotHeight = " + referenceId + "_yWidth;");
//		componentJavaScript.add("		                " + id + "_plotLeft = String(Number(" + id + "_plotLeft) + Number(dataPlotLeft) + Number(sectionLeft) + Number(" + referenceId + "_tStart));");
//		componentJavaScript.add("		                " + id + "_plotTop = String(Number(" + id + "_plotTop) + Number(dataPlotTop) + Number(sectionTop) + Number(" + referenceId + "_yStart));"); 
//		componentJavaScript.add("						if (" + referenceId + "_browser == \"MSIE\") {");
//		componentJavaScript.add("							if (!event) event = window.event;");
//		componentJavaScript.add("							event.returnValue = false;"); 
//		componentJavaScript.add("		                	" + id + "_dragStartLeft = event.clientX;");
//		componentJavaScript.add("		                	" + id + "_dragStartTop = document.body.scrollTop + event.clientY;");
//		componentJavaScript.add("						}");
//		componentJavaScript.add("						else {");
//		componentJavaScript.add("							event.preventDefault();"); 
//		componentJavaScript.add("		                	" + id + "_dragStartLeft = event.clientX;");
//		componentJavaScript.add("		                	" + id + "_dragStartTop = window.pageYOffset + event.clientY;");
//		componentJavaScript.add("						}");  
//		componentJavaScript.add("			        } else {");
//		componentJavaScript.add("				        " + referenceId + "_plotZoomClick = false;");
//		componentJavaScript.add("		                " + referenceId + "_plot.style.cursor = \"default\";");
//		componentJavaScript.add("						if (" + referenceId + "_browser == \"MSIE\") {");
//		componentJavaScript.add("							if (!event) event = window.event;");
//		componentJavaScript.add("							event.returnValue = false;"); 
//		componentJavaScript.add("		               		" + id + "_dragStopLeft = event.clientX;");
//		componentJavaScript.add("		                	" + id + "_dragStopTop = document.body.scrollTop + event.clientY;");
//		componentJavaScript.add("						}");
//		componentJavaScript.add("						else {");
//		componentJavaScript.add("							event.preventDefault();"); 
//		componentJavaScript.add("		                " + id + "_dragStopLeft = event.clientX;");
//		componentJavaScript.add("		                " + id + "_dragStopTop = window.pageYOffset + event.clientY;");
//		componentJavaScript.add("						}"); 
//		componentJavaScript.add("		                millisPerPx = (" + referenceId + "_selectedToInMillis - " + referenceId + "_selectedFromInMillis) / Number(" + id + "_plotWidth);");
//		componentJavaScript.add("		                dyPerPx = (" + referenceId + "_yMax - " + referenceId + "_yMin) / Number(" + id + "_plotHeight);");		
//		componentJavaScript.add("		                " + referenceId + "_selectedFromInMillis = parseInt(" + referenceId + "_selectedFromInMillis + (millisPerPx * (Number(" + id + "_dragStartLeft) - Number(" + id + "_plotLeft))));");
//		componentJavaScript.add("		                " + referenceId + "_selectedToInMillis = parseInt(" + referenceId + "_selectedFromInMillis + (millisPerPx * (Number(" + id + "_dragStopLeft) - Number(" + id + "_dragStartLeft))));");
//		componentJavaScript.add("		                " + referenceId + "_yMax = parseInt(" + referenceId + "_yMax - (dyPerPx * (Number(" + id + "_dragStartTop) - Number(" + id + "_plotTop))));");
//		componentJavaScript.add("		                " + referenceId + "_yMin = parseInt(" + referenceId + "_yMax - (dyPerPx * (Number(" + id + "_dragStopTop) - Number(" + id + "_dragStartTop))));");
//		componentJavaScript.add("        				" + referenceId + "_plotReloadCounter = 0;");
//		componentJavaScript.add("						" + referenceId + "_getPlotData();");
//		componentJavaScript.add("	    				handleAll(" + referenceId + "_selectedFromInMillis, " + referenceId + "_selectedToInMillis, \"" + referenceId + "\");");
//		componentJavaScript.add("			        }");
//		componentJavaScript.add("			     }");
		
		componentJavaScript.add("	             function " + referenceId + "_plotMouseMove(event) {");
		componentJavaScript.add("	             	var dragX;");  
		componentJavaScript.add("	             	var dragY;");  
		componentJavaScript.add("					if (" + referenceId + "_plotZoomClick == true) {"); 
		componentJavaScript.add("						var element = document.getElementById(" + referenceId + "_zoomFrameId);"); 
		componentJavaScript.add("						element.style.visibility = \"visible\";"); 
		componentJavaScript.add("						if (" + referenceId + "_browser == \"MSIE\") {");
		componentJavaScript.add("							if (!event) event = window.event;");
		componentJavaScript.add("							event.returnValue = false;"); 
		componentJavaScript.add("		                	dragX = event.clientX;");
		componentJavaScript.add("		                	dragY = document.body.scrollTop + event.clientY;");
		componentJavaScript.add("						}");
		componentJavaScript.add("						else {");
		componentJavaScript.add("							event.preventDefault();"); 
		componentJavaScript.add("		                	dragX = event.clientX;");
		componentJavaScript.add("		                	dragY = window.pageYOffset + event.clientY;");
		componentJavaScript.add("						}"); 
		componentJavaScript.add("						element.style.left = " + id + "_dragStartLeft;"); 
		componentJavaScript.add("						element.style.top = " + id + "_dragStartTop;"); 
		componentJavaScript.add("						element.style.width = dragX - " + id + "_dragStartLeft;"); 
		componentJavaScript.add("						element.style.height = dragY - " + id + "_dragStartTop;"); 
		componentJavaScript.add("					}");  
		componentJavaScript.add("			     }");

		componentJavaScript.add("	             function " + referenceId + "_plotMouseDown(event) {");
		componentJavaScript.add("	             	var dragX;");  
		componentJavaScript.add("	             	var dragY;");  
		componentJavaScript.add("					var element = document.getElementById(" + referenceId + "_zoomFrameId);"); 
		componentJavaScript.add("					element.style.visibility = \"visible\";"); 
		componentJavaScript.add("					if (" + referenceId + "_browser == \"MSIE\") {");
		componentJavaScript.add("						if (!event) event = window.event;");
		componentJavaScript.add("						event.returnValue = false;"); 
		componentJavaScript.add("		                dragX = event.clientX;");
		componentJavaScript.add("		                dragY = document.body.scrollTop + event.clientY;");
		componentJavaScript.add("					}");
		componentJavaScript.add("					else {");
		componentJavaScript.add("						event.preventDefault();"); 
		componentJavaScript.add("		                dragX = event.clientX;");
		componentJavaScript.add("		                dragY = window.pageYOffset + event.clientY;");
		componentJavaScript.add("					}");  
		componentJavaScript.add("					if (" + referenceId + "_plotZoomClick == false) {");
		componentJavaScript.add("						" + id + "_dragStartLeft = dragX;"); 
		componentJavaScript.add("						" + id + "_dragStartTop = dragY;"); 
		componentJavaScript.add("					}");  
		componentJavaScript.add("					" + referenceId + "_plotZoomClick = true;");
		componentJavaScript.add("					element.style.left = " + id + "_dragStartLeft;"); 
		componentJavaScript.add("					element.style.top = " + id + "_dragStartTop;");
		componentJavaScript.add("					element.style.width = dragX - " + id + "_dragStartLeft;"); 
		componentJavaScript.add("					element.style.height = dragY - " + id + "_dragStartTop;"); 
//		componentJavaScript.add("					if (dragX > " + id + "_dragStartLeft) {");
//		componentJavaScript.add("						if (dragY > " + id + "_dragStartTop) {");
//		componentJavaScript.add("							element.style.left = " + id + "_dragStartLeft;"); 
//		componentJavaScript.add("							element.style.top = " + id + "_dragStartTop;");
//		componentJavaScript.add("							element.style.width = dragX - " + id + "_dragStartLeft;"); 
//		componentJavaScript.add("							element.style.height = dragY - " + id + "_dragStartTop;"); 
//		componentJavaScript.add("			     		}");
//		componentJavaScript.add("						else {");
//		componentJavaScript.add("							element.style.left = " + id + "_dragStartLeft;"); 
//		componentJavaScript.add("							element.style.top = dragY;");
//		componentJavaScript.add("							element.style.width = dragX - " + id + "_dragStartLeft;"); 
//		componentJavaScript.add("							element.style.height = " + id + "_dragStartTop - dragY;"); 
//		componentJavaScript.add("			     		}");
//		componentJavaScript.add("			     	}");
//		componentJavaScript.add("					else {");
//		componentJavaScript.add("						if (dragY > " + id + "_dragStartTop) {");
//		componentJavaScript.add("							element.style.left = dragX;"); 
//		componentJavaScript.add("							element.style.top = " + id + "_dragStartTop;");
//		componentJavaScript.add("							element.style.width = " + id + "_dragStartLeft - dragX;"); 
//		componentJavaScript.add("							element.style.height = dragY - " + id + "_dragStartTop;"); 
//		componentJavaScript.add("			     		}");
//		componentJavaScript.add("						else {");
//		componentJavaScript.add("							element.style.left = dragX;"); 
//		componentJavaScript.add("							element.style.top = dragY;");
//		componentJavaScript.add("							element.style.width = " + id + "_dragStartLeft - dragX;"); 
//		componentJavaScript.add("							element.style.height = " + id + "_dragStartTop - dragY;"); 
//		componentJavaScript.add("			     		}");
//		componentJavaScript.add("			     	}");
		componentJavaScript.add("			     }");
		
		componentJavaScript.add("	             function " + referenceId + "_plotMouseUp(event) {");
		componentJavaScript.add("		            var sectionLeft;");
		componentJavaScript.add("		            var sectionTop;");
		componentJavaScript.add("		            var dataPlotLeft;");
		componentJavaScript.add("		            var dataPlotTop;");
		componentJavaScript.add("		            var millisPerPx = 0;");
		componentJavaScript.add("		            var dyPerPx = 0;");
		componentJavaScript.add("					var element = document.getElementById(" + referenceId + "_zoomFrameId);"); 
		componentJavaScript.add("					" + referenceId + "_plotZoomClick = false;");
		componentJavaScript.add("					if (" + referenceId + "_browser == \"MSIE\") {");
		componentJavaScript.add("						if (!event) event = window.event;");
		componentJavaScript.add("						event.returnValue = false;"); 
		componentJavaScript.add("		                " + id + "_dragStopLeft = event.clientX;");
		componentJavaScript.add("		                " + id + "_dragStopTop = document.body.scrollTop + event.clientY;");
		componentJavaScript.add("					}");
		componentJavaScript.add("					else {");
		componentJavaScript.add("						event.preventDefault();"); 
		componentJavaScript.add("		                " + id + "_dragStopLeft = event.clientX;");
		componentJavaScript.add("		                " + id + "_dragStopTop = window.pageYOffset + event.clientY;");
		componentJavaScript.add("					}");  
		componentJavaScript.add("					element.style.left = 0;"); 
		componentJavaScript.add("					element.style.top = 0;"); 
		componentJavaScript.add("					element.style.width = 1;"); 
		componentJavaScript.add("					element.style.height = 1;"); 
		componentJavaScript.add("					element.style.visibility = \"hidden\";"); 
		componentJavaScript.add("		            sectionLeft = document.getElementById(" + referenceId + "_sectionId).style.left.replace(\"px\", \"\");"); 
		componentJavaScript.add("		            sectionTop = document.getElementById(" + referenceId + "_sectionId).style.top.replace(\"px\", \"\");");
		componentJavaScript.add("		            dataPlotLeft = document.getElementById(" + referenceId + "_dataPlotId).style.left.replace(\"px\", \"\");");
		componentJavaScript.add("		            dataPlotTop = document.getElementById(" + referenceId + "_dataPlotId).style.top.replace(\"px\", \"\");");
		componentJavaScript.add("		            " + id + "_plotLeft = document.getElementById(" + referenceId + "_dataPlotId + \"_plot\").style.left.replace(\"px\", \"\");");
		componentJavaScript.add("		            " + id + "_plotWidth = " + referenceId + "_tWidth;");
		componentJavaScript.add("		            " + id + "_plotTop = document.getElementById(" + referenceId + "_dataPlotId + \"_plot\").style.top.replace(\"px\", \"\");"); 
		componentJavaScript.add("		            " + id + "_plotHeight = " + referenceId + "_yWidth;");
		componentJavaScript.add("		            " + id + "_plotLeft = String(Number(" + id + "_plotLeft) + Number(dataPlotLeft) + Number(sectionLeft) + Number(" + referenceId + "_tStart));");
		componentJavaScript.add("		            " + id + "_plotTop = String(Number(" + id + "_plotTop) + Number(dataPlotTop) + Number(sectionTop) + Number(" + referenceId + "_yStart));"); 
		componentJavaScript.add("		            millisPerPx = (" + referenceId + "_selectedToInMillis - " + referenceId + "_selectedFromInMillis) / Number(" + id + "_plotWidth);");
		componentJavaScript.add("		            dyPerPx = (" + referenceId + "_yMax - " + referenceId + "_yMin) / Number(" + id + "_plotHeight);");		
		componentJavaScript.add("		            " + referenceId + "_selectedFromInMillis = parseInt(" + referenceId + "_selectedFromInMillis + (millisPerPx * (Number(" + id + "_dragStartLeft) - Number(" + id + "_plotLeft))));");
		componentJavaScript.add("		            " + referenceId + "_selectedToInMillis = parseInt(" + referenceId + "_selectedFromInMillis + (millisPerPx * (Number(" + id + "_dragStopLeft) - Number(" + id + "_dragStartLeft))));");
		componentJavaScript.add("		            " + referenceId + "_yMax = parseInt(" + referenceId + "_yMax - (dyPerPx * (Number(" + id + "_dragStartTop) - Number(" + id + "_plotTop))));");
		componentJavaScript.add("		            " + referenceId + "_yMin = parseInt(" + referenceId + "_yMax - (dyPerPx * (Number(" + id + "_dragStopTop) - Number(" + id + "_dragStartTop))));");
		componentJavaScript.add("        			" + referenceId + "_plotReloadCounter = 0;");
		componentJavaScript.add("			        " + referenceId + "_getPlotData();");
		componentJavaScript.add("	    			handleAll(" + referenceId + "_selectedFromInMillis, " + referenceId + "_selectedToInMillis, \"" + referenceId + "\");");
		componentJavaScript.add("			     }");

		return componentJavaScript;
	}
	
	/**
	 * Returns data plot
	 * @param parameterList parameters for plot requested
	 * @return plot
	 */
	public ByteArrayOutputStream getPlot(String parameterList) {
		
		ByteArrayOutputStream plot = new ByteArrayOutputStream();

		if (plotDataStatus.equals("oldData")) {
			plotDataStatus = "readingData";
			plotGenerator = new PlotGenerator(parameterList);
			plotGenerator.addObserver(this);
		}
		else if (plotDataStatus.equals("newData")) {
			plot = dataPlot;
			plotDataStatus = "oldData";
		}
		
		return plot;
	}
	
	/**
	 * Returns data table
	 * @param parameterList parameters for plot requested
	 * @return table
	 */
	public String[] getTable(String parameterList) {
		
		String[] table = null;
		
		ArrayList<String> list = new ArrayList<String>();
		list.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		list.add("<html>");
		list.add("   <head>");
		list.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		list.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		list.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		list.add("		<title>Table loading...</title>");
		list.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
//		list.add("		<link rel=\"shortcut icon\"  type=\"image/png\" href=\"" + web2cdocs + directorySeparator + "Web2cLogo.png\">");
		list.add("      <script type=\"text/javascript\">");
		list.add("         function initPage() {");	
		list.add("       		setTimeout(\"reloadTable()\", 1000);");
		list.add("         }");	
		list.add("       	function reloadTable() {");
		list.add("       		var bool = true;");
		list.add("         		location.reload(bool);");	
		list.add("         }");	
		list.add("      </script>");
		list.add("	 </head>");
		if (tableReloadCounter <= tableMaxReloadCounter) {
			list.add("	 <body onload=\"initPage()\">");
			list.add("		<div>Table loading...</div>");
			list.add("	 </body>");
		}
		else {
			list.add("	 <body>");
			list.add("		<div>Timeout! Error generating table.</div>");
			list.add("	 </body>");
		}
		list.add("</html>");

		if (tableDataStatus.equals("oldData")) {
			tableGenerator = new TableGenerator(parameterList);
			tableGenerator.addObserver(this);			
			table = new String[list.size()];
			for (int i = 0; i < list.size(); i++) {
				table[i] = list.get(i);
			}
			tableDataStatus = "readingData";
			tableReloadCounter = 0;
		}
		else if (tableDataStatus.equals("readingData")) {
			table = new String[list.size()];
			for (int i = 0; i < list.size(); i++) {
				table[i] = list.get(i);
			}
			tableDataStatus = "readingData";
			tableReloadCounter = tableReloadCounter + 1;
		}
		else if (tableDataStatus.equals("newData")) {
			table = dataTable;
			tableDataStatus = "oldData";
			tableReloadCounter = 0;
		}
		
		return table;
	}	
	
	/**
	 * Gets the finally generated plot / table
	 * @param generator plot/table generator
	 * @param data generated plot/table
	 */
	public void update(Observable generator, Object data) {
		
		if (generator.equals(plotGenerator)) {
			dataPlot = (ByteArrayOutputStream)data;
			plotDataStatus = "newData";		
		}
		
		if (generator.equals(tableGenerator)) {
			dataTable = (String[])data;
			tableDataStatus = "newData";		
		}
	}
	
	@SuppressWarnings("unchecked")
	private IntervalCategoryDataset createIntervalCategoryDataset(String[] name) {
		
		int i = 0;
		int j = 0;
		int k = 0;
		Calendar calendarFirst = null;
		Calendar calendarLast = null;
		TaskSeries taskSeries = null;
		TaskSeriesCollection taskSeriesCollection = null;
		ArrayList<ArrayList<String>> states = null;
		ArrayList<ArrayList<Long>> firstOccurrences = null;
		ArrayList<ArrayList<Long>> lastOccurrences = null;
		
		calendarFirst = Calendar.getInstance();	
		calendarLast = Calendar.getInstance();	
		
		states = new ArrayList<ArrayList<String>>();
		firstOccurrences = new ArrayList<ArrayList<Long>>();
		lastOccurrences = new ArrayList<ArrayList<Long>>();
		for (i = 0; i < name.length; i++) {
			ArrayList<String> stateList = new ArrayList<String>();
			ArrayList<Long> firstOccurence = new ArrayList<Long>();
			ArrayList<Long> lastOccurence = new ArrayList<Long>();
			for (j = 0; j < dataArrayLength[i]; j++) {
				if (dataValuesAsText[i][j] != null) {
					if (!stateList.contains(dataValuesAsText[i][j])) {
						stateList.add(dataValuesAsText[i][j]);
						firstOccurence.add((long)(dataTimeStampsInSeconds[i][j] * 1000.0));
					}
				}
			}
			for (Iterator<String> it = stateList.iterator(); it.hasNext();) {
				String state = (String)it.next();
				int lastIndex = 0;
				if (state != null) {
					for (j = 0; j < dataArrayLength[i]; j++) {
						if (dataValuesAsText[i][j].equals(state)) {
							lastIndex = j;
						}
					}
					lastOccurence.add((long)(dataTimeStampsInSeconds[i][lastIndex] * 1000.0));
				}
			}
			states.add(stateList);
			firstOccurrences.add(firstOccurence);
			lastOccurrences.add(lastOccurence);
		}
		
        taskSeries = new TaskSeries("");		
	    taskSeriesCollection = new TaskSeriesCollection();
		for (i = 0; i < name.length; i++) {
			ArrayList<String> stateList = new ArrayList<String>();
			ArrayList<Long> firstOccurence = new ArrayList<Long>();
			ArrayList<Long> lastOccurence = new ArrayList<Long>();
			stateList = (ArrayList<String>)(states.toArray()[i]);
			firstOccurence = (ArrayList<Long>)(firstOccurrences.toArray()[i]);
			lastOccurence = (ArrayList<Long>)(lastOccurrences.toArray()[i]);
//			boolean firstTimeNull = true;
			for (k = 0; k < stateList.toArray().length; k++) {
				calendarFirst.setTimeInMillis((Long)(firstOccurence.toArray()[k]));
				calendarLast.setTimeInMillis((Long)(lastOccurence.toArray()[k]));
				Task mainTask = null;
				if (!((String)(stateList.toArray()[k])).equals("Null")) {
					mainTask = new Task((String)(stateList.toArray())[k], calendarFirst.getTime(), calendarLast.getTime());
				}
//				else {
//					if (firstTimeNull == true) {
//						mainTask = new Task(name[i], calendarFirst.getTime(), calendarLast.getTime());
//						mainTask.addSubtask(new Task(name[i], calendarFirst.getTime(), calendarFirst.getTime()));
//						mainTask.addSubtask(new Task(name[i], calendarLast.getTime(), calendarLast.getTime()));
//						firstTimeNull = false;
//					}
//				}
				boolean lastState = false;
				boolean newState = false;
				int first = 0;
				int last = 0;
				for (j = 0; j < dataArrayLength[i]; j++) {
					if (dataValuesAsText[i][j].equals(stateList.toArray()[k])) {
						newState = true;
					}
					else {
						newState = false;
					}
					if (lastState == false && newState == true) {
						first = j;
						lastState = true;
					}
					else if (lastState == true && newState == false) {
						last = j;
						lastState = false;
						calendarFirst.setTimeInMillis((long)(dataTimeStampsInSeconds[i][first] * 1000.0));
						calendarLast.setTimeInMillis((long)(dataTimeStampsInSeconds[i][last] * 1000.0));
						if (!((String)(stateList.toArray()[k])).equals("Null")) {
							mainTask.addSubtask(new Task((String)(stateList.toArray())[k], calendarFirst.getTime(), calendarLast.getTime()));
						}
					}
					else if (lastState == true && newState == true && j == dataArrayCompletionLength - 1) {
						last = j;
						lastState = false;
						calendarFirst.setTimeInMillis((long)(dataTimeStampsInSeconds[i][first] * 1000.0));
						calendarLast.setTimeInMillis((long)(dataTimeStampsInSeconds[i][last] * 1000.0));
						if (!((String)(stateList.toArray()[k])).equals("Null")) {
							mainTask.addSubtask(new Task((String)(stateList.toArray())[k], calendarFirst.getTime(), calendarLast.getTime()));
						}
					}
				}
				if (mainTask != null) {
					taskSeries.add(mainTask);
				}
			}
		}
		
		taskSeriesCollection.add(taskSeries);
	
		return taskSeriesCollection;
	}
	
	private XYDataset createXYDataset(int i, String dataChannelName) {
		
		int j = 0;
		Calendar calendar = null;
		
		TimeSeries series = new TimeSeries(dataChannelName, Millisecond.class);	
		calendar = Calendar.getInstance();	

		for (j = 0; j < dataArrayLength[i]; j++) {
			if (dataTimeStampsInSeconds[i][j] != 0) {
				calendar.setTimeInMillis((long)(dataTimeStampsInSeconds[i][j] * 1000.0));
				series.addOrUpdate(new Millisecond(calendar.getTime()),(double)(dataValues[i][j]));
			}
		}
		
		TimeSeriesCollection dataset = new TimeSeriesCollection();
		dataset.addSeries(series);
		
		return dataset;
	}

	/**
	 * Handles client action request 
	 * @param parameter1 first action parameter
	 * @param parameter2 second action parameter
	 * @return returns plotArea and dataArea bounds
	 */
	public String doAction (String parameter1, String parameter2) {

		String str = null;	
		
		str = "<zoom><tstart>" + tStart + "</tstart><twidth>" + tWidth + "</twidth><ystart>" + yStart + "</ystart><ywidth>" + yWidth + "</ywidth></zoom>";

		return str;
	}
	
	private void getArchiveData(String[] dataChannelNames, long from, long to, String[] plotType) {
		
		int i = 0;
		int j = 0;
		String[] data = null;
		
		if (to > from) {
			numberOfDataTraces = dataChannelNames.length;
			data = new String[numberOfDataTraces];
			
			dataArrayCompletionLength = 0;
			for (i = 0; i < numberOfDataTraces; i++) {
				dataChannelNames[i] = dataChannelNames[i].replace("()", "");
				data[i] = caa.getArchiveData(dataChannelNames[i] + "(" + from + ";" + to + ")");
				if (data[i].split("</timeStamp>").length > dataArrayCompletionLength) {
					dataArrayCompletionLength = data[i].split("</timeStamp>").length;
				}
			}
			
			archiveDataPlottable = true;
			dataValues = new double[numberOfDataTraces][dataArrayCompletionLength];		
			dataValuesAsText = new String[numberOfDataTraces][dataArrayCompletionLength];
			dataTimeStampsInSeconds = new int[numberOfDataTraces][dataArrayCompletionLength];
			dataArrayLength = new int[numberOfDataTraces];
			for (i = 0; i < numberOfDataTraces; i++) {	
				String[] dataPairs = data[i].split("</timeStamp>");
				dataArrayLength[i] = dataPairs.length;
				if (!data[i].equals("")) {
					for (j = 0; j < dataArrayLength[i]; j++) {
						if (plotType[i].equals("state") || plotType[i].contains("bit")) {
							dataValuesAsText[i][j] = dataPairs[j].split("</value><timeStamp>")[0].replace("<value>", "");
						}
						else {
							if ((plotType[i].equals("log") || plotType[i].equals("logstep")) && (Double.valueOf(dataPairs[j].split("</value><timeStamp>")[0].replace("<value>", "")) <= 0.0)) {
								archiveDataPlottable = false;
							}
							dataValues[i][j] = Double.valueOf(dataPairs[j].split("</value><timeStamp>")[0].replace("<value>", ""));							
						}
						dataTimeStampsInSeconds[i][j] = Integer.valueOf(dataPairs[j].split("</value><timeStamp>")[1]);
					}
				}
			}
		}
	}

	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		int xLeft = 0;
		int yTop = 0;
		int xRight= 0;
		int yBottom = 0;

		backgroundColor = white;
		borderStyle = none;
		borderWidth = thin;
		borderColor = black;
		toolTipText = "To zoom in, click left and right of region of interest.";
		axis = multi;
		type = "lin";
		min = "0";
		max = "0";
		line = "dot";
		inline = false;
		axisName = "";

		xLeft = 0;
		yTop = 470;
		xRight = 1260;
		yBottom = 830;
		marginLeft = ((Integer)(xLeft)).toString() + "px";
		marginTop = ((Integer)(yTop)).toString() + "px";
		width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
		height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("id")) {
				id = referenceId + "_" + value;
			}
			if (name.equals("bounds") && !value.equals("")) {
				xLeft = Integer.valueOf(value.split(",")[0]);
				yTop = Integer.valueOf(value.split(",")[1]);
				xRight = Integer.valueOf(value.split(",")[2]);
				yBottom = Integer.valueOf(value.split(",")[3]);
				marginLeft = ((Integer)(xLeft)).toString() + "px";
				marginTop = ((Integer)(yTop)).toString() + "px";
				width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
				height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
			}
			if (name.equals("backgroundColor")) {
				backgroundColor = value;
			}					
			if (name.equals("border")) {
				if (value.equals("thin")) {
					borderWidth= thin;
					borderStyle = solid;
				}
				else if (value.equals("thick")) {
					borderStyle = solid;
					borderWidth= thick;
				}
				else {
					borderStyle = none;
				}	
			}
			if (name.equals("borderColor")) {
				borderColor = value;
			}		
			if (name.equals("tooltipText")) {
				if (!value.equals("") && !value.equals(" ")) toolTipText = value;
			}	
			if (name.equals("plotAxis")) {
				if (value.equals("") || value.equals(" ")) axis = multi;
				else axis = value;
			}	
			if (name.equals("plotType")) {
				if (!value.equals("") && !value.equals(" ")) type = value;
			}	
			if (name.equals("yMin")) {
				if (!value.equals("") && !value.equals(" ")) min = value;
			}	
			if (name.equals("yMax")) {
				if (!value.equals("") && !value.equals(" ")) max = value;
			}		
			if (name.equals("lineType")) {
				if (!value.equals("") && !value.equals(" ")) line = value;
			}		
			if (name.equals("inline")) {
				if (value.equals("yes")) inline = true;
				else inline = false;
			}		
			if (name.equals("axisName")) {
				if (!value.equals("") && !value.equals(" ")) axisName = value;
			}			
		}		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class PlotGenerator extends Observable implements Runnable {

		private Thread generator = null;
		private String parameterList = "";
		
		/**	
		 * @param list parameters for plot requested
		 */
		public PlotGenerator (String list) {
			
			super();
			
			parameterList = list;
		
			if (generator == null) {
				generator = new Thread(this);
				generator.start();
			}
		}
		
		/**
		 * Generates page
		 */
		public void run() {	

			int i = 0;
			int j = 0;
			ByteArrayOutputStream plotOut = null;
			String[] dataChannel = null;
			String[] uri = null;
			String[] name = null;
			String[] plotType = null;
			String[] plotMin = null;
			String[] plotMax = null;
			Calendar calendar = null;
			long from = 0;
			long to = 0;
			int yMin = 0;
			int yMax = 0;	
			int lineOrLogPlot = 0;
			int statePlot = 0;
			int bitPlot = 0;
			String fromDate = "";
			String toDate = "";
			JFreeChart chart = null;
			String plotTitle = "";
			ChartPanel chartPanel = null;;
			BufferedImage bf = null;
			Rectangle2D dataArea = null;
			Rectangle2D plotArea = null;
			int plotWidth = 0;
			int plotHeight = 0;
			int error = 0;
			
			calendar = Calendar.getInstance();
			long now = calendar.getTimeInMillis();
			
			parameterList = parameterList.replaceAll("%23", "#");
			
			plotOut = new ByteArrayOutputStream();
			if (parameterList.contains("NaN") || parameterList.contains("undefined") || parameterList.contains("<value></value>") || parameterList.contains("!DOCTYPE")) {
				error = error + 1;
			}
			else {
				dataChannel = (parameterList).split("</value>")[0].replace("<trace>", "").split("</trace>");
				uri = new String[dataChannel.length];
				name = new String[dataChannel.length];
				plotType = new String[dataChannel.length];
				plotMin = new String[dataChannel.length];
				plotMax = new String[dataChannel.length];
				for (i = 0; i < dataChannel.length; i++) {	
					uri[i] = dataChannel[i].split("</uri>")[0].replace("<uri>", "");
					name[i] = dataChannel[i].split("</uri>")[1].split("</name>")[0].replace("<name>", "");
					plotType[i] = dataChannel[i].split("</uri>")[1].split("</name>")[1].split("</plotType>")[0].replace("<plotType>", "");
					plotMin[i] = dataChannel[i].split("</uri>")[1].split("</name>")[1].split("</plotType>")[1].split("</plotMin>")[0].replace("<plotMin>", "");
					plotMax[i] = dataChannel[i].split("</uri>")[1].split("</name>")[1].split("</plotType>")[1].split("</plotMin>")[1].replace("<plotMax>", "").replace("</plotMax>", "");
				}
				from = Math.round(Double.valueOf((parameterList).split("</value>")[1].replace("<value>", "")));		
				if (from >= now) {
					from = now - 86400000;  //24h
				}
				to = Math.round(Double.valueOf((parameterList).split("</value>")[2].replace("<value>", "")));
				if (to >= now) {
					to = now;
				}
				if (from >= to) {
					from = to - 86400000;  //24h
				}			
				calendar.setTimeInMillis((long)(from));
				fromDate = calendar.getTime().toString();		
				calendar.setTimeInMillis((long)(to));
				toDate = calendar.getTime().toString();
				yMin = Integer.valueOf((parameterList).split("</value>")[3].replace("<value>", ""));
				yMax = Integer.valueOf((parameterList).split("</value>")[4].replace("<value>", ""));
				if (yMin > yMax) {
					int valMin = yMin;
					int valMax = yMax;
					yMin = valMax;
					yMax = valMin;
				}
				
				getArchiveData(uri, from, to, plotType);
				if (archiveDataPlottable == false) {
					error = error + 1;
				}
				else {
					try {
						for (i = 0; (i < numberOfDataTraces) && (i < maxTraces); i++) {
							if (plotType[i].equals("state")) {
								statePlot = statePlot + 1;
							}
							else if (plotType[i].contains("bit")) {
								bitPlot = bitPlot + 1;
							}
							else {
								lineOrLogPlot = lineOrLogPlot + 1;
							}
						}
						if (statePlot > 0) {   //"state": Gantt chart
							IntervalCategoryDataset dataset = createIntervalCategoryDataset(name);
						    chart = ChartFactory.createGanttChart(plotTitle + fromDate + " - " + toDate, name[0], "Date", dataset, false, true, false); 
					        CategoryPlot plot = chart.getCategoryPlot();
					        GanttRenderer renderer = (GanttRenderer)plot.getRenderer();	
							renderer.setMaximumBarWidth(0.1);
							CategoryAxis categoryAxis = plot.getDomainAxis();
							categoryAxis.setMaximumCategoryLabelWidthRatio(10.0f);
					        ValueAxis rangeAxis = plot.getRangeAxis();
							rangeAxis.setLowerMargin(0);
							rangeAxis.setUpperMargin(0);
							plot.setRangeAxis(rangeAxis);
							rangeAxis.setAutoRange(false);
							rangeAxis.setRange(from, to);						
							if (inline == true) {
								AxisSpace space = new AxisSpace();
								space.setLeft(axisSpaceLeft);
								space.setRight(axisSpaceRight);
								plot.setFixedDomainAxisSpace(space);
							}					
						}
						else if (bitPlot > 0) {  //"bit": Gantt chart
							for (i = 0; i < name.length; i++) {
								for (j = 0; j < dataArrayLength[i]; j++) {	
									if (Double.valueOf(dataValuesAsText[i][j]) == 0.0) {
										dataValuesAsText[i][j] = "Null";
									}
									else {
										int value = Float.valueOf(dataValuesAsText[i][j]).intValue();
										int bit = Integer.valueOf(plotType[i].replace("bit(", "").replace(")", ""));
										int bitValue = (value >>> bit) & 0x1;
										if (bitValue == 1) {
											dataValuesAsText[i][j] = name[i];
										}
										else {
											dataValuesAsText[i][j] = "Null";
										}
									}
								}
							}
							IntervalCategoryDataset dataset = createIntervalCategoryDataset(name);
						    chart = ChartFactory.createGanttChart(plotTitle + fromDate + " - " + toDate, name[0], "Date", dataset, false, true, false); 
					        CategoryPlot plot = chart.getCategoryPlot();
					        GanttRenderer renderer = (GanttRenderer)plot.getRenderer();
							for (i = 0; i < name.length; i++) {
								renderer.setSeriesPaint(i, new Color(0,128,0));
							}
							renderer.setMaximumBarWidth(0.1);
							CategoryAxis categoryAxis = plot.getDomainAxis();
							categoryAxis.setMaximumCategoryLabelWidthRatio(10.0f);
							ValueAxis rangeAxis = plot.getRangeAxis();
							rangeAxis.setLowerMargin(0);
							rangeAxis.setUpperMargin(0);
							plot.setRangeAxis(rangeAxis);	
							rangeAxis.setAutoRange(false);
							rangeAxis.setRange(from, to);						
							if (inline == true) {
								AxisSpace space = new AxisSpace();
								space.setLeft(axisSpaceLeft);
								space.setRight(axisSpaceRight);
								plot.setFixedDomainAxisSpace(space);
							}
						}
						else if (lineOrLogPlot > 0) {  //"lin", "log", "step": XY Chart
							XYDataset dataset = createXYDataset(0, plotTitle);
							chart = ChartFactory.createTimeSeriesChart(plotTitle + fromDate + " - " + toDate , "Date", name[0], dataset, true, true, false);
							XYPlot plot = chart.getXYPlot();
							ValueAxis domainAxis = plot.getDomainAxis();
							domainAxis.setLowerMargin(0);
							domainAxis.setUpperMargin(0);
							plot.setDomainAxis(domainAxis);	
							domainAxis.setAutoRange(false);
							domainAxis.setRange(from , to);						
							if (inline == true) {
								AxisSpace space = new AxisSpace();
								space.setLeft(axisSpaceLeft);
								space.setRight(axisSpaceRight);
								plot.setFixedRangeAxisSpace(space);
							}				
							if (axis.equals(single)){
								if (type.equals("lin") || type.equals("step")) {
									NumberAxis axis = new NumberAxis(axisName);
									if (!min.equals(max)) {
										axis.setAutoRange(false);
										double maxValue = Double.valueOf(max);
										double minValue = Double.valueOf(min);
										axis.setRange(minValue + ((maxValue - minValue) * (yMin / 100.0)) , minValue + ((maxValue - minValue) * (yMax / 100.0)));
									}
									else {
										axis.setAutoRange(true);
									}
									axis.setLabelPaint(Color.BLACK);
									axis.setTickLabelPaint(Color.BLACK);
									plot.setRangeAxis(0,axis);
									plot.setRangeAxisLocation(0, AxisLocation.BOTTOM_OR_LEFT);
								}
								if (type.equals("log") || type.equals("logstep")) {
									LogarithmicAxis axis = new LogarithmicAxis(axisName);
									axis.setLog10TickLabelsFlag(true);
									axis.setAllowNegativesFlag(false);
									if (!min.equals(max)) {
										axis.setAutoRange(false);
										double maxValue = Double.valueOf(max);
										double maxLogValue = Math.log10(maxValue);
										double minValue = Double.valueOf(min);
										double minLogValue = Math.log10(minValue);
										axis.setRange(minValue * Math.pow(10, ((maxLogValue - minLogValue) * (yMin / 100.0))) , minValue * Math.pow(10, ((maxLogValue - minLogValue) * (yMax / 100.0))));
									}
									else {
										axis.setAutoRange(true);
									}
									axis.setLabelPaint(Color.BLACK);
									axis.setTickLabelPaint(Color.BLACK);
									plot.setRangeAxis(0,axis);
									plot.setRangeAxisLocation(0, AxisLocation.BOTTOM_OR_LEFT);
								}
								for (i = 0; (i < numberOfDataTraces) && (i < maxTraces); i++) {
									plotType[i] = type;
								}
							}
							for (i = 0; (i < numberOfDataTraces) && (i < maxTraces); i++) {
								dataset = createXYDataset(i, name[i]);
								plot.setDataset(i, dataset);
								if (axis.equals(multi)){
									if (plotType[i].equals("lin") || plotType[i].equals("step")) {
										NumberAxis axis = new NumberAxis(name[i]);
										axis.setLabelPaint(traceColor[i]);
										axis.setTickLabelPaint(traceColor[i]);
										if (!plotMin[i].equals(plotMax[i])) {
											axis.setAutoRange(false);
											double maxValue = Double.valueOf(plotMax[i]);
											double minValue = Double.valueOf(plotMin[i]);
											axis.setRange(minValue + ((maxValue - minValue) * (yMin / 100.0)) , minValue + ((maxValue - minValue) * (yMax / 100.0)));
										}
										else {
											axis.setAutoRange(true);
										}
										plot.setRangeAxis(i,axis);
										plot.setRangeAxisLocation(i, AxisLocation.BOTTOM_OR_LEFT);
										plot.mapDatasetToRangeAxis(i, i);
									}
									if (plotType[i].equals("log") || plotType[i].equals("logstep")) {
										LogarithmicAxis axis = new LogarithmicAxis(name[i]);
										axis.setLabelPaint(traceColor[i]);
										axis.setTickLabelPaint(traceColor[i]);
										axis.setLog10TickLabelsFlag(true);
										axis.setAllowNegativesFlag(false);
										if (!plotMin[i].equals(plotMax[i])) {
											axis.setAutoRange(false);
											double maxValue = Double.valueOf(plotMax[i]);
											double maxLogValue = Math.log10(maxValue);
											double minValue = Double.valueOf(plotMin[i]);
											double minLogValue = Math.log10(minValue);
											axis.setRange(minValue * Math.pow(10, ((maxLogValue - minLogValue) * (yMin / 100.0))) , minValue * Math.pow(10, ((maxLogValue - minLogValue) * (yMax / 100.0))));
										}
										else {
											axis.setAutoRange(true);
											axis.setAutoRangeNextLogFlag(true);
										}
										plot.setRangeAxis(i,axis);
										plot.setRangeAxisLocation(i, AxisLocation.BOTTOM_OR_LEFT);
										plot.mapDatasetToRangeAxis(i, i);
									}
								}
								if (plotType[i].equals("lin") || plotType[i].equals("log")) {
									if (line.equals("dot")) {
										XYDotRenderer renderer = new XYDotRenderer();  //dots only
										renderer.setDotHeight(3);
										renderer.setDotWidth(3);
										renderer.setSeriesPaint(0, traceColor[i]);
										plot.setRenderer(i, renderer);
									}
									else if (line.equals("line")) {
										StandardXYItemRenderer renderer = new StandardXYItemRenderer();  //lines only
										renderer.setSeriesPaint(0, traceColor[i]);
										plot.setRenderer(i, renderer);
									}
									else if (line.equals("linedot")) {
										XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();  //lines and dots etc.
										renderer.setLinesVisible(true);
										renderer.setShapesVisible(true);
										renderer.setShapesFilled(false);
										renderer.setSeriesPaint(0, traceColor[i]);
										plot.setRenderer(i, renderer);
									}
								}
								else if (plotType[i].equals("step") || plotType[i].equals("logstep"))  {
									XYStepRenderer renderer = new XYStepRenderer();
									renderer.setSeriesPaint(0, traceColor[i]);
									plot.setRenderer(i, renderer);
								}
							}
						}
						else {
							System.out.println("Error generating chart!");			
						}
					
						plotWidth = Integer.valueOf(horizontalArchivePlotSize.substring(0, horizontalArchivePlotSize.length() - 2));
						plotHeight = Integer.valueOf(verticalArchivePlotSize.substring(0, verticalArchivePlotSize.length() - 2));
						
						chartPanel = new ChartPanel(chart);
						chartPanel.setSize(plotWidth, plotHeight);
						bf = new BufferedImage(plotWidth, plotHeight, BufferedImage.TYPE_INT_RGB);
						chartPanel.paint(bf.getGraphics());
						dataArea = chartPanel.getChartRenderingInfo().getPlotInfo().getDataArea();
						plotArea = chartPanel.getChartRenderingInfo().getPlotInfo().getPlotArea();
						
						double chartWidth = chartPanel.getBounds().width / chartPanel.getScaleX();
						double chartHeight = chartPanel.getBounds().height / chartPanel.getScaleY();
						int dataAreaWidth = dataArea.getBounds().width;
						int dataAreaHeight = dataArea.getBounds().height;
						double scalingFactorX = (plotWidth - (chartWidth - dataAreaWidth)) / dataAreaWidth;  
						double scalingFactorY = (plotHeight - (chartHeight - dataAreaHeight)) / dataAreaHeight;  					
						tStart = dataArea.getBounds().x;
						tWidth = (int)((double)dataArea.getBounds().width * scalingFactorX);
						yStart = dataArea.getBounds().y;  
						yWidth = (int)((double)dataArea.getBounds().height * scalingFactorY);
//					
//						System.out.println("chart (width/height): " + (chartPanel.getBounds().width / chartPanel.getScaleX()) + " / " + (chartPanel.getBounds().height / chartPanel.getScaleY()));
//						System.out.println("plot (width/height): " + plotArea.getBounds().width + " / " + plotArea.getBounds().height);					
//						System.out.println("data (width/height): " + dataArea.getBounds().width + " / " + dataArea.getBounds().height);
//						System.out.println("scalingFactor  (x/y): " + scalingFactorX + " / " + scalingFactorY);
//						System.out.println("tFrom/tTo yFrom/yTo: " + from + "/" + to + " " + yMin + "/" + yMax);
//						System.out.println("tStart/tWidth yStart/yWidth: " + tStart + "/" + tWidth + " " + yStart + "/" + yWidth);

						ChartUtilities.writeChartAsPNG(plotOut, chart, plotWidth, plotHeight);
					}
					catch (Exception e) {
						error = error + 1;			
						System.out.println(e.toString());
						e.printStackTrace();
						System.out.println("Error generating chart!");
					}
				}
			}
		
			if (error > 0) {
				plotWidth = Integer.valueOf(horizontalArchivePlotSize.substring(0, horizontalArchivePlotSize.length() - 2));
				plotHeight = Integer.valueOf(verticalArchivePlotSize.substring(0, verticalArchivePlotSize.length() - 2));
			    bf = new BufferedImage(plotWidth, plotHeight, BufferedImage.TYPE_INT_ARGB);
			    Graphics2D gr2D = bf.createGraphics();
			    Font font = new Font("TimesRoman", Font.BOLD, 20);
			    gr2D.setFont(font);
			    String message = "Error generating chart!";
			    FontMetrics fontMetrics = gr2D.getFontMetrics();
			    int stringWidth = fontMetrics.stringWidth(message);
			    int stringHeight = fontMetrics.getAscent();
			    gr2D.setPaint(Color.black);
			    gr2D.drawString(message, (plotWidth - stringWidth) / 2, plotHeight / 2 + stringHeight / 4);
			    try {
					ImageIO.write(bf, "PNG", plotOut);
				} 
			    catch (IOException e1) {
					e1.printStackTrace();
				}
			}
			
			setChanged();
			notifyObservers(plotOut);
			
			generator.interrupt();
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class TableGenerator extends Observable implements Runnable {

		private Thread generator = null;
		private String parameterList = "";
		
		/**	
		 * @param list parameters for plot requested
		 */
		public TableGenerator (String list) {
			
			super();
			
			parameterList = list;
		
			if (generator == null) {
				generator = new Thread(this);
				generator.start();
			}
		}
		
		/**
		 * Generates page
		 */
		public void run() {	
			
			int i = 0;
			int n = 0;
			int j = 0;
			String[] dataChannel = null;
			String[] uri = null;
			String[] name = null;
			String[] plotType = null;
			String[] plotMin = null;
			String[] plotMax = null;
			long from = 0;
			long to = 0;
			Calendar calendar = null;
			String[] out = null;
			
			calendar = Calendar.getInstance();
			
			parameterList = parameterList.replaceAll("%23", "#");
			
			if (parameterList.contains("NaN") || parameterList.contains("undefined") || parameterList.contains("<value></value>") || parameterList.contains("!DOCTYPE")) {
				out = new String[1];
				out[0] = ("Error generating table!");
			}
			else {			
				dataChannel = (parameterList).split("</value>")[0].replace("<trace>", "").split("</trace>");
				uri = new String[dataChannel.length];
				name = new String[dataChannel.length];
				plotType = new String[dataChannel.length];
				plotMin = new String[dataChannel.length];
				plotMax = new String[dataChannel.length];
				for (i = 0; i < dataChannel.length; i++) {	
					uri[i] = dataChannel[i].split("</uri>")[0].replace("<uri>", "");
					name[i] = dataChannel[i].split("</uri>")[1].split("</name>")[0].replace("<name>", "");
					plotType[i] = dataChannel[i].split("</uri>")[1].split("</name>")[1].split("</plotType>")[0].replace("<plotType>", "");
					plotMin[i] = dataChannel[i].split("</uri>")[1].split("</name>")[1].split("</plotType>")[1].split("</plotMin>")[0].replace("<plotMin>", "");
					plotMax[i] = dataChannel[i].split("</uri>")[1].split("</name>")[1].split("</plotType>")[1].split("</plotMin>")[1].replace("<plotMax>", "").replace("</plotMax>", "");
				}
				from = Long.valueOf((parameterList).split("</value>")[1].replace("<value>", ""));
				to = Long.valueOf((parameterList).split("</value>")[2].replace("<value>", ""));
				getArchiveData(uri, from, to, plotType);
			
				n = 0;
				out = new String[(numberOfDataTraces * dataArrayCompletionLength) + 1];
				out[n] =  "Description, Value, Time, Timestamp/ms" ;
				for (i = 0; i < numberOfDataTraces; i++) {
					for (j = 0; j < dataArrayLength[i]; j++) {
						n = n + 1;
						if (dataTimeStampsInSeconds[i][j] != 0) {
							calendar.setTimeInMillis((long)(dataTimeStampsInSeconds[i][j] * 1000.0));
							out[n] = name[i].toString() + ", " + ((Double)(dataValues[i][j])).toString() + ", " + calendar.getTime().toString() + ", " + calendar.getTimeInMillis();
						}
					}
				}
			}
			
			setChanged();
			notifyObservers(out);
			
			generator.interrupt();
		}
	}
}
