package de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents;

import java.util.ArrayList;
import java.util.Iterator;

import de.desy.web2cToolkit.web2cArchive.web2cArchiveAccess.Web2cCommonArchiveAccess;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerDataPlot;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerDataSelector;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * @author R.Bacher, DESY
 */
public class Web2cArchiveViewerSection implements IWeb2cConstants {

	private ArrayList<Object> sectionsElements = new ArrayList<Object>();
	private ArrayList<ArrayList<String>> sectionsElementsHTML = new ArrayList<ArrayList<String>>();
	private ArrayList<ArrayList<String>> sectionsElementsJavaScript = new ArrayList<ArrayList<String>>();
	private String hostURL = null;
	private String referenceId = null;
	private String referencePoint = null;
	private String id = null;
	private String width = null;
	private String height = null;
	private String marginLeft = null;
	private String marginTop = null;
	private String borderStyle = null;
	private String borderWidth = null;
	private String borderColor = null;
	private String backgroundColor = null;
	private Web2cCommonArchiveAccess caa = null;
	private String fromTime = "";
	private String toTime = "";

	private String error = "";

	/**
	 * Section
	 * @param url URL of host servicing the request
	 * @param refId ID of reference component which component belongs to
	 * @param refPoint coordinates (left upper corner) of reference component which component belongs to
	 * @param attributeNames list of section's valid attribute names
	 * @param attributeValues list of actual section's attribute values
	 * @param elementsNames list of names of components which belong to the actual section
	 * @param elementsAttributeNames list of valid attribute names of components which belong to the actual section
	 * @param elementsAttributeValues list of actual attribute values of components which belong to the actual section
	 * @param commonArchiveAccess actual common archive access instance
	 * @param from start time of archive data interval
	 * @param to end time of archive data interval
	 */
	public Web2cArchiveViewerSection(String url, String refId, String refPoint, ArrayList<String> attributeNames, ArrayList<String> attributeValues, ArrayList<String> elementsNames, ArrayList<ArrayList<String>> elementsAttributeNames, ArrayList<ArrayList<String>> elementsAttributeValues, Web2cCommonArchiveAccess commonArchiveAccess, String from, String to) {
		
		hostURL = url;
		referenceId = refId;
		referencePoint = refPoint;
		caa = commonArchiveAccess;
		fromTime = from;
		toTime = to;
		decodeAttributes(attributeNames, attributeValues);
		setSectionElements(elementsNames, elementsAttributeNames, elementsAttributeValues);
	}
	
	/**
	 * @return component id
	 */
	public String getId() {
		
		return id;
	}
	
	/**
	 * Returns a list of components which belong to the actual section
	 * @return list
	 */
	public ArrayList<Object> getComponents() {
		
		ArrayList<Object> list = new ArrayList<Object>();
		for (Iterator<Object> it = sectionsElements.iterator(); it.hasNext();) {
			list.add(it.next());
		}
		
		return list;
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> sectionHTML = new ArrayList<String>();	

		int backButtonWidth = 30;
		int backButtonHeight = 30;
		int backButtonXPos = Integer.valueOf(width.split("p")[0]) - 30;
		int backButtonYPos = 0;
		
		sectionHTML.add("			<div id=\"" + id + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "\">");
		for(Iterator<ArrayList<String>> it1 = sectionsElementsHTML.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				sectionHTML.add((String)it2.next());
			}
		}
		sectionHTML.add("			</div>");
		sectionHTML.add("			<input type=\"button\" id=\"" + id + "_backButton\" value=\"X\" title=\"\" onclick=\"backButton_click()\" style=\"position:absolute; left:" + String.valueOf(backButtonXPos) + "px" + "; top:" + String.valueOf(backButtonYPos) + "px" + "; width:" + String.valueOf(backButtonWidth) + "px" + "; height:" + String.valueOf(backButtonHeight) + "px" + "; border-style:outset; color:black; background-color:white; font-size:18px; font-weight:bold\">");
		sectionHTML.add("			<img id=\"" + id + "_zoomFrame\" src=\"web2cdocs/zoomframe.gif\" title=\"\" alt=\"\" style=\"visibility:hidden; position:absolute; left:0px; top:0px; width:1px; height:1px; border-style:dashed; border-width:thin; border-color:black\">");
		
		return sectionHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> sectionJavaScript = new ArrayList<String>();	
		
		sectionJavaScript.add("	    //ArchiveViewerSection: " + id);
		sectionJavaScript.add("	        var " + referenceId + "_sectionId = \"" + id + "\";");
		sectionJavaScript.add("                var " + referenceId + "_zoomFrameId = \"" + id + "_zoomFrame\";");
		for(Iterator<ArrayList<String>> it1 = sectionsElementsJavaScript.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				sectionJavaScript.add((String)it2.next());
			}
		}

		sectionJavaScript.add("		function backButton_click() {");
		sectionJavaScript.add("			document.clear();");  
		sectionJavaScript.add("			window.close();");  
		sectionJavaScript.add("		};");  
		
		return sectionJavaScript;
	}
	
	public String getError() {
		
		return error;
	}

	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		int xLeft = 0;
		int yTop = 0;
		int xRight= 0;
		int yBottom = 0;
		int xRef = 0;
		int yRef = 0;

		borderStyle = none;
		borderWidth = thin;
		borderColor = black;
		backgroundColor = white;

		xRef = Integer.valueOf((referencePoint).split(",")[0]);
		yRef = Integer.valueOf((referencePoint).split(",")[1]);
		xLeft = 0 + xRef;
		yTop = 0 + yRef;
		xRight = 1260 + xRef;
		yBottom = 830 + yRef;
		marginLeft = ((Integer)(xLeft)).toString() + "px";
		marginTop = ((Integer)(yTop)).toString() + "px";
		width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
		height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("id")) {
				id = referenceId + "_" + value;
			}
			if (name.equals("bounds") && !value.equals("")) {
				xLeft = Integer.valueOf(value.split(",")[0]) + xRef;
				yTop = Integer.valueOf(value.split(",")[1]) + yRef;
				xRight = Integer.valueOf(value.split(",")[2]) + xRef;
				yBottom = Integer.valueOf(value.split(",")[3]) + yRef;
				marginLeft = ((Integer)(xLeft)).toString() + "px";
				marginTop = ((Integer)(yTop)).toString() + "px";
				width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
				height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
			}			
			if (name.equals("border")) {
				if (value.equals("thin")) {
					borderWidth= thin;
					borderStyle = solid;
				}
				else if (value.equals("thick")) {
					borderStyle = solid;
					borderWidth= thick;
				}
				else {
					borderStyle = none;
				}	
			}
			if (name.equals("borderColor")) {
				borderColor = value;
			}
			if (name.equals("backgroundColor")) {
				backgroundColor = value;
			}
		}		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}
	
	private void setSectionElements(ArrayList<String> elementsNames, ArrayList<ArrayList<String>> elementsAttributeNames, ArrayList<ArrayList<String>> elementsAttributeValues) {
		
		Iterator<String> it3 = elementsNames.iterator();
		Iterator<ArrayList<String>> it2 = elementsAttributeValues.iterator();
		for (Iterator<ArrayList<String>> it1 = elementsAttributeNames.iterator(); it1.hasNext();) {
			String element = (String)it3.next();
			ArrayList<String> names = (ArrayList<String>)it1.next();
			ArrayList<String> values = (ArrayList<String>)it2.next();
			if (element == "ArchiveViewerDataSelector") {
				Web2cArchiveViewerDataSelector value = new Web2cArchiveViewerDataSelector(hostURL, referenceId, names, values, fromTime, toTime);
				sectionsElements.add((Object)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
			}
			if (element == "ArchiveViewerDataPlot") {
				Web2cArchiveViewerDataPlot value = new Web2cArchiveViewerDataPlot(referenceId, names, values, caa);
				sectionsElements.add((Object)value);
				sectionsElementsHTML.add(value.getHTML());
				sectionsElementsJavaScript.add(value.getJavaScript());
			}
		}
	}
}
