package de.desy.web2cToolkit.web2cEditor;

import java.util.ArrayList;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;


/**
 * @author R.Bacher, DESY
 */
public class Web2cEditorActionButton implements IWeb2cConstants {

	private String hostURL = null;
	private String referenceId = null;
	private String id = null;
	private String width = null;
	private String height = null;
	private String marginLeft = null;
	private String marginTop = null;
	private String caption = null;
	private String color = null;
	private String fontSize = null;
	private String fontThickness= null;
	private String backgroundColor = null;
	private String toolTipText = null;
	private String uri = null;
	private String userPrivilege = null;
	
	/**
	 * Buttons on Editor Page
	 * @param url URL of host servicing the request
	 * @param refId ID of reference component which component belongs to
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 */
	public Web2cEditorActionButton(String url, String refId, ArrayList<String> names, ArrayList<String> values, String privilege) {

		hostURL = url;
		referenceId = refId;
		userPrivilege = layman;
		if (userPrivilege == null) {
			userPrivilege = layman;
		}
		else {
			userPrivilege = privilege;
		}
		decodeAttributes(names, values);
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		String html = null;
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		
		html = "			<input type=\"button\" id=\"" + id + "\" value=\"" + caption + "\" title=\"" + toolTipText + "\"";
		if ((uri.split(":")[1]).substring(2).equals("addComponent")) {
			html = html + " onclick=\"addComponent()\"";
		}
		else if ((uri.split(":")[1]).substring(2).equals("removeComponent")) {
			html = html + " onclick=\"removeComponent()\"";
		}
		else if ((uri.split(":")[1]).substring(2).equals("showPreview")) {
			html = html + " onclick=\"showPreview()\"";
		}
		else if ((uri.split(":")[1]).substring(2).equals("saveToFile")) {
			html = html + " onclick=\"saveToFile()\"";
		}
		else if ((uri.split(":")[1]).substring(2).equals("readFromFile")) {
			html = html + " onclick=\"readFromFile()\"";
		}
		else if ((uri.split(":")[1]).substring(2).equals("confirmAttributeChanges")) {
			html = html + " onclick=\"confirmAttributeChanges()\"";
		}
		else if ((uri.split(":")[1]).substring(2).equals("resetPage")) {
			html = html + " onclick=\"resetPage()\"";
		}
		else if ((uri.split(":")[1]).substring(2).equals("nextComponent")) {
			html = html + " onclick=\"nextComponent()\"";
		}
		else {}
		html = html + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:" + color + "; background-color:" + backgroundColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + ";\">";	
		if ((uri.split(":")[1]).substring(2).equals("saveToFile") && (userPrivilege.equals(layman))) {
			html = "";
		}
		componentHTML.add(html);
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		componentJavaScript.add("	//EditorActionButton: " + id);
		componentJavaScript.add("	var resetTimer;");
		
		componentJavaScript.add("	function resetAll() {");
		componentJavaScript.add("	    var parentElement = document.getElementById(drawingFrameId);");
		componentJavaScript.add("		var childElements = parentElement.childNodes;");
		componentJavaScript.add("	    if (childElements.length > childOffset) {");
		componentJavaScript.add("		    selectedElement = childElements[childElements.length - 1];");
		componentJavaScript.add("			removeComponent();");
		componentJavaScript.add("	    } else {");
		componentJavaScript.add("	        clearInterval(resetTimer);");
		componentJavaScript.add("	    }");
		componentJavaScript.add("	}");
		
		if ((uri.split(":")[1]).substring(2).equals("addComponent")) {
			componentJavaScript.add("	function addComponent() {");
			componentJavaScript.add("		try {");
			componentJavaScript.add("			var now = new Date();"); 
			componentJavaScript.add("			var elementId;");
			componentJavaScript.add("			var elementType;");
			componentJavaScript.add("			if (selectedElement != null) {");
			componentJavaScript.add("				selectedElement.style.borderStyle = \"solid\";"); 
			componentJavaScript.add("			}");
			componentJavaScript.add("			var componentList = document.getElementById(componentListId);");
			componentJavaScript.add("			var parentElement = document.getElementById(drawingFrameId);"); 
			componentJavaScript.add("			var childElement = document.createElement(\"img\");");
			componentJavaScript.add("			childElement.id = String(now.getTime());");             
			componentJavaScript.add("			childElement.style.borderWidth = \"1px\";");              
			componentJavaScript.add("			childElement.style.borderStyle = \"solid\";");              
			componentJavaScript.add("			childElement.style.borderColor = \"black\";");
			componentJavaScript.add("			childElement.style.position = \"absolute\";");
			componentJavaScript.add("			childElement.style.left = \"0px\";");
			componentJavaScript.add("			childElement.style.top = \"0px\";"); 
			componentJavaScript.add("			childElement.style.minWidth = \"1px\";");
			componentJavaScript.add("			childElement.style.minHeight = \"1px\";");
			componentJavaScript.add("			if (browser == \"MSIE\") {");
			componentJavaScript.add("				childElement.style.width = \"50px\";");
			componentJavaScript.add("				childElement.style.height = \"50px\";");
			componentJavaScript.add("			}");
			componentJavaScript.add("			else {");
			componentJavaScript.add("				childElement.style.width = String(50 - (2 * parseInt(childElement.style.borderWidth.substring(0, childElement.style.borderWidth.length - 2)))) + \"px\";");
			componentJavaScript.add("				childElement.style.height = String(50 -(2 * parseInt(childElement.style.borderWidth.substring(0, childElement.style.borderWidth.length - 2)))) + \"px\";");
			componentJavaScript.add("			}");  
			componentJavaScript.add("			parentElement.appendChild(childElement);");
			componentJavaScript.add("			elementId = childElement.id;");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Section\") {elementType = \"Section\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"EmbeddedPage\") {elementType = \"EmbeddedPage\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Label\") {elementType = \"Label\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"LabelList\") {elementType = \"LabelList\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Text\") {elementType = \"Text\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"TextList\") {elementType = \"TextList\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Value\") {elementType = \"Value\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"ValueList\") {elementType = \"ValueList\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Indicator\") {elementType = \"Indicator\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"IndicatorList\") {elementType = \"IndicatorList\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Image\") {elementType = \"Image\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"AnimatedImage\") {elementType = \"AnimatedImage\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Button\") {elementType = \"Button\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Slider\") {elementType = \"Slider\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"WheelSwitch\") {elementType = \"WheelSwitch\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Input\") {elementType = \"Input\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Ticker\") {elementType = \"Ticker\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Time\") {elementType = \"Time\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Trend\") {elementType = \"Trend\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Histo\") {elementType = \"Histo\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Video\") {elementType = \"Video\";}");
			componentJavaScript.add("			if (componentList.options[componentList.options.selectedIndex].value == \"Chat\") {elementType = \"Chat\";}");
			componentJavaScript.add("			if (selectedElement != null) {");
			componentJavaScript.add("				selectedElement.style.backgroundColor = \"white\";");
			componentJavaScript.add("			}");
			componentJavaScript.add("			selectedElement = childElement;");
			componentJavaScript.add("			selectedElement.style.borderStyle = \"dotted\";"); 
			componentJavaScript.add("			parentElement.style.borderStyle = \"solid\";"); 
			componentJavaScript.add("			if (browser == \"MSIE\") {");
			componentJavaScript.add("				selectedElement.attachEvent(\"onclick\", selectedElementClick);");
			componentJavaScript.add("				selectedElement.attachEvent(\"onmousedown\", selectedElementMouseDown);");
			componentJavaScript.add("				selectedElement.attachEvent(\"onmouseup\", selectedElementMouseUp);");
			componentJavaScript.add("			}");
			componentJavaScript.add("			else {");
			componentJavaScript.add("				selectedElement.addEventListener(\"click\", selectedElementClick, false);");
			componentJavaScript.add("				selectedElement.addEventListener(\"mousedown\", selectedElementMouseDown, false);");
			componentJavaScript.add("				selectedElement.addEventListener(\"mouseup\", selectedElementMouseUp, false);");
			componentJavaScript.add("			}");         
			componentJavaScript.add("			xhr.open(\"post\", \"/web2cToolkit/Web2cEditor?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><command>\" + \"addComponent\" + \"</command><value>\" + elementId + \"</value><value>\" + elementType + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
			componentJavaScript.add("			xhr.onreadystatechange = updateAttributeList;"); 
			componentJavaScript.add("			xhr.send(null);");  
			componentJavaScript.add("		} catch (e) {");
			componentJavaScript.add("			window.alert(\"Error adding Web2cComponent\");");
			componentJavaScript.add("		}");
			componentJavaScript.add("	}");
		}
		else if ((uri.split(":")[1]).substring(2).equals("removeComponent")) {
			componentJavaScript.add("	function removeComponent() {");
			componentJavaScript.add("		try {");
			componentJavaScript.add("			var now = new Date();");
			componentJavaScript.add("			var elementType = \" \";");
			componentJavaScript.add("			var parentElement = document.getElementById(drawingFrameId);");
			componentJavaScript.add("			var childElement = selectedElement;");
			componentJavaScript.add("			elementId = childElement.id;");
			componentJavaScript.add("			if (parentElement.childNodes.length >= childOffset) {");
			componentJavaScript.add("				parentElement.removeChild(childElement);");     
			componentJavaScript.add("			}");      
			componentJavaScript.add("			xhr.open(\"post\", \"/web2cToolkit/Web2cEditor?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><command>\" + \"removeComponent\" + \"</command><value>\" + elementId + \"</value><value>\" + elementType + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);");
			componentJavaScript.add("			xhr.onreadystatechange = function() {");  
			componentJavaScript.add("				if (xhr.readyState == 4) {");
			componentJavaScript.add("					if (parentElement.childNodes.length == childOffset) {");
			componentJavaScript.add("						selectElement(parentElement);");
			componentJavaScript.add("					}");
			componentJavaScript.add("					else {");
			componentJavaScript.add("						selectElement(parentElement.childNodes[parentElement.childNodes.length - 1]);");
			componentJavaScript.add("					}");
			componentJavaScript.add("				}");
			componentJavaScript.add("			}");
			componentJavaScript.add("			xhr.send(null);"); 
			componentJavaScript.add("		} catch (e) {");
			componentJavaScript.add("			window.alert(\"Error removing Web2cComponent\");");
			componentJavaScript.add("		}");
			componentJavaScript.add("	}");
		}
		else if ((uri.split(":")[1]).substring(2).equals("showPreview")) {
			componentJavaScript.add("	function showPreview() {");
		    componentJavaScript.add("		configFileName = document.getElementById(textInputId).value;"); 
			componentJavaScript.add("		newWindow = window.open(\"" + hostURL.replace("Web2cEditor", "") + "Web2c?param=(file)\" + configFileName);");
			componentJavaScript.add("	}");
		}
		else if ((uri.split(":")[1]).substring(2).equals("saveToFile")) {
			if (userPrivilege.equals(expert)) {
			    componentJavaScript.add("	function saveToFile() {");
			    componentJavaScript.add("		var overWrite;");
			    componentJavaScript.add("		try {");
			    componentJavaScript.add("			var now = new Date();");
			    componentJavaScript.add("			var param = \" \";");
			    componentJavaScript.add("			configFileName = document.getElementById(textInputId).value;"); 
				componentJavaScript.add("			xhr.open(\"post\", \"/web2cToolkit/Web2cEditor?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><command>\" + \"checkFile\" + \"</command><value>\" + configFileName + \"</value><value>\" + param + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);");           
				componentJavaScript.add("           xhr.onreadystatechange = function() {"); 
				componentJavaScript.add("               if (xhr.readyState == 4) {"); 
				componentJavaScript.add("                   if (xhr.responseText.substring(7, xhr.responseText.length - 8) == \"fileExists\") {"); 
				componentJavaScript.add("                   	overWrite = window.confirm(configFileName + \" already exists! Overwrite?\");");	   
				componentJavaScript.add("                    	if (overWrite == true) {");   
			    componentJavaScript.add("							now = new Date();");
				componentJavaScript.add("							xhr.open(\"post\", \"/web2cToolkit/Web2cEditor?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><command>\" + \"saveToFile\" + \"</command><value>\" + configFileName + \"</value><value>\" + param + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);");
				componentJavaScript.add("                			xhr.onreadystatechange = function() {"); 
				componentJavaScript.add("               				if (xhr.readyState == 4) {"); 
				componentJavaScript.add("                   				if (xhr.responseText.substring(7, xhr.responseText.length - 8) == \"fileNotSaved\") {"); 
			    componentJavaScript.add("										window.alert(\"File not saved! Component hierarchy inconsistent or attribute values incomplete! Please, enter/complete attribute values and confirm changes!\");"); 
			    componentJavaScript.add("                    				}");		    
			    componentJavaScript.add("                    			}");
			    componentJavaScript.add("                    		}"); 
			    componentJavaScript.add("							xhr.send(null);"); 
				componentJavaScript.add("                    	}");
				componentJavaScript.add("                   }");
				componentJavaScript.add("                   else {");  
			    componentJavaScript.add("						now = new Date();");
				componentJavaScript.add("						xhr.open(\"post\", \"/web2cToolkit/Web2cEditor?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><command>\" + \"saveToFile\" + \"</command><value>\" + configFileName + \"</value><value>\" + param + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);");
				componentJavaScript.add("                		xhr.onreadystatechange = function() {"); 
				componentJavaScript.add("               			if (xhr.readyState == 4) {"); 
				componentJavaScript.add("                   			if (xhr.responseText.substring(7, xhr.responseText.length - 8) == \"fileNotSaved\") {"); 
			    componentJavaScript.add("									window.alert(\"File not saved! Component hierarchy inconsistent or attribute values incomplete! Please, enter/complete attribute values and confirm changes!\");"); 
			    componentJavaScript.add("                    			}");		    
			    componentJavaScript.add("                    		}");
			    componentJavaScript.add("                    	}"); 
			    componentJavaScript.add("						xhr.send(null);"); 
				componentJavaScript.add("                   }");
				componentJavaScript.add("               }");
				componentJavaScript.add("           }");  
			    componentJavaScript.add("			xhr.send(null);"); 
			    componentJavaScript.add("		} catch (e) {");
			    componentJavaScript.add("			window.alert(\"Error saving to file\");");
			    componentJavaScript.add("		}");
			    componentJavaScript.add("	}");
			}
		}
		else if ((uri.split(":")[1]).substring(2).equals("readFromFile")) {
			componentJavaScript.add("	function readFromFile() {");
			componentJavaScript.add("		try {");
			componentJavaScript.add("			var param = \" \";");
			componentJavaScript.add("			var now = new Date();");
			componentJavaScript.add("			var elementId;");
			componentJavaScript.add("			var response1;"); 
			componentJavaScript.add("			var response2;"); 
			componentJavaScript.add("			var i;"); 
			componentJavaScript.add("			var j;"); 
			componentJavaScript.add("			var k;"); 
			componentJavaScript.add("			var bounds;"); 
			componentJavaScript.add("			var x1;"); 
			componentJavaScript.add("			var x2;"); 
			componentJavaScript.add("			var y1;"); 
			componentJavaScript.add("			var y2;"); 
			componentJavaScript.add("			var width;"); 
			componentJavaScript.add("			var height;"); 
			componentJavaScript.add("			var element1;"); 
			componentJavaScript.add("			var element2;");  
			componentJavaScript.add("			if (selectedElement != null) {");
			componentJavaScript.add("				selectedElement.style.borderStyle = \"solid\";"); 
			componentJavaScript.add("			}");
			componentJavaScript.add("			configFileName = document.getElementById(textInputId).value;");       
			componentJavaScript.add("			var componentList = document.getElementById(componentListId);");
			componentJavaScript.add("			var parentElement = document.getElementById(drawingFrameId);");       
			componentJavaScript.add("			xhr.open(\"post\", \"/web2cToolkit/Web2cEditor?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><command>\" + \"readFromFile\" + \"</command><value>\" + configFileName + \"</value><value>\" + param + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
			componentJavaScript.add("			xhr.onreadystatechange = function() {");  
			componentJavaScript.add("				if (xhr.readyState == 4) {");
			componentJavaScript.add("					try {");
			componentJavaScript.add("						response1 = xhr.responseText.substring(0, xhr.responseText.length - 8).split(\"</value>\");");	
			componentJavaScript.add("						for (i = 0; i < response1.length; i++) {");
			componentJavaScript.add("							response2 = response1[i].replace(/<value>/, \"\").split(\"&#044;\");");
			componentJavaScript.add("							var attribute = response2[0].substring(response2[0].indexOf(\"[\") + 1,response2[0].lastIndexOf(\"]\"));");
			componentJavaScript.add("							var serverData = response2[0].split(\"]\");");
			componentJavaScript.add("							var value = serverData[1];");
			componentJavaScript.add("							elementId = value;");
			componentJavaScript.add("							element1 = document.getElementById(attributeListId + \"_typeAttribute\");");
			componentJavaScript.add("							element1.innerHTML = \"type\";");
			componentJavaScript.add("							element2= document.getElementById(attributeListId + \"_typeValue\");");
			componentJavaScript.add("							element2.innerHTML = \" \";");
			componentJavaScript.add("							element1 = document.getElementById(attributeListId + \"_sectionIdAttribute\");");
			componentJavaScript.add("							element1.innerHTML = \"sectionId\";");
			componentJavaScript.add("							element2= document.getElementById(attributeListId + \"_sectionIdValue\");");
			componentJavaScript.add("							element2.value = \" \";");
			componentJavaScript.add("							for (k = 0; k < attributeListRows; k++) {");
			componentJavaScript.add("								element1 = document.getElementById(attributeListId + \"_cellAttribute\" + String(k));");
			componentJavaScript.add("								element1.innerHTML = \" \";");
			componentJavaScript.add("								element2 = document.getElementById(attributeListId + \"_cellValue\" + String(k));");
			componentJavaScript.add("								element2.value = \" \";");
			componentJavaScript.add("							}"); 
			componentJavaScript.add("							attribute = response2[1].substring(response2[1].indexOf(\"[\") + 1,response2[1].lastIndexOf(\"]\"));");
			componentJavaScript.add("							serverData = response2[1].split(\"]\");");
			componentJavaScript.add("							value = serverData[1];");
			componentJavaScript.add("							element1 = document.getElementById(attributeListId + \"_typeAttribute\");");
			componentJavaScript.add("							element1.innerHTML = attribute;");
			componentJavaScript.add("							element2= document.getElementById(attributeListId + \"_typeValue\");");
			componentJavaScript.add("							element2.innerHTML = value;");
			componentJavaScript.add("							attribute = response2[2].substring(response2[2].indexOf(\"[\") + 1,response2[2].lastIndexOf(\"]\"));");
			componentJavaScript.add("							serverData = response2[2].split(\"]\");");
			componentJavaScript.add("							value = serverData[1];");
			componentJavaScript.add("							element1 = document.getElementById(attributeListId + \"_sectionIdAttribute\");");
			componentJavaScript.add("							element1.innerHTML = attribute;");
			componentJavaScript.add("							element2 = document.getElementById(attributeListId + \"_sectionIdValue\");");
			componentJavaScript.add("							element2.value = value;");
			componentJavaScript.add("							for (j = 3; j < response2.length; j++) {");
			componentJavaScript.add("								attribute = response2[j].substring(response2[j].indexOf(\"[\") + 1,response2[j].lastIndexOf(\"]\"));");
			componentJavaScript.add("								serverData = response2[j].split(\"]\");");
			componentJavaScript.add("								value = serverData[1];");
			componentJavaScript.add("								element1 = document.getElementById(attributeListId + \"_cellAttribute\" + String(j-3));");
			componentJavaScript.add("								element1.innerHTML = attribute;");
			componentJavaScript.add("								element2 = document.getElementById(attributeListId + \"_cellValue\" + String(j-3));");
			componentJavaScript.add("								element2.value = value;");
			componentJavaScript.add("								if (attribute == \"bounds\") {");
			componentJavaScript.add("									bounds = value.split(\",\");");
			componentJavaScript.add("									x1 = Math.round(bounds[0] / 2);");
			componentJavaScript.add("									y1 = Math.round(bounds[1] / 2);");
			componentJavaScript.add("									x2 = Math.round(bounds[2] / 2);");
			componentJavaScript.add("									y2 = Math.round(bounds[3] / 2);");
			componentJavaScript.add("									width = parseInt(x2 - x1);");
			componentJavaScript.add("									height = parseInt(y2 - y1);");
			componentJavaScript.add("									var childElement = document.createElement(\"img\");");
			componentJavaScript.add("									childElement.id = elementId;");             
			componentJavaScript.add("									childElement.style.borderWidth = \"1px\";");              
			componentJavaScript.add("									childElement.style.borderStyle = \"solid\";");              
			componentJavaScript.add("									childElement.style.borderColor = \"black\";");
			componentJavaScript.add("									childElement.style.backgroundColor = \"white\";"); 	
			componentJavaScript.add("									childElement.style.position = \"absolute\";");
			componentJavaScript.add("									childElement.style.left = x1 + \"px\";");
			componentJavaScript.add("									childElement.style.top = y1 + \"px\";"); 
			componentJavaScript.add("			                        childElement.style.minWidth = \"1px\";");
			componentJavaScript.add("			                        childElement.style.minHeight = \"1px\";");
			componentJavaScript.add("									if (browser == \"MSIE\") {");
			componentJavaScript.add("										childElement.style.width = String(width) + \"px\";");
			componentJavaScript.add("										childElement.style.height = String(height) + \"px\";");
			componentJavaScript.add("									}");
			componentJavaScript.add("									else {");
			componentJavaScript.add("										childElement.style.width = String(width - (2 * parseInt(childElement.style.borderWidth.substring(0, childElement.style.borderWidth.length - 2)))) + \"px\";");
			componentJavaScript.add("										childElement.style.height = String(height - (2 * parseInt(childElement.style.borderWidth.substring(0, childElement.style.borderWidth.length - 2)))) + \"px\";");
			componentJavaScript.add("									}");  
			componentJavaScript.add("									parentElement.appendChild(childElement);");	      
			componentJavaScript.add("									if (browser == \"Microsoft Internet Explorer\") {");
			componentJavaScript.add("										childElement.attachEvent(\"onclick\", selectedElementClick);");
			componentJavaScript.add("										childElement.attachEvent(\"onmousedown\", selectedElementMouseDown);");
			componentJavaScript.add("										childElement.attachEvent(\"onmouseup\", selectedElementMouseUp);");
			componentJavaScript.add("									}");
			componentJavaScript.add("									if ((browser == \"Netscape\") || (browser == \"Opera\")) {");
			componentJavaScript.add("										childElement.addEventListener(\"click\", selectedElementClick, false);");
			componentJavaScript.add("										childElement.addEventListener(\"mousedown\", selectedElementMouseDown, false);");
			componentJavaScript.add("										childElement.addEventListener(\"mouseup\", selectedElementMouseUp, false);");
			componentJavaScript.add("									}");   
			componentJavaScript.add("								}");
			componentJavaScript.add("							}");
			componentJavaScript.add("							updateIcons(childElement);");
			componentJavaScript.add("						}");
			componentJavaScript.add("						selectElement(childElement);"); 	
			componentJavaScript.add("						selectedElement.style.borderStyle = \"dotted\";"); 
			componentJavaScript.add("						parentElement.style.borderStyle = \"solid\";"); 
			componentJavaScript.add("					} catch (e) {");
			componentJavaScript.add("						window.alert(\"Error reading from file\");");
			componentJavaScript.add("					}");
			componentJavaScript.add("				}"); 
			componentJavaScript.add("			}"); 
			componentJavaScript.add("			xhr.send(null);");   
			componentJavaScript.add("		} catch (e) {");
			componentJavaScript.add("			window.alert(\"Error reading from file\");");
			componentJavaScript.add("		}");
			componentJavaScript.add("	}");
		}
		else if ((uri.split(":")[1]).substring(2).equals("confirmAttributeChanges")) {
			componentJavaScript.add("	function confirmAttributeChanges() {");
			componentJavaScript.add("		try {");
			componentJavaScript.add("			var param = \" \";");
			componentJavaScript.add("			var notconfirmed = 0;");
			componentJavaScript.add("			var now = new Date();"); 
			componentJavaScript.add("			var elementId = selectedElement.id;"); 
			componentJavaScript.add("			updateIcons(selectedElement);");
			componentJavaScript.add("			var list = postAttributeList();"); 
			componentJavaScript.add("			if (list.indexOf(\"enter Id\") != -1) {");
			componentJavaScript.add("				window.alert(\"Changes not confirmed! Please, enter Id!\")");
			componentJavaScript.add("				notconfirmed = 1;");
			componentJavaScript.add("			}");
			componentJavaScript.add("			if (list.indexOf(\"enter sectionId\") != -1) {");
			componentJavaScript.add("				window.alert(\"Changes not confirmed! Please, enter sectionId!\")");
			componentJavaScript.add("				notconfirmed = notconfirmed + 1;");
			componentJavaScript.add("			}");
			componentJavaScript.add("			if (notconfirmed == 0) {");
			componentJavaScript.add("				xhr.open(\"post\", \"/web2cToolkit/Web2cEditor?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><command>\" + \"confirmAttributeChanges\" + \"</command><value>\" + elementId + \"</value><value>\" + list + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);");
			componentJavaScript.add("				xhr.onreadystatechange = function() {};");   
			componentJavaScript.add("				xhr.send(null);"); 
			componentJavaScript.add("			}");
			componentJavaScript.add("		} catch (e) {");
			componentJavaScript.add("			window.alert(\"Error conforming attribute changes\");");
			componentJavaScript.add("		}"); 
			componentJavaScript.add("	}");
		}
		else if ((uri.split(":")[1]).substring(2).equals("resetPage")) {
			componentJavaScript.add("	function resetPage() {");
			componentJavaScript.add("		try {");
			componentJavaScript.add("			resetTimer = setInterval(resetAll, 200);");
			componentJavaScript.add("		} catch (e) {");
			componentJavaScript.add("			window.alert(\"Error resetting page\");");
			componentJavaScript.add("		}");
			componentJavaScript.add("	}");
		}
		else if ((uri.split(":")[1]).substring(2).equals("nextComponent")) {
			componentJavaScript.add("	function nextComponent() {");
			componentJavaScript.add("		try {");
			componentJavaScript.add("			var index;");
			componentJavaScript.add("			var nextElement;");
			componentJavaScript.add("			var parentElement = document.getElementById(drawingFrameId);");
			componentJavaScript.add("			var childElements = parentElement.childNodes;");
			componentJavaScript.add("			var element = selectedElement;");
			componentJavaScript.add("			for (var i = childOffset; i < childElements.length; i++) {");
			componentJavaScript.add("				if (element == childElements[i]) {;");
			componentJavaScript.add("					index = i;");
			componentJavaScript.add("				}");
			componentJavaScript.add("			}");
			componentJavaScript.add("			if (index != (childElements.length - 1)) {");
			componentJavaScript.add("				selectElement(childElements[index + 1]);");
			componentJavaScript.add("			}");
			componentJavaScript.add("			else {");
			componentJavaScript.add("				selectElement(childElements[childOffset]);");
			componentJavaScript.add("			}");
			componentJavaScript.add("		} catch (e) {");
			componentJavaScript.add("			window.alert(\"Error selecting next component\");");
			componentJavaScript.add("		}");
			componentJavaScript.add("	}");
		}
		else {}

		return componentJavaScript;
	}
	
	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		int xLeft = 0;
		int yTop = 0;
		int xRight= 0;
		int yBottom = 0;
		
		caption = "";
		color = black;
		backgroundColor = white;		
		fontSize = medium;
		fontThickness = normal;
		toolTipText = "";
		uri = "";
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("id")) {
				id = referenceId + "_" + value;
			}
			if (name.equals("bounds")) {
				xLeft = Integer.valueOf(value.split(",")[0]);
				yTop = Integer.valueOf(value.split(",")[1]);
				xRight = Integer.valueOf(value.split(",")[2]);
				yBottom = Integer.valueOf(value.split(",")[3]);
				marginLeft = ((Integer)(xLeft)).toString() + "px";
				marginTop = ((Integer)(yTop)).toString() + "px";
				width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
				height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
			}
			if (name.equals("caption")) {
				caption = value;
			}			
			if (name.equals("tooltipText")) {
				toolTipText = value;
			}			
			if (name.equals("uri")) {
				uri = value;
			}
		}
	}	
}
