package de.desy.web2cToolkit.web2cEditor;

import java.util.ArrayList;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;


/**
 * @author R.Bacher, DESY
 */
public class Web2cEditorDrawingFrame implements IWeb2cConstants {

	private String referenceId = null;
	private String id = null;
	private String width = null;
	private String height = null;
	private String marginLeft = null;
	private String marginTop = null;
	private String backgroundColor = null;
	private String toolTipText = null;
	private String borderStyle = null;
	private String borderWidth = null;
	private String borderColor = null;
	
	/**
	 * Drawing Frame
	 * @param refId ID of reference component which component belongs to
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 */
	public Web2cEditorDrawingFrame(String refId, ArrayList<String> names, ArrayList<String> values) {
		
		referenceId = refId;
		decodeAttributes(names, values);
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> sectionHTML = new ArrayList<String>();
		int buttonWidth = 25;
		int buttonHeight = 25;
		String minButtonLeft = "";
		String maxButtonLeft = "";
		String buttonTop = "";
		
		minButtonLeft = Integer.valueOf(width.substring(0, width.length() - 2)) - (2 * buttonWidth) + "px";
		maxButtonLeft = Integer.valueOf(width.substring(0, width.length() - 2)) - buttonWidth + "px";
		buttonTop = "0px";
		
		sectionHTML.add("			<div id=\"" + id + "\" title=\"" + toolTipText + "\" style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + ";\">");
		sectionHTML.add("				<input type=\"button\" id=\"" + id + "_min\" value=\"" + "<" + "\" title=\"" + "Minimizes Drawing Frame" + "\" onclick=\"minimizeFrame()\" style=\"position:absolute; left:" + minButtonLeft + "; top:" + buttonTop + "; width:" + String.valueOf(buttonWidth) + "px" + "; height:" + String.valueOf(buttonHeight) + "px" + "; color:" + black + "; background-color:" + gray + "; font-size:" + medium + "; font-weight:" + normal + ";\">");
		sectionHTML.add("				<input type=\"button\" id=\"" + id + "_max\" value=\"" + ">" + "\" title=\"" + "Maximizes Drawing Frame" + "\" onclick=\"maximizeFrame()\" style=\"position:absolute; left:" + maxButtonLeft + "; top:" + buttonTop + "; width:" + String.valueOf(buttonWidth) + "px" + "; height:" + String.valueOf(buttonHeight) + "px" + "; color:" + black + "; background-color:" + gray + "; font-size:" + medium + "; font-weight:" + normal + ";\">");
		sectionHTML.add("			</div>");
		
		return sectionHTML;

	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		componentJavaScript.add("	//EditorDrawingFrame: " + id);
		componentJavaScript.add("	var drawingFrameId = \"" + id + "\";");  
		componentJavaScript.add("	var drawingFrame;");
		componentJavaScript.add("	var minButton;");  
		componentJavaScript.add("	var maxButton;");  
		componentJavaScript.add("	var minButtonTop;");  
		componentJavaScript.add("	var maxButtonTop;");  
		componentJavaScript.add("	var minButtonLeft;");  
		componentJavaScript.add("	var maxButtonLeft;");  
		componentJavaScript.add("	var drawingFrameTop;");  
		componentJavaScript.add("	var drawingFrameLeft;");  
		componentJavaScript.add("	var magnification = 1;");     
		componentJavaScript.add("	var selectedElement;");       
		componentJavaScript.add("	var childs;");      
		componentJavaScript.add("	var childOffset;");  		
		componentJavaScript.add("	var topStart;");
		componentJavaScript.add("	var leftStart;");         
		componentJavaScript.add("	var widthStart;");
		componentJavaScript.add("	var heightStart;");                  
		componentJavaScript.add("	var elementLeft;");                 
		componentJavaScript.add("	var elementTop;"); 
		componentJavaScript.add("	var dragStartTop;");  
		componentJavaScript.add("	var dragStartLeft;");  
		componentJavaScript.add("	var dragging = false;");       
		componentJavaScript.add("	var resizing = false;");  
	       
		componentJavaScript.add("	function initializeDrawingFrame() {");
		componentJavaScript.add("		drawingFrame = document.getElementById(drawingFrameId);");  
		componentJavaScript.add("		drawingFrame.style.zIndex = 98;"); 
		componentJavaScript.add("		drawingFrameTop = drawingFrame.style.top;");
		componentJavaScript.add("		drawingFrameLeft = drawingFrame.style.left;");
		componentJavaScript.add("		minButton = document.getElementById(\"" + id + "_min\");");  
		componentJavaScript.add("		minButton.style.zIndex = 99;"); 
		componentJavaScript.add("		minButtonTop = minButton.style.top;");
		componentJavaScript.add("		minButtonLeft = minButton.style.left;");
		componentJavaScript.add("		maxButton = document.getElementById(\"" + id + "_max\");");  
		componentJavaScript.add("		maxButton.style.zIndex = 100;"); 
		componentJavaScript.add("		maxButtonTop = maxButton.style.top;");
		componentJavaScript.add("		maxButtonLeft = maxButton.style.left;");
		componentJavaScript.add("		selectedElement = drawingFrame");
		componentJavaScript.add("		childOffset = drawingFrame.childNodes.length;");
		componentJavaScript.add("		if (browser == \"MSIE\") {");
		componentJavaScript.add("			drawingFrame.attachEvent(\"onclick\", selectedElementClick);");
		componentJavaScript.add("			drawingFrame.attachEvent(\"onmousedown\", selectedElementMouseDown);");
		componentJavaScript.add("			drawingFrame.attachEvent(\"onmouseup\", selectedElementMouseUp);");
		componentJavaScript.add("		}");
		componentJavaScript.add("		else {");
		componentJavaScript.add("			drawingFrame.addEventListener(\"click\", selectedElementClick, false);");
		componentJavaScript.add("			drawingFrame.addEventListener(\"mousedown\", selectedElementMouseDown, false);");
		componentJavaScript.add("			drawingFrame.addEventListener(\"mouseup\", selectedElementMouseUp, false);");
		componentJavaScript.add("		}");  
		componentJavaScript.add("	    selectElement(drawingFrame);");
		componentJavaScript.add("	}");   
		
		componentJavaScript.add("	function minimizeFrame() {"); 
		componentJavaScript.add("		if (magnification == 2) {");
		componentJavaScript.add("			drawingFrame.style.top = drawingFrameTop;"); 
		componentJavaScript.add("			drawingFrame.style.left = drawingFrameLeft;"); 
		componentJavaScript.add("			drawingFrame.style.borderWidth = String(Math.round(parseInt(drawingFrame.style.borderWidth.substring(0, drawingFrame.style.borderWidth.length - 2)) / 2)) + \"px\";"); 
		componentJavaScript.add("			drawingFrame.style.width = String(Math.round(parseInt(drawingFrame.style.width.substring(0, drawingFrame.style.width.length - 2)) / 2)) + \"px\";");
		componentJavaScript.add("			drawingFrame.style.height = String(Math.round(parseInt(drawingFrame.style.height.substring(0, drawingFrame.style.height.length - 2)) / 2)) + \"px\";");	 
		componentJavaScript.add("			minButton.style.top = \"0px\";"); 
		componentJavaScript.add("			minButton.style.left = minButtonLeft;"); 
		componentJavaScript.add("			maxButton.style.top = \"0px\";"); 
		componentJavaScript.add("			maxButton.style.left = maxButtonLeft;"); 
		componentJavaScript.add("			magnification = 1;"); 
		componentJavaScript.add("			childs = drawingFrame.childNodes;");
		componentJavaScript.add("			for (var i = childOffset; i < childs.length; i++) {");
		componentJavaScript.add("				childs[i].style.top = String(Math.round(parseInt(childs[i].style.top.substring(0, childs[i].style.top.length - 2)) / 2)) + \"px\";");
		componentJavaScript.add("				childs[i].style.left = String(Math.round(parseInt(childs[i].style.left.substring(0, childs[i].style.left.length - 2)) / 2)) + \"px\";");
		componentJavaScript.add("				childs[i].style.width = String(Math.round(parseInt(childs[i].style.width.substring(0, childs[i].style.width.length - 2)) / 2)) + \"px\";");
		componentJavaScript.add("				childs[i].style.height = String(Math.round(parseInt(childs[i].style.height.substring(0, childs[i].style.height.length - 2)) / 2)) + \"px\";");
		componentJavaScript.add("				childs[i].style.borderWidth = \"1px\";"); 
		componentJavaScript.add("			}"); 
		componentJavaScript.add("		}");
		componentJavaScript.add("	}");     
		
		componentJavaScript.add("	function maximizeFrame() {"); 
		componentJavaScript.add("		if (magnification == 1) {");
		componentJavaScript.add("			drawingFrame.style.top = \"0px\";"); 
		componentJavaScript.add("			drawingFrame.style.left = \"0px\";"); 
		componentJavaScript.add("			drawingFrame.style.borderWidth = String(Math.round(parseInt(drawingFrame.style.borderWidth.substring(0, drawingFrame.style.borderWidth.length - 2)) * 2)) + \"px\";"); 
		componentJavaScript.add("			drawingFrame.style.width = String(Math.round(parseInt(drawingFrame.style.width.substring(0, drawingFrame.style.width.length - 2)) * 2)) + \"px\";");
		componentJavaScript.add("			drawingFrame.style.height = String(Math.round(parseInt(drawingFrame.style.height.substring(0, drawingFrame.style.height.length - 2)) * 2)) + \"px\";");	 
		componentJavaScript.add("			minButton.style.top = \"0px\";"); 
		componentJavaScript.add("			maxButton.style.top = \"0px\";"); 
		componentJavaScript.add("			magnification = 2;"); 
		componentJavaScript.add("			childs = drawingFrame.childNodes;");
		componentJavaScript.add("			if (browser == \"MSIE\") {");
		componentJavaScript.add("				minButton.style.left = String(Math.round(parseInt(drawingFrame.style.width.substring(0, drawingFrame.style.width.length - 2)) - (2 * parseInt(minButton.style.width.substring(0, minButton.style.width.length - 2))) - (2 * parseInt(drawingFrame.style.borderWidth.substring(0, drawingFrame.style.borderWidth.length - 2))))) + \"px\";"); 
		componentJavaScript.add("				maxButton.style.left = String(Math.round(parseInt(drawingFrame.style.width.substring(0, drawingFrame.style.width.length - 2)) - (1 * parseInt(minButton.style.width.substring(0, minButton.style.width.length - 2))) - (2 * parseInt(drawingFrame.style.borderWidth.substring(0, drawingFrame.style.borderWidth.length - 2))))) + \"px\";"); 
		componentJavaScript.add("			}");
		componentJavaScript.add("			else {");
		componentJavaScript.add("				minButton.style.left = String(Math.round(parseInt(drawingFrame.style.width.substring(0, drawingFrame.style.width.length - 2)) - (2 * parseInt(minButton.style.width.substring(0, minButton.style.width.length - 2))))) + \"px\";"); 
		componentJavaScript.add("				maxButton.style.left = String(Math.round(parseInt(drawingFrame.style.width.substring(0, drawingFrame.style.width.length - 2)) - (1 * parseInt(minButton.style.width.substring(0, minButton.style.width.length - 2))))) + \"px\";"); 
		componentJavaScript.add("			}");
		componentJavaScript.add("			for (var i = childOffset; i < childs.length; i++) {");
		componentJavaScript.add("				childs[i].style.top = String(Math.round(parseInt(childs[i].style.top.substring(0, childs[i].style.top.length - 2)) * 2)) + \"px\";");
		componentJavaScript.add("				childs[i].style.left = String(Math.round(parseInt(childs[i].style.left.substring(0, childs[i].style.left.length - 2)) * 2)) + \"px\";");
		componentJavaScript.add("				childs[i].style.width = String(Math.round(parseInt(childs[i].style.width.substring(0, childs[i].style.width.length - 2)) * 2)) + \"px\";");
		componentJavaScript.add("				childs[i].style.height = String(Math.round(parseInt(childs[i].style.height.substring(0, childs[i].style.height.length - 2)) * 2)) + \"px\";");
		componentJavaScript.add("				childs[i].style.borderWidth = \"2px\";"); 
		componentJavaScript.add("			}");  
		componentJavaScript.add("		}");
		componentJavaScript.add("	}");             
		
		componentJavaScript.add("	function selectElement(element) {");
		componentJavaScript.add("		try {");
		componentJavaScript.add("			var now = new Date();");             
		componentJavaScript.add("			var elementBounds;");
		componentJavaScript.add("			var borderWidth;");   
		componentJavaScript.add("			if (selectedElement != null) {");
		componentJavaScript.add("				selectedElement.style.borderStyle = \"solid\";"); 
		componentJavaScript.add("			}");
		componentJavaScript.add("			var elementId = element.id;");
		componentJavaScript.add("			element.style.borderStyle = \"dotted\";"); 
		componentJavaScript.add("			if (elementId != drawingFrameId) {");
		componentJavaScript.add("				borderWidth = parseInt(element.style.borderWidth.substring(0, element.style.borderWidth.length - 2));"); 
		componentJavaScript.add("				var x1 = parseInt(element.style.left.substring(0, element.style.left.length - 2));");
		componentJavaScript.add("				var y1 = parseInt(element.style.top.substring(0, element.style.top.length - 2));");
		componentJavaScript.add("				if (browser == \"MSIE\") {");
		componentJavaScript.add("					var x2 = x1 + parseInt(element.style.width.substring(0, element.style.width.length - 2));");
		componentJavaScript.add("					var y2 = y1 + parseInt(element.style.height.substring(0, element.style.height.length - 2));");
		componentJavaScript.add("				}");
		componentJavaScript.add("				else {");
		componentJavaScript.add("					var x2 = x1 + parseInt(element.style.width.substring(0, element.style.width.length - 2)) + (2 * borderWidth);");
		componentJavaScript.add("					var y2 = y1 + parseInt(element.style.height.substring(0, element.style.height.length - 2)) + (2 * borderWidth);");
		componentJavaScript.add("				}");
		componentJavaScript.add("				if (magnification == 1) {");
		componentJavaScript.add("					x1 = Math.round(x1 * 2);");
		componentJavaScript.add("					y1 = Math.round(y1 * 2);");
		componentJavaScript.add("					x2 = Math.round(x2 * 2);");
		componentJavaScript.add("					y2 = Math.round(y2 * 2);");
		componentJavaScript.add("				}");
		componentJavaScript.add("				elementBounds = String(x1) + \",\" + String(y1) + \",\" + String(x2) + \",\" + String(y2);");
		componentJavaScript.add("			}");
		componentJavaScript.add("			selectedElement = element;");  
		componentJavaScript.add("			xhr.open(\"post\", \"/web2cToolkit/Web2cEditor?param=<action><sessionId>\" + " + referenceId + "_sessionId + \"</sessionId><command>\" + \"selectElement\" + \"</command><value>\" + elementId + \"</value><value>\" + elementBounds + \"</value><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);");
		componentJavaScript.add("			xhr.onreadystatechange = updateAttributeList;");   
		componentJavaScript.add("			xhr.send(null);"); 
		componentJavaScript.add("		} catch (e) {");
		componentJavaScript.add("			window.alert(\"Error selecting Web2cComponent\");");
		componentJavaScript.add("		}");
		componentJavaScript.add("	}");      
		
		componentJavaScript.add("	function selectedElementClick(event) {"); 
		componentJavaScript.add("		try {"); 
		componentJavaScript.add("			var element;");
		componentJavaScript.add("			if (browser == \"MSIE\") {");
		componentJavaScript.add("				if (!event) event = window.event;");
		componentJavaScript.add("				event.returnValue = false;");
		componentJavaScript.add("				event.cancelBubble = true;");
		componentJavaScript.add("				element = event.srcElement;");
		componentJavaScript.add("			}");
		componentJavaScript.add("			else {");
		componentJavaScript.add("				event.preventDefault();");
		componentJavaScript.add("				event.stopPropagation();");
		componentJavaScript.add("				element = event.target;");
		componentJavaScript.add("			}");  
		componentJavaScript.add("			if ((element != minButton) && (element != maxButton)) {");
		componentJavaScript.add("				selectElement(element);");
		componentJavaScript.add("			}");
		componentJavaScript.add("		} catch (e) {");
		componentJavaScript.add("			window.alert(\"Error selecting Web2cComponent\");");
		componentJavaScript.add("		}");
		componentJavaScript.add("	}");   
		     
		componentJavaScript.add("	function selectedElementMouseDown(event) {"); 
		componentJavaScript.add("		var element;");
		componentJavaScript.add("		var elementWidth;");
		componentJavaScript.add("		var elementHeight;");
		componentJavaScript.add("		var borderWidth;");             
		componentJavaScript.add("		var sectionLeft;");           
		componentJavaScript.add("		var drawingFrameLeft;");           
		componentJavaScript.add("		var sectionTop;");           
		componentJavaScript.add("		var drawingFrameTop;");
		componentJavaScript.add("		var drawingFrameBorderWidth;");
		componentJavaScript.add("		var leftPos;"); 
		componentJavaScript.add("		var topPos;"); 	
		componentJavaScript.add("		if (browser == \"MSIE\") {");
		componentJavaScript.add("			if (!event) event = window.event;");
		componentJavaScript.add("			event.returnValue = false;");
		componentJavaScript.add("			element = event.srcElement;");
		componentJavaScript.add("		}");
		componentJavaScript.add("		else {");
		componentJavaScript.add("			event.preventDefault();");
		componentJavaScript.add("			event.stopPropagation();");
		componentJavaScript.add("			element = event.target;");
		componentJavaScript.add("		}");	
		componentJavaScript.add("		resizing = false;");	
		componentJavaScript.add("		dragging = false;");
		componentJavaScript.add("		element.style.cursor = \"default\";"); 
		componentJavaScript.add("		if (String(element.id) != String(drawingFrameId)) {");
		componentJavaScript.add("		   dragStartLeft = event.clientX;");
		componentJavaScript.add("		   dragStartTop = event.clientY;");
		componentJavaScript.add("		   borderWidth = parseInt(element.style.borderWidth.substring(0, element.style.borderWidth.length - 2));");  
		componentJavaScript.add("		   widthStart = parseInt(element.style.width.substring(0, element.style.width.length - 2));");
		componentJavaScript.add("		   heightStart = parseInt(element.style.height.substring(0, element.style.height.length - 2));");
		componentJavaScript.add("		   leftPos = document.getElementById(sectionId).style.left;"); 
		componentJavaScript.add("		   sectionLeft = parseInt(leftPos.substring(0, leftPos.length - 2));");
		componentJavaScript.add("		   leftPos = document.getElementById(drawingFrameId).style.left;"); 
		componentJavaScript.add("		   drawingFrameLeft = parseInt(leftPos.substring(0, leftPos.length - 2));");
		componentJavaScript.add("		   topPos = document.getElementById(sectionId).style.top;"); 
		componentJavaScript.add("		   sectionTop = parseInt(topPos.substring(0, topPos.length - 2));");
		componentJavaScript.add("		   topPos = document.getElementById(drawingFrameId).style.top;"); 
		componentJavaScript.add("		   leftPos = document.getElementById(drawingFrameId).style.borderWidth;"); 
		componentJavaScript.add("		   drawingFrameBorderWidth = parseInt(leftPos.substring(0, leftPos.length - 2));");
		componentJavaScript.add("		   if (browser == \"MSIE\") {");
		componentJavaScript.add("			   elementWidth = widthStart - (2 * borderWidth);");
		componentJavaScript.add("			   elementHeight = heightStart - (2 * borderWidth);");
		componentJavaScript.add("		   }");         
		componentJavaScript.add("		   else {");
		componentJavaScript.add("			   elementWidth = widthStart;");
		componentJavaScript.add("			   elementHeight = heightStart;");
		componentJavaScript.add("		   }");         
		componentJavaScript.add("		   leftStart = parseInt(element.style.left.substring(0, element.style.left.length - 2));");
		componentJavaScript.add("		   topStart = parseInt(element.style.top.substring(0, element.style.top.length - 2));");
		componentJavaScript.add("		   elementLeft = sectionLeft + drawingFrameLeft + drawingFrameBorderWidth + leftStart;");
		componentJavaScript.add("		   elementTop = sectionTop + drawingFrameTop + drawingFrameBorderWidth + topStart;");
		componentJavaScript.add("		   if (((dragStartLeft >= (elementLeft + borderWidth + elementWidth - 20)) && (dragStartLeft < (elementLeft + borderWidth + elementWidth + borderWidth + 100))) || ((dragStartTop >= (elementTop + borderWidth + elementHeight - 20)) && (dragStartTop < (elementTop + borderWidth + elementHeight + borderWidth + 100)))) {");       
		componentJavaScript.add("			   resizing = true;");
		componentJavaScript.add("			   dragging = false;");
		componentJavaScript.add("			   element.style.cursor = \"nw-resize\";");	
		componentJavaScript.add("		   }");         
		componentJavaScript.add("		   else {");
		componentJavaScript.add("			   dragging = true;");
		componentJavaScript.add("			   resizing = false;");
		componentJavaScript.add("			   element.style.cursor = \"move\";");
		componentJavaScript.add("		   }"); 
		componentJavaScript.add("		   if (browser == \"MSIE\") {");
		componentJavaScript.add("			   element.attachEvent(\"onmousemove\", selectedElementMouseMove);");
		componentJavaScript.add("			   drawingFrame.attachEvent(\"onmousemove\", selectedElementMouseMove);");
		componentJavaScript.add("		   }");  
		componentJavaScript.add("		   else {");
		componentJavaScript.add("			   element.addEventListener(\"mousemove\", selectedElementMouseMove, false);");
		componentJavaScript.add("			   drawingFrame.addEventListener(\"mousemove\", selectedElementMouseMove, false);");
		componentJavaScript.add("		   }");  
		componentJavaScript.add("		}"); 
		componentJavaScript.add("	}");     
		
		componentJavaScript.add("	function selectedElementMouseMove(event) {"); 
		componentJavaScript.add("		var element;");
		componentJavaScript.add("		if (browser == \"MSIE\") {");
		componentJavaScript.add("			if (!event) event = window.event;");
		componentJavaScript.add("			event.returnValue = false;");
		componentJavaScript.add("			element = event.srcElement;");
		componentJavaScript.add("		}");
		componentJavaScript.add("		else {");
		componentJavaScript.add("			event.preventDefault();");
		componentJavaScript.add("			event.stopPropagation();");
		componentJavaScript.add("			element = event.target;");
		componentJavaScript.add("		}");
		componentJavaScript.add("		if (String(element.id) != String(drawingFrameId)) {");
		componentJavaScript.add("			if (dragging == true) {");
		componentJavaScript.add("				element.style.left = String(leftStart + (event.clientX - dragStartLeft)) + \"px\";");
		componentJavaScript.add("				element.style.top = String(topStart + (event.clientY - dragStartTop)) + \"px\";");
		componentJavaScript.add("			}"); 
		componentJavaScript.add("			else {");
		componentJavaScript.add("				if (resizing == true) {");
		componentJavaScript.add("					element.style.width = String(widthStart + (event.clientX - dragStartLeft)) + \"px\";");
		componentJavaScript.add("					element.style.height = String(heightStart + (event.clientY - dragStartTop)) + \"px\";");
		componentJavaScript.add("				}"); 
		componentJavaScript.add("			}");
		componentJavaScript.add("		}"); 
		componentJavaScript.add("	}");       
		
		componentJavaScript.add("	function selectedElementMouseUp(event) {"); 
		componentJavaScript.add("		var element;");
		componentJavaScript.add("		if (browser == \"MSIE\") {");
		componentJavaScript.add("			if (!event) event = window.event;");
		componentJavaScript.add("			event.returnValue = false;");
		componentJavaScript.add("			element = event.srcElement;");
		componentJavaScript.add("			element.detachEvent(\"onmousemove\", selectedElementMouseMove);");
		componentJavaScript.add("			drawingFrame.detachEvent(\"onmousemove\", selectedElementMouseMove);");
		componentJavaScript.add("		}");
		componentJavaScript.add("		else {");
		componentJavaScript.add("			event.preventDefault();");
		componentJavaScript.add("			event.stopPropagation();");
		componentJavaScript.add("			element = event.target;");
		componentJavaScript.add("			element.removeEventListener(\"mousemove\", selectedElementMouseMove, false);");
		componentJavaScript.add("			drawingFrame.removeEventListener(\"mousemove\", selectedElementMouseMove, false);");
		componentJavaScript.add("		}");
		componentJavaScript.add("		if (String(element.id) != String(drawingFrameId)) {");
		componentJavaScript.add("		    dragging = false;");
		componentJavaScript.add("		    resizing = false;");
		componentJavaScript.add("		    element.style.cursor = \"default\";"); 
		componentJavaScript.add("		}");  
		componentJavaScript.add("	}");  

		return componentJavaScript;

	}
	
	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		int xLeft = 0;
		int yTop = 0;
		int xRight= 0;
		int yBottom = 0;

		backgroundColor = white;	
		toolTipText = "";		
		borderStyle = solid;
		borderWidth = thin;
		borderColor = black;
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("id")) {
				id = referenceId + "_" + value;
			}
			if (name.equals("bounds")) {
				xLeft = Integer.valueOf(value.split(",")[0]);
				yTop = Integer.valueOf(value.split(",")[1]);
				xRight = Integer.valueOf(value.split(",")[2]);
				yBottom = Integer.valueOf(value.split(",")[3]);
				marginLeft = ((Integer)(xLeft)).toString() + "px";
				marginTop = ((Integer)(yTop)).toString() + "px";
				width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
				height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
			}		
			if (name.equals("tooltipText")) {
				toolTipText = value;
			}
		}		
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}	
}
