package de.desy.web2cToolkit.web2cEditor;

import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import de.desy.web2cToolkit.web2cEditor.Web2cEditorSection;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cConfigReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cConfigWriter;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cJavascriptWriter;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;


/**
 * @author R.Bacher, DESY
 */
public class Web2cEditorPage implements IWeb2cConstants {

	private ArrayList<ArrayList<String>> sectionsHTML = new ArrayList<ArrayList<String>>();
	private ArrayList<ArrayList<String>> sectionsJavaScript = new ArrayList<ArrayList<String>>();	
	private ArrayList<Web2cEditorSection> sections = new ArrayList<Web2cEditorSection>();
	private ArrayList<ArrayList<String>> elementAttributes = new ArrayList<ArrayList<String>>();
	private String hostURL = null;
	private String homeDir = null;
	private String webDir = null;
	private String title = null;
	private String referenceId = "editorPage";
	private String referencePoint = "0,0";
	private String userName = null;
	private String userAddress = null;
	private ArrayList<String> expertName = null;
	private ArrayList<String> expertAddress = null;
	private String userPrivilege = null;
	private String networkPrivilege = null;
	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	private String web2cSession = null;

	/**
	 * Page
	 * @param url URL of host servicing the request
	 * @param session Web2c session id
	 * @param home directory which contains user-specific files
	 * @param web directory which contains Web2cToolkit-specific files 
	 * @param configFile actual configuration file
	 * @param name actual user name
	 * @param ipAddress actual IP address of user
	 * @param logger debug logger
	 * @param listener debug flag listener
	 */
	public Web2cEditorPage(String url, String session, String home, String web, String configFile, String name, String ipAddress, Web2cFileLogger logger, Web2cKeyBoardListener listener) {
		
		hostURL = url;
		web2cSession = session;	
		homeDir = home;
		webDir = web;
		userPrivilege = layman;
		networkPrivilege = layman;
		userName = "guest";
		if (name == null) {
			userName = "guest";
		}
		else {
			userName = name;
		}
		if (ipAddress == null) {
			userAddress = "";
		}
		else {
			userAddress = ipAddress;
		}
		debugLogger = logger;
		debugListener = listener;
		pageExecute(new File(configFile));
		getComponents();
		addDrawingFrame();
	}

	/**
	 * Returns full contents of page (HTML tags, JavaScript snippets) 
	 * @return list
	 */
	public ArrayList<Object> getComponents() {
		
		ArrayList<Object> list = new ArrayList<Object>();
		for (Iterator<Web2cEditorSection> it1 = sections.iterator(); it1.hasNext();) {
			Web2cEditorSection section = (Web2cEditorSection)it1.next();
			list.add((Object)section);
			for (Iterator<Object> it2 = (section.getComponents()).iterator(); it2.hasNext();) {
				list.add(it2.next());
			}
		}
		
		return list;
	}
	
	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> pageHTML = new ArrayList<String>();	
		
		for(Iterator<ArrayList<String>> it1 = sectionsHTML.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				pageHTML.add((String)it2.next());
			}
		}

		
		return pageHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> pageJavaScript = new ArrayList<String>();	
		
		pageJavaScript.add("	//EditorPage: " + "editorPage");
		pageJavaScript.add("		var " + referenceId + "_sessionId = " + web2cSession + ";");
		pageJavaScript.add("		var xhr;");   
		pageJavaScript.add("		var browser;");      
		
		pageJavaScript.add("		function initPage() {");
		pageJavaScript.add("			handleBrowserSpecifica();");
		pageJavaScript.add("			initXhr();");
		pageJavaScript.add("			initializeDrawingFrame();");
		pageJavaScript.add("		}");
		
		pageJavaScript.add("		function handleBrowserSpecifica() {");
		pageJavaScript.add("			var browserType = navigator.appName;");  
		pageJavaScript.add("			if (browserType == \"Microsoft Internet Explorer\") {");
		pageJavaScript.add("				browser = \"MSIE\";");
		pageJavaScript.add("			}");
		pageJavaScript.add("			if (browser == \"MSIE\") {");
		pageJavaScript.add("				var element = document.getElementsByTagName(\"DIV\");");
		pageJavaScript.add("				for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("					if (element[i].style.borderStyle == \"solid\") {");
		pageJavaScript.add("						var widthandunits = element[i].style.width.split(\"p\");");
		pageJavaScript.add("						var heightandunits = element[i].style.height.split(\"p\");");
		pageJavaScript.add("						var width = widthandunits[0];");
		pageJavaScript.add("						var height = heightandunits[0];");
		pageJavaScript.add("						var bordersizeandunits = element[i].style.borderWidth.split(\"p\");");
		pageJavaScript.add("						var bordersize = bordersizeandunits[0];");
		pageJavaScript.add("						element[i].style.width = String(parseInt(width) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("						element[i].style.height = String(parseInt(height) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("					}");
		pageJavaScript.add("				}");
		pageJavaScript.add("				var element = document.getElementsByTagName(\"TEXTAREA\");");
		pageJavaScript.add("				for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("					if (element[i].style.borderStyle == \"solid\") {");
		pageJavaScript.add("						var widthandunits = element[i].style.width.split(\"p\");");
		pageJavaScript.add("						var heightandunits = element[i].style.height.split(\"p\");");
		pageJavaScript.add("						var width = widthandunits[0];");
		pageJavaScript.add("						var height = heightandunits[0];");
		pageJavaScript.add("						var bordersizeandunits = element[i].style.borderWidth.split(\"p\");");
		pageJavaScript.add("						var bordersize = bordersizeandunits[0];");
		pageJavaScript.add("						element[i].style.width = String(parseInt(width) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("						element[i].style.height = String(parseInt(height) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("					}");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("			else {");
		pageJavaScript.add("				var element = document.getElementsByTagName(\"TEXTAREA\");");
		pageJavaScript.add("				for (var i = 0; i < element.length; i++) {");
		pageJavaScript.add("					if (element[i].style.borderStyle != \"none none none none\") {");
		pageJavaScript.add("						var widthandunits = element[i].style.width.split(\"p\");");
		pageJavaScript.add("						var heightandunits = element[i].style.height.split(\"p\");");
		pageJavaScript.add("						var width = widthandunits[0];");
		pageJavaScript.add("						var height = heightandunits[0];");
		pageJavaScript.add("						var topandunits = element[i].style.top.split(\"p\");");
		pageJavaScript.add("						var top = topandunits[0];");
		pageJavaScript.add("						var bordersizeandunits = element[i].style.borderWidth.split(\"p\");");
		pageJavaScript.add("						var bordersize = bordersizeandunits[0];");
		pageJavaScript.add("						element[i].style.width = String(parseInt(width) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("						element[i].style.height = String(parseInt(height) + (2 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("						element[i].style.top = String(parseInt(top) - (1 * parseInt(bordersize))) + \"px\";");
		pageJavaScript.add("					}");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("		}");
		pageJavaScript.add("		function initXhr() {"); 
		pageJavaScript.add("			try {");                                                  
		pageJavaScript.add("				xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		pageJavaScript.add("			} catch (event1) {");                             
		pageJavaScript.add("				try {");
		pageJavaScript.add("					xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		pageJavaScript.add("				} catch (event2) {");
		pageJavaScript.add("					xhr = false;");
		pageJavaScript.add("				}");
		pageJavaScript.add("			}");
		pageJavaScript.add("			if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		pageJavaScript.add("				xhr = new XMLHttpRequest();");                 
		pageJavaScript.add("			}"); 																	
		pageJavaScript.add("		}");     
		
		for(Iterator<ArrayList<String>> it1 = sectionsJavaScript.iterator(); it1.hasNext();) {
			for(Iterator<String> it2 = ((ArrayList<String>)it1.next()).iterator(); it2.hasNext();) {
				pageJavaScript.add((String)it2.next());
			}
		}
		
		return pageJavaScript;
	}
	
	/**
	 * Returns a list of components which belong to the actual section
	 * @return list
	 */
	public ArrayList<String> getPage() {
				
		int i = 0;
		ArrayList<String> page = new ArrayList<String>();
		Calendar calendar = Calendar.getInstance();
		File jScriptDir = null;
		File jScriptFile = null;
		String[] fileList = null;
		
		jScriptDir = new File(webDir);
		fileList = jScriptDir.list();
		for (i = 0; i < fileList.length; i++) {
			if (fileList[i].contains("Web2cEditor_")) {
				jScriptFile = new File(webDir + File.separator + fileList[i]);
				if ((System.currentTimeMillis() - jScriptFile.lastModified() > (deleteFileTimeout * 1000))) jScriptFile.delete();
			}
		}

		Web2cJavascriptWriter javascriptWriter = new Web2cJavascriptWriter(new File(webDir + File.separator + "Web2cEditor_" + calendar.getTimeInMillis() + ".js"), debugLogger, debugListener);
		javascriptWriter.setJavascript(getJavaScript());

		page.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		page.add("<html>");
		page.add("	<head>");
		page.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		page.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		page.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		page.add("       <meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		page.add("		<title>" + title + "</title>");
		page.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
//		page.add("		<link rel=\"shortcut icon\" href=\"" + web2cdocs + directorySeparator + "Web2cLogo.png\">");
		page.add("	</head>");
		page.add("	<script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + "Web2cEditor_" + calendar.getTimeInMillis() + ".js\"></script>");
		page.add("	<body onload=\"initPage()\">");
		for(Iterator<String> it = getHTML().iterator(); it.hasNext();) {
			page.add((String)it.next());
		}
		page.add("	</body>");
		page.add("</html>");
		
		return page;
	}

	/**
	 * Handles client action requests
	 * @param action action requested
	 * @param parameter1 first action parameter
	 * @param parameter2 second action parameter
	 */
	public String doAction(String action, String parameter1, String parameter2) {
		
		String str = null;	
		String attribute = null;
		ArrayList<String> attributes = new ArrayList<String>();	

		str = "";
		if (action.equals("checkFile")) {	
			String configFileName = null;	

			configFileName = parameter1;
			if (userPrivilege == expert) {
				Web2cConfigWriter configFileWriter = new Web2cConfigWriter(new File(homeDir + File.separator + configFileName), debugLogger, debugListener);
				if (configFileWriter.checkFileExists() == false) {
					str = "fileNotExists";
				}
				else {
					str = "fileExists";
				}
			}
			str = "<value>" + str + "</value>";
		}
		
		else if (action.equals("saveToFile")) {
			String configFileName = null;
			ArrayList<String> out = null;
			String sectionId = null;
		    int dataUncomplete = 0;
			int i = 0;
			int j = 0;

			try {
				configFileName = parameter1;
				out = new ArrayList<String>();
				for (Iterator<ArrayList<String>> it1 = elementAttributes.iterator(); it1.hasNext();) {
					attributes = new ArrayList<String>();
					attributes = (ArrayList<String>)it1.next();
					for (Iterator<String> it2 = attributes.iterator(); it2.hasNext();) {
						attribute = (String)it2.next();
						if (attribute.split("]").length == 1) attribute = attribute + " ";
						String value = attribute.split("]")[1];
						if (value.equals("Page")) {
							out.add("<Page>");
						    out.add("    <PageAttributes>");
						    for (i = 3; i < attributes.size(); i++) {
								attribute = (String)(attributes.get(i));
								if (attribute.split("]").length == 1) attribute = attribute + " ";
								String pageTag = (attribute.split("]")[0]).substring(1);
								String pageValue = attribute.split("]")[1];
								if (pageTag.equals("expert")) {
									for (j = 0; j < (pageValue.split(";")).length; j++) {
										out.add("        <" + pageTag + ">" + pageValue.split(";")[j] +  "</" + pageTag + ">");
									}
								}
								else if (pageTag.equals("supervisor")) {
									for (j = 0; j < (pageValue.split(";")).length; j++) {
										out.add("        <" + pageTag + ">" + pageValue.split(";")[j] +  "</" + pageTag + ">");
									}
								}
								else if (pageTag.equals("ipAddress")) {
									for (j = 0; j < (pageValue.split(";")).length; j++) {
										out.add("        <" + pageTag + ">" + pageValue.split(";")[j] +  "</" + pageTag + ">");
									}
								}
								else {
									out.add("        <" + pageTag + ">" + pageValue +  "</" + pageTag + ">");
								}
						    }
					        out.add("    </PageAttributes>");
					        out.add("    <PageElements>");
						}
					}
				}
				for (Iterator<ArrayList<String>> it1 = elementAttributes.iterator(); it1.hasNext();) {
					attributes = new ArrayList<String>();
					attributes = (ArrayList<String>)it1.next();
					for (Iterator<String> it2 = attributes.iterator(); it2.hasNext();) {
						attribute = (String)it2.next();
						if (attribute.split("]").length == 1) attribute = attribute + " ";
						String value = attribute.split("]")[1];
						if (value.equals("Section")) {
							out.add("          <Section>");
						    out.add("               <SectionAttributes>");
							String sectionBounds = "";
							int sectionX1 = 0;
							int sectionY1 = 0;
						    for (i = 3; i < attributes.size(); i++) {
								attribute = (String)(attributes.get(i));
								if (attribute.split("]").length == 1) attribute = attribute + " ";
								String sectionTag = (attribute.split("]")[0]).substring(1);
								String sectionValue = attribute.split("]")[1];
								if (sectionTag.equals("bounds")) {
									sectionBounds = (String)sectionValue;
									sectionX1 = Integer.valueOf(sectionBounds.split(",")[0]);
									sectionY1 = Integer.valueOf(sectionBounds.split(",")[1]);
									String bounds = "";
									bounds = bounds + String.valueOf(Integer.valueOf(sectionValue.split(",")[0])) + ",";
									bounds = bounds + String.valueOf(Integer.valueOf(sectionValue.split(",")[1])) + ",";
									bounds = bounds + String.valueOf(Integer.valueOf(sectionValue.split(",")[2])) + ",";
									bounds = bounds + String.valueOf(Integer.valueOf(sectionValue.split(",")[3]));
									sectionValue = bounds;
								}
								if (sectionTag.equals("id")) {
									sectionId = sectionValue;
								}
								out.add("                    <" + sectionTag + ">" + sectionValue +  "</" + sectionTag + ">");
						    }
					        out.add("               </SectionAttributes>");
					        out.add("               <SectionElements>");
							for (Iterator<ArrayList<String>> it3 = elementAttributes.iterator(); it3.hasNext();) {
								ArrayList<String> sectionElementAttributes = new ArrayList<String>();
								sectionElementAttributes = (ArrayList<String>)it3.next();
								for (Iterator<String> it4 = sectionElementAttributes.iterator(); it4.hasNext();) {
									attribute = (String)it4.next();
									if (attribute.split("]").length == 1) attribute = attribute + " ";
									String sectionElementTag = (attribute.split("]")[0]).substring(1);
									String sectionElementValue = attribute.split("]")[1];
									if ((sectionElementTag.equals("type")) && (sectionElementValue.equals("AnimatedImage") || 
										sectionElementValue.equals("Image") || sectionElementValue.equals("Indicator") ||
										sectionElementValue.equals("IndicatorList") || sectionElementValue.equals("Label") || 
										sectionElementValue.equals("LabelList") || sectionElementValue.equals("Ticker") || 
										sectionElementValue.equals("Time") || sectionElementValue.equals("Trend") || 
										sectionElementValue.equals("Slider") || sectionElementValue.equals("WheelSwitch") ||
										sectionElementValue.equals("Histo") || sectionElementValue.equals("Value") || 
										sectionElementValue.equals("ValueList") || sectionElementValue.equals("Text") || 
										sectionElementValue.equals("TextList") || sectionElementValue.equals("Button") || 
										sectionElementValue.equals("Video") || sectionElementValue.equals("Input") ||
										sectionElementValue.equals("Chat"))) {
										if (((sectionElementAttributes.get(2)).split("]")[1]).equals(sectionId)) {
											out.add("                    <" + sectionElementValue + ">");
										    for (i = 3; i < sectionElementAttributes.size(); i++) {
										    	attribute = (String)(sectionElementAttributes.get(i));
												if (attribute.split("]").length == 1) attribute = attribute + " ";
												String elementTag = (attribute.split("]")[0]).substring(1);
												String elementValue = attribute.split("]")[1];
												if (elementTag.equals("bounds")) {
													String bounds = "";
													bounds = bounds + String.valueOf((Integer.valueOf(elementValue.split(",")[0]) - sectionX1)) + ",";
													bounds = bounds + String.valueOf((Integer.valueOf(elementValue.split(",")[1]) - sectionY1)) + ",";
													bounds = bounds + String.valueOf((Integer.valueOf(elementValue.split(",")[2]) - sectionX1)) + ",";
													bounds = bounds + String.valueOf((Integer.valueOf(elementValue.split(",")[3]) - sectionY1));
													elementValue = bounds;
													out.add("                        <" + elementTag + ">" + elementValue +  "</" + elementTag + ">");
												}
												else if ((elementTag.equals("uri")) && 
													(sectionElementValue.equals("Trend") || sectionElementValue.equals("Histo") || 
													 sectionElementValue.equals("IndicatorList") || sectionElementValue.equals("LabelList") || 
													 sectionElementValue.equals("ValueList") || sectionElementValue.equals("TextList"))) {
													elementValue = elementValue.replace(");",") ");
													if (elementValue.equals("") || elementValue.equals(" ")) {
														out.add("                        <" + elementTag + ">" + elementValue +  "</" + elementTag + ">");
													}
													else {
														for (j = 0; j < (elementValue.split(" ")).length; j++) {
															String val = elementValue.split(" ")[j];
															out.add("                        <" + elementTag + ">" + val +  "</" + elementTag + ">");
														}
													}
												}
												else if ((elementTag.equals("caption")) && 
													(sectionElementValue.equals("Trend") || sectionElementValue.equals("Histo") || 
													 sectionElementValue.equals("Label") ||sectionElementValue.equals("LabelList") || 
													 sectionElementValue.equals("Slider") || sectionElementValue.equals("Button") || 
													 sectionElementValue.equals("Chat") || sectionElementValue.equals("Video") ||
													 sectionElementValue.equals("Input") || sectionElementValue.equals("WheelSwitch"))) {
													for (j = 0; j < (elementValue.split(";")).length; j++) {
														String val = elementValue.split(";")[j];
														out.add("                        <" + elementTag + ">" + val +  "</" + elementTag + ">");
													}
												}
												else if ((elementTag.equals("condition")) && 
														(sectionElementValue.equals("Trend") || sectionElementValue.equals("Histo") || 
														 sectionElementValue.equals("Button") || sectionElementValue.equals("Text") || 
														 sectionElementValue.equals("TextList"))) {
														for (j = 0; j < (elementValue.split(";")).length; j++) {
															String val = elementValue.split(";")[j];
															out.add("                        <" + elementTag + ">" + val +  "</" + elementTag + ">");
														}
													}
												else if ((elementTag.equals("legend") || elementTag.equals("yMin") ||elementTag.equals("yMax")) && 
														(sectionElementValue.equals("Trend") || sectionElementValue.equals("Histo"))) {
														for (j = 0; j < (elementValue.split(";")).length; j++) {
															String val = elementValue.split(";")[j];
															out.add("                        <" + elementTag + ">" + val +  "</" + elementTag + ">");
														}
													}
												else {
													out.add("                        <" + elementTag + ">" + elementValue +  "</" + elementTag + ">");
												}
										    }
											out.add("                    </" + sectionElementValue + ">");
										}
									}
								}
							}
					        out.add("               </SectionElements>");
							out.add("          </Section>");
						}
					}
				}
				for (Iterator<ArrayList<String>> it1 = elementAttributes.iterator(); it1.hasNext();) {
					attributes = new ArrayList<String>();
					attributes = (ArrayList<String>)it1.next();
					for (Iterator<String> it2 = attributes.iterator(); it2.hasNext();) {
						attribute = (String)it2.next();
						if (attribute.split("]").length == 1) attribute = attribute + " ";
						String value = attribute.split("]")[1];
						if (value.equals("EmbeddedPage")) {
							out.add("          <EmbeddedPage>");
						    for (i = 3; i < attributes.size(); i++) {
								attribute = (String)(attributes.get(i));
								if (attribute.split("]").length == 1) attribute = attribute + " ";
								String embeddedPageTag = (attribute.split("]")[0]).substring(1);
								String embeddedPageValue = attribute.split("]")[1];
								out.add("               <" + embeddedPageTag + ">" + embeddedPageValue +  "</" + embeddedPageTag + ">");
						    }
							out.add("          </EmbeddedPage>");
						}
					}
				}
			    out.add("     </PageElements>");
			    out.add("</Page>");
			    
			    for (Iterator<String> it1 = out.iterator(); it1.hasNext();) {
			    	String line = (String)it1.next();
			    	if (line.contains("enter Id") || line.contains("enter sectionId")) {
			    		dataUncomplete = dataUncomplete + 1;
			    	}
			    }
			} catch(Exception e) {
				dataUncomplete = 1;
				System.out.println("Error saving user's config file!");
				System.err.println(e.toString());
				e.printStackTrace();
			}
		    if (dataUncomplete == 0) {
				if (userPrivilege == expert) {
					Web2cConfigWriter configFileWriter = new Web2cConfigWriter(new File(homeDir + File.separator + configFileName), debugLogger, debugListener);
					configFileWriter.setXML(out);
				}
		    	str = "<value>fileSaved</value>";
		    }
		    else {
		    	str = "<value>fileNotSaved</value>";
		    }
			
		}
		else if (action.equals("readFromFile")) {
			String configFileName = null;
			int i = 0;
			int j = 0;
			String caption = null;
			String legend = null;
			String yMin = null;
			String yMax = null;
			String condition = null;
			String uri = null;
			String expert = null;
			String superVisor = null;
			String ipAddress = null;
			
			configFileName = parameter1;
			Web2cConfigReader configFileReader = new Web2cConfigReader(new File(homeDir + File.separator + configFileName), debugLogger, debugListener);
			ArrayList<String> pageAttributeNames = new ArrayList<String>();   //all attribute tags of the page
			ArrayList<String> pageAttributeValues = new ArrayList<String>();  //all attribute values of the page
			ArrayList<ArrayList<String>> embeddedPagesAttributeNames = new ArrayList<ArrayList<String>>();
			ArrayList<ArrayList<String>> embeddedPagesAttributeValues = new ArrayList<ArrayList<String>>();
			ArrayList<ArrayList<String>> sectionsAttributeNames = new ArrayList<ArrayList<String>>();
			ArrayList<ArrayList<String>> sectionsAttributeValues = new ArrayList<ArrayList<String>>();
			ArrayList<ArrayList<String>> sectionsElements = new ArrayList<ArrayList<String>>();
			ArrayList<ArrayList<ArrayList<String>>> sectionsElementsAttributeNames = new ArrayList<ArrayList<ArrayList<String>>>();
			ArrayList<ArrayList<ArrayList<String>>> sectionsElementsAttributeValues = new ArrayList<ArrayList<ArrayList<String>>>();
			pageAttributeNames = configFileReader.getPageAttributeNames();
			pageAttributeValues = configFileReader.getPageAttributeValues();
			sectionsAttributeNames = configFileReader.getSectionsAttributeNames();
			sectionsAttributeValues = configFileReader.getSectionsAttributeValues();
			sectionsElements = configFileReader.getSectionsElements();
			sectionsElementsAttributeNames = configFileReader.getSectionsElementsAttributeNames();
			sectionsElementsAttributeValues = configFileReader.getSectionsElementsAttributeValues();
			embeddedPagesAttributeNames = configFileReader.getEmbeddedPagesAttributeNames();
			embeddedPagesAttributeValues = configFileReader.getEmbeddedPagesAttributeValues();
			
			j = 0;
			str = "<value>";
			expert = "[expert]";
			superVisor = "[supervisor]";
			ipAddress = "[ipAddress]";
			elementAttributes.clear();
			attributes = new ArrayList<String>();
			attributes.add("[uniqueId]" + "editorPage_drawingFrame");
			str = str + "[uniqueId]" + "editorPage_drawingFrame" + "&#044;";
			attributes.add("[type]Page");
			str = str + "[type]Page" + "&#044;";
			attributes.add("[section]n.a.");
			str = str + "[section]n.a." + "&#044;";
			for (i = 0; i < pageAttributeNames.size(); i++) {
				if (pageAttributeNames.get(i).equals("expert")) {
					expert = expert + pageAttributeValues.get(i) + ";";
				}
				else if (pageAttributeNames.get(i).equals("supervisor")) {
					superVisor = superVisor + pageAttributeValues.get(i) + ";";
				}
				else if (pageAttributeNames.get(i).equals("ipAddress")) {
					ipAddress = ipAddress + pageAttributeValues.get(i) + ";";
				}
				else {
					attributes.add("[" + pageAttributeNames.get(i) + "]" + pageAttributeValues.get(i));
					str = str + "[" + pageAttributeNames.get(i) + "]" + pageAttributeValues.get(i) + "&#044;";
				}
			}
			if (!(expert.equals("[expert]"))) {
				expert = expert.substring(0, expert.length() - 1);
				attributes.add(expert);
				str = str + expert + "&#044;";
			}
			if (!(superVisor.equals("[supervisor]"))) {
				superVisor = superVisor.substring(0, superVisor.length() - 1);
				attributes.add(superVisor);
				str = str + superVisor + "&#044;";
			}
			if (!(ipAddress.equals("[ipAddress]"))) {
				ipAddress = ipAddress.substring(0, ipAddress.length() - 1);
				attributes.add(ipAddress);
				str = str + ipAddress + "&#044;";
			}
			str = str.substring(0, str.length() - 6);
			elementAttributes.add(attributes);
			str = str + "</value><value>";
			Iterator<ArrayList<String>> it1 = sectionsAttributeNames.iterator();
			Iterator<ArrayList<String>> it2 = sectionsAttributeValues.iterator();
			Iterator<ArrayList<ArrayList<String>>> it4 = sectionsElementsAttributeNames.iterator();
			Iterator<ArrayList<ArrayList<String>>> it5 = sectionsElementsAttributeValues.iterator();
			for (Iterator<ArrayList<String>> it3 = sectionsElements.iterator(); it3.hasNext();) {
				ArrayList<String> attributeNames = (ArrayList<String>)it1.next();     // all attribute tags of a specific section element within a page
				ArrayList<String> attributeValues = (ArrayList<String>)it2.next();    // all attribute values of a specific section element within a page
				attributes = new ArrayList<String>();
				attributes.add("[uniqueId]" + String.valueOf(j));
				str = str + "[uniqueId]" + String.valueOf(j) + "&#044;";
				j = j + 1;
				attributes.add("[type]Section");
				str = str + "[type]Section" + "&#044;";
				attributes.add("[section]n.a.");
				str = str + "[section]n.a." + "&#044;";
				for (i = 0; i < attributeNames.size(); i++) {
					if (attributeNames.get(i).equals("bounds")) {
						String bounds = "";
						bounds = bounds + String.valueOf(Integer.valueOf(((String)attributeValues.get(i)).split(",")[0])) + ",";
						bounds = bounds + String.valueOf(Integer.valueOf(((String)attributeValues.get(i)).split(",")[1])) + ",";
						bounds = bounds + String.valueOf(Integer.valueOf(((String)attributeValues.get(i)).split(",")[2])) + ",";
						bounds = bounds + String.valueOf(Integer.valueOf(((String)attributeValues.get(i)).split(",")[3]));
						attributes.add("[" + attributeNames.get(i) + "]" + bounds);
						str = str + "[" + attributeNames.get(i) + "]" + bounds + "&#044;";
					}
					else {
						attributes.add("[" + attributeNames.get(i) + "]" + attributeValues.get(i));
						str = str + "[" + attributeNames.get(i) + "]" + attributeValues.get(i) + "&#044;";
					}
				}
				str = str.substring(0, str.length() - 6);
				elementAttributes.add(attributes);
				str = str + "</value><value>";
				ArrayList<String> elementsNames = (ArrayList<String>)it3.next();      // all elements (type) of a specific section element within a page
				ArrayList<ArrayList<String>> elementsAttributeNames = (ArrayList<ArrayList<String>>)it4.next();
				ArrayList<ArrayList<String>> elementsAttributeValues = (ArrayList<ArrayList<String>>)it5.next();
				Iterator<String> it6 = elementsNames.iterator();
				Iterator<ArrayList<String>> it7 = elementsAttributeValues.iterator();
				for (Iterator<ArrayList<String>> it8 = elementsAttributeNames.iterator(); it8.hasNext();) {
					String element = (String)it6.next();         // specific element (type) within a specific section 
					ArrayList<String> names = (ArrayList<String>)it8.next();     // all attribute tags of a specific element within a specific section within a page
					ArrayList<String> values = (ArrayList<String>)it7.next();    // all attribute value of a specific element within a specific section within a page
					String sectionBounds = "";
					int sectionX1 = 0;
					int sectionY1 = 0;
					uri = "[uri]";
					caption = "[caption]";
					legend = "[legend]";
					yMin = "[yMin]";
					yMax = "[yMax]";
					condition = "[condition]";
					attributes = new ArrayList<String>();
					attributes.add("[uniqueId]" + String.valueOf(j));
					str = str + "[uniqueId]" + String.valueOf(j) + "&#044;";
					j = j + 1;
					attributes.add("[type]" + element);
					str = str + "[type]" + element + "&#044;";
					for (i = 0; i < attributeNames.size(); i++) {
						if (attributeNames.get(i).equals("id")) {
							attributes.add("[section]" + attributeValues.get(i));
							str = str + "[section]" + attributeValues.get(i) + "&#044;";
						}
						if (attributeNames.get(i).equals("bounds")) {
							sectionBounds = (String)attributeValues.get(i);
							sectionX1 = Integer.valueOf(sectionBounds.split(",")[0]);
							sectionY1 = Integer.valueOf(sectionBounds.split(",")[1]);
						}
					}
					for (i = 0; i < names.size(); i++) {
						if (names.get(i).equals("bounds")) {
							String bounds = "";
							bounds = bounds + String.valueOf((Integer.valueOf(((String)values.get(i)).split(",")[0]) + sectionX1)) + ",";
							bounds = bounds + String.valueOf((Integer.valueOf(((String)values.get(i)).split(",")[1]) + sectionY1)) + ",";
							bounds = bounds + String.valueOf((Integer.valueOf(((String)values.get(i)).split(",")[2]) + sectionX1)) + ",";
							bounds = bounds + String.valueOf((Integer.valueOf(((String)values.get(i)).split(",")[3]) + sectionY1));
							attributes.add("[" + names.get(i) + "]" + bounds);
							str = str + "[" + names.get(i) + "]" + bounds + "&#044;";
						}
						else if (names.get(i).equals("uri")) {
							uri = uri + values.get(i) + ";";
						}
						else if (names.get(i).equals("caption")) {
							caption = caption + values.get(i) + ";";
						}
						else if (names.get(i).equals("condition")) {
							condition = condition + values.get(i) + ";";
						}
						else if (names.get(i).equals("legend")) {
							legend = legend + values.get(i) + ";";
						}
						else if (names.get(i).equals("yMin")) {
							yMin = yMin + values.get(i) + ";";
						}
						else if (names.get(i).equals("yMax")) {
							yMax = yMax + values.get(i) + ";";
						}
						else {
							attributes.add("[" + names.get(i) + "]" + values.get(i));
							str = str + "[" + names.get(i) + "]" + values.get(i) + "&#044;";
						}
					}
					if (!(caption.equals("[caption]"))) {
						caption = caption.substring(0, caption.length() - 1);
						attributes.add(caption);
						str = str + caption + "&#044;";
					}
					if (!(condition.equals("[condition]"))) {
						condition = condition.substring(0, condition.length() - 1);
						attributes.add(condition);
						str = str + condition + "&#044;";
					}
					if (!(yMin.equals("[yMin]"))) {
						yMin = yMin.substring(0, yMin.length() - 1);
						attributes.add(yMin);
						str = str + yMin + "&#044;";
					}
					if (!(yMax.equals("[yMax]"))) {
						yMax = yMax.substring(0, yMax.length() - 1);
						attributes.add(yMax);
						str = str + yMax + "&#044;";
					}
					if (!(legend.equals("[legend]"))) {
						legend = legend.substring(0, legend.length() - 1);
						attributes.add(legend);
						str = str + legend + "&#044;";
					}
					if (!(uri.equals("[uri]"))) {
						uri = uri.substring(0, uri.length() - 1);
						attributes.add(uri);
						str = str + uri + "&#044;";
					}
					str = str.substring(0, str.length() - 6);
					elementAttributes.add(attributes);
					str = str + "</value><value>";
				}
			}
			if (embeddedPagesAttributeNames != null) {
				ArrayList<String> attributeNames = new ArrayList<String>();
				ArrayList<String> attributeValues = new ArrayList<String>();
				Iterator<ArrayList<String>> it9 = embeddedPagesAttributeValues.iterator();
				for (Iterator<ArrayList<String>> it10 = embeddedPagesAttributeNames.iterator(); it10.hasNext();) {
					attributeNames = (ArrayList<String>)it10.next();   // all attribute tags of a specific embedded page within a page
					attributeValues = (ArrayList<String>)it9.next();   // all attribute values of a specific embedded page within a page
					attributes = new ArrayList<String>();
					attributes.add("[uniqueId]" + String.valueOf(j));
					str = str + "[uniqueId]" + String.valueOf(j) + "&#044;";
					j = j + 1;
					attributes.add("[type]EmbeddedPage");
					str = str + "[type]Page" + "&#044;";
					attributes.add("[section]n.a.");
					str = str + "[section]n.a." + "&#044;";
					for (i = 0; i < attributeNames.size(); i++) {
						if (attributeNames.get(i).equals("bounds")) {
							String bounds = "";
							bounds = bounds + String.valueOf(Integer.valueOf(((String)attributeValues.get(i)).split(",")[0])) + ",";
							bounds = bounds + String.valueOf(Integer.valueOf(((String)attributeValues.get(i)).split(",")[1])) + ",";
							bounds = bounds + String.valueOf(Integer.valueOf(((String)attributeValues.get(i)).split(",")[2])) + ",";
							bounds = bounds + String.valueOf(Integer.valueOf(((String)attributeValues.get(i)).split(",")[3]));
							attributes.add("[" + attributeNames.get(i) + "]" + bounds);
							str = str + "[" + attributeNames.get(i) + "]" + bounds + "&#044;";
						}
						else {
							attributes.add("[" + attributeNames.get(i) + "]" + attributeValues.get(i));
							str = str + "[" + attributeNames.get(i) + "]" + attributeValues.get(i) + "&#044;";
						}
					}
					str = str.substring(0, str.length() - 6);
					elementAttributes.add(attributes);
					str = str + "</value><value>";
				}
			}
			str = str.substring(0, str.length() - 7);
			
		}
		else if (action.equals("addComponent")) {
			String elementId = null;
			String elementType = null;

			elementId = parameter1;
			elementType = parameter2;
			attributes = new ArrayList<String>();
			if (elementType.equals("Section")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "n.a.");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[backgroundColor]" + white);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[extensionManifest]" + " ");
			}
			else if (elementType.equals("EmbeddedPage")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "n.a.");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("AnimatedImage")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[blinking]" + " ");
				attributes.add("[blinkingEnabled]" + no);
				attributes.add("[sound]" + " ");
				attributes.add("[soundEnabled]" + no);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[alarmThreshold]" + "0.0");
				attributes.add("[lowIcon]" + " ");
				attributes.add("[highIcon]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Image")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Button")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[superComponent]" + no);
				attributes.add("[keepHistory]" + "true");
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[caption]" + " ");
				attributes.add("[condition]" + " ");
				attributes.add("[image]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Slider")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[minimum]" + "0");
				attributes.add("[maximum]" + "10");
				attributes.add("[superComponent]" + no);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[caption]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("WheelSwitch")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[superComponent]" + no);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[caption]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Input")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[superComponent]" + no);
				attributes.add("[fieldLength]" +  " ");
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[caption]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Indicator")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[blinking]" + " ");
				attributes.add("[blinkingEnabled]" + no);
				attributes.add("[sound]" + " ");
				attributes.add("[soundEnabled]" + no);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[trueValue]" + "0.0");
				attributes.add("[falseIcon]" + redIcon);
				attributes.add("[trueIcon]" + greenIcon);
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("IndicatorList")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[headerCaption]" + " ");
				attributes.add("[headerColor]" + black);
				attributes.add("[headerBackgroundColor]" + gray);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[blinking]" + " ");
				attributes.add("[blinkingEnabled]" + no);
				attributes.add("[sound]" + " ");
				attributes.add("[soundEnabled]" + no);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[trueValue]" + "0.0");
				attributes.add("[falseIcon]" + redIcon);
				attributes.add("[trueIcon]" + greenIcon);
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Label")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[alignment]" + left);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[caption]" + " ");
			}
			else if (elementType.equals("LabelList")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[headerCaption]" + " ");
				attributes.add("[headerColor]" + black);
				attributes.add("[headerBackgroundColor]" + gray);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[alignment]" + left);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[caption]" + " ");
			}
			else if (elementType.equals("Text")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[blinking]" + " ");
				attributes.add("[blinkingEnabled]" + no);
				attributes.add("[sound]" + " ");
				attributes.add("[soundEnabled]" + no);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[alignment]" + center);
				attributes.add("[scrollBar]" + " ");
				attributes.add("[condition]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("TextList")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[headerCaption]" + " ");
				attributes.add("[headerColor]" + black);
				attributes.add("[headerBackgroundColor]" + gray);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[blinking]" + " ");
				attributes.add("[blinkingEnabled]" + no);
				attributes.add("[sound]" + " ");
				attributes.add("[soundEnabled]" + no);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[alignment]" + center);
				attributes.add("[condition]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Video")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[alignment]" + center);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[caption]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Ticker")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[blinking]" + " ");
				attributes.add("[blinkingEnabled]" + no);
				attributes.add("[sound]" + " ");
				attributes.add("[soundEnabled]" + no);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Time")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[alignment]" + center);
				attributes.add("[offset]" + " ");
			}
			else if (elementType.equals("Trend")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[plotType]" + linePlot);
				attributes.add("[plotAxis]" + multi);
				attributes.add("[plotColor]" + "red,blue,magenta,green,pink,yellow,orange,cyan,darkGray,black");
				attributes.add("[timeInterval]" + timeInterval3600);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[function]" + " ");
				attributes.add("[caption]" + " ");
				attributes.add("[condition]" + " ");
				attributes.add("[yMin]" + "0.0");
				attributes.add("[yMax]" + "0.0");
				attributes.add("[legend]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Histo")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[plotType]" + linePlot);
				attributes.add("[plotAxis]" + multi);
				attributes.add("[plotOrder]" + normal);
				attributes.add("[plotColor]" + "red,blue,magenta,green,pink,yellow,orange,cyan,darkGray,black");
				attributes.add("[xMin]" + "0.0");
				attributes.add("[xMax]" + "0.0");
				attributes.add("[xLabel]" + " ");
				attributes.add("[fontSize]" + "medium");
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[function]" + " ");
				attributes.add("[caption]" + " ");
				attributes.add("[condition]" + " ");
				attributes.add("[yMin]" + "0.0");
				attributes.add("[yMax]" + "0.0");
				attributes.add("[legend]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Value")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[alarmThreshold]" + "0.0");
				attributes.add("[minLimit]" + "0.0");
				attributes.add("[lowColor]" + black);
				attributes.add("[highColor]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[blinking]" + " ");
				attributes.add("[blinkingEnabled]" + no);
				attributes.add("[sound]" + " ");
				attributes.add("[soundEnabled]" + no);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[alignment]" + left);
				attributes.add("[format]" + expformat);
				attributes.add("[function]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("ValueList")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[headerCaption]" + " ");
				attributes.add("[headerColor]" + black);
				attributes.add("[headerBackgroundColor]" + gray);
				attributes.add("[alarmThreshold]" + "0.0");
				attributes.add("[minLimit]" + "0.0");
				attributes.add("[lowColor]" + black);
				attributes.add("[highColor]" + black);
				attributes.add("[backgroundColor]" + white);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[blinking]" + " ");
				attributes.add("[blinkingEnabled]" + no);
				attributes.add("[sound]" + " ");
				attributes.add("[soundEnabled]" + no);
				attributes.add("[extensionManifest]" + " ");
				attributes.add("[expertComponent]" + no);
				attributes.add("[alignment]" + left);
				attributes.add("[format]" + expformat);
				attributes.add("[function]" + " ");
				attributes.add("[uri]" + " ");
			}
			else if (elementType.equals("Chat")) {
				attributes.add("[uniqueId]" + elementId);
				attributes.add("[type]" + elementType);
				attributes.add("[sectionId]" + "enter sectionId");
				attributes.add("[id]" + "enter Id");
				attributes.add("[bounds]" + " ");
				attributes.add("[tooltipText]" + " ");
				attributes.add("[color]" + black);
				attributes.add("[backgroundColor]" + gray);
				attributes.add("[fontSize]" + "medium");
				attributes.add("[fontThickness]" + normal);
				attributes.add("[border]" + none);
				attributes.add("[borderColor]" + black);
				attributes.add("[blinking]" + " ");
				attributes.add("[blinkingEnabled]" + no);
				attributes.add("[sound]" + " ");
				attributes.add("[soundEnabled]" + no);
				attributes.add("[expertComponent]" + no);
				attributes.add("[prompt]" + "chat");
				attributes.add("[alignment]" + left);
				attributes.add("[caption]" + " ");
			}
			elementAttributes.add(attributes);
			for (Iterator<String> it = attributes.iterator(); it.hasNext();) {
				attribute = (String)it.next();
				str = str + attribute + "&#044;";
			}
			str = str.substring(0, str.length() - 6);
			str = "<value>" + str + "</value>";

		}
		
		else if (action.equals("removeComponent")) {
			int index = 0;
			int m = 0;
			String elementId = null;

			elementId = parameter1;
			for (Iterator<ArrayList<String>> it = elementAttributes.iterator(); it.hasNext();) {
				ArrayList<String> list = (ArrayList<String>)it.next();
				if ((list.get(0).split("]")[1]).equals(elementId)) {
					index = m;
				}
				m = m + 1;
			}
			elementAttributes.remove(index);
			str = "<value></value>";
		}
		
		else if (action.equals("confirmAttributeChanges")) {
			String elementId = null;
			String[] attributeList = null;
			
			elementId = parameter1;
			attributeList = parameter2.split("&#044;");
			for (Iterator<ArrayList<String>> it = elementAttributes.iterator(); it.hasNext();) {
				ArrayList<String> list = (ArrayList<String>)it.next();
				if ((list.get(0).split("]")[1]).equals(elementId)) {
					list.set(0, "[uniqueId]" + elementId);
					str = str + String.valueOf(list.get(0)) + "&#044;";
					for (int i = 0; i < attributeList.length; i++) {
						if (!(attributeList[i].split("]"))[0].equals("[") && !(attributeList[i].split("]"))[0].equals("[ ")) {
							if ((attributeList[i].split("]")).length != 1) {
								list.set(i + 1, (attributeList[i].split("]"))[0] + "]" + (attributeList[i].split("]"))[1]);
							}
							else {
								list.set(i + 1, (attributeList[i].split("]"))[0] + "]" + " ");						
							}
						    str = str + String.valueOf(list.get(i+1)) + "&#044;";
						}
					}
					str = str.substring(0, str.length() - 6);
				}
			}
			str = "<value>" + str + "</value>";
			
		}
		
		else if (action.equals("selectElement")) {
			String elementId = null;
			String elementBounds = null;
			
			elementId = parameter1;
			elementBounds = parameter2;
			for (Iterator<ArrayList<String>> it = elementAttributes.iterator(); it.hasNext();) {
				ArrayList<String> list = (ArrayList<String>)it.next();
					if ((list.get(0).split("]")[1]).equals(elementId)) {
						if (!(elementId.equals("editorPage_drawingFrame"))) {
							for (int i = 1; i < list.size(); i++) {
								if ((list.get(i).split("]")[0]).equals("[bounds")) {
									list.set(i, "[bounds]" + elementBounds);
								}
							}
						}
						for (int i = 0; i < list.size(); i++) {
							str = str + String.valueOf(list.get(i)) + "&#044;";
						}
					}
			}
			str = str.substring(0, str.length() - 6);
			str = "<value>" + str + "</value>";

		}
		
		return str;
	}
	
	private void addDrawingFrame() {
		
		ArrayList<String> attributes = new ArrayList<String>();	
		attributes = new ArrayList<String>();
		attributes.add("[uniqueId]" + "editorPage_drawingFrame");
		attributes.add("[type]" + "Page");
		attributes.add("[sectionId]" + "n.a.");
		attributes.add("[caption]" + "enter title");
		attributes.add("[timeInterval]" + "2");
		attributes.add("[message]" + " ");
		attributes.add("[reconnect]" + "false");
		attributes.add("[sound]" + " ");
		attributes.add("[scale]" + " ");
		attributes.add("[debug]" + "no");
		attributes.add("[expert]" + " ");
		attributes.add("[supervisor]" + " ");
		attributes.add("[exclusive]" + " ");
		attributes.add("[ipAddress]" + "*.*.*.*");
		elementAttributes.add(attributes);
	}
	
	private void pageExecute(File configFile) {
		
		Web2cConfigReader configReader = new Web2cConfigReader(configFile, debugLogger, debugListener);
		decodeAttributes(configReader.getPageAttributeNames(), configReader.getPageAttributeValues());
		setPageElements(configReader.getSectionsAttributeNames(), configReader.getSectionsAttributeValues(), configReader.getSectionsElements(), configReader.getSectionsElementsAttributeNames(), configReader.getSectionsElementsAttributeValues());
	}
	
	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		expertName = new ArrayList<String>();
		expertAddress = new ArrayList<String>();
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("title")) {
				title = value;
			}		
			if (name.equals("expert")) {
				expertName.add(value);
			}					
			if (name.equals("ipAddress")) {
				expertAddress.add(value);
			}
		}	
		
		for (Iterator<String> it = expertName.iterator(); it.hasNext();) {
			if ((it.next()).equals(userName)) {
				userPrivilege = expert;
			}
		}		
		for (Iterator<String> it = expertAddress.iterator(); it.hasNext();) {
			String nextAddress = it.next().toString();
			if (nextAddress.replace(".", ">").split(">")[3].equals("*") || nextAddress.replace(".", ">").split(">")[3].equals(userAddress.replace(".", ">").split(">")[3])) {
				if (nextAddress.replace(".", ">").split(">")[2].equals("*") || nextAddress.replace(".", ">").split(">")[2].equals(userAddress.replace(".", ">").split(">")[2])) {
					if (nextAddress.replace(".", ">").split(">")[1].equals("*") || nextAddress.replace(".", ">").split(">")[1].equals(userAddress.replace(".", ">").split(">")[1])) {
						if (nextAddress.replace(".", ">").split(">")[0].equals("*") || nextAddress.replace(".", ">").split(">")[0].equals(userAddress.replace(".", ">").split(">")[0])) {
							networkPrivilege = expert;
						}
					}
				}
			}
		}
		
		if (userPrivilege == expert && networkPrivilege == expert) {
			userPrivilege = expert;
		}
		else if (userPrivilege == expert && expertAddress.isEmpty()) {
			userPrivilege = expert;
		}
		else {
			userPrivilege = layman;
		}
		
		title = title + " (User: " + userName;
		if (userPrivilege == expert) {
			title = title + ", Privilege: expert";
		}
		else {
			title = title + ", Privilege: layman";
		}
		title = title + ", IP Address: " + userAddress;
		title = title + ")";
	}
	
	private void setPageElements(ArrayList<ArrayList<String>> sectionsAttributeNames, ArrayList<ArrayList<String>> sectionsAttributeValues, ArrayList<ArrayList<String>> sectionsElements, ArrayList<ArrayList<ArrayList<String>>> sectionsElementsAttributeNames, ArrayList<ArrayList<ArrayList<String>>> sectionsElementsAttributeValues) {
		
		Iterator<ArrayList<String>> it1 = sectionsAttributeNames.iterator();
		Iterator<ArrayList<String>> it2 = sectionsAttributeValues.iterator();
		Iterator<ArrayList<ArrayList<String>>> it4 = sectionsElementsAttributeNames.iterator();
		Iterator<ArrayList<ArrayList<String>>> it5 = sectionsElementsAttributeValues.iterator();
		for (Iterator<ArrayList<String>> it3 = sectionsElements.iterator(); it3.hasNext();) {
			ArrayList<String> attributeNames = (ArrayList<String>)it1.next();
			ArrayList<String> attributeValues = (ArrayList<String>)it2.next();
			ArrayList<String> elementsNames = (ArrayList<String>)it3.next();
			ArrayList<ArrayList<String>> elementsAttributeNames = (ArrayList<ArrayList<String>>)it4.next();
			ArrayList<ArrayList<String>> elementsAttributeValues = (ArrayList<ArrayList<String>>)it5.next();
			Web2cEditorSection section = new Web2cEditorSection(hostURL, referenceId, referencePoint, attributeNames, attributeValues, elementsNames, elementsAttributeNames, elementsAttributeValues, userPrivilege);
			sections.add(section);
			sectionsHTML.add(section.getHTML());
			sectionsJavaScript.add(section.getJavaScript());
		}
	}
}
