package de.desy.web2cToolkit.web2cEditor;

import java.util.ArrayList;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;


/**
 * @author R.Bacher, DESY
 */
public class Web2cEditorTextInput implements IWeb2cConstants {

	private String referenceId = null;
	private String id = null;
	private String width = null;
	private String height = null;
	private String marginLeft = null;
	private String marginTop  = null;
	private String toolTipText = null;
	private String backgroundColor = null;
	private String textAlignment = null;
	private String fontSize = null;
	private String fontThickness= null;
	private String borderStyle = null;
	private String borderWidth = null;
	private String borderColor = null;

	/**
	 * Buttons on Editor Page
	 * @param refId ID of reference component which component belongs to
	 * @param names list of valid attribute names
	 * @param values list of actual attribute values
	 */
	public Web2cEditorTextInput(String refId, ArrayList<String> names, ArrayList<String> values) {

		referenceId = refId;
		decodeAttributes(names, values);
	}

	/**
	 * Returns component's HTML tags 
	 * @return HTML tags
	 */
	public ArrayList<String> getHTML() {
		
		ArrayList<String> componentHTML = new ArrayList<String>();	
		String html = null;
		html = "			<input type=\"text\" id=\"" + id + "\" value=\"Enter Configuration File Name\" title=\"" + toolTipText + "\"";
		html = html + " style=\"position:absolute; left:" + marginLeft + "; top:" + marginTop + "; width:" + width + "; height:" + height + "; color:black; background-color:" + backgroundColor + "; border-style:" + borderStyle + "; border-width:" + borderWidth + "; border-color:" + borderColor + "; font-size:" + fontSize + "; font-weight:" + fontThickness + "; text-align:" + textAlignment + ";\">";	
		
		componentHTML.add(html);
		
		return componentHTML;
	}

	/**
	 * Returns component's JavaScript code snippets
	 * @return JavaScript code snippets
	 */
	public ArrayList<String> getJavaScript() {
		
		ArrayList<String> componentJavaScript = new ArrayList<String>();	
		
		componentJavaScript.add("	//EditorTextInput: " + id);
		componentJavaScript.add("	var textInputId = \"" + id + "\";");
		componentJavaScript.add("	var configFileName;");

		return componentJavaScript;
	}
	
	private void decodeAttributes(ArrayList<String> attributeNames, ArrayList<String> attributeValues) {
		
		int xLeft = 0;
		int yTop = 0;
		int xRight= 0;
		int yBottom = 0;

		toolTipText = "";
		backgroundColor = white;
		textAlignment = left;	
		fontSize = medium;
		fontThickness = normal;	
		borderStyle = solid;
		borderWidth = thin;
		borderColor = black;
		
		for (int i = 0; i < attributeNames.toArray().length; i++) {
			String name = attributeNames.get(i);
			String value = attributeValues.get(i);
			if (name.equals("id")) {
				id = referenceId + "_" + value;
			}
			if (name.equals("bounds")) {
				xLeft = Integer.valueOf(value.split(",")[0]);
				yTop = Integer.valueOf(value.split(",")[1]);
				xRight = Integer.valueOf(value.split(",")[2]);
				yBottom = Integer.valueOf(value.split(",")[3]);
				marginLeft = ((Integer)(xLeft)).toString() + "px";
				marginTop = ((Integer)(yTop)).toString() + "px";
				width = ((Integer)(xRight - xLeft + 1)).toString() + "px";
				height = ((Integer)(yBottom - yTop + 1)).toString() + "px";
			}
			if (name.equals("backgroundColor")) {
				backgroundColor = value;
			}			
			if (name.equals("alignment")) {
				textAlignment = value;
			}						
			if (name.equals("fontSize")) {
				if (value.equals("small")) {
					fontSize = small;
				}
				else if (value.equals("medium")) {
					fontSize = medium;
				}
				else {
					fontSize = large;
				}
			}			
			if (name.equals("fontThickness")) {
				if (value.equals("normal")) {
					fontThickness = normal;
				}
				else {
					fontThickness = bold;
				}
			}						
			if (name.equals("border")) {
				if (value.equals("thin")) {
					borderWidth= thin;
					borderStyle = solid;
				}
				else if (value.equals("thick")) {
					borderStyle = solid;
					borderWidth= thick;
				}
				else {
					borderStyle = none;
				}	
			}
			if (name.equals("borderColor")) {
				borderColor = value;
			}
			if (name.equals("borderColor")) {
				borderColor = value;
			}
			if (name.equals("tooltipText")) {
				toolTipText = value;
			}
		}	
		if (borderStyle.equals(solid)) {
			width = String.valueOf(Integer.valueOf(width.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
			height = String.valueOf(Integer.valueOf(height.split("p")[0]) - (2 * Integer.valueOf(borderWidth.split("p")[0]))) + "px";
		}
	}
}
