package de.desy.web2cToolkit.web2cLogbook;

import java.util.ArrayList;
import java.util.Iterator;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2cMessenger.Web2cSMTPInterface;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cLogbookInitFileWriterAndReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;

/**
 * @author R.Bacher, DESY
 */
public class Web2cLogbookAutoReporter implements IWeb2cConstants {

	private boolean serviceAvailable = false;
	private boolean serviceAutoEnabledOption = false;
	private ArrayList<String> fileList = new ArrayList<String>();
	private boolean iniFileRead = false;
	private Web2cCommonDeviceAccess commonDeviceAccess = null;
	private Web2cSMTPInterface smtpInterface = null;
	private volatile Double[] constructionProgress = new Double[1];
	private ArrayList<Web2cLogbookPage> web2cLogbookAutoPageList = new  ArrayList<Web2cLogbookPage>();
	private Web2cLogbookInitFileWriterAndReader logbookInitFileWriterAndReader = null;
	private String homeDir = null;
	private String logbookDir = null;
	
    /**
     * Web2cLogbook auto-reporter
     * Provides auto-reporting service
     * Constructor
	 * @param home directory which contains user-specific files
	 * @param logbookHome directory which contains logbook-specific files
	 * @param cda actual common device access instance
	 * @param smtp SMTP service interface instance
	 * @param readerAndWriter logbook init file reader/writer
     */
	public Web2cLogbookAutoReporter(String home, String logbookHome, Web2cCommonDeviceAccess cda, Web2cSMTPInterface smtp, Web2cLogbookInitFileWriterAndReader readerAndWriter) {

		homeDir = home;
		logbookDir = logbookHome;
		commonDeviceAccess = cda;
		smtpInterface = smtp;
		logbookInitFileWriterAndReader = readerAndWriter;
	}	
	
	/**
	* Returns the service auto-enabled-option.
	* @return option
	*/
	public String getServiceAutoEnabledOption() {
		
		String ret = "no";
		
		if (serviceAutoEnabledOption == true) ret = "yes";
		else ret = "no";
			
		return ret;
	}	
	
	/**
	 * Returns the list of logbook files to be scanned for auto-reporting
	 * @return file list
	 */
	public ArrayList<String> getAutoReportingFileList() {
		
		return fileList;
	}
	
	/**
	* Arms the service
	*/
	public void armService() {
		
		readInitializationFile();
		initializeService();
		startLogbookAutoReporting();
	}
	
	/**
	* Disarms the service
	*/
	public void disarmService() {

		stopLogbookAutoReporting();
		iniFileRead = false;
		serviceAvailable = false;	
	}
	
	/**
	 * Starts auto-reporting service
	 */
	public void startLogbookAutoReporting() {
		
		if (serviceAvailable == true && serviceAutoEnabledOption == true) {
			for (Iterator<Web2cLogbookPage> it = web2cLogbookAutoPageList.iterator(); it.hasNext();) {
				Web2cLogbookPage autoLogbookPage = it.next(); 
				autoLogbookPage.startReportCreation();
			}
		}
	}
	
	/**
	 * Terminated auto-reporting service
	 */
	public void stopLogbookAutoReporting() {
		
		for (Iterator<Web2cLogbookPage> it = web2cLogbookAutoPageList.iterator(); it.hasNext();) {
			Web2cLogbookPage autoLogbookPage = it.next(); 
			autoLogbookPage.stopReportCreation();
		}
	}
	
	/**
	* Returns the service status.
	* @return status
	*/
	public boolean getServiceAvailable() {
			
		return serviceAvailable;
	}
	
    /**
     * Reads the Web2cLogbook initialization file
     */
	private void readInitializationFile() {

		if (logbookInitFileWriterAndReader.getInitFileReadStatus() == true) {
			fileList = logbookInitFileWriterAndReader.getAutoReportingFileList();
			if (logbookInitFileWriterAndReader.getServiceAutoEnabledOption().equals("yes")) serviceAutoEnabledOption = true;
			else serviceAutoEnabledOption = false;			
			iniFileRead = true;
		}
		else {
			iniFileRead = false;
			System.out.println("Logbook auto-reporting service not available!");
		}
	}    
	
	/**
     * Initializes the auto-reporting service
     */
	private void initializeService() {
		
		if (!web2cLogbookAutoPageList.isEmpty()) {
			stopLogbookAutoReporting();
			web2cLogbookAutoPageList.clear();
		}

		if (iniFileRead == true) {	
			int i = 0;
			for (Iterator<String> it = fileList.iterator(); it.hasNext();) {
				String fileName = it.next(); 
				Web2cLogbookPage autoLogbookPage = new Web2cLogbookPage("AutoSession " + i, homeDir, logbookDir, fileName, "Web2cLogbookAutoReporter", "127.0.0.1", commonDeviceAccess, smtpInterface, logbookInitFileWriterAndReader, constructionProgress);
				web2cLogbookAutoPageList.add(autoLogbookPage);		
				i = i + 1;
			}
			serviceAvailable = true;
		}
		else {
			serviceAvailable = false;
			System.out.println("Logbook auto-reporting service not available!");
		}
	}
}