package de.desy.web2cToolkit.web2cLogbook;

import java.io.File;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2cMessenger.Web2cSMTPInterface;
import de.desy.web2cToolkit.web2cUtil.web2cEscape.Web2cASCIICharacterEscaper;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cLogbookConfigReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cLogbookInitFileWriterAndReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cLogbookRecordWriterAndReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * Page
 * @author R.Bacher, DESY
 */
public class Web2cLogbookPage implements IWeb2cConstants, Observer {

	private String userName = null;
	private String userAddress = null;
	private String userPrivilege = null;
	private String networkPrivilege = null;
	private Web2cCommonDeviceAccess commonDeviceAccess = null;
	private String web2cSession = null;
	private String pageConfigFile = null;
	private String homeDir = null;
	private String logbookDir = null;
	private String logbookName = null;
	private String logbookPeriode = null;
	private ArrayList<String> expertNameList = new ArrayList<String>();
	private ArrayList<String> supervisorNameList = new ArrayList<String>();
	private ArrayList<String> expertAddressList = new ArrayList<String>();
	private ArrayList<String> exclusiveNameList = new ArrayList<String>();
	private ArrayList<String> categoryList = new ArrayList<String>();
	private ArrayList<String> importanceList = new ArrayList<String>();
	private ArrayList<String> reportCaptionList = new ArrayList<String>();
	private ArrayList<String> reportCategoryList = new ArrayList<String>();
	private ArrayList<String> reportImportanceList = new ArrayList<String>();
	private ArrayList<String> reportModeList = new ArrayList<String>();
	private ArrayList<String> reportConditionList = new ArrayList<String>();
	private ArrayList<ArrayList<String>> reportRecipientsList = new ArrayList<ArrayList<String>>();
	private ArrayList<ArrayList<String>> reportTextLinesList = new ArrayList<ArrayList<String>>();	
	private MonitorLinkRegisterer monitorlinkRegisterer = null;
	private ArrayList<String> monitorList = new ArrayList<String>();
	private ArrayList<MonitorLinkUpdater> monitorLinkUpdaterList = new ArrayList<MonitorLinkUpdater>();
	private Web2cASCIICharacterEscaper characterEscaper = null;
	private boolean exclusive = false;
	private String registerStatus = "idle";
	private String registerError = "";
	private Web2cLogbookConfigReader logbookConfigReader = null;
	private Web2cLogbookRecordWriterAndReader logbookRecordWriterAndReader = null;
	private ReportCreator reportCreator = null;
	private Web2cSMTPInterface smtpInterface = null;
	private volatile Double[] constructionProgress = new Double[1];
	private Web2cLogbookInitFileWriterAndReader logbookInitFileWriterAndReader = null;
	

	/**
	 * Page
	 * @param session Web2cLogbook session id
	 * @param home directory which contains user-specific files
	 * @param logbookHome directory which contains logbook-specific files
	 * @param configFile actual configuration file
	 * @param name actual user name
	 * @param ipAddress actual IP address of user
	 * @param cda actual common device access instance
	 * @param smtp SMTP service interface instance
	 * @param readerAndWriter logbook init file reader/writer
	 * @param progress construction progress
	 */
	public Web2cLogbookPage(String session, String home, String logbookHome, String configFile, String name, String ipAddress, Web2cCommonDeviceAccess cda, Web2cSMTPInterface smtp, Web2cLogbookInitFileWriterAndReader readerAndWriter, Double[] progress) {

		web2cSession = session;	
		userPrivilege = layman;
		networkPrivilege = layman;
		userName = "guest";
		userAddress = "";
		if (name == null) {
			userName = "guest";
		}
		else {
			userName = name;
		}
		if (ipAddress == null) {
			userAddress = "";
		}
		else {
			userAddress = ipAddress;
		}
		homeDir = home;
		logbookDir = logbookHome;
		commonDeviceAccess = cda;
		smtpInterface = smtp;
		logbookInitFileWriterAndReader = readerAndWriter;
		constructionProgress = progress;
		pageConfigFile = configFile;
		registerStatus = "idle";
		registerError = "";
		characterEscaper = new Web2cASCIICharacterEscaper();
		constructPage();
	}
	
	/**
	 * Returns the session id of the page
	 * @return session id
	 */
	public String getSessionId() {
		
		return web2cSession;
	}

	/**
	 * Returns the control system monitors attached to this page
	 * @return list of monitors
	 */
	public ArrayList<String> getMonitors() {
		
		return monitorList;
	}	
	
	/**
	 * Returns the corresponding logbook viewer page (contains GWT widgets)
	 * @return list
	 */
	public ArrayList<String> getPage() {

		ArrayList<String> out = new ArrayList<String>();	
		
		out.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		out.add("<html>");
		out.add("   <head>");
		out.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		out.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		out.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		out.add("       <meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		out.add("       <title>Web2cToolkit Logbook</title>");
		out.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
		out.add("		<link type=\"text/css\" rel=\"stylesheet\" href=\"Web2cLogbookViewerPage.css\">");
		out.add("       <script language=\"javascript\" src=\"de.desy.web2cToolkit.web2cLogbook.web2cGWT.Web2cLogbookViewerPage/de.desy.web2cToolkit.web2cLogbook.web2cGWT.Web2cLogbookViewerPage.nocache.js\"></script>");
		out.add("   </head>");
		out.add("	<body>");
		out.add("		<input type=\"hidden\" id=\"sessionId\" name=\"sessionId\" value=\"" + web2cSession + "\">");
		out.add("		<input type=\"hidden\" id=\"logbookName\" name=\"logbookName\" value=\"" + logbookName + "\">");
		out.add("	</body>");
		out.add("</html>");

		return out;
	}
	
	/**
	 * Returns link registering error
	 * @return error string
	 */
	public String getError() {
		
		return registerError;
	}
	
	/**
	 * Terminates link updating
	 */
	public void stopUpdate() {

		synchronized (monitorLinkUpdaterList) {
			for (Iterator<MonitorLinkUpdater> it = monitorLinkUpdaterList.iterator(); it.hasNext();) {
				MonitorLinkUpdater linkUpdater = (MonitorLinkUpdater)(it.next());
				linkUpdater.stopUpdate();
			}
		}
	}
	
	/**
	 * Starts report creation
	 */
	public void startReportCreation() {
		
		reportCreator.startReportCreator();
	}
	
	/**
	 * Terminates report creation
	 */
	public void stopReportCreation() {
		
		reportCreator.stopReportCreator();
	}
	
	/**
	 * Returns the video receivers attached to this page
	 * @return list of receivers
	 */
	public ArrayList<String> getVideoReceivers() {
		
		return null;
	}
	
	/**
	 * Returns common device access instance attached to this page 
	 * @return common device access
	 */
	public Web2cCommonDeviceAccess getCommonDeviceAccess() {
		
		return commonDeviceAccess;
	}
	
	/**
	 * Executes requested action
	 * @param action action to be performed
	 * @return data
	 */
	public String doAction(String action) {
		
		String[] str = null;
		String ret = "";
		String command = "";
		String[] parameter = null;
		
		str = action.replace("<do>", "").split("</do><parameter>");
		command = str[0];
		if (str[1].contains("<parameter>")) {
			parameter = str[1].replaceAll("</parameter>", "").split("<parameter>");
		}
		else {
			parameter = new String[1];
			parameter[0] = str[1].replaceAll("</parameter>", "");
		}
		
		
		if (exclusive == false) {
			if (str.length > 1) {
				if (command.equals("getRecord")) ret = logbookRecordWriterAndReader.getRecord(parameter[0], Integer.valueOf(parameter[1]));
				if (command.equals("getFirstRecord"))  ret = logbookRecordWriterAndReader.getFirstRecord(parameter[0], Integer.valueOf(parameter[1]));
				if (command.equals("getMostRecentRecord"))  ret = logbookRecordWriterAndReader.getMostRecentRecord(parameter[0], Integer.valueOf(parameter[1]));
				if (command.equals("getPreviousRecord"))  ret = logbookRecordWriterAndReader.getPreviousRecord(parameter[0], Integer.valueOf(parameter[1]));
				if (command.equals("getNextRecord"))  ret = logbookRecordWriterAndReader.getNextRecord(parameter[0], Integer.valueOf(parameter[1]));
				if (userPrivilege.equals(expert)) {
					if (command.equals("editRecord")) ret = String.valueOf(logbookRecordWriterAndReader.editRecord(parameter[0], Integer.valueOf(parameter[1]), parameter[2], userName));
					if (command.equals("appendRecord")) ret = String.valueOf(logbookRecordWriterAndReader.appendRecord(parameter[0], parameter[1], userName));
					if (command.equals("generateReport")) ret = generateLogbookReport(parameter[0], userName);
				}
				if (userPrivilege.equals(supervisor)) {
					if (command.equals("editRecord")) ret = String.valueOf(logbookRecordWriterAndReader.editRecord(parameter[0], Integer.valueOf(parameter[1]), parameter[2], userName));
					if (command.equals("appendRecord")) ret = String.valueOf(logbookRecordWriterAndReader.appendRecord(parameter[0], parameter[1], userName));
					if (command.equals("deleteRecord")) ret = String.valueOf(logbookRecordWriterAndReader.deleteRecord(parameter[0], Integer.valueOf(parameter[1]), parameter[2]));
					if (command.equals("generateReport")) ret = generateLogbookReport(parameter[0], userName);
				}
				if (command.equals("getReportList")) ret = getLogbookReportList();				
				if (command.equals("getLogbookTree")) ret = logbookRecordWriterAndReader.getLogbookTree(-1);
				if (command.equals("getCategoryList")) ret = getLogbookCategoryList();
				if (command.equals("getImportanceList")) ret = getLogbookImportanceList();
				if (command.equals("getRecipientList")) ret = getLogbookRecipientList();
				if (command.equals("sendNotification")) sendNotification(parameter[0], Integer.valueOf(parameter[1]), parameter[2]);
				ret = "<return>" + ret + "</return>";
			}
		}
		else {
			ret = "<return><return>";
		}
		
		return ret;
	}
	
	/**
	 * Update from the monitor link registerer
	 * @param registerer monitor link registerer
	 * @param status final register status
	 */
	public void update(Observable registerer, Object status) {

		registerError = (String)status;
		registerStatus = "registered";	
	}
	
	/**
	 * Returns a list (as String) of all predefined logbook reports
	 * @return list
	 */
	private String getLogbookReportList() {
		
		String str = "";
		for (Iterator<String> it = reportCaptionList.iterator(); it.hasNext();) {
			str = str + "<report>" + it.next() + "</report>";
		}
		
		return str;
		
	}
	
	/**
	 * Returns a list (as String) of all predefined recipients of notifications
	 * @return list
	 */
	private String getLogbookRecipientList() {
		
		String str = "";
		for (Iterator<String> it = logbookInitFileWriterAndReader.getNotificationRecipientList().iterator(); it.hasNext();) {
			str = str + "<recipient>" + it.next() + "</recipient>";
		}
		
		return str;
		
	}
	
	/**
	 * Returns a list (as String) of all predefined category keywords
	 * @return list
	 */
	private String getLogbookCategoryList() {
		
		String str = "";
		for (Iterator<String> it = categoryList.iterator(); it.hasNext();) {
			str = str + "<category>" + it.next() + "</category>";
		}
		
		return str;
		
	}
	
	/**
	 * Returns a list (as String) of all predefined importance levels
	 * @return list
	 */
	private String getLogbookImportanceList() {
		
		String str = "";
		for (Iterator<String> it = importanceList.iterator(); it.hasNext();) {
			str = str + "<importance>" + it.next() + "</importance>";
		}
		
		return str;
		
	}
	
	/**
	 * Sends a notification 
	 * @param fileName file name
	 * @param index record index
	 * @param notification address
	 */
	private void sendNotification(String fileName, int index, String address) {
		if (smtpInterface.getServiceAvailable() == true) {
			String from = userName;
			String subject = "Message from Web2cLogbook \"" + logbookName + "\"";
			String message = logbookRecordWriterAndReader.getRecord(fileName, index);
			smtpInterface.sendMessage(from, address, subject, message);
		}
	}
	
	/**
	 * Generates a logbook report
	 * @param reportName name of report to be generated
	 * @param author author's name
	 * @return report text (XML)
	 */
	private String generateLogbookReport(String reportName, String author) {
		
		String reportString = "";
		
		if (registerError.equals("")) {
			synchronized (monitorList) {
				synchronized (monitorLinkUpdaterList) {				
					Iterator<ArrayList<String>> it5 = reportRecipientsList.iterator();
					Iterator<ArrayList<String>> it4 = reportTextLinesList.iterator();
					Iterator<String> it3 = reportImportanceList.iterator();
					Iterator<String> it2 = reportCategoryList.iterator();
					for (Iterator<String> it1 = reportCaptionList.iterator(); it1.hasNext();) {
						String caption = (String)(it1.next());
						String category = (String)(it2.next());
						String importance = (String)(it3.next());
						ArrayList<String> textLines = (ArrayList<String>)it4.next();
						ArrayList<String> recipients = (ArrayList<String>)it5.next();
						if (caption.equals(reportName)) {		
							reportString = "<record>";
							reportString = reportString + "<caption>" + caption + "</caption>";
							reportString = reportString + "<category>" + category + "</category>";
							reportString = reportString + "<importance>" + importance + "</importance>";
							reportString = reportString + "<text>";
							for (Iterator<String> it = textLines.iterator(); it.hasNext();) {
								String line = (String)it.next();
								String description = line.split(",")[0];
								String format = line.split(",")[1];
								String uri = line.split(",")[2];
								reportString = reportString + description;
								String ret = monitorLinkUpdaterList.get(monitorList.indexOf(uri)).getMostRecentUpdate();		
								if (format.equals("") || format.equals(" ")) {  //process ret as string as it is
									reportString = reportString + ret + "; ";
								}
								else if (format.contains("x")) {                //fixed-length string, process ret as string
									if (format.length() <= ret.length()) {
										reportString = reportString + ret.substring(0, format.length() - 1) + "; ";
									}
									else {
										reportString = reportString + ret + "; ";
									}
								}
								else {                                          //number format, process ret as number
									DecimalFormat formatter = new DecimalFormat(format.replace("n", "#"), new DecimalFormatSymbols(localSettings));
									reportString = reportString + formatter.format(Double.valueOf(ret)) + "; ";
								}
							}					
							reportString = reportString.substring(0, reportString.length() - 2) + "</text>" + "</record>";					
							logbookRecordWriterAndReader.appendRecord(logbookRecordWriterAndReader.getMostRecentLogbookFile(), reportString, userName);
							if (smtpInterface.getServiceAvailable() == true) {
								String from = userName;
								String subject = "Message from Web2cLogbook \"" + logbookName + "\"";
								String message = reportString;
								for (Iterator<String> it = recipients.iterator(); it.hasNext();) {
									smtpInterface.sendMessage(from, it.next(), subject, message);
								}
							}
						}
					}
				}
			}
		}
		
		return reportString;
	}
	
	/**
	 * Constructs the page
	 */
	private void constructPage() {
		
		try {
			logbookConfigReader = new Web2cLogbookConfigReader(new File(homeDir + File.separator + pageConfigFile));
			logbookName = logbookConfigReader.getCaption();
			logbookPeriode = logbookConfigReader.getPeriode();
			
			expertNameList = logbookConfigReader.getExpertNameList();
			supervisorNameList = logbookConfigReader.getSupervisorNameList();
			exclusiveNameList = logbookConfigReader.getExclusiveNameList();
			expertAddressList = logbookConfigReader.getExpertAddressList();
			categoryList = logbookConfigReader.getCategoryKeywordList();
			importanceList = logbookConfigReader.getImportanceLevelList();
			
			reportCaptionList = logbookConfigReader.getReportCaptionList();
			reportCategoryList = logbookConfigReader.getReportCategoryList();
			reportImportanceList = logbookConfigReader.getReportImportanceList();
			reportModeList = logbookConfigReader.getReportModeList();
			reportConditionList = logbookConfigReader.getReportConditionList();
			reportRecipientsList = logbookConfigReader.getReportRecipientsList();
			reportTextLinesList = logbookConfigReader.getReportTextLinesList();
			
			logbookRecordWriterAndReader = new Web2cLogbookRecordWriterAndReader(logbookName, logbookPeriode, logbookDir);

			if (registerStatus.equals("idle")) {
				registerStatus = "registering";	
				monitorlinkRegisterer = new MonitorLinkRegisterer(logbookConfigReader.getMonitorList());
				monitorlinkRegisterer.addObserver(this);
			}
			else if (registerStatus.equals("registered")) {
				registerStatus = "idle";
				registerError = "";
			}
		}
		catch (Exception e) {}
		
		exclusive = false;
		if (exclusiveNameList.isEmpty()) {
			exclusive = false;
		}
		else {
			exclusive = true;
			for (Iterator<String> it = exclusiveNameList.iterator(); it.hasNext();) {
				String item = it.next().toString();
				if (item.equals(userName)) {
					exclusive = false;
				}
			}
		}

		networkPrivilege = layman;
		if (expertAddressList.isEmpty()) {
			networkPrivilege = layman;
		}
		else {
			for (Iterator<String> it = expertAddressList.iterator(); it.hasNext();) {
				String nextAddress = it.next().toString();
				if (nextAddress.replace(".", ">").split(">")[3].equals("*") || nextAddress.replace(".", ">").split(">")[3].equals(userAddress.replace(".", ">").split(">")[3])) {
					if (nextAddress.replace(".", ">").split(">")[2].equals("*") || nextAddress.replace(".", ">").split(">")[2].equals(userAddress.replace(".", ">").split(">")[2])) {
						if (nextAddress.replace(".", ">").split(">")[1].equals("*") || nextAddress.replace(".", ">").split(">")[1].equals(userAddress.replace(".", ">").split(">")[1])) {
							if (nextAddress.replace(".", ">").split(">")[0].equals("*") || nextAddress.replace(".", ">").split(">")[0].equals(userAddress.replace(".", ">").split(">")[0])) {
								networkPrivilege = expert;
							}
						}
					}
				}
			}
		}
		
		if (userAddress.equals("*.*.*.*")) {
			userPrivilege = supervisor;
		}
		else {
			userPrivilege = layman;
			if (expertNameList.isEmpty() && supervisorNameList.isEmpty()) {
				userPrivilege = layman;
			}
			if (!expertNameList.isEmpty()) {
				for (Iterator<String> it = expertNameList.iterator(); it.hasNext();) {
					String item = it.next().toString();;
					if (item.indexOf("(") < 0) {
						if (item.equals(userName)) {
							userPrivilege = expert;
						}
					}
					else {
						String name = item.substring(0, item.indexOf("("));
						String address = item.substring(item.indexOf("(") + 1, item.indexOf(")"));
						if (name.equals(userName)) {
							if (address.replace(".", ">").split(">")[3].equals("*") || address.replace(".", ">").split(">")[3].equals(userAddress.replace(".", ">").split(">")[3])) {
								if (address.replace(".", ">").split(">")[2].equals("*") || address.replace(".", ">").split(">")[2].equals(userAddress.replace(".", ">").split(">")[2])) {
									if (address.replace(".", ">").split(">")[1].equals("*") || address.replace(".", ">").split(">")[1].equals(userAddress.replace(".", ">").split(">")[1])) {
										if (address.replace(".", ">").split(">")[0].equals("*") || address.replace(".", ">").split(">")[0].equals(userAddress.replace(".", ">").split(">")[0])) {
											networkPrivilege = expert;
											userPrivilege = expert;
										}
										else {
											networkPrivilege = layman;
											userPrivilege = layman;
										}
									}
								}
							}
						}
					}
				}		
			}
			if (!supervisorNameList.isEmpty()) {
				for (Iterator<String> it = supervisorNameList.iterator(); it.hasNext();) {
					String item = it.next().toString();;
					if (item.indexOf("(") < 0) {
						if (item.equals(userName)) {
							userPrivilege = supervisor;
						}
					}
					else {
						String name = item.substring(0, item.indexOf("("));
						String address = item.substring(item.indexOf("(") + 1, item.indexOf(")"));
						if (name.equals(userName)) {
							if (address.replace(".", ">").split(">")[3].equals("*") || address.replace(".", ">").split(">")[3].equals(userAddress.replace(".", ">").split(">")[3])) {
								if (address.replace(".", ">").split(">")[2].equals("*") || address.replace(".", ">").split(">")[2].equals(userAddress.replace(".", ">").split(">")[2])) {
									if (address.replace(".", ">").split(">")[1].equals("*") || address.replace(".", ">").split(">")[1].equals(userAddress.replace(".", ">").split(">")[1])) {
										if (address.replace(".", ">").split(">")[0].equals("*") || address.replace(".", ">").split(">")[0].equals(userAddress.replace(".", ">").split(">")[0])) {
											networkPrivilege = expert;
											userPrivilege = supervisor;
										}
										else {
											networkPrivilege = layman;
											userPrivilege = layman;
										}
									}
								}
							}
						}
					}
				}
			}
			
			if (userPrivilege == expert && networkPrivilege == expert) {
				userPrivilege = expert;
			}
			else if (userPrivilege == supervisor && networkPrivilege == expert) {
				userPrivilege = supervisor;
			}
			else {
				userPrivilege = layman;
			}
		}	
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class MonitorLinkRegisterer extends Observable implements Runnable {

		private Thread registerer = null;
		private ArrayList<String> monitorLinkList = null;
		private String error = "";
		
		/**	
		 * @param list list of monitor links to be registered
		 */
		public MonitorLinkRegisterer (ArrayList<String> list) {
			
			super();
			
			monitorLinkList = list;
		
			if (registerer == null) {
				registerer = new Thread(this);
				registerer.start();
			}
		}
		
		/**
		 * Registers monitor links
		 */
		public void run() {	

					error = "";
					double index = 1.0;
					for (Iterator<String> it = monitorLinkList.iterator(); it.hasNext();) {
						String prop = (String)(it.next());
						String linkError = "";
						if (prop.split(":")[0].equalsIgnoreCase("tine") || prop.split(":")[0].equalsIgnoreCase("tango") || prop.split(":")[0].equalsIgnoreCase("epics") || prop.split(":")[0].equalsIgnoreCase("doocs")) {
							commonDeviceAccess.registerMonitor(prop);
							linkError = commonDeviceAccess.getError();
							if (linkError.equals("")) {
								monitorList.add(prop);
								monitorLinkUpdaterList.add(new MonitorLinkUpdater(prop));
							}
							error = error + linkError;
							constructionProgress[0] = (index / monitorLinkList.size()) * 100.0;
							index = index + 1.0;
						}
					}
			
			if (error.equals("")) {
				reportCreator = new ReportCreator();
				reportCreator.startReportCreator();
			}
			
			setChanged();
			notifyObservers(error);
			
			registerer.interrupt();
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class MonitorLinkUpdater implements Runnable {

		private Thread updater = null;
		boolean updaterStarted = false;
		private String monitorLinkAddress = "";
		private String update = null;
		
		/**	
		 * @param address address of monitor link to be updated
		 */
		public MonitorLinkUpdater (String address) {
			
			super();
			
			monitorLinkAddress = address;
		
			if (updater == null) {
				updater = new Thread(this);
				updaterStarted = true;
				updater.start();
			}
		}
		
		/**
		 * Updates requested monitor link
		 */
		public void run() {	
			
			while (updaterStarted == true) {
				update = characterEscaper.decodeString((commonDeviceAccess.getMonitorData(monitorLinkAddress).split(">")[1].split("<")[0]));	
				try {Thread.sleep(logbookUpdateInterval * 1000);}
				catch (InterruptedException e) {}
			}
			
			updater.interrupt();
		}
		
		/**
		 * Returns the most recent update
		 * @return update
		 */
		public String getMostRecentUpdate() {
			
			return update;
		}
		
		/**
		 * Terminates updating cache data
		 */
		public void stopUpdate() {
			
			updaterStarted = false;
		}
		
		/**
		 * Returns the updater status
		 * @return updater status
		 */
		public boolean getUpdaterStatus() {
			
			return updaterStarted;
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class ReportCreator implements Runnable {

		private Thread creator = null;
		boolean creatorStarted = false;
		ArrayList<Boolean> conditionFulfilledList = new ArrayList<Boolean>();
		
		/**	Creates reports if trigger conditions fulfilled
		 */
		public ReportCreator() {
			
			super();
		
			for (int i = 0; i < reportCaptionList.size(); i++) conditionFulfilledList.add(false);
			creatorStarted = false;
			if (creator == null) {
				creator = new Thread(this);
			}
		}
		
		/**
		 * Creates reports
		 */
		public void run() {	
			
			while (creatorStarted == true) {

				int index = 0;
				Iterator<Boolean> it4 = conditionFulfilledList.iterator();
				Iterator<String> it3 = reportCaptionList.iterator();
				Iterator<String> it2 = reportConditionList.iterator();
				for (Iterator<String> it1 = reportModeList.iterator(); it1.hasNext();) {
					String mode = (String)(it1.next());
					String condition = (String)(it2.next());
					String conditionUri = condition.split("=")[0];
					String conditionValue = condition.split("=")[1];
					String reportName = (String)(it3.next());
					boolean fulFilled = (boolean)(it4.next());
					if (mode.equals("auto")) {
						if (conditionValue.equals(monitorLinkUpdaterList.get(monitorList.indexOf(conditionUri)).getMostRecentUpdate())) {
							if (fulFilled == false) {
								generateLogbookReport(reportName, "Web2cLogbook Report Creator");
								fulFilled = true;
							}
						}
						else {
							fulFilled = false;
						}
					}
					conditionFulfilledList.set(index, fulFilled);
					index = index + 1;
				}
				
				try {Thread.sleep(logbookUpdateInterval * 1000);}
				catch (InterruptedException e) {}
			}
			
			creator.interrupt();
		}
		
		/**
		 * Start report creation
		 */
		public void startReportCreator() {
			
			if (creator == null) {
				creator = new Thread(this);
			}
			creatorStarted = true;
			creator.start();
		}
		
		/**
		 * Terminates report creation
		 */
		public void stopReportCreator() {
			
			creatorStarted = false;
		}
		
		/**
		 * Returns the report creator status
		 * @return report creator status
		 */
		public boolean getReportCreatorStatus() {
			
			return creatorStarted;
		}
	}
}
