package de.desy.web2cToolkit.web2cLogbook.web2cGWT.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * Logbook Viewer GWT Page Remote Service Interface (Client)
 * @author R.Bacher, DESY
 */
@RemoteServiceRelativePath("Web2cLogbookViewerRemoteService")
public interface IWeb2cLogbookViewerRemoteService extends RemoteService {
	
	/**
	 * Utility class for simplifying access to the instance of async service.
	 */
	public static class Util {
		private static IWeb2cLogbookViewerRemoteServiceAsync instance;
		public static IWeb2cLogbookViewerRemoteServiceAsync getInstance(){
			if (instance == null) {
				instance = GWT.create(IWeb2cLogbookViewerRemoteService.class);
			}
			return instance;
		}
	}
	
	/**
	 * Returns the logbook record with the index specified
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record requested
	 * @return record (XML)
	 */
	public String getRecord(String sessionId, String fileName, String recordIndex);
	
	/**
	 * Returns the first logbook record
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record requested
	 * @return record (XML)
	 */
	public String getFirstRecord(String sessionId, String fileName, String recordIndex);
	
	/**
	 * Returns the previous logbook record
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record currently displayed
	 * @return record (XML)
	 */
	public String getPreviousRecord(String sessionId, String fileName, String recordIndex);
	
	/**
	 * Returns the next logbook record
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record currently displayed
	 * @return record (XML)
	 */
	public String getNextRecord(String sessionId, String fileName, String recordIndex);
	
	/**
	 * Returns the most recent logbook record
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record requested
	 * @return record (XML)
	 */
	public String getMostRecentRecord(String sessionId, String fileName, String recordIndex);
	
	/**
	 * Edits and saves the logbook record with the index specified
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record to be edited
	 * @param record record (XML)
	 * @return index of recorded edited
	 */
	public String editRecord(String sessionId, String fileName, String recordIndex, String record);
	
	/**
	 * Appends and saves a new logbook record
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param record record (XML)
	 * @return index of recorded appended
	 */
	public String appendRecord(String sessionId, String fileName, String record);
	
	/**
	 * Delets the logbook record with the index specified
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record to be deleted
	 * @param record record (XML)
	 * @return index of recorded deleted
	 */
	public String deleteRecord(String sessionId, String fileName, String recordIndex, String record);
	
	/**
	 * Sends a notification the address specified
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record to be sent
	 * @param address address
	 */
	public void sendNotification(String sessionId, String fileName, String recordIndex, String address);
	
	/**
	 * Returns the logbook tree
	 * @param sessionId id of associated Web2c session
	 * @return tree (XML)
	 */
	public String getLogbookTree(String sessionId);
	
	/**
	 * Returns the list of recipients
	 * @param sessionId id of associated Web2c session
	 * @return list (XML)
	 */
	public String getRecipients(String sessionId);
	
	/**
	 * Returns the list of category keywords
	 * @param sessionId id of associated Web2c session
	 * @return list (XML)
	 */
	public String getCategoryKeywords(String sessionId);
	
	/**
	 * Returns the list of importance levels
	 * @param sessionId id of associated Web2c session
	 * @return list (XML)
	 */
	public String getImportanceLevels(String sessionId);	
	
	/**
	 * Returns the list of available levels
	 * @param sessionId id of associated Web2c session
	 * @return list (XML)
	 */
	public String getReportList(String sessionId);
	
	/**
	 * Returns the report text
	 * @param sessionId id of associated Web2c session
	 * @param reportName name of report to be written
	 * @return report (XML)
	 */
	public String writeReport(String sessionId, String reportName);
}
