package de.desy.web2cToolkit.web2cLogbook.web2cGWT.client;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * Logbook Viewer GWT Page Remote Service Interface (Client)
 * @author R.Bacher, DESY
 */
public interface IWeb2cLogbookViewerRemoteServiceAsync {
	
	/**
	 * Returns the logbook record with the index specified
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record requested
	 * @param callback the callback to return record (XML)
	 */
	public void getRecord(String sessionId, String fileName, String recordIndex, AsyncCallback<String> callback);
	
	/**
	 * Returns the first logbook record
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record requested
	 * @param callback the callback to return record (XML)
	 */
	public void getFirstRecord(String sessionId, String fileName, String recordIndex, AsyncCallback<String> callback);
	
	/**
	 * Returns the previous logbook record
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record currently displayed
	 * @param callback the callback to return record (XML)
	 */
	public void getPreviousRecord(String sessionId, String fileName, String recordIndex, AsyncCallback<String> callback);
	
	/**
	 * Returns the next logbook record
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record currently displayed
	 * @param callback the callback to return record (XML)
	 */
	public void getNextRecord(String sessionId, String fileName, String recordIndex, AsyncCallback<String> callback);
	
	/**
	 * Returns the most recent logbook record
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record requested
	 * @param callback the callback to return record (XML)
	 */
	public void getMostRecentRecord(String sessionId, String fileName, String recordIndex, AsyncCallback<String> callback);
	
	/**
	 * Edits and saves the logbook record with the index specified
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record to be edited
	 * @param record record (XML)
	 * @param callback the callback to return index of recorded edited
	 */
	public void editRecord(String sessionId, String fileName, String recordIndex, String record, AsyncCallback<String> callback);
	
	/**
	 * Appends and saves a new logbook record
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param record record (XML)
	 * @param callback the callback to return index of recorded appended
	 */
	public void appendRecord(String sessionId, String fileName, String record, AsyncCallback<String> callback);
	
	/**
	 * Delets the logbook record with the index specified
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record to be deleted
	 * @param record record (XML)
	 * @param callback the callback to return index of recorded deleted
	 */
	public void deleteRecord(String sessionId, String fileName, String recordIndex, String record, AsyncCallback<String> callback);
	
	/**
	 * Sends a notification the address specified
	 * @param sessionId id of associated Web2c session
	 * @param fileName fileName containing the record
	 * @param recordIndex index of record to be sent
	 * @param address address
	 */
	public void sendNotification(String sessionId, String fileName, String recordIndex, String address, AsyncCallback<Void> callback);
	
	/**
	 * Returns the logbook tree
	 * @param sessionId id of associated Web2c session
	 * @param callback the callback to return tree (XML)
	 */
	public void getLogbookTree(String sessionId, AsyncCallback<String> callback);
	
	/**
	 * Returns the list of recipients
	 * @param sessionId id of associated Web2c session
	 * @param callback the callback to return list (XML)
	 */
	public void getRecipients(String sessionId, AsyncCallback<String> callback);
	
	/**
	 * Returns the list of category keywords
	 * @param sessionId id of associated Web2c session
	 * @param callback the callback to return list (XML)
	 */
	public void getCategoryKeywords(String sessionId, AsyncCallback<String> callback);
	
	/**
	 * Returns the list of importance levels
	 * @param sessionId id of associated Web2c session
	 * @param callback the callback to return list (XML)
	 */
	public void getImportanceLevels(String sessionId, AsyncCallback<String> callback);	
	
	/**
	 * Returns the list of available levels
	 * @param sessionId id of associated Web2c session
	 * @param callback the callback to return list (XML)
	 */
	public void getReportList(String sessionId, AsyncCallback<String> callback);
	
	/**
	 * Returns the report text
	 * @param sessionId id of associated Web2c session
	 * @param reportName name of report to be written
	 * @param callback the callback to return report (XML)
	 */
	public void writeReport(String sessionId, String reportName, AsyncCallback<String> callback);
}
