package de.desy.web2cToolkit.web2cManager.web2cGWT.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * Manager GWT Page Remote Service Interface (Client)
 * @author R.Bacher, DESY
 */
@RemoteServiceRelativePath("Web2cManagerRemoteService")
public interface IWeb2cManagerRemoteService extends RemoteService {
	
	/**
	 * Utility class for simplifying access to the instance of async service.
	 */
	public static class Util {
		private static IWeb2cManagerRemoteServiceAsync instance;
		public static IWeb2cManagerRemoteServiceAsync getInstance(){
			if (instance == null) {
				instance = GWT.create(IWeb2cManagerRemoteService.class);
			}
			return instance;
		}
	}
	
	/**
	 * Returns the names of the implemented servlets
	 * @return servlet name
	 */
	public String[] getServletNames();
	
	/**
	 * Returns the URL of the servlet specified
	 * @param name servlet name
	 * @param sessionId Web2cManagerPage sessionId
	 * @param hostAndPort host name and port number (i.e. host:xxxx)
	 * @return servlet URL
	 */
	public String getServletDiagnosticsPage(String name, String sessionId, String hostAndPort);
	
	/**
	 * Returns the config file name list
	 * @return list
	 */
	public String[] getAllConfigFileNames();
	
	/**
	 * Returns the list of files to be scanned for early link registering
	 * @return file list
	 */
	public String[] getEarlyRegisteringFiles();
	
	/**
	 * Saves the toolkit initialization parameters
	 * @param text salutation text
	 * @param list early-registering files
	 */
	public void saveToolkitInitializationParameters(String text, String[] list);
	
	/**
	 * Returns the salutation text
	 * @return salutation text
	 */
	public String getSalutationText();
	
	/**
	 * Returns the user names stored in the user registry
	 * @return list of user name
	 */
	public String[] getUserNames();
	
	/**
	 * Returns the password belonging to the user name specified
	 * @param text user name
	 * @return password
	 */
	public String getUserPassword(String text);
	
	/**
	 * Saves user name / user password in the user registry
	 * @param name user name
	 * @param password user password
	 * @return modified list of user names
	 */
	public String[] saveUserRegistry(String name, String password);
	
	/**
	 * Removes a user entry in the user registry
	 * @param name user name
	 * @return modified list of user names
	 */
	public String[] removeRegistryEntry(String name);
	
	/**
	 * Sets the Twitter id
	 * @param text id
	 */
	public void setTwitterId(String text);
	
	/**
	 * Saves the Twitter id
	 * @param text id
	 */
	public void saveTwitterId(String text);
	
	/**
	 * Returns the Twitter id
	 * @return id
	 */
	public String getTwitterId();
	
	/**
	 * Sets the Twitter password
	 * @param text password
	 */
	public void setTwitterPassword(String text);
	
	/**
	 * Saves the Twitter password
	 * @param text password
	 */
	public void saveTwitterPassword(String text);
	
	/**
	 * Returns the Twitter password
	 * @return password
	 */
	public String getTwitterPassword();
	
	/**
	 * Sets the Twitter logging-option
	 * @param text option (yes, no)
	 */
	public void setTwitterLoggingOption(String text);
	
	/**
	 * Saves the Twitter logging-option
	 * @param text option (yes, no)
	 */
	public void saveTwitterLoggingOption(String text);
	
	/**
	 * Returns the Twitter logging-option
	 * @return option
	 */
	public String getTwitterLoggingOption();
	
	/**
	 * Does a twitter permit request
	 */
	public String doRequestTwitterPermit();
	
	/**
	 * Returns the most recently published twitter status
	 */
	public String doGetTwitterStatus();
	
	/**
	 * Starts publishing a twitter status
	 */
	public String doStartPublishingTwitterStatus();
	
	/**
	 * Stops publishing a twitter status
	 */
	public String doStopPublishingTwitterStatus();
	
	/**
	 * Returns the availability status of the twitter service
	 * @return status
	 */
	public Boolean getTwitterServiceStatus();
	
	/**
	 * Returns the publishing status of the twitter service
	 * @return status
	 */
	public Boolean getTwitterPublishingStatus();	
	
	/**
	 * Returns the most recently published twitter status
	 * @return tweet
	 */
	public String getMostRecentTwitterStatus();
	
	/**
	 * Arms the twitter service
	 */
	public void doArmTwitterService();
	
	/**
	 * Disarms the twitter service
	 */
	public void doDisarmTwitterService();
	
	/**
	 * Sets the SMTP id
	 * @param text id
	 */
	public void setSMTPId(String text);
	
	/**
	 * Saves the SMTP id
	 * @param text id
	 */
	public void saveSMTPId(String text);
	
	/**
	 * Returns the SMTP id
	 * @return id
	 */
	public String getSMTPId();
	
	/**
	 * Sets the SMTP password
	 * @param text password
	 */
	public void setSMTPPassword(String text);
	
	/**
	 * Saves the SMTP password
	 * @param text password
	 */
	public void saveSMTPPassword(String text);
	
	/**
	 * Returns the SMTP password
	 * @return password
	 */
	public String getSMTPPassword();
	
	/**
	 * Sets the SMTP host
	 * @param text host
	 */
	public void setSMTPHost(String text);
	
	/**
	 * Saves the SMTP host
	 * @param text host
	 */
	public void saveSMTPHost(String text);
	
	/**
	 * Returns the SMTP host
	 * @return host
	 */
	public String getSMTPHost();
	
	/**
	 * Sets the SMTP logging-option
	 * @param text option (yes, no)
	 */
	public void setSMTPLoggingOption(String text);
	
	/**
	 * Saves the SMTP logging-option
	 * @param text option (yes, no)
	 */
	public void saveSMTPLoggingOption(String text);
	
	/**
	 * Returns the SMTP logging-option
	 * @return option
	 */
	public String getSMTPLoggingOption();
	
	/**
	 * Arms the SMTP service
	 */
	public void doArmSMTPService();
	
	/**
	 * Disarms the SMTP service
	 */
	public void doDisarmSMTPService();
	
	/**
	 * Returns the availability status of the SMTP service
	 * @return status
	 */
	public Boolean getSMTPServiceStatus();
	
	/**
	 * Returns the message sending status of the SMTP service
	 * @return status
	 */
	public String getSMTPMessageSendingStatus();
	
	/**
	 * Sets the web communication debug flag
	 * @param text option (yes, no)
	 */
	public void setWebCommunicationDebugFlag(String text);
	
	/**
	 * Sets the control system communication debug flag
	 * @param text option (yes, no)
	 */
	public void setControlSystemCommunicationDebugFlag(String text);	
	
	/**
	 * Sets the configuration file parsing debug flag
	 * @param text option (yes, no)
	 */
	public void setConfigurationFileParsingDebugFlag(String text);
	
	/**
	 * Modifies and saves the toolkit init file entry (name / type)
	 * @param name file name
	 * @param type file type
	 */
	public void saveToolkitInitFile(String name, String type);
	
	/**
	 * Returns the implemented toolkit init file types
	 * @return file types
	 */	
	public String[] getInitFileTypes();
	
	/**
	 * Returns the init file name associated with the file type specified
	 * @param type file type
	 * @return file name
	 */
	public String getInitFileName(String type);	
	
	/**
	 * Returns the log file name list
	 * @return list
	 */
	public String[] getAllLogFileNames();
	
	/**
	 * Clears the log file specified
	 * @param file name
	 */
	public void clearLogFile(String name);
	
	/**
	 * Returns the log file specified
	 * @param file name
	 * @param numberOfLines number of lines to be returned
	 * @return file content
	 */
	public String[] getLogFile(String name, String numberOfLines);
	
	/**
	 * Sets the Java garbage collector enabled/disabled flag
	 * @param flag
	 */
	public void setJavaGarbageCollection(String flag);
	
	/**
	 * Returns the Java garbage collector enabled/disabled flag
	 * @return flag
	 */
	public String getJavaGarbageCollection();
	
	/**
	 * Perform Java garbage collection
	 */
	public void doJavaGarbageCollection();
	
	/**
	 * Sets the Web2cToolkit garbage collector enabled/disabled flag
	 * @param flag
	 */
	public void setWeb2cToolkitGarbageCollection(String flag);
	
	/**
	 * Returns the Web2cToolkit garbage collector enabled/disabled flag
	 * @return flag
	 */
	public String getWeb2cToolkitGarbageCollection();
	
	/**
	 * Returns the JVM's heap space usage [%]
	 * @param sessionId Web2cManagerPage sessionId
	 * @return percentaged heap space usage
	 */
	public String getPercentagedHeapSpaceUsage(String sessionId);
	
	/**
	 * Returns the JVM's cpu usage [%]
	 * @param sessionId Web2cManagerPage sessionId
	 * @return percentaged cpu usage
	 */
	public String getPercentagedCPUUsage(String sessionId);
	
	/**
	 * Returns the number of classes loaded by the JVM
	 * @param sessionId Web2cManagerPage sessionId
	 * @return loaded class count
	 */
	public String getLoadedClassCount(String sessionId);
}
