package de.desy.web2cToolkit.web2cManager.web2cGWT.client;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * Manager GWT Page Remote Service Interface (Client)
 * @author R.Bacher, DESY
 */
public interface IWeb2cManagerRemoteServiceAsync {
	
	/**
	 * Returns the names of the implemented servlets
	 * @param callback the callback to return servlet name
	 */
	public void getServletNames(AsyncCallback<String[]> callback);
	
	/**
	 * Returns the URL of the servlet specified
	 * @param name servlet name
	 * @param sessionId Web2cManagerPage sessionId
	 * @param hostAndPort host name and port number (i.e. host:xxxx)
	 * @param callback the callback to return servlet URL
	 */
	public void getServletDiagnosticsPage(String name, String sessionId, String hostAndPort, AsyncCallback<String> callback);
	
	/**
	 * Returns the config file name list
	 * @param callback the callback to return list
	 */
	public void getAllConfigFileNames(AsyncCallback<String[]> callback);
	
	/**
	 * Returns the list of files to be scanned for early link registering
	 * @param callback the callback to return file list
	 */
	public void getEarlyRegisteringFiles(AsyncCallback<String[]> callback);
	
	/**
	 * Saves the toolkit initialization parameters
	 * @param text salutation text
	 * @param list early-registering files
	 */
	public void saveToolkitInitializationParameters(String text, String[] list, AsyncCallback<Void> callback);
	
	/**
	 * Returns the salutation text
	 * @param callback the callback to return salutation text
	 */
	public void getSalutationText(AsyncCallback<String> callback);
	
	/**
	 * Returns the user names stored in the user registry
	 * @param callback the callback to return list of user name
	 */
	public void getUserNames(AsyncCallback<String[]> callback);
	
	/**
	 * Returns the password belonging to the user name specified
	 * @param text user name
	 * @param callback the callback to return password
	 */
	public void getUserPassword(String text, AsyncCallback<String> callback);
	
	/**
	 * Saves user name / user password in the user registry
	 * @param name user name
	 * @param password user password
	 * @param callback the callback to return modified list of user names
	 */
	public void saveUserRegistry(String name, String password, AsyncCallback<String[]> callback);
	
	/**
	 * Removes a user entry in the user registry
	 * @param name user name
	 * @param callback the callback to return modified list of user names
	 */
	public void removeRegistryEntry(String name, AsyncCallback<String[]> callback);
	
	/**
	 * Sets the Twitter id
	 * @param text id
	 */
	public void setTwitterId(String text, AsyncCallback<Void> callback);
	
	/**
	 * Saves the Twitter id
	 * @param text id
	 */
	public void saveTwitterId(String text, AsyncCallback<Void> callback);
	
	/**
	 * Returns the Twitter id
	 * @param callback the callback to return id
	 */
	public void getTwitterId(AsyncCallback<String> callback);
	
	/**
	 * Sets the Twitter password
	 * @param text password
	 */
	public void setTwitterPassword(String text, AsyncCallback<Void> callback);
	
	/**
	 * Saves the Twitter password
	 * @param text password
	 */
	public void saveTwitterPassword(String text, AsyncCallback<Void> callback);
	
	/**
	 * Returns the Twitter password
	 * @param callback the callback to return password
	 */
	public void getTwitterPassword(AsyncCallback<String> callback);
	
	/**
	 * Sets the Twitter logging-option
	 * @param text option (yes, no)
	 */
	public void setTwitterLoggingOption(String text, AsyncCallback<Void> callback);
	
	/**
	 * Saves the Twitter logging-option
	 * @param text option (yes, no)
	 */
	public void saveTwitterLoggingOption(String text, AsyncCallback<Void> callback);
	
	/**
	 * Returns the Twitter logging-option
	 * @param callback the callback to return option
	 */
	public void getTwitterLoggingOption(AsyncCallback<String> callback);
	
	/**
	 * Does a twitter permit request
	 */
	public void doRequestTwitterPermit(AsyncCallback<String> callback);
	
	/**
	 * Returns the most recently published twitter status
	 */
	public void doGetTwitterStatus(AsyncCallback<String> callback);
	
	/**
	 * Starts publishing a twitter status
	 */
	public void doStartPublishingTwitterStatus(AsyncCallback<String> callback);
	
	/**
	 * Stops publishing a twitter status
	 */
	public void doStopPublishingTwitterStatus(AsyncCallback<String> callback);
	
	/**
	 * Returns the availability status of the twitter service
	 * @param callback the callback to return status
	 */
	public void getTwitterServiceStatus(AsyncCallback<Boolean> callback);
	
	/**
	 * Returns the publishing status of the twitter service
	 * @param callback the callback to return status
	 */
	public void getTwitterPublishingStatus(AsyncCallback<Boolean> callback);	
	
	/**
	 * Returns the most recently published twitter status
	 * @param callback the callback to return tweet
	 */
	public void getMostRecentTwitterStatus(AsyncCallback<String> callback);
	
	/**
	 * Arms the twitter service
	 */
	public void doArmTwitterService(AsyncCallback<Void> callback);
	
	/**
	 * Disarms the twitter service
	 */
	public void doDisarmTwitterService(AsyncCallback<Void> callback);
	
	/**
	 * Sets the SMTP id
	 * @param text id
	 */
	public void setSMTPId(String text, AsyncCallback<Void> callback);
	
	/**
	 * Saves the SMTP id
	 * @param text id
	 */
	public void saveSMTPId(String text, AsyncCallback<Void> callback);
	
	/**
	 * Returns the SMTP id
	 * @param callback the callback to return id
	 */
	public void getSMTPId(AsyncCallback<String> callback);
	
	/**
	 * Sets the SMTP password
	 * @param text password
	 */
	public void setSMTPPassword(String text, AsyncCallback<Void> callback);
	
	/**
	 * Saves the SMTP password
	 * @param text password
	 */
	public void saveSMTPPassword(String text, AsyncCallback<Void> callback);
	
	/**
	 * Returns the SMTP password
	 * @param callback the callback to return password
	 */
	public void getSMTPPassword(AsyncCallback<String> callback);
	
	/**
	 * Sets the SMTP host
	 * @param text host
	 */
	public void setSMTPHost(String text, AsyncCallback<Void> callback);
	
	/**
	 * Saves the SMTP host
	 * @param text host
	 */
	public void saveSMTPHost(String text, AsyncCallback<Void> callback);
	
	/**
	 * Returns the SMTP host
	 * @param callback the callback to return host
	 */
	public void getSMTPHost(AsyncCallback<String> callback);
	
	/**
	 * Sets the SMTP logging-option
	 * @param text option (yes, no)
	 */
	public void setSMTPLoggingOption(String text, AsyncCallback<Void> callback);
	
	/**
	 * Saves the SMTP logging-option
	 * @param text option (yes, no)
	 */
	public void saveSMTPLoggingOption(String text, AsyncCallback<Void> callback);
	
	/**
	 * Returns the SMTP logging-option
	 * @param callback the callback to return option
	 */
	public void getSMTPLoggingOption(AsyncCallback<String> callback);
	
	/**
	 * Arms the SMTP service
	 */
	public void doArmSMTPService(AsyncCallback<Void> callback);
	
	/**
	 * Disarms the SMTP service
	 */
	public void doDisarmSMTPService(AsyncCallback<Void> callback);
	
	/**
	 * Returns the availability status of the SMTP service
	 * @param callback the callback to return status
	 */
	public void getSMTPServiceStatus(AsyncCallback<Boolean> callback);
	
	/**
	 * Returns the message sending status of the SMTP service
	 * @param callback the callback to return status
	 */
	public void getSMTPMessageSendingStatus(AsyncCallback<String> callback);
	
	/**
	 * Sets the web communication debug flag
	 * @param text option (yes, no)
	 */
	public void setWebCommunicationDebugFlag(String text, AsyncCallback<Void> callback);
	
	/**
	 * Sets the control system communication debug flag
	 * @param text option (yes, no)
	 */
	public void setControlSystemCommunicationDebugFlag(String text, AsyncCallback<Void> callback);	
	
	/**
	 * Sets the configuration file parsing debug flag
	 * @param text option (yes, no)
	 */
	public void setConfigurationFileParsingDebugFlag(String text, AsyncCallback<Void> callback);
	
	/**
	 * Modifies and saves the toolkit init file entry (name / type)
	 * @param name file name
	 * @param type file type
	 */
	public void saveToolkitInitFile(String name, String type, AsyncCallback<Void> callback);
	
	/**
	 * Returns the implemented toolkit init file types
	 * @param callback the callback to return file types
	 */	
	public void getInitFileTypes(AsyncCallback<String[]> callback);
	
	/**
	 * Returns the init file name associated with the file type specified
	 * @param type file type
	 * @param callback the callback to return file name
	 */
	public void getInitFileName(String type, AsyncCallback<String> callback);	
	
	/**
	 * Returns the log file name list
	 * @param callback the callback to return list
	 */
	public void getAllLogFileNames(AsyncCallback<String[]> callback);
	
	/**
	 * Clears the log file specified
	 * @param file name
	 */
	public void clearLogFile(String name, AsyncCallback<Void> callback);
	
	/**
	 * Returns the log file specified
	 * @param file name
	 * @param numberOfLines number of lines to be returned
	 * @param callback the callback to return file content
	 */
	public void getLogFile(String name, String numberOfLines, AsyncCallback<String[]> callback);
	
	/**
	 * Sets the Java garbage collector enabled/disabled flag
	 * @param flag
	 */
	public void setJavaGarbageCollection(String flag, AsyncCallback<Void> callback);
	
	/**
	 * Returns the Java garbage collector enabled/disabled flag
	 * @param callback the callback to return flag
	 */
	public void getJavaGarbageCollection(AsyncCallback<String> callback);
	
	/**
	 * Perform Java garbage collection
	 */
	public void doJavaGarbageCollection(AsyncCallback<Void> callback);
	
	/**
	 * Sets the Web2cToolkit garbage collector enabled/disabled flag
	 * @param flag
	 */
	public void setWeb2cToolkitGarbageCollection(String flag, AsyncCallback<Void> callback);
	
	/**
	 * Returns the Web2cToolkit garbage collector enabled/disabled flag
	 * @param callback the callback to return flag
	 */
	public void getWeb2cToolkitGarbageCollection(AsyncCallback<String> callback);
	
	/**
	 * Returns the JVM's heap space usage [%]
	 * @param sessionId Web2cManagerPage sessionId
	 * @param callback the callback to return percentaged heap space usage
	 */
	public void getPercentagedHeapSpaceUsage(String sessionId, AsyncCallback<String> callback);
	
	/**
	 * Returns the JVM's cpu usage [%]
	 * @param sessionId Web2cManagerPage sessionId
	 * @param callback the callback to return percentaged cpu usage
	 */
	public void getPercentagedCPUUsage(String sessionId, AsyncCallback<String> callback);
	
	/**
	 * Returns the number of classes loaded by the JVM
	 * @param sessionId Web2cManagerPage sessionId
	 * @param callback the callback to return loaded class count
	 */
	public void getLoadedClassCount(String sessionId, AsyncCallback<String> callback);
}
