package de.desy.web2cToolkit.web2cManager.web2cGWT.server;

import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import de.desy.web2cToolkit.web2cManager.web2cGWT.client.IWeb2cManagerRemoteService;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;
import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/**
 * Manager GWT Page Remote Service (Server)
 * @author R.Bacher, DESY
 */
@SuppressWarnings("serial")
public class Web2cManagerRemoteService extends RemoteServiceServlet implements IWeb2cManagerRemoteService, IWeb2cConstants {

	public String[] getServletNames() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getServletNames();
	}
	
	public String getServletDiagnosticsPage(String servletName, String managerSessionId, String hostAndPort) {
		
		String diagnosticsPageURL = "http://" + hostAndPort + Web2cContextHandler.getInstance(getServletContext()).getServletHandler(servletName).doSingleSignOnLogin(Web2cContextHandler.getInstance(getServletContext()).getServletHandler("Web2cManager").getSessionHandler(managerSessionId), "DiagnosticsPage");
		
		return diagnosticsPageURL;
	}

	public String[] getAllConfigFileNames() {
		
		ArrayList<String> fileList = Web2cContextHandler.getInstance(getServletContext()).getListOfAllFiles(new File(Web2cContextHandler.getInstance(getServletContext()).getUserDir()), "xml");			
		String[] files = new String[fileList.size()];
		int i = 0;
		for (Iterator<String> it = fileList.iterator(); it.hasNext();) {
			files[i] = (String)(it.next());
			i = i + 1;
		}
		
		return files;
	}

	public void saveToolkitInitializationParameters(String salutation, String[] earlyRegisteringFiles) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cInitWriterAndReader().setSalutation(salutation);		
	
		ArrayList<String> fileList = new ArrayList<String>();		
		for(int i = 0; i < earlyRegisteringFiles.length; i++) {
			fileList.add(earlyRegisteringFiles[i]);
		}		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cInitWriterAndReader().setEarlyRegisteringFileList(fileList);
	}

	public String getSalutationText() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cInitWriterAndReader().getSalutation();
	}

	public String[] getEarlyRegisteringFiles() {
		
		ArrayList<String> fileList = Web2cContextHandler.getInstance(getServletContext()).getWeb2cInitWriterAndReader().getEarlyRegisteringFileList();
		String[] files = new String[fileList.size()];
		int i = 0;
		for (Iterator<String> it = fileList.iterator(); it.hasNext();) {
			files[i] = (String)(it.next());
			i = i + 1;
		}
		
		return files;
	}

	public void setTwitterId(String text) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().setServiceId(text);
	}

	public void saveTwitterId(String text) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().saveServiceId(text);
	}

	public String getTwitterId() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().getServiceId();
	}

	public void setTwitterPassword(String text) {		
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().setServicePassword(text);		
	}

	public void saveTwitterPassword(String text) {		
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().saveServicePassword(text);		
	}

	public String getTwitterPassword() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().getServicePassword();
	}

	public void setTwitterLoggingOption(String text) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().setServiceLoggingOption(text);			
	}

	public void saveTwitterLoggingOption(String text) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().saveServiceLoggingOption(text);			
	}

	public String getTwitterLoggingOption() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().getServiceLoggingOption();
	}

	public String doRequestTwitterPermit() {	
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().getAuthorizationURL();
	}

	public String doStartPublishingTwitterStatus() {		
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().startPublishingStatus();
		
		return "Try to start updating twitter status ...   Return status (true = started): " + getTwitterPublishingStatus();
	}

	public String doStopPublishingTwitterStatus() {		
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().stopPublishingStatus();
		
		return "Try to stop updating twitter status ...";
	}

	public String doGetTwitterStatus() {		
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().stopPublishingStatus();
	
		return "Last Twitter status from " + Calendar.getInstance().getTime().toString() + ": " + getMostRecentTwitterStatus();
	}

	public Boolean getTwitterServiceStatus() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().getServiceAvailable();
	}

	public Boolean getTwitterPublishingStatus() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().getServiceStarted();
	}

	public String getMostRecentTwitterStatus() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().getMessage();
	}

	public void doArmTwitterService() {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().armService();
	}

	public void doDisarmTwitterService() {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cTwitterInterface().disarmService();
	}
	
	public void setSMTPId(String text) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().setServiceId(text);					
	}

	public void saveSMTPId(String text) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().saveServiceId(text);	
	}

	public String getSMTPId() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().getServiceId();
	}

	public void setSMTPPassword(String text) {	
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().setServicePassword(text);			
		
	}

	public void saveSMTPPassword(String text) {	
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().saveServicePassword(text);		
	}

	public String getSMTPPassword() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().getServicePassword();
	}

	public void setSMTPLoggingOption(String text) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().setServiceLoggingOption(text);			
	}

	public void saveSMTPLoggingOption(String text) {		
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().saveServiceLoggingOption(text);			
	}

	public String getSMTPLoggingOption() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().getServiceLoggingOption();
	}

	public void setSMTPHost(String text) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().setServiceHostName(text);				
	}

	public void saveSMTPHost(String text) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().saveServiceHostName(text);	
	}

	public String getSMTPHost() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().getServiceHostName();
	}

	public void doArmSMTPService() {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().armService();
	}

	public void doDisarmSMTPService() {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().disarmService();
	}

	public Boolean getSMTPServiceStatus() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().getServiceAvailable();
	}

	public String getSMTPMessageSendingStatus() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cSMTPInterface().getMessageSendingStatus();
	}

	public String[] getUserNames() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cRegistryWriterAndReader().getUserNames();
	}

	public String getUserPassword(String name) {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cRegistryWriterAndReader().getPassword(name);
	}

	public String[] saveUserRegistry(String name, String password) {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cRegistryWriterAndReader().addOrModifyRegistryEntry(name, password);
	}

	public String[] removeRegistryEntry(String name) {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cRegistryWriterAndReader().removeRegistryEntry(name);
	}

	public void setWebCommunicationDebugFlag(String text) {

		boolean debugFlag = false;
		
		if (text.equals("yes")) debugFlag = true;
		else debugFlag = false;
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cKeyBoardListener().setDebugWebCommunication(debugFlag);
	}

	public void setControlSystemCommunicationDebugFlag(String text) {

		boolean debugFlag = false;
		
		if (text.equals("yes")) debugFlag = true;
		else debugFlag = false;
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cKeyBoardListener().setDebugControlSystemCommunication(debugFlag);
	}

	public void setConfigurationFileParsingDebugFlag(String text) {

		boolean debugFlag = false;
		
		if (text.equals("yes")) debugFlag = true;
		else debugFlag = false;
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2cKeyBoardListener().setDebugConfigurationFileParsing(debugFlag);
	}

	public void saveToolkitInitFile(String name, String type) {
		
		Web2cContextHandler.getInstance(getServletContext()).getWeb2ToolkitInitFilesWriterAndReader().saveInitFile(name, type);
	}

	public String[] getInitFileTypes() {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2ToolkitInitFilesWriterAndReader().getInitFileTypes();
	}

	public String getInitFileName(String type) {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2ToolkitInitFilesWriterAndReader().getInitFileName(type);
	}

	public String[] getAllLogFileNames() {
		
		ArrayList<String> fileList = Web2cContextHandler.getInstance(getServletContext()).getListOfAllFiles(new File(Web2cContextHandler.getInstance(getServletContext()).getLoggingDir()), "log");			
		String[] files = new String[fileList.size()];
		int i = 0;
		for (Iterator<String> it = fileList.iterator(); it.hasNext();) {
			files[i] = (String)(it.next());
			i = i + 1;
		}
		
		return files;
	}

	public void clearLogFile(String name) {
		
		Web2cContextHandler.getInstance(getServletContext()).getFileLogger(name).clearLog();
	}

	public String[] getLogFile(String name, String numberOfLines) {
		
		return Web2cContextHandler.getInstance(getServletContext()).getFileLogger(name).getLog(numberOfLines);
	}

	public void setJavaGarbageCollection(String flag) {
		
		if (flag.equals("on")) Web2cContextHandler.getInstance(getServletContext()).setJavaGarbageCollection(true);
		else Web2cContextHandler.getInstance(getServletContext()).setJavaGarbageCollection(false);
	}

	public String getJavaGarbageCollection() {

		if (Web2cContextHandler.getInstance(getServletContext()).getJavaGarbageCollection() == true) return "on";
		else return "off";
	}

	public void setWeb2cToolkitGarbageCollection(String flag) {
		
		if (flag.equals("on")) Web2cContextHandler.getInstance(getServletContext()).setWeb2cToolkitGarbageCollection(true);
		else Web2cContextHandler.getInstance(getServletContext()).setWeb2cToolkitGarbageCollection(false);
	}

	public String getWeb2cToolkitGarbageCollection() {

		if (Web2cContextHandler.getInstance(getServletContext()).getWeb2cToolkitGarbageCollection() == true) return "on";
		else return "off";
	}

	public void doJavaGarbageCollection() {
		
		Web2cContextHandler.getInstance(getServletContext()).doJavaGarbageCollection();
	}

	public String getPercentagedHeapSpaceUsage(String sessionId) {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cManagerServlet().doPostAction("<sessionId>" + sessionId + "</sessionId><do>getHeapMemoryUsage</do><parameter></parameter>");
	}

	public String getPercentagedCPUUsage(String sessionId) {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cManagerServlet().doPostAction("<sessionId>" + sessionId + "</sessionId><do>getCPUUsage</do><parameter></parameter>");
	}

	public String getLoadedClassCount(String sessionId) {
		
		return Web2cContextHandler.getInstance(getServletContext()).getWeb2cManagerServlet().doPostAction("<sessionId>" + sessionId + "</sessionId><do>getLoadedClassCount</do><parameter></parameter>");
	}
}
