package de.desy.web2cToolkit.web2cServlets;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Iterator;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cPage;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cServletHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;

/**
 * @author R.Bacher, DESY
 * @version de.desy.web2cToolkit.web2cUtil.web2cInterfaces:IWeb2cConstants.java (web2cVersion)
 * @see <a href="http://adweb.desy.de/mcs/web2cToolkit/web2c_UserManual.htm">How-to-Use Instructions</a>
 */
@SuppressWarnings("serial")
public class Web2c extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet, IWeb2cConstants {

	private Web2cContextHandler contextHandler = null;
	private Web2cServletHandler servletHandler = null;
	
	/**
	 * Web2c Synoptic Display Servlet: handles client login, requests and updates
	 */
	public Web2c() {

		super();
	}   
	
	/**
	 * Initializes the servlet
	 * @param config servlet config
	 * @throws ServletException 
	 */
	public void init(ServletConfig config) throws ServletException {	
		
		super.init(config);
		
		if (contextHandler == null) {
			contextHandler = Web2cContextHandler.getInstance(config.getServletContext());
			contextHandler.addToServletList(this);
		}
		
		if (servletHandler == null) {
			servletHandler = new Web2cServletHandler(config, this);
		}
		
		servletHandler.doEarlyLinkRegistering();
	}	
	
	/**
	 * Returns the servlet handler associated to this servlet
	 * @returns servlet handler
	 */
	public Web2cServletHandler getServletHandler() {
		
		return servletHandler;
	}
	
	/**
	 * Get Method
	 * @param request client request
	 * @param response servlet response
	 * @throws ServletException 
	 * @throws IOException 
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		try {			
			if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			servletHandler.getWeb2cKeyBoardListener().getKeyboardInput();			
		    if (request.getParameter("param") != null) {	
		    	String servletInputParameter = (request.getParameter("param")).substring(0, 6);	
		    	String requestParameter = (request.getParameter("param")).substring(6);
				if (servletInputParameter.compareToIgnoreCase("<plot>") == 0) {
					response.setContentType("image/png");
					doGetPlot(requestParameter).writeTo(response.getOutputStream());
				}				
				else if (servletInputParameter.compareToIgnoreCase("<vimg>") == 0){
					response.setContentType("image/png");
					doGetVideoImage(requestParameter).writeTo(response.getOutputStream());
				}
				else if (servletInputParameter.compareToIgnoreCase("<hplt>") == 0) {
					response.setContentType("image/png");
					ChartUtilities.writeChartAsPNG(response.getOutputStream(), (JFreeChart)(doGetHistoryPlot(requestParameter)[0]), ((Integer)(doGetHistoryPlot(requestParameter)[1])).intValue(), ((Integer)(doGetHistoryPlot(requestParameter)[2])).intValue());
				}
				else if (servletInputParameter.compareToIgnoreCase("<htxt>") == 0) {
					String str = doGetHistory(requestParameter);
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else if (servletInputParameter.compareToIgnoreCase("(page)") == 0) {
					String str = doGetPage(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else if (servletInputParameter.compareToIgnoreCase("(load)") == 0) {
					String str = servletHandler.doGetLoadingProgressPage(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);
					response.setContentType("text/html");	
					response.getWriter().print(str);
				}
				else if (servletInputParameter.compareToIgnoreCase("(open)") == 0) {	
					request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
					String str = servletHandler.doGetLogin(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString(), request.getSession().getId(), request.getSession().getCreationTime(), "real", false);
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);			
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else if (servletInputParameter.compareToIgnoreCase("(file)") == 0) {	
					request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
					String str = servletHandler.doGetLogin(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString(), request.getSession().getId(), request.getSession().getCreationTime(), "real", true);
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);			
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else if (servletInputParameter.compareToIgnoreCase("(simu)") == 0) {
					request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
					String str = servletHandler.doGetLogin(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString(), request.getSession().getId(), request.getSession().getCreationTime(), "simulation", true);
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);		
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else {				
					response.setContentType("text/html");
					System.out.println("Web2c-GET: Unknown servlet input parameter specified!");
				}
		    }	
		    else {				
				response.setContentType("text/html");
		    	System.out.println("Web2c-GET: No servlet input parameter specified!");
		    }
		}
		catch (Exception e) {
			servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();				
			response.setContentType("text/html");
			System.out.println("Web2c-GET: Error processing servlet: error processing parameter " + (request.getParameter("param")).substring(0, 6));
		}
		response.flushBuffer();
	}  	
	
	/**
	 * Post Method
	 * @param request client request
	 * @param response servlet response
	 * @throws ServletException 
	 * @throws IOException 
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		try {			
			if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			servletHandler.getWeb2cKeyBoardListener().getKeyboardInput();
			if (request.getParameter("param") != null) {
				String servletInputParameter = request.getParameter("param");
				if (servletInputParameter.compareToIgnoreCase("checkAlive") == 0) {
					String str = servletHandler.doPostCheckAlive();
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);		
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
	    		else if (servletInputParameter.compareToIgnoreCase("login") == 0) {
	    			request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
	    			String str = servletHandler.doPostLogin(request.getRemoteAddr(), request.getSession().getId(), request.getSession().getCreationTime(), true);	
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);					
	    			response.setContentType("text/html");
					response.getWriter().print(str);
	    		}
				else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<update>") == 0) {
					String str = doPostUpdate((request.getParameter("param")).substring(8));
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);				
					response.setContentType("text/html");
					response.getWriter().print(str);
			    }
			    else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<action>") == 0) {
			    	String str = doPostAction((request.getParameter("param")).substring(8));
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);				
					response.setContentType("text/html");
					response.getWriter().print(str);
			    }
			    else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<elogin>") == 0) {
					request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
					String str = servletHandler.doPostElogin(servletInputParameter.substring(8), request.getRemoteAddr());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);				
					response.setContentType("text/html");
					response.getWriter().print(str);
			    }
			    else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<ldchck>") == 0) {
			    	String str = servletHandler.doPostCheckPageLoadingStatus(servletInputParameter.substring(8));
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);				
					response.setContentType("text/html");
					response.getWriter().print(str);
			    }
			    else {
					response.setContentType("text/html");
			    	System.out.println("Web2c-POST: Unknown servlet input parameter specified!");
			    }
			}
			else {
				response.setContentType("text/html");
		    	System.out.println("Web2c-POST: No servlet input parameter specified!");
			}	
		}
		catch(Exception e) {
			servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			response.setContentType("text/html");
			String servletInputParameter = request.getParameter("param");
			if ((servletInputParameter.compareToIgnoreCase("checkAlive") == 0) || (servletInputParameter.compareToIgnoreCase("login") == 0)) {
				System.out.println("Web2c-POST: Error processing servlet: error processing parameter " + servletInputParameter);
			}
			else {
				System.out.println("Web2c-POST: Error processing servlet: error processing parameter " + servletInputParameter.substring(0,8));
			}
		}
		response.flushBuffer();
	}
	
	/**
	 * Handles client request to get plot chart as byte stream
	 * @param parameter request parameter
	 * @return chart as byte stream
	 */
	private ByteArrayOutputStream doGetPlot(String parameter) {

		String sessionId = servletHandler.parsePostRequestParameter(parameter, "sessionId")[0];
		String id = servletHandler.parsePostRequestParameter(parameter, "id")[0];
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			return servletHandler.getWeb2cImageCache().getChartFromCache(associatedSessionHandler, id);
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to get video image as byte stream
	 * @param parameter request parameter
	 * @return video image as byte stream
	 */
	private ByteArrayOutputStream doGetVideoImage(String parameter) {

		String sessionId = servletHandler.parsePostRequestParameter(parameter, "sessionId")[0];
		String id = servletHandler.parsePostRequestParameter(parameter, "id")[0];
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			return servletHandler.getWeb2cImageCache().getVideoFromCache(associatedSessionHandler, id);
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to get history chart as byte stream
	 * @param parameter request parameter
	 * @return object-type array ([0]:chart (byte stream), [1]:chart width (integer), [2]:chart height (integer))
	 */
	private Object[] doGetHistoryPlot(String parameter) {

		String sessionId = servletHandler.parsePostRequestParameter(parameter, "sessionId")[0];
		String value1 = servletHandler.parsePostRequestParameter(parameter, "value")[0];
		String value2 = servletHandler.parsePostRequestParameter(parameter, "value")[1];
		String value3 = servletHandler.parsePostRequestParameter(parameter, "value")[2];
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			Object[] retVals = new Object[3];
			retVals[0] = associatedSessionHandler.getWeb2cPage().getHistory().getChart(value1, value2, value3);
			retVals[1] = associatedSessionHandler.getWeb2cPage().getHistory().getChartWidth();
			retVals[2] = associatedSessionHandler.getWeb2cPage().getHistory().getChartHeight();
			return retVals;
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to get history chart page
	 * @param parameter request parameter
	 * @return data string
	 */
	private String doGetHistory(String parameter) {

		String sessionId = servletHandler.parsePostRequestParameter(parameter, "sessionId")[0];
		String value1 = servletHandler.parsePostRequestParameter(parameter, "value")[0];
		String value2 = "";
		try {
			value2 = servletHandler.parsePostRequestParameter(parameter, "value")[1];
		}
		catch (Exception e) {
			value2 = "";
		}
		String value3 = "";
		try {
			value3 = servletHandler.parsePostRequestParameter(parameter, "value")[2];
		}
		catch (Exception e) {
			value3 = "";
		}
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			StringBuffer strBfr = new StringBuffer();
			for (Iterator<String> it1= associatedSessionHandler.getWeb2cPage().getHistory().getPage(value1, value2, value3).iterator(); it1.hasNext();) {
				String out = (String)it1.next();
				strBfr.append(out);
			}
			return strBfr.toString();
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to load finally generated page
	 * @param parameter request parameter
	 * @param remoteAddress client address
	 * @param requestedURL by client requested URL
	 * @return html code
	 */
	public String doGetPage(String parameter, String remoteAddress, String requestedURL) {

		String sessionId = servletHandler.parseGetRequestParameter(parameter, "sessionId");
		String fileName = servletHandler.parseGetRequestParameter(parameter, "fileName");
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			StringBuffer strBfr = new StringBuffer();
			if (associatedSessionHandler.getAttribute("loginStatus").equals("in")) {
				if (fileName.equals("DiagnosticsPage")) {
					for (Iterator<String> it = servletHandler.getWeb2cProbe().getStatus().iterator(); it.hasNext();) {
						strBfr.append((String)it.next());
					}
				}
				else {
					Web2cPage web2cPage = associatedSessionHandler.getWeb2cPage();
					String error = web2cPage.getError().replace("null", "");
					if (!error.equals("")) {
						strBfr.append(servletHandler.getPageCreationErrorPage((String)associatedSessionHandler.getAttribute("loadedConfigFileName")));
						strBfr.append("<div>----->>>>></div>");
						strBfr.append("<div>Reasons identified:</div>");
						for (int i = 0; i < error.split(";").length; i++ ) {
							strBfr.append("<div> \"" + error.split(";")[i] + "\" </div>");
						}
					}
					else {
						for (Iterator<String> it1 = web2cPage.getPage().iterator(); it1.hasNext();) {
							String out = (String)it1.next();
							strBfr.append(out);
						}
					}
				}
			}
			else {
				strBfr.append("");
			}
			servletHandler.getSessionLogger().appendLog("IP Address: " + remoteAddress + " User Name: " + (String)associatedSessionHandler.getAttribute("userName") + " Page loaded: " + (String)associatedSessionHandler.getAttribute("loadedConfigFileName"));			
			return strBfr.toString();
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to get update
	 * @param parameter request parameter
	 * @return update string
	 */
	private String doPostUpdate(String parameter) {

		synchronized (servletHandler.getWeb2cSessionHandlerList()) {
			servletHandler.getWeb2cGarbageCollector().removeDeadSessions(servletHandler.getWeb2cSessionHandlerList());
			servletHandler.getWeb2cGarbageCollector().clearCache(servletHandler.getWeb2cImageCache());
		}
		String sessionId = servletHandler.parsePostRequestParameter(parameter, "sessionId")[0];
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
			String str = associatedSessionHandler.getWeb2cPage().getUpdate();
			return "<return>" + str + "</return>";
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to perform action
	 * @param parameter request parameter
	 * @return return string
	 */
	private String doPostAction(String parameter) {

		String sessionId = servletHandler.parsePostRequestParameter(parameter, "sessionId")[0];
		String value = servletHandler.parsePostRequestParameter(parameter, "value")[0];
		String id = servletHandler.parsePostRequestParameter(parameter, "id")[0];
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			if (value.equals("logout")) {
				synchronized (servletHandler.getWeb2cSessionHandlerList()) {
					servletHandler.getWeb2cGarbageCollector().removeSession(servletHandler.getWeb2cSessionHandlerList(), associatedSessionHandler);
				}
			}
			else {	
				associatedSessionHandler.getWeb2cPage().doAction(value, id);
			}
			return "";
		}
		else {
			return null;
		}
	}
}