package de.desy.web2cToolkit.web2cServlets;

import java.io.IOException;
import java.util.Calendar;
import java.util.Iterator;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cServletHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;

/**
 * @author R.Bacher, DESY
 * @version de.desy.web2cToolkit.web2cUtil.web2cInterfaces:IWeb2cConstants.java (web2cVersion)
 * @see <a href="http://adweb.desy.de/mcs/web2cToolkit/web2c_UserManual.htm">How-to-Use Instructions</a>
 */
@SuppressWarnings("serial")
public class Web2cGateway extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet, IWeb2cConstants {

	private Web2cContextHandler contextHandler = null;
	private Web2cServletHandler servletHandler = null;
	
	/**
	 * Web2cGateway Servlet: http gateway to the control systems attached
	 */
	public Web2cGateway() {

		super();
	}  
	
	/**
	 * Initializes the servlet
	 * @param config servlet config
	 * @throws ServletException 
	 */
	public void init(ServletConfig config) throws ServletException {	
		
		super.init(config);
		
		if (contextHandler == null) {
			contextHandler = Web2cContextHandler.getInstance(config.getServletContext());
			contextHandler.addToServletList(this);
		}
		
		if (servletHandler == null) {
			servletHandler = new Web2cServletHandler(config, this);
		}
		
		servletHandler.doEarlyLinkRegistering();
	}	
	
	/**
	 * Returns the servlet handler associated to this servlet
	 * @throws servlet handler
	 */
	public Web2cServletHandler getServletHandler() {
		
		return servletHandler;
	}
	
	/**
	 * Get Method
	 * @param request client request
	 * @param response servlet response
	 * @throws ServletException 
	 * @throws IOException 
	 */
	protected synchronized void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		try {				
			if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			servletHandler.getWeb2cKeyBoardListener().getKeyboardInput();		
		    if (request.getParameter("param") != null) {	
		    	String servletInputParameter = (request.getParameter("param")).substring(0, 6);	
		    	String requestParameter = (request.getParameter("param")).substring(6);
				if (servletInputParameter.compareToIgnoreCase("(page)") == 0) {
					String str = doGetPage(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else if (servletInputParameter.compareToIgnoreCase("(load)") == 0) {
					String str = servletHandler.doGetLoadingProgressPage(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);
					response.setContentType("text/html");	
					response.getWriter().print(str);
				}				
				else if (servletInputParameter.compareToIgnoreCase("(file)") == 0) {
					request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
					String str = servletHandler.doGetLogin(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString(), request.getSession().getId(), request.getSession().getCreationTime(), "real", true);
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);			
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else {				
					response.setContentType("text/html");
					System.out.println("Web2cGateway-GET: Unknown servlet input parameter specified!");
				}
		    }	
		    else {				
				response.setContentType("text/html");
		    	System.out.println("Web2cGateway-GET: No servlet input parameter specified!");
		    }
		}
		catch (Exception e) {
			servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();				
			response.setContentType("text/html");
			System.out.println("Web2cGateway-GET: Error processing servlet: error processing parameter " + (request.getParameter("param")).substring(0, 6));
		}
		response.flushBuffer();
	}  	
	
	/**
	 * Post Method
	 * @param request client request
	 * @param response servlet response
	 * @throws ServletException 
	 * @throws IOException 
	 */
	protected synchronized void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		try {		
			if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			servletHandler.getWeb2cKeyBoardListener().getKeyboardInput();
			if (request.getParameter("param") != null) {
				String servletInputParameter = request.getParameter("param");
		    	synchronized(servletHandler.getWeb2cSessionHandlerList()) {
		    		if (servletInputParameter.compareToIgnoreCase("checkAlive") == 0) {
						String str = servletHandler.doPostCheckAlive();
						if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);		
						response.setContentType("text/html");
						response.getWriter().print(str);
		    		}
		    		else if (servletInputParameter.compareToIgnoreCase("login") == 0) {
		    			request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
		    			String str = servletHandler.doPostLogin(request.getRemoteAddr(), request.getSession().getId(), request.getSession().getCreationTime(), true);	
						if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);					
		    			response.setContentType("text/html");
						response.getWriter().print(str);
		    		}
		    		else if (servletInputParameter.compareToIgnoreCase("loginGuest") == 0) {
		    			request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
		    			String str = servletHandler.doPostLogin(request.getRemoteAddr(), request.getSession().getId(), request.getSession().getCreationTime(), false);	
						if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);					
		    			response.setContentType("text/html");
						response.getWriter().print(str);
		    		}
		    		else if (servletInputParameter.compareToIgnoreCase("loginExpert") == 0) {
		    			request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
		    			String str = servletHandler.doPostLogin(request.getRemoteAddr(), request.getSession().getId(), request.getSession().getCreationTime(), true);	
						if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);					
		    			response.setContentType("text/html");
						response.getWriter().print(str);
		    		}
		    		else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<action>") == 0) {
				    	String str = doPostAction((request.getParameter("param")).substring(8));
						if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);				
						response.setContentType("text/html");
						response.getWriter().print(str);
				    }
				    else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<elogin>") == 0) {
						request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
						String str = servletHandler.doPostElogin(servletInputParameter.substring(8), request.getRemoteAddr());
						if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);				
						response.setContentType("text/html");
						response.getWriter().print(str);
				    }
				    else {
						response.setContentType("text/html");
				    	System.out.println("Web2cGateway-POST: Unknown servlet input parameter specified!");
				    }
		    	}
			}
			else {
				response.setContentType("text/html");
		    	System.out.println("Web2cGateway-POST: No servlet input parameter specified!");
			}
		}
		catch(Exception e) {
			servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			response.setContentType("text/html");
			String servletInputParameter = request.getParameter("param");
			if ((servletInputParameter.compareToIgnoreCase("checkAlive") == 0) || (servletInputParameter.compareToIgnoreCase("login") == 0)) {
				System.out.println("Web2c-Gateway: Error processing servlet: error processing parameter " + servletInputParameter);
			}
			else {
				System.out.println("Web2c-Gateway: Error processing servlet: error processing parameter " + servletInputParameter.substring(0,8));
			}
		}
		response.flushBuffer();
	}
	
	/**
	 * Handles client request to get a page after successful login procedure
	 * @param parameter request parameter
	 * @param remoteAddress client address
	 * @param requestedURL by client requested URL
	 * @return html code
	 */
	public String doGetPage(String parameter, String remoteAddress, String requestedURL) {

		synchronized (servletHandler.getWeb2cSessionHandlerList()) {
			servletHandler.getWeb2cGarbageCollector().removeDeadSessions(servletHandler.getWeb2cSessionHandlerList());
		}
		String sessionId = servletHandler.parseGetRequestParameter(parameter, "sessionId");
		String page = servletHandler.parseGetRequestParameter(parameter, "fileName");
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			StringBuffer strBfr = new StringBuffer();	
			if (associatedSessionHandler.getAttribute("loginStatus").equals("in")) {
				associatedSessionHandler.setAttribute("loadedConfigFileName", (String)associatedSessionHandler.getAttribute("requestedConfigFileName"));
				if (page.equals("DiagnosticsPage")) {
					for (Iterator<String> it = servletHandler.getWeb2cGatewayProbe().getStatus().iterator(); it.hasNext();) {
						strBfr.append((String)it.next());
					}
				}
				else {
					strBfr.append("");
				}	
			}
			else {
				strBfr.append("");
			}	
			return strBfr.toString();
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to perform action
	 * @param parameter request parameter
	 * @return return data
	 */
	private String doPostAction(String parameter) {

		String sessionId = servletHandler.parsePostRequestParameter(parameter, "sessionId")[0];
		String value = servletHandler.parsePostRequestParameter(parameter, "value")[0];
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {			
				String str = "";
				if (value.equals("logout")) {
					synchronized (servletHandler.getWeb2cSessionHandlerList()) {
						servletHandler.getWeb2cGarbageCollector().removeSession(servletHandler.getWeb2cSessionHandlerList(), associatedSessionHandler);
					}
					str = "<return>removed<return>";
				}
				else {
					associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
					str = associatedSessionHandler.getWeb2cGatewayPage().doAction(value);
				}	
				return str;
		}
		else {
			return null;
		}
	}
}