package de.desy.web2cToolkit.web2cServlets;

import java.io.IOException;
import java.lang.management.ClassLoadingMXBean;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.MemoryUsage;
import java.lang.management.OperatingSystemMXBean;
import java.lang.management.RuntimeMXBean;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cServletHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;

/**
 * @author R.Bacher, DESY
 * @version see de.desy.web2cToolkit.web2cUtil.web2cInterfaces:IWeb2cConstants.java (web2cVersion)
 * @see <a href="http://adweb.desy.de/mcs/web2cToolkit/web2c_UserManual.htm">How-to-Use Instructions</a>
 */
@SuppressWarnings("serial")
public class Web2cManager extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet, IWeb2cConstants {

	private Web2cContextHandler contextHandler = null;
	private Web2cServletHandler servletHandler = null;
	
	/**
	 * Web2cManger Servlet: manages the Web2cToolkit servlets
	 */
	public Web2cManager() {

		super();
	}     
	
	/**
	 * Initializes the servlet
	 * @param config servlet config
	 * @throws ServletException 
	 */
	public void init(ServletConfig config) throws ServletException {	
		
		super.init(config);
		
		if (contextHandler == null) {
			contextHandler = Web2cContextHandler.getInstance(config.getServletContext());
			contextHandler.addToServletList(this);
		}
		
		if (servletHandler == null) {
			servletHandler = new Web2cServletHandler(config, this);
		}
	}	
	
	/**
	 * Destroys the servlet
	 */
	public void destroy() {
		
		servletHandler.getWeb2cPerformanceMonitor().stopMonitor();
	}
	
	/**
	 * Returns the servlet handler associated to this servlet
	 * @throws servlet handler
	 */
	public Web2cServletHandler getServletHandler() {
		
		return servletHandler;
	}
	
	/**
	 * Get Method
	 * @param request client request
	 * @param response servlet response
	 * @throws ServletException 
	 * @throws IOException 
	 */
	protected synchronized void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		try {			
			if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			servletHandler.getWeb2cKeyBoardListener().getKeyboardInput();			
		    if (request.getParameter("param") != null) {	
		    	String servletInputParameter = (request.getParameter("param")).substring(0, 6);	
		    	String requestParameter = (request.getParameter("param")).substring(6);
				if (servletInputParameter.compareToIgnoreCase("(file)") == 0) {	
					request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
					String str = servletHandler.doGetLogin(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString(), request.getSession().getId(), request.getSession().getCreationTime(), "real", true);
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);			
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else if (servletInputParameter.compareToIgnoreCase("(load)") == 0) {
					String str = servletHandler.doGetLoadingProgressPage(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);
					response.setContentType("text/html");	
					response.getWriter().print(str);
				}
				else if (servletInputParameter.compareToIgnoreCase("(page)") == 0) {
					String str = doGetPage(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else {				
					response.setContentType("text/html");
					System.out.println("Web2cManager-GET: Unknown servlet input parameter specified!");
				}
			}	
			else {				
				response.setContentType("text/html");
				System.out.println("Web2cManager-GET: No servlet input parameter specified!");
			}
		}
		catch (Exception e) {
			servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();				
			response.setContentType("text/html");
			System.out.println("Web2c-Manager: Error processing servlet: error processing parameter " + (request.getParameter("param")).substring(0, 6));
		}
		response.flushBuffer();
	}
	
	/**
	 * Post Method
	 * @param request client request
	 * @param response servlet response
	 * @throws ServletException 
	 * @throws IOException 
	 */
	protected synchronized void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		try {			
			if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			servletHandler.getWeb2cKeyBoardListener().getKeyboardInput();
			if (request.getParameter("param") != null) {
				String servletInputParameter = request.getParameter("param");
	    		if (servletInputParameter.compareToIgnoreCase("checkAlive") == 0) {
					String str = servletHandler.doPostCheckAlive();
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);		
					response.setContentType("text/html");
					response.getWriter().print(str);
	    		}
	    		else if (servletInputParameter.compareToIgnoreCase("login") == 0) {
	    			request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
	    			String str = servletHandler.doPostLogin(request.getRemoteAddr(), request.getSession().getId(), request.getSession().getCreationTime(), true);	
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);					
	    			response.setContentType("text/html");
					response.getWriter().print(str);
	    		}
	    		else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<elogin>") == 0) {
					request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
					String str = servletHandler.doPostElogin(servletInputParameter.substring(8), request.getRemoteAddr());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);				
					response.setContentType("text/html");
					response.getWriter().print(str);
	    		}
	    		else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<action>") == 0) {
			    	String str = doPostAction((request.getParameter("param")).substring(8));
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);				
					response.setContentType("text/html");
					response.getWriter().print(str);
	    		}
			    else {
					response.setContentType("text/html");
			    	System.out.println("Web2cManager-POST: Unknown servlet input parameter specified!");
			    }
			}
			else {
				response.setContentType("text/html");
		    	System.out.println("Web2cManager-POST: No servlet input parameter specified!");
			}
		}
		catch(Exception e) {
			servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			response.setContentType("text/html");
			String servletInputParameter = request.getParameter("param");
			if ((servletInputParameter.compareToIgnoreCase("checkAlive") == 0) || (servletInputParameter.compareToIgnoreCase("login") == 0)) {
				System.out.println("Web2cManager-POST: Error processing servlet: error processing parameter " + servletInputParameter);
			}
			else {
				System.out.println("Web2cManager-POST: Error processing servlet: error processing parameter " + servletInputParameter.substring(0,8));
			}
		}
		response.flushBuffer();
	}
	
	/**
	 * Returns the manager page
	 * @param sessionHandler session handler
	 * @param servletURL servlet URL
	 * @return HTML page
	 */
	private ArrayList<String> getManagerPage(Web2cSessionHandler sessionHandler, String servletURL) {

		ArrayList<String> out = new ArrayList<String>();	
		
		out.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		out.add("<html>");
		out.add("   <head>");
		out.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		out.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		out.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		out.add("       <meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		out.add("       <title>Web2cToolkit Manager</title>");
		out.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
		out.add("		<link type=\"text/css\" rel=\"stylesheet\" href=\"Web2cManagerPage.css\">");
		out.add("       <script language=\"javascript\" src=\"de.desy.web2cToolkit.web2cManager.web2cGWT.Web2cManagerPage/de.desy.web2cToolkit.web2cManager.web2cGWT.Web2cManagerPage.nocache.js\"></script>");
		out.add("   </head>");
		out.add("	<body>");
		out.add("		<input type=\"hidden\" id=\"sessionId\" name=\"sessionId\" value=\"" + sessionHandler.getAttribute("id") + "\">");
		out.add("	</body>");
		out.add("</html>");
		
		return out;
	}
	
	/**
	 * Handles client request to get a page after successful login procedure
	 * @param parameter request parameter
	 * @param remoteAddress client address
	 * @param requestedURL by client requested URL
	 * @return html code
	 */
	public String doGetPage(String parameter, String remoteAddress, String requestedURL) {

		synchronized (servletHandler.getWeb2cSessionHandlerList()) {
			servletHandler.getWeb2cGarbageCollector().removeDeadSessions(servletHandler.getWeb2cSessionHandlerList());
		}
		String sessionId = servletHandler.parseGetRequestParameter(parameter, "sessionId");
		String page = servletHandler.parseGetRequestParameter(parameter, "fileName");
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			StringBuffer strBfr = new StringBuffer();	
			if (associatedSessionHandler.getAttribute("loginStatus").equals("in")) {
				associatedSessionHandler.setAttribute("loadedConfigFileName", (String)associatedSessionHandler.getAttribute("requestedConfigFileName"));
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));	
				if (page.equals("ManagerPage")) {
					for (Iterator<String> it = getManagerPage(associatedSessionHandler, requestedURL.toString()).iterator(); it.hasNext();) {
						String str = (String)it.next();
						strBfr.append(str);
					}			
				}
				else if (page.equals("DiagnosticsPage")) {
					for (Iterator<String> it = servletHandler.getWeb2cManagerProbe().getStatus().iterator(); it.hasNext();) {
						strBfr.append((String)it.next());
					}
				}
				else {
					strBfr.append("");
				}	
			}
			else {
				strBfr.append("");
			}
			return strBfr.toString();
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to perform action
	 * @param parameter request parameter
	 * @return return data
	 */
	public String doPostAction(String parameter) {
		
		String ret = "";

		synchronized (servletHandler.getWeb2cSessionHandlerList()) {
			servletHandler.getWeb2cGarbageCollector().removeDeadSessions(servletHandler.getWeb2cSessionHandlerList());
		}
		String sessionId = servletHandler.parsePostRequestParameter(parameter, "sessionId")[0];
		String action = servletHandler.parsePostRequestParameter(parameter, "do")[0];
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {	
			if (associatedSessionHandler.getAttribute("loginStatus").equals("in")) {
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				if (action.equals("getHeapMemoryUsage")) ret = String.valueOf(servletHandler.getWeb2cPerformanceMonitor().getHeapMemoryUsage());
				if (action.equals("getCPUUsage")) ret = String.valueOf(servletHandler.getWeb2cPerformanceMonitor().getProcessCPUUsage());
				if (action.equals("getLoadedClassCount")) ret = String.valueOf(servletHandler.getWeb2cPerformanceMonitor().getLoadedClassCount());
				if (action.equals("doJavaGarbageCollection")) servletHandler.getWeb2cPerformanceMonitor().doGarbageCollection();
				return "<return>" + ret + "</return>";
			}
			else {
				return "<return></return>";
			}
		}
		else {
			return null;
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class Web2cPerformanceMonitor implements Runnable {

		private Thread monitor = null;
		private boolean monitorStarted = false;
		private MemoryMXBean memoryMXBean = null;
		private int percentagedHeapMemoryUsage = -1;
		private ClassLoadingMXBean classLoadingMxBean = null;
		private int loadedClassCount = -1;    
		private MBeanServer mBeanServer = null;
	    private RuntimeMXBean runtimeMXBean = null;
	    private OperatingSystemMXBean operatingSystemMXBean = null;
		private int percentagedProcessCPUUsage = -1;
		private int numberOfProcessors = 1;
		
		/**	
		 * Determines JVM performance measures
		 */
		public Web2cPerformanceMonitor() {
			
			super();
		
			if (monitor == null) {
				memoryMXBean = ManagementFactory.getMemoryMXBean();
				classLoadingMxBean = ManagementFactory.getClassLoadingMXBean();
				mBeanServer = ManagementFactory.getPlatformMBeanServer();
				operatingSystemMXBean = (OperatingSystemMXBean)ManagementFactory.getOperatingSystemMXBean();
				runtimeMXBean = ManagementFactory.getRuntimeMXBean();	        
		        numberOfProcessors = operatingSystemMXBean.getAvailableProcessors();
		        if (numberOfProcessors < 1) numberOfProcessors = 1;
				monitor = new Thread(this);				
			}
		}
		
		/**
		 * Retrieves and publishes continuously performance measures from the JVM
		 * Implemented measures: heap memory usage [%], process CPU usage [%], loaded class count
		 */
		public void run() {	 
			
	        long lastCpuTime = 0;
	        long cpuTime = 0;
	        long lastProcessTime = 0;
	        long processTime = 0;
	        
	    	monitorStarted = true;  
			System.out.println("ManagerMonitor started!");	
	        lastProcessTime = runtimeMXBean.getUptime();
	        try {lastCpuTime = (Long)mBeanServer.getAttribute(new ObjectName("java.lang:type=OperatingSystem"), "ProcessCpuTime");} catch(Exception ex) {}

	        while (monitorStarted == true) {
				try {			    			
					Thread.sleep(1000);		
					
					MemoryUsage memoryUsage  = memoryMXBean.getHeapMemoryUsage();
			    	double usage = (double)(memoryUsage.getUsed()) / (double)(memoryUsage.getCommitted());			    	
			    	percentagedHeapMemoryUsage = (int)(usage * 100.0);		
			    	
			    	loadedClassCount = classLoadingMxBean.getLoadedClassCount();		
			    	
			    	processTime = runtimeMXBean.getUptime();
			    	try {cpuTime = (Long)mBeanServer.getAttribute(new ObjectName("java.lang:type=OperatingSystem"), "ProcessCpuTime");} catch(Exception ex) {}
			    	percentagedProcessCPUUsage = (int)(((((float)((cpuTime - lastCpuTime))) / ((float)(processTime - lastProcessTime) * 1000000)) / numberOfProcessors) * 100.0);
			    	lastCpuTime = cpuTime;
			    	lastProcessTime = processTime;
				} 
				catch (InterruptedException e) {}
			}
			
			monitor.interrupt();   
			System.out.println("ManagerMonitor stopped!");
		}
		
		/**
		 * Starts monitor
		 */
		public void startMonitor() {	

			monitor.start();
		}
		
		/**
		 * Stops monitor
		 */
	    public void stopMonitor() {
	    	
	    	monitorStarted = false;
	    }
		
		/**
		 * Returns the percentaged heap memory usage (used / committed)
		 */
	    public int getHeapMemoryUsage() {
	    	
	    	return percentagedHeapMemoryUsage;
	    }
		
		/**
		 * Returns the number of classes loaded
		 */
	    public int getLoadedClassCount() {
	    	
	    	return loadedClassCount;
	    }
		
		/**
		 * Returns the percentaged process CPU usage
		 */
	    public int getProcessCPUUsage() {
	    	
	    	return percentagedProcessCPUUsage;
	    }
	    
		/**
		 * Requests JAVA garbage collection
		 */
	    public void doGarbageCollection() {
	    	
	    	memoryMXBean.gc();
	    }
	}
}