package de.desy.web2cToolkit.web2cServlets;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cContextHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cServletHandler;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;

/**
 * @author R.Bacher, DESY
 * @version see de.desy.web2cToolkit.web2cUtil.web2cInterfaces:IWeb2cConstants.java (web2cVersion)
 * @see <a href="http://adweb.desy.de/mcs/web2cToolkit/web2c_UserManual.htm">How-to-Use Instructions</a>
 */
@SuppressWarnings("serial")
public class Web2cMessenger extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet, IWeb2cConstants {

	private Web2cContextHandler contextHandler = null;
	private Web2cServletHandler servletHandler = null;
	
	/**
	 * Web2cMessager Servlet: performs messaging
	 * Services implemented: Twitter, SMTP
	 */
	public Web2cMessenger() {

		super();
	}     
	
	/**
	 * Initializes the servlet
	 * @param config servlet config
	 * @throws ServletException 
	 */
	public void init(ServletConfig config) throws ServletException {	
		
		super.init(config);
		
		if (contextHandler == null) {
			contextHandler = Web2cContextHandler.getInstance(config.getServletContext());
			contextHandler.addToServletList(this);
		}
		
		if (servletHandler == null) {
			servletHandler = new Web2cServletHandler(config, this);
		}
	}	
	
	/**
	 * Destroys the servlet
	 */
	public void destroy() {
		
		ArrayList<String> monitors = servletHandler.getWeb2cTwitterInterface().getMonitors();		
		if (monitors != null && !monitors.isEmpty()) {
			for (Iterator<String> it = monitors.iterator(); it.hasNext();) {
				String monitorId = (String)it.next();
				servletHandler.getWeb2cCommonDeviceAccess().removeRequester(monitorId);
			}
		}
		
		servletHandler.getWeb2cTwitterInterface().disarmService();
		servletHandler.getWeb2cSMTPInterface().disarmService();
	} 	
	
	/**
	 * Returns the servlet handler associated to this servlet
	 * @throws servlet handler
	 */
	public Web2cServletHandler getServletHandler() {
		
		return servletHandler;
	}
	
	/**
	 * Get Method
	 * @param request client request
	 * @param response servlet response
	 * @throws ServletException 
	 * @throws IOException 
	 */
	protected synchronized void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		try {			
			if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			servletHandler.getWeb2cKeyBoardListener().getKeyboardInput();			
		    if (request.getParameter("param") != null) {	
		    	String servletInputParameter = (request.getParameter("param")).substring(0, 6);	
		    	String requestParameter = (request.getParameter("param")).substring(6);
				if (servletInputParameter.compareToIgnoreCase("(file)") == 0) {
					request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
					String str = servletHandler.doGetLogin(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString(), request.getSession().getId(), request.getSession().getCreationTime(), "real", true);
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);			
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else if (servletInputParameter.compareToIgnoreCase("(load)") == 0) {
					String str = servletHandler.doGetLoadingProgressPage(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);
					response.setContentType("text/html");	
					response.getWriter().print(str);
				}
				else if (servletInputParameter.compareToIgnoreCase("(page)") == 0) {
					String str = doGetPage(requestParameter, request.getRemoteAddr(), request.getRequestURL().toString());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);
					response.setContentType("text/html");
					response.getWriter().print(str);
				}
				else {			
					response.setContentType("text/html");
					System.out.println("Web2cMessenger-GET: Unknown servlet input parameter specified!");
				}
			}	
			else {			
				response.setContentType("text/html");
				System.out.println("Web2cMessenger-GET: No servlet input parameter specified!");
			}
		}
		catch (Exception e) {
			servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();				
			response.setContentType("text/html");
			System.out.println("Web2cMessenger-GET: Error processing servlet: error processing parameter " + (request.getParameter("param")).substring(0, 6));
		}
		response.flushBuffer();
	}
	
	/**
	 * Post Method
	 * @param request client request
	 * @param response servlet response
	 * @throws ServletException 
	 * @throws IOException 
	 */
	protected synchronized void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		
		try {		
			if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			servletHandler.getWeb2cKeyBoardListener().getKeyboardInput();
			if (request.getParameter("param") != null) {
				String servletInputParameter = request.getParameter("param");
	    		if (servletInputParameter.compareToIgnoreCase("checkAlive") == 0) {
					String str = servletHandler.doPostCheckAlive();
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);		
					response.setContentType("text/html");
					response.getWriter().print(str);
	    		}
	    		if (servletInputParameter.compareToIgnoreCase("login") == 0) {
	    			request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
	    			String str = servletHandler.doPostLogin(request.getRemoteAddr(), request.getSession().getId(), request.getSession().getCreationTime(), true);	
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);					
	    			response.setContentType("text/html");
					response.getWriter().print(str);
	    		}
	    		else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<action>") == 0) {
			    	String str = doPostAction((request.getParameter("param")).substring(8));
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);				
					response.setContentType("text/html");
					response.getWriter().print(str);
	    		}
	    		else if (servletInputParameter.substring(0,8).compareToIgnoreCase("<elogin>") == 0) {
					request.getSession().setMaxInactiveInterval(servletHandler.getHTTPSessionTimeout());
					String str = servletHandler.doPostElogin(servletInputParameter.substring(8), request.getRemoteAddr());
					if (servletHandler.getWeb2cKeyBoardListener().getDebugWebCommunication()) servletHandler.getDebugLogger().appendLog("response to " + request.getRemoteAddr() + ": " + str);				
					response.setContentType("text/html");
					response.getWriter().print(str);
	    		}
			    else {
					response.setContentType("text/html");
			    	System.out.println("Web2cMessenger-GET-POST: Unknown servlet input parameter specified!");
			    }
			}
			else {
				response.setContentType("text/html");
		    	System.out.println("Web2cMessenger-GET-POST: No servlet input parameter specified!");
			}
		}
		catch(Exception e) {
			servletHandler.getDebugLogger().appendLog("request from " + request.getRemoteAddr() + ": " + request.getParameter("param"));
			System.out.println("Time: " + String.valueOf(Calendar.getInstance().getTime().toString()));
			System.err.println(e.toString());
			e.printStackTrace();
			response.setContentType("text/html");
			String servletInputParameter = request.getParameter("param");
			if ((servletInputParameter.compareToIgnoreCase("checkAlive") == 0) || (servletInputParameter.compareToIgnoreCase("login") == 0)) {
				System.out.println("Web2cMessenger-POST: Error processing servlet: error processing parameter " + servletInputParameter);
			}
			else {
				System.out.println("Web2cMessenger-POST: Error processing servlet: error processing parameter " + servletInputParameter.substring(0,8));
			}
		}
		response.flushBuffer();
	}
	
	/**
	 * Returns Twitter control page
	 * @param sessionHandler session handler
	 * @return HTML page
	 */
	private ArrayList<String> getTwitterControlPage(Web2cSessionHandler sessionHandler) {

		ArrayList<String> out = new ArrayList<String>();

		String sessionId = (String)(sessionHandler.getAttribute("id"));
		
		out.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		out.add("<html>");
		out.add("   <head>");
		out.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		out.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		out.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		out.add("       <meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		out.add("       <title>Twitter Control</title>");
		out.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
		out.add("       <script type=\"text/javascript\">");
		out.add("			var xhr;");
		out.add("			var statusLine;");
		out.add("			function initPage() {");
		out.add("				initXhr();");
		out.add("				statusLine = document.getElementById(\"statusLine\");");
		out.add("			}");
		out.add("			function initXhr() {"); 
		out.add("				try {");                                                  
		out.add("					xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		out.add("				} catch (event1) {");                             
		out.add("					try {");
		out.add("						xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		out.add("					} catch (event2) {");
		out.add("						xhr = false;");
		out.add("					}");
		out.add("				}");
		out.add("				if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		out.add("					xhr = new XMLHttpRequest();");                 
		out.add("				}"); 																	
		out.add("			}");
		out.add("			function requestPermit() {");
		out.add("				var now = new Date();");
		out.add("				xhr.open(\"post\", \"/web2cToolkit/" + getServletName() + "?param=<action><sessionId>" + sessionId + "</sessionId><do>requestTwitterPermit</do><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
		out.add("				xhr.onreadystatechange = function() {");
		out.add("					if (xhr.readyState == 4) {");
		out.add("						var ret = xhr.responseText.replace(\"<return>\", \"\").replace(\"</return>\", \"\");");
		out.add("						var url = ret.replace(\"Enable twitter connection: \", \"\");");
		out.add("						statusLine.innerHTML = ret;");
		out.add("						newWindow = window.open(url);");
		out.add("           		};");   
		out.add("           	};"); 
		out.add("				xhr.send(null);");  
		out.add("			}");
		out.add("			function getStatus() {");
		out.add("				var now = new Date();");
		out.add("				xhr.open(\"post\", \"/web2cToolkit/" + getServletName() + "?param=<action><sessionId>" + sessionId + "</sessionId><do>getTwitterStatus</do><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
		out.add("				xhr.onreadystatechange = function() {");
		out.add("					if (xhr.readyState == 4) {");
		out.add("						var ret = xhr.responseText.replace(\"<return>\", \"\").replace(\"</return>\", \"\");");
		out.add("						statusLine.innerHTML = ret;");
		out.add("           		};");   
		out.add("           	};"); 
		out.add("				xhr.send(null);");  
		out.add("			}");
		out.add("			function startService() {");
		out.add("				var now = new Date();");
		out.add("				xhr.open(\"post\", \"/web2cToolkit/" + getServletName() + "?param=<action><sessionId>" + sessionId + "</sessionId><do>startTwitterService</do><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
		out.add("				xhr.onreadystatechange = function() {");
		out.add("					if (xhr.readyState == 4) {");
		out.add("						var ret = xhr.responseText.replace(\"<return>\", \"\").replace(\"</return>\", \"\");");
		out.add("						statusLine.innerHTML = ret;");
		out.add("           		};");   
		out.add("           	};"); 
		out.add("				xhr.send(null);");  
		out.add("			}");
		out.add("			function stopService() {");
		out.add("				var now = new Date();");
		out.add("				xhr.open(\"post\", \"/web2cToolkit/" + getServletName() + "?param=<action><sessionId>" + sessionId + "</sessionId><do>stopTwitterService</do><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
		out.add("				xhr.onreadystatechange = function() {");
		out.add("					if (xhr.readyState == 4) {");
		out.add("						var ret = xhr.responseText.replace(\"<return>\", \"\").replace(\"</return>\", \"\");");
		out.add("						statusLine.innerHTML = ret;");
		out.add("           		};");   
		out.add("           	};"); 
		out.add("				xhr.send(null);");  
		out.add("			}");
		out.add("       </script>");
		out.add("   </head>");
		out.add("	<body onload=\"initPage()\">");
		out.add("		<div align=\"center\"><b><h1>Twitter Control Page</h1></b></div>");
		out.add("		<div align=\"center\">");
		out.add("			<table cellspacing=\"0\" class=\"lined\">");
		out.add("				<tr><td colspan=\"3\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"request\" id=\"request\" value=\"Request Permit\" onclick=\"requestPermit()\"></center></td></tr>");
		out.add("				<tr><td colspan=\"3\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"start\" id=\"start\" value=\"Start Service\" onclick=\"startService()\"></center></td></tr>");
		out.add("				<tr><td colspan=\"3\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"start\" id=\"stop\" value=\"Stop Service\" onclick=\"stopService()\"></center></td></tr>");
		out.add("				<tr><td colspan=\"3\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"status\" id=\"status\" value=\"Get Status\" onclick=\"getStatus()\"></center></td></tr>");
		out.add("			</table>");
		out.add("		</div>");
		out.add("		<p></p><p></p><div align=\"center\" name=\"statusLine\" id=\"statusLine\"></div>");
		out.add("	</body>");
		out.add("</html>");
		
		return out;
	}
	
	/**
	 * Returns Send-Message page
	 * @param sessionHandler session handler
	 * @return HTML page
	 */
	private ArrayList<String> getSendMessagePage(Web2cSessionHandler sessionHandler) {

		ArrayList<String> out = new ArrayList<String>();

		String sessionId = (String)(sessionHandler.getAttribute("id"));
		
		out.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		out.add("<html>");
		out.add("   <head>");
		out.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		out.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		out.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		out.add("       <meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		out.add("       <title>Send Message (SMTP)</title>");
		out.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
		out.add("       <script type=\"text/javascript\">");
		out.add("			var xhr;");
		out.add("			var statusLine;");
		out.add("			function initPage() {");
		out.add("				initXhr();");
		out.add("				statusLine = document.getElementById(\"statusLine\");");
		out.add("			}");
		out.add("			function initXhr() {"); 
		out.add("				try {");                                                  
		out.add("					xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		out.add("				} catch (event1) {");                             
		out.add("					try {");
		out.add("						xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		out.add("					} catch (event2) {");
		out.add("						xhr = false;");
		out.add("					}");
		out.add("				}");
		out.add("				if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		out.add("					xhr = new XMLHttpRequest();");                 
		out.add("				}"); 																	
		out.add("			}");
		out.add("			function sendMessage() {");
		out.add("				var now = new Date();");
		out.add("				var from = document.getElementById(\"from\").value;");
		out.add("				var to = document.getElementById(\"to\").value;");
		out.add("				var subject = document.getElementById(\"subject\").value;");
		out.add("				var message = document.getElementById(\"message\").value;");
		out.add("				xhr.open(\"post\", \"/web2cToolkit/" + getServletName() + "?param=<action><sessionId>" + sessionId + "</sessionId><do>sendMessage</do><service>SMTP</service><from>\" + from + \"</from><to>\" + to + \"</to><subject>\" + subject + \"</subject><message>\" + message + \"</message><timeStamp>\" + String(now.getTime()) + \"</timeStamp></action>\", true);"); 
		out.add("				xhr.onreadystatechange = function() {");
		out.add("					if (xhr.readyState == 4) {");
		out.add("						var ret = xhr.responseText.replace(\"<return>\", \"\").replace(\"</return>\", \"\");");
		out.add("						statusLine.innerHTML = ret;");
		out.add("           		};");   
		out.add("           	};"); 
		out.add("				xhr.send(null);");  
		out.add("			}");
		out.add("       </script>");
		out.add("   </head>");
		out.add("	<body onload=\"initPage()\">");
		out.add("		<div align=\"center\"><b><h1>Send Message Page</h1></b></div>");
		out.add("		<div align=\"center\">");
		out.add("			<table cellspacing=\"0\" class=\"lined\">");
		out.add("				<tr><td align=\"left\"><b>From:    </b></td><td class=\"control\" height=\"50\"><textarea name=\"from\" id=\"from\" rows=\"1\" cols=\"100\"></textarea></td></tr>");
		out.add("				<tr><td align=\"left\"><b>To:      </b></td><td class=\"control\" height=\"50\"><textarea name=\"to\" id=\"to\" rows=\"1\" cols=\"100\"></textarea></td></tr>");
		out.add("				<tr><td align=\"left\"><b>Subject: </b></td><td class=\"control\" height=\"50\"><textarea name=\"subject\" id=\"subject\" rows=\"1\" cols=\"100\"></textarea></td></tr>");
		out.add("				<tr><td align=\"left\"><b>Message: </b></td><td class=\"control\" height=\"50\"><textarea name=\"message\" id=\"message\" rows=\"20\" cols=\"100\"></textarea></td></tr>");
		out.add("				<tr><td> </td><td class=\"control\" height=\"25\"></td></tr>");
		out.add("				<tr><td colspan=\"3\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"send\" id=\"send\" value=\"Send Message\" onclick=\"sendMessage()\"></center></td></tr>");
		out.add("			</table>");
		out.add("		</div>");
		out.add("		<p></p><p></p><div align=\"center\" name=\"statusLine\" id=\"statusLine\"></div>");
		out.add("	</body>");
		out.add("</html>");
		
		return out;
	}
	
	/**
	 * Handles client request to get a messenger page (e.g. TwitterControlPage, SendMessagePage) after successful login procedure
	 * @param parameter request parameter
	 * @param remoteAddress client address
	 * @param requestedURL by client requested URL
	 * @return html code
	 */
	public String doGetPage(String parameter, String remoteAddress, String requestedURL) {

		synchronized (servletHandler.getWeb2cSessionHandlerList()) {
			servletHandler.getWeb2cGarbageCollector().removeDeadSessions(servletHandler.getWeb2cSessionHandlerList());
		}
		String sessionId = servletHandler.parseGetRequestParameter(parameter, "sessionId");
		String page = servletHandler.parseGetRequestParameter(parameter, "fileName");
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			StringBuffer strBfr = new StringBuffer();	
			if (associatedSessionHandler.getAttribute("loginStatus").equals("in")) {
				associatedSessionHandler.setAttribute("loadedConfigFileName", (String)associatedSessionHandler.getAttribute("requestedConfigFileName"));
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));	
				if (page.equals("TwitterControlPage")) {
					for (Iterator<String> it = getTwitterControlPage(associatedSessionHandler).iterator(); it.hasNext();) {
						strBfr.append((String)it.next());
					}			
				}
				else if (page.equals("SendMessagePage")) {
					for (Iterator<String> it = getSendMessagePage(associatedSessionHandler).iterator(); it.hasNext();) {
						strBfr.append((String)it.next());
					}	
				}
				else if (page.equals("DiagnosticsPage")) {
					for (Iterator<String> it = servletHandler.getWeb2cMessengerProbe().getStatus().iterator(); it.hasNext();) {
						strBfr.append((String)it.next());
					}
				}
				else {
					strBfr.append("");
				}	
			}
			else {
				strBfr.append("");
			}
			return strBfr.toString();
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to perform action
	 * @param paramater request parameter
	 * @return return data
	 */
	public String doPostAction(String parameter) {

		synchronized (servletHandler.getWeb2cSessionHandlerList()) {
			servletHandler.getWeb2cGarbageCollector().removeDeadSessions(servletHandler.getWeb2cSessionHandlerList());
		}
		String sessionId = servletHandler.parsePostRequestParameter(parameter, "sessionId")[0];
		String action = servletHandler.parsePostRequestParameter(parameter, "do")[0];
		Web2cSessionHandler associatedSessionHandler = servletHandler.getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {			
			String str = "";
			if (associatedSessionHandler.getAttribute("loginStatus").equals("in")) {
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				if (action.equals("sendMessage")) {  
					String service = servletHandler.parsePostRequestParameter(parameter, "service")[0];
					if (service.equals("SMTP")) {
						String senderAddress = servletHandler.parsePostRequestParameter(parameter, "from")[0];
						String receiverAddress = servletHandler.parsePostRequestParameter(parameter, "to")[0];
						String subjectText = servletHandler.parsePostRequestParameter(parameter, "subject")[0];
						String messageText = servletHandler.parsePostRequestParameter(parameter, "message")[0];
						str = "<return>SMTP sendMessage:" + servletHandler.getWeb2cSMTPInterface().sendMessage(senderAddress, receiverAddress, subjectText, messageText) + "</return>";
					}
					else {
						str = "<return>" + service + ": Messenger service not implemented</return>";
					}
				}
				else if (action.equals("getTwitterStatus")) {  
					str = "<return>Last Twitter status from " + Calendar.getInstance().getTime().toString() + ": " + servletHandler.getWeb2cTwitterInterface().getMessage() + "</return>";
				}
				else if (action.equals("startTwitterService")) {  
					servletHandler.getWeb2cTwitterInterface().startPublishingStatus();
					str = "<return>Try to start updating twitter status ...   Return status (true = started): " + servletHandler.getWeb2cTwitterInterface().getServiceStarted() + "</return>";				
				}
				else if (action.equals("stopTwitterService")) {  
					servletHandler.getWeb2cTwitterInterface().stopPublishingStatus();
					str = "<return>Try to stop updating twitter status ...</return>";
				}
				else if (action.equals("requestTwitterPermit")) {  
					str = "<return>Grant twitter connection: " + servletHandler.getWeb2cTwitterInterface().getAuthorizationURL() + "</return>";
				}
				else {
					str = "<return>" + action + ": Messenger action not implemented</return>";
				}
			}
			return str;
		}
		else {
			return null;
		}
	}
}