package de.desy.web2cToolkit.web2cUtil.web2cDebug;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;

/**
 * @author R.Bacher, DESY
 */
public class Web2cFileLogger{

	private File logFile = null;
	private boolean withTimeStamp = true;
	
	/**
	 * File Logger
	 * @param file log file
	 * @param timeStamp time Stamp flag (with / without)
	 */
	public Web2cFileLogger(File file, boolean timeStamp) {
		
		logFile = file;		
		withTimeStamp = timeStamp;
	}	
	
	/**
	 * Returns the file content (reads the last specified number of lines form the file)
	 * @param number number of lines
	 * @return file content
	 */
	public String[] getLog(String number) { 
		
		ArrayList<String> list = new ArrayList<String>();
		String[] content = null;
		int startAtLine = 0;
		int numberOfLines = 0;
		int fileLength = 0;
		
		try {
			BufferedReader in = new BufferedReader(new FileReader(logFile.getCanonicalFile()));
			boolean eof = false;
			while(!eof) {
				String line = in.readLine();
				if (line == null) eof = true;
				else {
					list.add(line);
				}
			}
		} 
		catch (FileNotFoundException e) {System.out.println("Error opening log file: " + logFile.getName());}
		catch (IOException e) {System.out.println("Error reading log file: " + logFile.getName());}
		
		fileLength = list.size();
		if (number.equals("All")) {
			startAtLine = 0;
			numberOfLines = fileLength;
		}
		else {
			numberOfLines = Integer.valueOf(number);
			startAtLine = fileLength - numberOfLines;
			if (startAtLine < 0) {
				startAtLine = 0;
				numberOfLines = fileLength;
			}
		}
		
		content = new String[numberOfLines];
		for (int i = startAtLine; i < fileLength; i++) {
			content[i] = list.get(i);
		}
		
		return content;
	}
	
	/**
	 * Appends log message to file
	 * @param logMessage log message
	 */
	public void appendLog(String logMessage) {

		BufferedWriter out = null;
		Calendar calendar = Calendar.getInstance(); 
		
		try {	
			if (out == null) {
				out = new BufferedWriter(new FileWriter(logFile.getCanonicalFile(), true));
			}
			if (withTimeStamp) {
				out.write(calendar.getTime().toString() + " " + logMessage);
			}
			else {
				out.write(logMessage);
			}
			out.newLine();
			out.flush();
			out.close();
		} catch (IOException e) {
			System.out.println("Error accessing log file " + logFile.getName());
		}
	}
	
	/**
	 * Clears the log file
	 */
	public void clearLog() {
		
		BufferedWriter out = null;
		
		try {
			out = new BufferedWriter(new FileWriter(logFile.getCanonicalFile()));
			out.flush();
			out.close();
		} catch (IOException e) {
			System.out.println("Error clearing log file " + logFile.getName());
		}
	}
}
