package de.desy.web2cToolkit.web2cUtil.web2cDiagnostics;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cGarbageCollector;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;

/**
 * @author R.Bacher, DESY
 */
public class Web2cArchiveViewerProbe implements IWeb2cConstants {
	
	private ArrayList<Web2cSessionHandler> web2cSessionHandlerList = null;
	private Web2cGarbageCollector garbageCollector = null;
	private Calendar calendar = null;
	private Calendar sessionCreationTime = null;
	private String serviceStarted = "";
	private String versionNumber = "";
	private boolean garbageCollectionIsEnabled = garbageCollectionEnabled;
	private boolean sessionWatcherIsEnabled = sessionWatcherEnabled;

	/**
	 * Administration interface
	 * @param list session handler list
	 * @param gc garbage collector
	 */
	public Web2cArchiveViewerProbe(ArrayList<Web2cSessionHandler>list,  Web2cGarbageCollector gc) {
		
		super();
		web2cSessionHandlerList = list;
		garbageCollector = gc;
		calendar = Calendar.getInstance();
		sessionCreationTime = Calendar.getInstance();
		serviceStarted = String.valueOf(calendar.getTime().toString());
		versionNumber = web2cVersion;
	}
	
	/**
	 * Sets the Web2cToolkit garbage collector enabled/disabled flag
	 * @param flag
	 */
	public void setWeb2cToolkitGarbageCollection(boolean flag) {
		
		sessionWatcherIsEnabled = flag;	
	}
	
	/**
	 * Sets the Java garbage collector enabled/disabled flag
	 * @param flag
	 */
	public void setJavaGarbageCollection(boolean flag) {
		
		garbageCollectionIsEnabled = flag;
	}
	
	/**
	 * Returns the Web2cArchiveViewer status page
	 * return status page
	 */
	public ArrayList<String> getStatus() {

		synchronized (web2cSessionHandlerList) {
			garbageCollector.removeDeadSessions(web2cSessionHandlerList);
		}
		
		ArrayList<String> statusList = new ArrayList<String>();
		
		statusList.add("<div><b><h1>Web2cArchive Status Page (" + (DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL, localSettings)).format(calendar.getTime()) + ")</h1></b></div>");
		
		statusList.add("<div><b><h2>General:</h2></b></div>");
		statusList.add("<div>	Web2cToolkit version number: " + versionNumber + "</div>");		
		statusList.add("<div>	Web2cToolkit started at: " + serviceStarted + "</div>");	
		statusList.add("<br>");
		if (garbageCollectionIsEnabled == true)  statusList.add("<div>	Java garbage collection: enabled</div>");
		if (garbageCollectionIsEnabled == false) statusList.add("<div>	Java garbage collection: disabled</div>");
		if (sessionWatcherIsEnabled == true)  statusList.add("<div>	Web2cToolkit garbage collection: enabled</div>");
		if (sessionWatcherIsEnabled == false) statusList.add("<div>	Web2cToolkit garbage gollection: disabled</div>");
		statusList.add("<br>");
		statusList.add("<div>	HTTP session time out [s]: " + archiveViewerSessionTimeoutInterval + "</div>");
		statusList.add("<br>");
		statusList.add("<div>	Web2cArchiveViewer session time out [s]: " + archiveViewerInactivityTimeoutInterval + "</div>");
		statusList.add("<div>	Maximal number of allowed Web2cArchiveViewer sessions: " + maxArchiveViewerSessionNumber + "</div>");
		statusList.add("<div>	Number of active Web2cArchiveViewer sessions: " + web2cSessionHandlerList.size() + "</div>");
		statusList.add("<br>");
		
		statusList.add("<div><b><h2>Session Details:</h2></b></div>");
		for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
			Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();			
			calendar.setTimeInMillis(Long.valueOf(sessionHandler.getAttribute("lastPoll")));
			sessionCreationTime.setTimeInMillis(Long.valueOf(sessionHandler.getAttribute("id")));
			statusList.add("<div>	" + sessionHandler.getAttribute("clientInfo") + ", " + sessionHandler.getAttribute("httpSessionInfo") + "</div>");
			statusList.add("<div>	Web2cArchiveViewerSession id: " + (String)sessionHandler.getAttribute("id") + " (created at " + (DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, localSettings)).format(sessionCreationTime.getTime()) + "), User: " + (String)sessionHandler.getAttribute("userName") + ", Configuration file loaded: " + sessionHandler.getAttribute("loadedConfigFileName") + ", Last update requested at: " + (DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, localSettings)).format(calendar.getTime()) + "</div>");
			statusList.add("<br>");
		}
		
		return statusList;
	}
}
