package de.desy.web2cToolkit.web2cUtil.web2cDiagnostics;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2cMessenger.Web2cTwitterInterface;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cGarbageCollector;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;

/**
 * @author R.Bacher, DESY
 */
public class Web2cMessengerProbe implements IWeb2cConstants {
	
	private ArrayList<Web2cSessionHandler> web2cSessionHandlerList = null;
	private Web2cCommonDeviceAccess commonDeviceAccess = null;
	private Web2cTwitterInterface twitterInterface = null;
	private Calendar calendar = null;
	private Calendar sessionCreationTime = null;
	private String serviceStarted = "";
	private String versionNumber = "";
	private boolean garbageCollectionIsEnabled = garbageCollectionEnabled;
	private boolean sessionWatcherIsEnabled = sessionWatcherEnabled;

	/**
	 * Administration interface
	 * @param list session handler list
	 * @param twitter twitter interface
	 * @param cda common device access
	 * @param gc garbage collector
	 */
	public Web2cMessengerProbe(ArrayList<Web2cSessionHandler>list, Web2cTwitterInterface twitter, Web2cCommonDeviceAccess cda, Web2cGarbageCollector gc) {
		
		super();
		web2cSessionHandlerList = list;
		twitterInterface = twitter;
		commonDeviceAccess = cda;
		calendar = Calendar.getInstance();
		sessionCreationTime = Calendar.getInstance();
		serviceStarted = String.valueOf(calendar.getTime().toString());
		versionNumber = web2cVersion;
	}
	
	/**
	 * Sets the Web2cToolkit garbage collector enabled/disabled flag
	 * @param flag
	 */
	public void setWeb2cToolkitGarbageCollection(boolean flag) {
		
		sessionWatcherIsEnabled = flag;	
	}
	
	/**
	 * Sets the Java garbage collector enabled/disabled flag
	 * @param flag
	 */
	public void setJavaGarbageCollection(boolean flag) {
		
		garbageCollectionIsEnabled = flag;
	}
	
	/**
	 * Returns the Web2cMessenger status page
	 * return status page
	 */
	public ArrayList<String> getStatus() {
		
		ArrayList<String> statusList = new ArrayList<String>();
		
		statusList.add("<div><b><h1>Web2cMessenger Status Page (" + (DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL, localSettings)).format(calendar.getTime()) + ")</h1></b></div>");
		
		statusList.add("<div><b><h2>General:</h2></b></div>");
		statusList.add("<div>	Web2cToolkit version number: " + versionNumber + "</div>");		
		statusList.add("<div>	Web2cToolkit started at: " + serviceStarted + "</div>");	
		statusList.add("<br>");
		if (garbageCollectionIsEnabled == true)  statusList.add("<div>	Java garbage collection: enabled</div>");
		if (garbageCollectionIsEnabled == false) statusList.add("<div>	Java garbage collection: disabled</div>");
		if (sessionWatcherIsEnabled == true)  statusList.add("<div>	Web2cToolkit garbage collection: enabled</div>");
		if (sessionWatcherIsEnabled == false) statusList.add("<div>	Web2cToolkit garbage gollection: disabled</div>");
		statusList.add("<br>");
		statusList.add("<div>	HTTP session time out [s]: " + messengerSessionTimeoutInterval + "</div>");
		statusList.add("<br>");
		statusList.add("<div>	Web2cMessenger session time out [s]: " + messengerInactivityTimeoutInterval + "</div>");
		statusList.add("<div>	Maximal number of allowed Web2cMessenger sessions: " + maxMessengerSessionNumber + "</div>");
		statusList.add("<div>	Number of active Web2cMessenger sessions: " + web2cSessionHandlerList.size() + "</div>");
		statusList.add("<br>");
		
		statusList.add("<div><b><h2>Twitter:</h2></b></div>");
		if (twitterInterface.getServiceAvailable() == true) {
			statusList.add("<div>	Service: available</div>");
			if (twitterInterface.getServiceStarted() == true) {
				statusList.add("<div>	Publishing: started</div>");
				statusList.add("<div>	Last status published: " + twitterInterface.getMessage() + "</div>");
			}
			if (twitterInterface.getServiceStarted() == false) statusList.add("<div>	Publishing: stopped</div>");
		}
		if (twitterInterface.getServiceAvailable() == false) statusList.add("<div>	Service: not available</div>");
		statusList.add("<br>");
		
		statusList.add("<div><b><h2>Session Details:</h2></b></div>");
		for (Iterator<Web2cSessionHandler> it = web2cSessionHandlerList.iterator(); it.hasNext();) {
			Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();			
			calendar.setTimeInMillis(Long.valueOf(sessionHandler.getAttribute("lastPoll")));
			sessionCreationTime.setTimeInMillis(Long.valueOf(sessionHandler.getAttribute("id")));
			statusList.add("<div>	" + sessionHandler.getAttribute("clientInfo") + ", " + sessionHandler.getAttribute("httpSessionInfo") + "</div>");
			statusList.add("<div>	Web2cMessengerSession id: " + (String)sessionHandler.getAttribute("id") + " (created at " + (DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, localSettings)).format(sessionCreationTime.getTime()) + "), User: " + (String)sessionHandler.getAttribute("userName") + ", Configuration file loaded: " + sessionHandler.getAttribute("loadedConfigFileName") + ", Last update requested at: " + (DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, localSettings)).format(calendar.getTime()) + "</div>");
			statusList.add("<br>");
		}	
		
		statusList.add("<div><b><h2>Device Access Details:</h2></b></div>");
		statusList.add("<div>	Total Number of monitors: " + commonDeviceAccess.getPropertyList().size() + "</div>");
		int numberOfRequesters = 0;
		for (Iterator<Integer> it1 = commonDeviceAccess.getNumberOfRequesterList().iterator(); it1.hasNext();) {
			numberOfRequesters = numberOfRequesters + (int)it1.next();
		}
		statusList.add("<div>	Total Number of requesters: " + numberOfRequesters + "</div>");
		statusList.add("<br>");
		Iterator<Integer> it2 = commonDeviceAccess.getNumberOfRequesterList().iterator();
		for (Iterator<String> it1 = commonDeviceAccess.getPropertyList().iterator(); it1.hasNext();) {
			String property = (String)it1.next();
			int number = (int)it2.next();
			statusList.add("<div>	Monitor established for: " + property + ", Number of requesters: " + number + "</div>");
		}
		
		return statusList;
	}
}
