package de.desy.web2cToolkit.web2cUtil.web2cFileIO;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import java.util.Iterator;

/**
 * @author R.Bacher, DESY
 */
public class Web2cGatewayConfigReader implements IWeb2cConstants {
	
	private File configFile = null;		
	private ArrayList<String> expertNameList = new ArrayList<String>();
	private ArrayList<String> supervisorNameList = new ArrayList<String>();
	private ArrayList<String> expertAddressList = new ArrayList<String>();
	private ArrayList<String> exclusiveNameList = new ArrayList<String>();
	private ArrayList<String> monitorList = new ArrayList<String>();

	/**
	 * Reader for gateway config file 
	 * @param file name of gateway config file
	 */
	public Web2cGatewayConfigReader(File file) {
		
		configFile = file;
		parseInitFile();
	}
	
	/**
	 * Returns the list of monitor links for early link registering
	 * @return list
	 */
	public ArrayList<String> getMonitorList() {
		
		return monitorList;
	}
	
	/**
	 * Returns the list of expert names
	 * @return list
	 */
	public ArrayList<String> getExpertNameList() {
		
		return expertNameList;
	}
	
	/**
	 * Returns the list of supervisor names
	 * @return list
	 */
	public ArrayList<String> getSupervisorNameList() {
		
		return supervisorNameList;
	}
	
	/**
	 * Returns the list of exclusive user names
	 * @return list
	 */
	public ArrayList<String> getExclusiveNameList() {
		
		return exclusiveNameList;
	}
	
	/**
	 * Returns the list of expert addresses
	 * @return list
	 */
	public ArrayList<String> getExpertAddressList() {
		
		return expertAddressList;
	}
	
	@SuppressWarnings("unchecked")
	private void parseInitFile() {
		
		expertNameList.clear();
		supervisorNameList.clear();
		exclusiveNameList.clear();
		expertAddressList.clear();
		monitorList.clear();
		
	   	SAXBuilder builder = null;
    	Document configDoc = null;
    	Element config = null;
    	List<Element> configList = null;
    	Element configElement = null;
    	
		builder = new SAXBuilder();
		try {
			configDoc = builder.build(configFile);			
			config = configDoc.getRootElement();
			configList = (List<Element>)config.getChildren();   
			for (Iterator<Element> it1 = configList.iterator(); it1.hasNext();) {
				configElement = (Element)it1.next(); 
				String value = configElement.getValue();
				if (!value.equals("") && !value.equals(" ")) {
					if ((configElement.getName()).equals("expert")) expertNameList.add(value);
					if ((configElement.getName()).equals("supervisor")) supervisorNameList.add(value);
					if ((configElement.getName()).equals("exclusive")) exclusiveNameList.add(value);
					if ((configElement.getName()).equals("ipAddress")) expertAddressList.add(value);
					if ((configElement.getName()).equals("uri")) monitorList.add(value);
				}
			}
		} catch (JDOMException e) {
			System.out.println("Error parsing gateway config file.");
		} catch (IOException e) {
			System.out.println("Error reading gateway config file.");
		}
	}
}
