package de.desy.web2cToolkit.web2cUtil.web2cFileIO;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import java.util.Iterator;

/**
 * @author R.Bacher, DESY
 */
public class Web2cLogbookConfigReader implements IWeb2cConstants {
	
	private File configFile = null;		
	private String caption = "";
	private String periode = "";
	private ArrayList<String> expertNameList = new ArrayList<String>();
	private ArrayList<String> supervisorNameList = new ArrayList<String>();
	private ArrayList<String> expertAddressList = new ArrayList<String>();
	private ArrayList<String> exclusiveNameList = new ArrayList<String>();
	private ArrayList<String> categoryKeywordList = new ArrayList<String>();
	private ArrayList<String> importanceLevelList = new ArrayList<String>();
	private ArrayList<String> recipientNameList = new ArrayList<String>();
	private ArrayList<String> reportCaptionList = new ArrayList<String>();
	private ArrayList<String> reportCategoryList = new ArrayList<String>();
	private ArrayList<String> reportImportanceList = new ArrayList<String>();
	private ArrayList<String> reportModeList = new ArrayList<String>();
	private ArrayList<String> reportConditionList = new ArrayList<String>();
	private ArrayList<ArrayList<String>> reportRecipientsList = new ArrayList<ArrayList<String>>();
	private ArrayList<ArrayList<String>> reportTextLinesList = new ArrayList<ArrayList<String>>();	
	private ArrayList<String> monitorList = new ArrayList<String>();

	/**
	 * Reader for logbook config file 
	 * @param file name of logbook config file
	 */
	public Web2cLogbookConfigReader(File file) {
		
		configFile = file;
		readConfigFile();
	}
	
	/**
	 * Returns the list of monitor links
	 * @return list
	 */
	public ArrayList<String> getMonitorList() {
		
		return monitorList;
	}
	
	/**
	 * Returns the logbook caption
	 * @return text
	 */
	public String getCaption() {
		
		return caption;
	}
	
	/**
	 * Returns the logbook periode (i.e.: shift, day, month, year)
	 * @return text
	 */
	public String getPeriode() {
		
		return periode;
	}
	
	/**
	 * Returns the list of expert names
	 * @return list
	 */
	public ArrayList<String> getExpertNameList() {
		
		return expertNameList;
	}
	
	/**
	 * Returns the list of supervisor names
	 * @return list
	 */
	public ArrayList<String> getSupervisorNameList() {
		
		return supervisorNameList;
	}
	
	/**
	 * Returns the list of exclusive user names
	 * @return list
	 */
	public ArrayList<String> getExclusiveNameList() {
		
		return exclusiveNameList;
	}
	
	/**
	 * Returns the list of expert addresses
	 * @return list
	 */
	public ArrayList<String> getExpertAddressList() {
		
		return expertAddressList;
	}	
	
	/**
	 * Returns the list of category keywords
	 * @return list
	 */
	public ArrayList<String> getCategoryKeywordList() {
		
		return categoryKeywordList;
	}	
	
	/**
	 * Returns the list of importance levels
	 * @return list
	 */
	public ArrayList<String> getImportanceLevelList() {
		
		return importanceLevelList;
	}		
	
	/**
	 * Returns the list of recipient names
	 * @return list
	 */
	public ArrayList<String> getRecipientNameList() {
		
		return recipientNameList;
	}	
	
	/**
	 * Returns the list of report captions
	 * @return list
	 */
	public ArrayList<String> getReportCaptionList() {
		
		return reportCaptionList;
	}	
	
	/**
	 * Returns the list of reportcategory keywords
	 * @return list
	 */
	public ArrayList<String> getReportCategoryList() {
		
		return reportCategoryList;
	}
	
	/**
	 * Returns the list of report importance levels
	 * @return list
	 */
	public ArrayList<String> getReportImportanceList() {
		
		return reportImportanceList;
	}
	
	/**
	 * Returns the list of report modes (i.e. auto == report written if logging condition changed to true, manual == report written on user intervention)
	 * @return list
	 */
	public ArrayList<String> getReportModeList() {
		
		return reportModeList;
	}
	
	/**
	 * Returns the list of report logging condition (only applicable if mode == auto)
	 * @return list
	 */
	public ArrayList<String> getReportConditionList() {
		
		return reportConditionList;
	}
	
	/**
	 * Returns the list of report recipient lists
	 * @return list of lists
	 */
	public ArrayList<ArrayList<String>> getReportRecipientsList() {
		
		return reportRecipientsList;
	}
	
	/**
	 * Returns the list of report text line lists
	 * @return list of lists
	 */
	public ArrayList<ArrayList<String>> getReportTextLinesList() {
		
		return reportTextLinesList;
	}
	
	/**
	 * Reads the logbook config file
	 */
	@SuppressWarnings("unchecked")
	private void readConfigFile() {
		
		caption = "";
		periode = "";
		expertNameList.clear();
		supervisorNameList.clear();
		exclusiveNameList.clear();
		expertAddressList.clear();
		categoryKeywordList.clear();
		importanceLevelList.clear();
		recipientNameList.clear();
		reportCaptionList.clear();
		reportCategoryList.clear();
		reportImportanceList.clear();
		reportModeList.clear();
		reportConditionList.clear();
		reportRecipientsList.clear();
		reportTextLinesList.clear();
		
		caption = "Default Logbook";
		periode = "month";
		
	   	SAXBuilder builder = null;
    	Document configDoc = null;
    	Element config = null;
    	List<Element> configList = null;
    	Element configElement = null;
    	
		builder = new SAXBuilder();
		try {
			configDoc = builder.build(configFile);			
			config = configDoc.getRootElement();
			configList = (List<Element>)config.getChildren();   
			for (Iterator<Element> it = configList.iterator(); it.hasNext();) {
				configElement = (Element)it.next(); 
				String configElementValue = configElement.getValue();
				if ((configElement.getName()).equals("caption")) caption = configElementValue;
				if ((configElement.getName()).equals("periode")) periode = configElementValue;
				if ((configElement.getName()).equals("expert")) {
					List<Element> list = (List<Element>)configElement.getChildren();   
					for (Iterator<Element> it1 = list.iterator(); it1.hasNext();) {
						Element listElement = (Element)it1.next();
						if (!listElement.getValue().equals("") && !listElement.getValue().equals(" ")) expertNameList.add(listElement.getValue()); 
					}
				}
				if ((configElement.getName()).equals("supervisor")) {
					List<Element> list = (List<Element>)configElement.getChildren();   
					for (Iterator<Element> it1 = list.iterator(); it1.hasNext();) {
						Element listElement = (Element)it1.next();
						if (!listElement.getValue().equals("") && !listElement.getValue().equals(" ")) supervisorNameList.add(listElement.getValue());
					}
				}
				if ((configElement.getName()).equals("ipAddress")) {
					List<Element> list = (List<Element>)configElement.getChildren();   
					for (Iterator<Element> it1 = list.iterator(); it1.hasNext();) {
						Element listElement = (Element)it1.next();
						if (!listElement.getValue().equals("") && !listElement.getValue().equals(" ")) expertAddressList.add(listElement.getValue());
					}
				}
				if ((configElement.getName()).equals("exclusive")) {
					List<Element> list = (List<Element>)configElement.getChildren();   
					for (Iterator<Element> it1 = list.iterator(); it1.hasNext();) {
						Element listElement = (Element)it1.next();
						if (!listElement.getValue().equals("") && !listElement.getValue().equals(" ")) exclusiveNameList.add(listElement.getValue());
					}
				}
				if ((configElement.getName()).equals("category")) {
					List<Element> list = (List<Element>)configElement.getChildren();   
					for (Iterator<Element> it1 = list.iterator(); it1.hasNext();) {
						Element listElement = (Element)it1.next();
						if (!listElement.getValue().equals("") && !listElement.getValue().equals(" ")) categoryKeywordList.add(listElement.getValue());
					}
				}
				if ((configElement.getName()).equals("importance")) {
					List<Element> list = (List<Element>)configElement.getChildren();   
					for (Iterator<Element> it1 = list.iterator(); it1.hasNext();) {
						Element listElement = (Element)it1.next();
						if (!listElement.getValue().equals("") && !listElement.getValue().equals(" ")) importanceLevelList.add(listElement.getValue());
					}
				}
				if ((configElement.getName()).equals("recipient")) {
					List<Element> list = (List<Element>)configElement.getChildren();   
					for (Iterator<Element> it1 = list.iterator(); it1.hasNext();) {
						Element listElement = (Element)it1.next();
						if (!listElement.getValue().equals("") && !listElement.getValue().equals(" ")) recipientNameList.add(listElement.getValue());
					}
				}
				if ((configElement.getName()).equals("report")) {
					List<Element> recordList = (List<Element>)configElement.getChildren();   
					for (Iterator<Element> it1 = recordList.iterator(); it1.hasNext();) {
						Element record = (Element)it1.next(); 
						String recordValue = record.getValue();
						if (!recordValue.equals("") && !recordValue.equals(" ")) {
							List<Element> recordElementList = (List<Element>)record.getChildren();   
							for (Iterator<Element> it2 = recordElementList.iterator(); it2.hasNext();) {
								Element recordElement = (Element)it2.next(); 
								String recordElementValue = recordElement.getValue();
								if ((recordElement.getName()).equals("caption")) reportCaptionList.add(recordElementValue);
								if ((recordElement.getName()).equals("category")) reportCategoryList.add(recordElementValue);
								if ((recordElement.getName()).equals("importance")) reportImportanceList.add(recordElementValue);
								if ((recordElement.getName()).equals("mode")) reportModeList.add(recordElementValue);
								if ((recordElement.getName()).equals("condition")) {
									reportConditionList.add(recordElementValue);
									monitorList.add(recordElementValue.split("=")[0]);
								}
								if ((recordElement.getName()).equals("recipient")) {
									ArrayList<String> nameList = new ArrayList<String>();
									List<Element> list = (List<Element>)recordElement.getChildren();   
									for (Iterator<Element> it3 = list.iterator(); it3.hasNext();) {
										nameList.add(((Element)it3.next()).getValue()); 
									}
									reportRecipientsList.add(nameList); 
								}
								if ((recordElement.getName()).equals("text")) {
									ArrayList<String> lineList = new ArrayList<String>();
									List<Element> list = (List<Element>)recordElement.getChildren();   
									for (Iterator<Element> it3 = list.iterator(); it3.hasNext();) {
										String line = ((Element)it3.next()).getValue();
										lineList.add(line); 
										monitorList.add(line.split(",")[2]);
									}
									reportTextLinesList.add(lineList); 
								}
							}
						}
					}
				}
			}
		} catch (JDOMException e) {
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error parsing logbook config file.");
		} catch (IOException e) {			
			System.err.println(e.toString());
			e.printStackTrace();
			System.out.println("Error reading logbook config file.");
		}
	}
}
