package de.desy.web2cToolkit.web2cUtil.web2cFileIO;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * @author R.Bacher, DESY
 */
public class Web2cLogbookInitFileWriterAndReader implements IWeb2cConstants {

	private boolean serviceAutoEnabledOption = false;
	private ArrayList<String> fileList = new ArrayList<String>();
	private ArrayList<String> recipientList = new ArrayList<String>();
	private String initFileName = "";
	private boolean initFileRead = false;
	
    /**
     * Reads / Writes the Web2cLogbook init file
     * Constructor
     * @param name init file name
     */
	public Web2cLogbookInitFileWriterAndReader(String name) {

		initFileName = name;
		readInitializationFile();
	}	
	
	/**
	* Returns the init file read status
	* @return status
	*/
	public boolean getInitFileReadStatus() {
			
		return initFileRead;
	}	
	
	/**
	* Returns the service auto-enabled-option.
	* @return option
	*/
	public String getServiceAutoEnabledOption() {
		
		String ret = "no";
		
		if (serviceAutoEnabledOption == true) ret = "yes";
		else ret = "no";
			
		return ret;
	}		
	
	/**
	* Sets the service auto-enabled-option.
	* @param text option
	*/
	public void setServiceAutoEnabledOption(String text) {
		
		if (text.equals("yes")) serviceAutoEnabledOption  = true;
		else serviceAutoEnabledOption  = false;
	}		
	
	/**
	* Saves the service auto-enabled-option.
	* @param text option
	*/
	public void saveServiceAutoEnabledOption(String text) {
		
		setServiceAutoEnabledOption(text);
		writeInitializationFile();
		readInitializationFile();
	}
	
	/**
	 * Returns the list of logbook files to be scanned for auto-reporting
	 * @return file list
	 */
	public ArrayList<String> getAutoReportingFileList() {
		
		return fileList;
	}
	
	/**
	 * Sets the list of logbook files to be scanned for auto-reporting
	 * @param file list
	 */
	public void setAutoReportingFileList(ArrayList<String> list) {
		
		fileList = list;
	}
	
	/**
	 * Saves the list of logbook files to be scanned for auto-reporting
	 * @param file list
	 */
	public void saveAutoReportingFileList(ArrayList<String> list) {
		
		setAutoReportingFileList(list);
		writeInitializationFile();
		readInitializationFile();
	}
	
	/**
	 * Returns the list of notification recipients
	 * @return list
	 */
	public ArrayList<String> getNotificationRecipientList() {
		
		return recipientList;
	}
	
	/**
	 * Sets the list of notification recipients
	 * @param list
	 */
	public void setNotificationRecipientList(ArrayList<String> list) {
		
		recipientList = list;
	}
	
	/**
	 * Saves the list of notification recipients
	 * @param list
	 */
	public void saveANotificationRecipientList(ArrayList<String> list) {
		
		setAutoReportingFileList(list);
		writeInitializationFile();
		readInitializationFile();
	}
	
    /**
     * Write the Web2cLogbook initialization file
     */
	private void writeInitializationFile() {
		
	    try {
	    	BufferedWriter out = new BufferedWriter(new FileWriter((new File(registryDir + File.separator + initFileName)).getCanonicalFile()));
	    	out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); out.newLine();
	    	out.write("<logbook>"); out.newLine();
	    	out.write("	  <autoFile>"); out.newLine();   
			for (Iterator<String> it = fileList.iterator(); it.hasNext();) {
		    	out.write("	  	  <file>" + it.next() + "</file>"); out.newLine();  
			}
	    	out.write("	  </autoFile>"); out.newLine();
	    	out.write("	  <notification>"); out.newLine();   
			for (Iterator<String> it = recipientList.iterator(); it.hasNext();) {
		    	out.write("	  	  <recipient>" + it.next() + "</recipient>"); out.newLine();  
			}
	    	out.write("	  </notification>"); out.newLine();
	    	if (serviceAutoEnabledOption == true) out.write("   <autoReporting>yes</autoReporting>");
	    	else out.write("   <autoReporting>no</autoReporting>");
	    	out.newLine();
	    	out.write("</logbook>"); out.newLine();
			out.flush();
			out.close();
		} catch (IOException e) {
			System.out.println("Error writing Web2cLogbookInit file.");
		}
	}
	
    /**
     * Reads the Web2cLogbook initialization file
     */
	@SuppressWarnings("unchecked")
	private void readInitializationFile() {

		try {
			SAXBuilder builder = new SAXBuilder();
			Document root = builder.build(new File(registryDir + File.separator + initFileName));
			Element logbook = root.getRootElement();
			List<Element> logbookAttributeList = (List<Element>)logbook.getChildren();    
			for (Iterator<Element> it1 = logbookAttributeList.iterator(); it1.hasNext();) {
				Element logbookAttribute = (Element)it1.next(); 	
				if ((logbookAttribute.getName()).equals("autoFile")) {
					List<Element> autoFileList = (List<Element>)logbookAttribute.getChildren();  
					for (Iterator<Element> it2 = autoFileList.iterator(); it2.hasNext();) {
						Element file = (Element)it2.next(); 
						if (!file.getValue().equals("") && !file.getValue().equals(" ")) fileList.add(file.getValue());
					}
				} 	
				else if ((logbookAttribute.getName()).equals("notification")) {
					List<Element> notificationRecipientList = (List<Element>)logbookAttribute.getChildren();  
					for (Iterator<Element> it2 = notificationRecipientList.iterator(); it2.hasNext();) {
						Element recipient = (Element)it2.next(); 
						if (!recipient.getValue().equals("") && !recipient.getValue().equals(" ")) recipientList.add(recipient.getValue());
					}
				} 
				else if ((logbookAttribute.getName()).equals("autoReporting")) {
					if (logbookAttribute.getValue().equals("yes")) serviceAutoEnabledOption = true;
					else serviceAutoEnabledOption = false;
				}
			}
			initFileRead = true;
		} 
		catch (JDOMException e) {
			initFileRead = false;
			System.out.println("Error parsing logbook init file! Logbook auto-reporting service not available!");
		} 
		catch (IOException e) {
			initFileRead = false;
			System.out.println("Error reading logbook init file! Logbook auto-reporting service not available!");
		}
	} 
}