package de.desy.web2cToolkit.web2cUtil.web2cFileIO;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import java.util.Iterator;

/**
 * @author R.Bacher, DESY
 */
public class Web2cMessengerConfigReader implements IWeb2cConstants {
	
	private File configFile = null;		
	private ArrayList<String> expertNameList = new ArrayList<String>();
	private ArrayList<String> supervisorNameList = new ArrayList<String>();
	private ArrayList<String> expertAddressList = new ArrayList<String>();
	private ArrayList<String> exclusiveNameList = new ArrayList<String>();
	private ArrayList<String> monitorList = new ArrayList<String>();
	private ArrayList<String> serviceNameList = new ArrayList<String>();
	private ArrayList<ArrayList<String>> serviceConfigValueLists = new ArrayList<ArrayList<String>>();
	private ArrayList<ArrayList<String>> serviceConfigNameLists = new ArrayList<ArrayList<String>>();

	/**
	 * Reader for gateway config file 
	 * @param file name of gateway config file
	 */
	public Web2cMessengerConfigReader(File file) {
		
		configFile = file;
		parseConfigFile();
	}
	
	/**
	 * Returns the list of monitor links for early link registering
	 * @return list
	 */
	public ArrayList<String> getMonitorList() {
		
		return monitorList;
	}
	
	/**
	 * Returns the list of expert names
	 * @return list
	 */
	public ArrayList<String> getExpertNameList() {
		
		return expertNameList;
	}
	
	/**
	 * Returns the list of supervisor names
	 * @return list
	 */
	public ArrayList<String> getSupervisorNameList() {
		
		return supervisorNameList;
	}
	
	/**
	 * Returns the list of exclusive user names
	 * @return list
	 */
	public ArrayList<String> getExclusiveNameList() {
		
		return exclusiveNameList;
	}
	
	/**
	 * Returns the list of expert addresses
	 * @return list
	 */
	public ArrayList<String> getExpertAddressList() {
		
		return expertAddressList;
	}
	
	/**
	 * Returns the list of service names
	 * @return list
	 */
	public ArrayList<String> getServiceNameList() {
		
		return serviceNameList;
	}
	
	/**
	 * Returns the list of service config name lists
	 * @return list
	 */
	public ArrayList<ArrayList<String>> getServiceConfigNameLists() {
		
		return serviceConfigNameLists;
	}
	
	/**
	 * Returns the list of service config value lists
	 * @return list
	 */
	public ArrayList<ArrayList<String>> getServiceConfigValueLists() {
		
		return serviceConfigValueLists;
	}
	
	@SuppressWarnings("unchecked")
	private void parseConfigFile() {
		
		expertNameList.clear();
		supervisorNameList.clear();
		exclusiveNameList.clear();
		expertAddressList.clear();
		monitorList.clear();
		serviceNameList.clear();
		serviceConfigValueLists.clear();
		serviceConfigNameLists.clear();
		
	   	SAXBuilder builder = null;
    	Document configDoc = null;
    	Element config = null;
    	List<Element> configList = null;
    	Element configElement = null;
    	List<Element> serviceList = null;
    	Element serviceElement = null;
    	List<Element> serviceConfigList = null;
    	Element serviceConfigElement = null;
    	
		builder = new SAXBuilder();
		try {
			configDoc = builder.build(configFile);			
			config = configDoc.getRootElement();
			configList = (List<Element>)(config.getChildren());   
			for (Iterator<Element> it1 = configList.iterator(); it1.hasNext();) {
				configElement = (Element)it1.next(); 
				String configElementValue = configElement.getValue();
				if (!configElementValue.equals("") && !configElementValue.equals(" ")) {
					if ((configElement.getName()).equals("expert")) expertNameList.add(configElementValue);
					if ((configElement.getName()).equals("supervisor")) supervisorNameList.add(configElementValue);
					if ((configElement.getName()).equals("exclusive")) exclusiveNameList.add(configElementValue);
					if ((configElement.getName()).equals("ipAddress")) expertAddressList.add(configElementValue);
					if ((configElement.getName()).equals("uri")) monitorList.add(configElementValue);
					if ((configElement.getName()).equals("service")) {
						serviceList = (List<Element>)configElement.getChildren();
						for (Iterator<Element> it2 = serviceList.iterator(); it2.hasNext();) {
							serviceElement = (Element)it2.next(); 
							String serviceElementValue = serviceElement.getValue();
							String serviceName = serviceElement.getName();
//							System.out.println(serviceName);
							if (!serviceElementValue.equals("") && !serviceElementValue.equals(" ")) {
								serviceConfigList = (List<Element>)serviceElement.getChildren();
								ArrayList<String> configNameLists = new ArrayList<String>();
								ArrayList<String> configValueLists = new ArrayList<String>();
								for (Iterator<Element> it3 = serviceConfigList.iterator(); it3.hasNext();) {
									serviceConfigElement = (Element)it3.next(); 
									configNameLists.add(serviceConfigElement.getName());
									configValueLists.add(serviceConfigElement.getValue());
//									System.out.println(serviceConfigElement.getName() + " " + serviceConfigElement.getValue());
								}
								serviceNameList.add(serviceName);
								serviceConfigNameLists.add(configNameLists);
								serviceConfigValueLists.add(configValueLists);
							}
						}
					}
				}
			}
		} catch (JDOMException e) {
			System.out.println("Error parsing messenger config file.");
		} catch (IOException e) {
			System.out.println("Error reading messenger config file.");
		}
	}
}
