package de.desy.web2cToolkit.web2cUtil.web2cFileIO;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import java.util.Iterator;

/**
 * @author R.Bacher, DESY
 */
public class Web2cRegistryWriterAndReader {
	
	private File userRegistryFile = null;
	private ArrayList<String> name = null;
	private ArrayList<String> password= null;

	/**
	 * Writer and reader for Registry File 
	 * @param file name of registry file
	 */
	public Web2cRegistryWriterAndReader(File file) {
		
		userRegistryFile = file;
		name = new ArrayList<String>();
		password = new ArrayList<String>();
		readRegistry();
	}	
	
	/**
	 * Returns user names stored in the user registry
	 * @return user names
	 */
	public String[] getUserNames() {
		
		int i = 0;
		String[] list = new String[name.size()];

		for (Iterator<String> it1 = name.iterator(); it1.hasNext();) {
			list[i] = (String)(it1.next());
			i = i + 1;
		}
		
		return list;
	}	
	
	/**
	 * Adds or modifies a registry entry (user name and password)
	 * @param user name to be added or modified
	 * @param user password to be added or modified
	 * @return resulting list of user names
	 */
	public String[] addOrModifyRegistryEntry(String userName, String userPassword) {
		
		int index = -1;
		
		for (Iterator<String> it1 = name.iterator(); it1.hasNext();) {
			String entry = (String)(it1.next());
			if (entry.equals(userName)) {
				index = name.indexOf(entry);
			}
		}
		if (index == -1) {
			name.add(userName);
			password.add(userName);
		}
		else {
			name.set(index, userName);
			password.set(index, userPassword);
		}
		writeRegistry();
		readRegistry();
		
		return getUserNames();
	}
	
	/**
	 * Deletes a registry entry (user name and password)
	 * @param user name associated with the entry to be deleted
	 * @return resulting list of user names
	 */
	public String[] removeRegistryEntry(String userName) {
		
		int index = -1;
		
		for (Iterator<String> it1 = name.iterator(); it1.hasNext();) {
			String entry = (String)(it1.next());
			if (entry.equals(userName)) {
				index = name.indexOf(entry);
			}
		}
		if (index >= 0) {
			name.remove(index);
			password.remove(index);
			writeRegistry();
			readRegistry();
		}
		
		return getUserNames();
		
	}
	
	/**
	 * Returns the password associated with the user name specified
	 * @param userName user name
	 * @return user password
	 */
	public String getPassword(String userName) {
		
		String userPassword = null;
		
		Iterator<String> it2 = password.iterator();
		for (Iterator<String> it1 = name.iterator(); it1.hasNext();) {
			String str = (String)(it2.next());
			if (((String)(it1.next())).equals(userName)) {
				userPassword = str;
			}
		}
		
		return userPassword;
	}
	
    /**
     * Writes the user registry file
     */
	private void writeRegistry() {
		
	    try {
	    	BufferedWriter out = new BufferedWriter(new FileWriter(userRegistryFile.getCanonicalFile()));
	    	out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); out.newLine();
			out.write("<registry>"); out.newLine();	
			Iterator<String> it2 = password.iterator();
			for (Iterator<String> it1 = name.iterator(); it1.hasNext();) {
				out.write("   <user>"); out.newLine();
				out.write("      <name>" + (String)(it1.next()) + "</name>"); out.newLine();
				out.write("      <password>" + (String)(it2.next()) + "</password>"); out.newLine();
				out.write("   </user>"); out.newLine();
			}
			out.write("</registry>"); out.newLine();
			out.flush();
			out.close();
		} catch (IOException e) {
			System.out.println("Error writing user registry file.");
		}
	}
	
    /**
     * Reads the user registry file
     */
	@SuppressWarnings("unchecked")
	private void readRegistry() {
		
	   	SAXBuilder builder = null;
    	Document userRegistry = null;
    	Element registry = null;
    	List<Element> userList = null;
    	Element user = null;
    	List<Element> userAttributes = null;
    	Element userAttribute = null;

		builder = new SAXBuilder();
		try {
			userRegistry = builder.build(userRegistryFile);
		} catch (JDOMException e) {
			System.out.println("Error parsing user registry file.");
		} catch (IOException e) {
			System.out.println("Error reading user registry file.");
		}
		
		registry = userRegistry.getRootElement();
		userList = (List<Element>)registry.getChildren();   
		for (Iterator<Element> it1 = userList.iterator(); it1.hasNext();) {
			user = (Element)it1.next();  
			userAttributes = user.getChildren();
			for (Iterator<Element> it2 = userAttributes.iterator(); it2.hasNext();) {
				userAttribute = (Element)it2.next();  
				if ((userAttribute.getName()).equals("name")) {
					name.add(userAttribute.getValue());
				} 
				else if ((userAttribute.getName()).equals("password")) {
					password.add(userAttribute.getValue());
				}
			}
		}
	}
}
