package de.desy.web2cToolkit.web2cUtil.web2cFileIO;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import java.util.Iterator;

/**
 * @author R.Bacher, DESY
 */
public class Web2cToolkitInitFilesWriterAndReader implements IWeb2cConstants {
	
	private String web2cInitFileName = web2cInitFile;
	private String web2cTwitterInitFileName = web2cTwitterInitFile;
	private String web2cSMTPInitFileName = web2cSMTPInitFile;
	private String web2cLogbookInitFileName = web2cLogbookInitFile;
	private String web2cUserRegistryFileName = web2cUserRegistryFile;

	/**
	 * Writer and reader for Web2cToolkit init files
	 */
	public Web2cToolkitInitFilesWriterAndReader() {

		readInitFile();
	}
	
	/**
	 * Returns the Web2c init file name
	 * @return file name
	 */
	public String getWeb2cInitFileName() {
		
		if (web2cInitFileName.equals("") || web2cInitFileName.equals(" ")) web2cInitFileName = web2cInitFile;

		return web2cInitFileName;
	}
	
	/**
	 * Returns the Web2cMessenger (Twitter) init file name
	 * @return file name
	 */
	public String getWeb2cTwitterInitFileName() {
		
		if (web2cTwitterInitFileName.equals("") || web2cTwitterInitFileName.equals(" ")) web2cTwitterInitFileName = web2cTwitterInitFile;

		return web2cTwitterInitFileName;
	}
	
	/**
	 * Returns the Web2cMessenger (SMTP) init file name
	 * @return file name
	 */
	public String getWeb2cSMTPInitFileName() {
		
		if (web2cSMTPInitFileName.equals("") || web2cSMTPInitFileName.equals(" ")) web2cSMTPInitFileName = web2cSMTPInitFile;

		return web2cSMTPInitFileName;
	}
	
	/**
	 * Returns the Web2cLogbook init file name
	 * @return file name
	 */
	public String getWeb2cLogbookInitFileName() {
		
		if (web2cLogbookInitFileName.equals("") || web2cLogbookInitFileName.equals(" ")) web2cLogbookInitFileName = web2cLogbookInitFile;

		return web2cLogbookInitFileName;
	}
	
	/**
	 * Returns the Web2cToolkit user registry file name
	 * @return file name
	 */
	public String getWeb2cUserRegistryFileName() {
		
		if (web2cUserRegistryFileName.equals("") || web2cUserRegistryFileName.equals(" ")) web2cUserRegistryFileName = web2cUserRegistryFile;

		return web2cUserRegistryFileName;
	}	
	
	/**
	 * Returns the implemented Web2cToolkit init file types
	 * @return types
	 */
	public String[] getInitFileTypes() {
		
		String[] types = {"web2c", "web2cMessengerTwitter", "web2cMessengerSMTP", "web2cLogbook", "web2cUserRegistry"};

		return types;
	}
	
	/**
	 * Returns the init file name associated with the file type specified
	 * @return file name
	 */
	public String getInitFileName(String type) {
		
		String fileName = "";
		
		if (type.equals("web2c")) {
			fileName = web2cInitFileName;
		}
		if (type.equals("web2cMessengerTwitter")) {
			fileName = web2cTwitterInitFileName;
		}
		if (type.equals("web2cMessengerSMTP")) {
			fileName = web2cSMTPInitFileName;
		}
		if (type.equals("web2cLogbook")) {
			fileName = web2cLogbookInitFileName;
		}
		if (type.equals("web2cUserRegistry")) {
			fileName = web2cUserRegistryFileName;
		}

		return fileName;
	}
	
	/**
	 * Updates and saves the Web2cToolkit init file
	 * @param name file name
	 * @param type file type
	 */
	public void saveInitFile(String name, String type) {

		if (type.equals("web2c")) {
			web2cInitFileName = name;
		}
		if (type.equals("web2cMessengerTwitter")) {
			web2cTwitterInitFileName = name;
		}
		if (type.equals("web2cMessengerSMTP")) {
			web2cSMTPInitFileName = name;
		}
		if (type.equals("web2cLogbook")) {
			web2cLogbookInitFileName = name;
		}
		if (type.equals("web2cUserRegistry")) {
			web2cUserRegistryFileName = name;
		}
		writeInitFile();
		readInitFile();
	}
	
    /**
     * Writes the toolkit init file
     */
	private void writeInitFile() {
		
	    try {
	    	BufferedWriter out = new BufferedWriter(new FileWriter(new File(registryDir + File.separator + web2cToolkitInitFile).getCanonicalFile()));
	    	out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); out.newLine();
			out.write("<init>"); out.newLine();	
			out.write("   <file>"); out.newLine();
			out.write("      <name>" + web2cInitFileName + "</name>"); out.newLine();
			out.write("      <type>web2c</type>"); out.newLine();
			out.write("   </file>"); out.newLine();	
			out.write("   <file>"); out.newLine();
			out.write("      <name>" + web2cTwitterInitFileName + "</name>"); out.newLine();
			out.write("      <type>web2cMessengerTwitter</type>"); out.newLine();
			out.write("   </file>"); out.newLine();	
			out.write("   <file>"); out.newLine();
			out.write("      <name>" + web2cSMTPInitFileName + "</name>"); out.newLine();
			out.write("      <type>web2cMessengerSMTP</type>"); out.newLine();
			out.write("   </file>"); out.newLine();	
			out.write("   <file>"); out.newLine();
			out.write("      <name>" + web2cLogbookInitFileName + "</name>"); out.newLine();
			out.write("      <type>web2cLogbook</type>"); out.newLine();
			out.write("   </file>"); out.newLine();	
			out.write("   <file>"); out.newLine();
			out.write("      <name>" + web2cUserRegistryFileName + "</name>"); out.newLine();
			out.write("      <type>web2cUserRegistry</type>"); out.newLine();
			out.write("   </file>"); out.newLine();
			out.write("</init>"); out.newLine();
			out.flush();
			out.close();
		} catch (IOException e) {
			System.out.println("Error writing Web2cToolkit init file.");
		}
	}
	
    /**
     * Reads the toolkit init file
     */
	@SuppressWarnings("unchecked")
	private void readInitFile() {
		
		web2cInitFileName = web2cInitFile;
		web2cTwitterInitFileName = web2cTwitterInitFile;
		web2cSMTPInitFileName = web2cSMTPInitFile;
		web2cLogbookInitFileName = web2cLogbookInitFile;
		web2cUserRegistryFileName = web2cUserRegistryFile;
    	
	   	SAXBuilder builder = new SAXBuilder();
		try {
			Element init = (builder.build(registryDir + File.separator + web2cToolkitInitFile)).getRootElement();
			List<Element> files = (List<Element>)init.getChildren();   
			for (Iterator<Element> it1 = files.iterator(); it1.hasNext();) {
				Element file = (Element)it1.next();
				List<Element> fileAttributes = file.getChildren(); 
				String fileName = "";
				for (Iterator<Element> it2 = fileAttributes.iterator(); it2.hasNext();) { 
					Element fileAttribute = (Element)it2.next(); 
					if ((fileAttribute.getName()).equals("name")) {
						if (!fileAttribute.getValue().equals("") && !fileAttribute.getValue().equals(" ")) {
							fileName = fileAttribute.getValue();
						}
					} 
					if ((fileAttribute.getName()).equals("type")) {
						if (fileAttribute.getValue().equals("web2c")) {
							web2cInitFileName = fileName;		
						}
						if (fileAttribute.getValue().equals("web2cMessengerTwitter")) {
							web2cTwitterInitFileName = fileName;
						}
						if (fileAttribute.getValue().equals("web2cMessengerSMTP")) {
							web2cSMTPInitFileName = fileName;	
						}
						if (fileAttribute.getValue().equals("web2cLogbook")) {
							web2cLogbookInitFileName = fileName;
						}
						if (fileAttribute.getValue().equals("web2cUserRegistry")) {
							web2cUserRegistryFileName = fileName;
						}
					}
				}
			}
		} 
		catch (JDOMException e) {System.out.println("Error parsing Web2cToolkit init file.");} 
		catch (IOException e) {System.out.println("Error reading Web2cToolkit init file.");}
	}
}
