package de.desy.web2cToolkit.web2cUtil.web2cInterfaces;

import java.awt.Color;
import java.io.File;
import java.util.Locale;
import javax.media.Format;
import javax.media.format.JPEGFormat;

/**
 * @author R.Bacher, DESY
 */
public interface IWeb2cConstants {
	
// Web2cToolkit version number and basic settings
	/**
	 * version number
	 */
	static final String web2cVersion = "6.0.0 Beta (18.11.2011 14:35)";	
	/**
	 * salutation
	 */
	static final String web2cSalutation = "Web2cToolkit: Framework for Web-based Controls Clients";
	/**
	 * preference for local settings
	 */
	static final Locale localSettings = new Locale("en", "US");	
	/**
	 *  URL of help page 
	 */
	static final String helpPage = "http://adweb.desy.de/mcs/web2cToolkit/user_documentation.htm";
	

//Web2cToolkit miscellaneous system settings
	/**
	 *  user role
	 */
	static final String layman = "layman";
	/**
	 *  user role 
	 */
	static final String expert = "expert";
	/**
	 *  user role 
	 */
	static final String supervisor = "supervisor";
	
	/**
	 *  flag whether cache enabled or not
	 */
	static final boolean cacheEnabled = true;		
	/**
	 *  cache timeout interval [s]
	 */
	static final int cacheTimeoutInterval = 120;
	
	/**
	 *  flag whether Java garbage collection enabled or not
	 */
	static final boolean garbageCollectionEnabled = false;	
	/**
	 *  Number of update requests to trigger garbage collector
	 */
	static final int maxGarbageCollectionCounter = 1000;	
	
	/**
	 *  flag whether session watcher enabled or not
	 */
	static final boolean sessionWatcherEnabled = true;
	
	/**
	 *  maximum data size per control system link 
	 */
	static final int maxLinkDataSize = 2048;
	
	/**
	 *  gateway update interval [s] 
	 */
	static final int gatewayUpdateInterval = 1;
	
	/**
	 *  logbook update interval [s] 
	 */
	static final int logbookUpdateInterval = 1;
	
	/**
	 *  Twitter update interval [s]
	 */
	static final int twitterUpdateInterval = 180;
	
	/**
	 *  login page time out [ms]
	 */
	static final int loginTimeOut = 60000;
	
	/**
	 *  default video format 
	 */
	static final Format defaultVideoFormat = new JPEGFormat();	
	
	/**
	 * page default sound
	 */
	static final String pageSound = "beep.wav";
	
	/**
	 *  Length of persistence interval for visual and acoustic alarm notifications [update cycles]
	 */
	static final int alarmPersistenceInterval = 5;
	

// Web2cToolkit directory related settings
	/**
	 * directory name separator
	 */
	static final String directorySeparator = "/";
	/**
	 * name of directory for user-specific files
	 */
	static final String userdocs = "userdocs";
	/**
	 * name of directory for Web2cToolkit-specific files
	 */
	static final String web2cdocs = "web2cdocs";
	/**
	 * name of directory for log files
	 */
	static final String log = "log";
	/**
	 * name of directory for logbook files
	 */
	static final String logbooks = "logbooks";
	/**
	 * pathname for directory "userdocs" 
	 */
	static final String userDir = File.separator + userdocs;
	/**
	 * pathname for directory "web2cdocs"  
	 */
	static final String web2cDir = File.separator + web2cdocs;	
	/**
	 * pathname for directory "log"  
	 */
	static final String logDir = File.separator + log;
	/**
	 * pathname for directory "logbooks" 
	 */
	static final String logbooksDir = File.separator + logbooks;
	/**
	 * pathname for directory containing user registry file  
	 */
	static final String registryDir = System.getProperty("java.endorsed.dirs");	
	/**
	 * time-out for deleting .js files [s]
	 */
	static final long deleteFileTimeout = 120;
	
	
// Web2cToolkit ini files
	/**
	 * name of Web2cToolkit init file
	 */
	static final String web2cToolkitInitFile = "Web2cToolkitInit.xml";	
	/**
	 * name of web2c init file 
	 */
	static final String web2cInitFile = "Web2cInit.xml";	
	/**
	 * name of file containing user registry information
	 */
	static final String web2cUserRegistryFile = "Web2cUsers.xml";		
	/**
	 * name of file containing twitter access credentials
	 */
	static final String web2cTwitterInitFile = "Web2cTwitterInit.xml";		
	/**
	 * name of file containing smtp access credentials
	 */
	static final String web2cSMTPInitFile = "Web2cSMTPInit.xml";		
	/**
	 * name of logbook init File
	 */
	static final String web2cLogbookInitFile = "Web2cLogbookInit.xml";	
	
	
// Web2cToolkit application config files	
	/**
	 * name of file containing gateway configurations
	 */
	static final String gatewayConfigFile = "gateway.xml";	
	/**
	 * name of file containing messenger configurations
	 */
	static final String messengerConfigFile = "messenger.xml";	
	

// Web2cToolkit session log files
	/**
	 * name of file containing synoptic display session log information
	 */
	static final String synopticDisplaySessionLogFile = "synopticDisplaySessionLog.log";		
	/**
	 * name of file containing editor session log information
	 */
	static final String editorSessionLogFile = "editorSessionLog.log";
	/**
	 * name of file containing archive viewer session log information
	 */
	static final String archiveViewerSessionLogFile = "archiveViewerSessionLog.log";	
	/**
	 * name of file containing messenger session log information
	 */
	static final String messengerSessionLogFile = "messengerSessionLog.log";	
	/**
	 * name of file containing manager session log information
	 */
	static final String managerSessionLogFile = "managerSessionLog.log";	
	/**
	 * name of file containing gateway session log information
	 */
	static final String gatewaySessionLogFile = "gatewaySessionLog.log";	
	/**
	 * name of file containing logbook session log information
	 */
	static final String logbookSessionLogFile = "logbookSessionLog.log";	
	
	
// Web2cToolkit common-device-access log files
	/**
	 * name of file containing control system action log information (Web2c)
	 */
	static final String synopticDisplayCdaActionLogFile = "synopticDisplayCdaActionLog.log";	
	/**
	 * name of file containing control system action log information (Web2cMessenger)
	 */
	static final String messengerCdaActionLogFile = "messengerCdaActionLog.log";
	/**
	 * name of file containing control system action log information (Web2cGateway)
	 */
	static final String gatewayCdaActionLogFile = "gatewayCdaActionLog.log";
	/**
	 * name of file containing control system action log information (Web2cLogbook)
	 */
	static final String logbookCdaActionLogFile = "logbookCdaActionLog.log";
	

// Web2cToolkit application log files
	/**
	 * name of file containing synoptic display log information
	 */
	static final String synopticDisplayLogFile = "synopticDisplayLog.log";
	/**
	 * name of file containing editor log information
	 */
	static final String editorLogFile = "editorLog.log";
	/**
	 * name of file containing archive viewer log information
	 */
	static final String archiveViewerLogFile = "archiveViewerLog.log";	
	/**
	 * name of file containing messenger log information
	 */
	static final String messengerLogFile = "messengerLog.log";
	/**
	 * name of file containing manager log information
	 */
	static final String managerLogFile = "managerLog.log";	
	/**
	 * name of file containing gateway log information
	 */
	static final String gatewayLogFile = "gatewayLog.log";
	/**
	 * name of file containing logbook log information
	 */
	static final String logbookLogFile = "logbookLog.log";
	
	
// Http session and Web2cToolkit session timeout / max session settings
	/**
	 *  Maximum number of sessions
	 */
	static final int maxSessionNumber = 50;
	/**
	 *  Web2c session inactivity timeout interval [s]
	 */
	static final int inactivityTimeoutInterval = 180;	
	/**
	 *  Web2c http session inactivity timeout interval [s]
	 */
	static final int sessionTimeoutInterval = 300;
	
	/**
	 *  Maximum number of editor sessions
	 */
	static final int maxEditorSessionNumber = 20;
	/**
	 *  Web2cEditor session inactivity timeout interval [s]
	 */
	static final int editorInactivityTimeoutInterval = 1200;	
	/**
	 *  Web2cEditor http session inactivity timeout interval [s]
	 */
	static final int editorSessionTimeoutInterval = 1320;
	
	/**
	 *  Maximum number of archive viewer sessions
	 */
	static final int maxArchiveViewerSessionNumber = 50;
	/**
	 *  Web2cArchiveViewer session inactivity timeout interval [s]
	 */
	static final int archiveViewerInactivityTimeoutInterval = 600;	
	/**
	 *  Web2cArchiveViewer http session inactivity timeout interval [s]
	 */
	static final int archiveViewerSessionTimeoutInterval = 720;
	
	/**
	 *  Maximum number of messenger sessions
	 */
	static final int maxMessengerSessionNumber = 50;
	/**
	 *  Web2cMessenger session inactivity timeout interval [s]
	 */
	static final int messengerInactivityTimeoutInterval = 1200;	
	/**
	 *  Web2cMessenger http session inactivity timeout interval [s]
	 */
	static final int messengerSessionTimeoutInterval = 1320;
	
	/**
	 *  Maximum number of manager sessions
	 */
	static final int maxManagerSessionNumber = 10;
	/**
	 *  Web2cManager session inactivity timeout interval [s]
	 */
	static final int managerInactivityTimeoutInterval = 1200;	
	/**
	 *  Web2cManager http session inactivity timeout interval [s]
	 */
	static final int managerSessionTimeoutInterval = 1320;
	
	/**
	 *  Maximum number of gateway sessions
	 */
	static final int maxGatewaySessionNumber = 50;
	/**
	 *  Web2cGateway session inactivity timeout interval [s]
	 */
	static final int gatewayInactivityTimeoutInterval = 180;	
	/**
	 *  Web2cGateway http session inactivity timeout interval [s]
	 */
	static final int gatewaySessionTimeoutInterval = 300;
	
	/**
	 *  Maximum number of logbook sessions
	 */
	static final int maxLogbookSessionNumber = 50;
	/**
	 *  Web2cLogbook session inactivity timeout interval [s]
	 */
	static final int logbookInactivityTimeoutInterval = 1200;	
	/**
	 *  Web2cLogbook http session inactivity timeout interval [s]
	 */
	static final int logbookSessionTimeoutInterval = 1320;
	
	
// Web2cToolkit application-specific declarations
	/**
	 * font size 
	 */
	static final String tiny = "6px";
	/**
	 * font size 
	 */
	static final String small = "12px";
	/**
	 * font size 
	 */
	static final String medium = "18px";
	/**
	 * font size 
	 */
	static final String large = "24px";	
	/**
	 * font size 
	 */
	static final String extraLarge = "30px";
	/**
	 * font size 
	 */
	static final String huge = "36px";
	/**
	 * font size 
	 */
	static final String extraHuge = "42px";
	/**
	 * font size 
	 */
	static final String gigantic = "48px";	
	/**
	 * font size 
	 */
	static final String extraGigantic = "54px";
	
	/**
	 * exponential number format 
	 */
	static final String expformat = "0.000E00";
	/**
	 * decimal number format 
	 */
	static final String decformat = "n0.00";
	
	/**
	 * border style 
	 */
	static final String none = "none";
	/**
	 * border style 
	 */
	static final String solid = "solid";
	/**
	 * border thickness 
	 */
	static final String thin = "1px";	
	/**
	 * border thickness 
	 */
	static final String thick = "3px";
	
	/**
	 * font weight 
	 */
	static final String normal = "normal";
	/**
	 * font weight 
	 */
	static final String bold = "bold";
	
	/**
	 * text alignment 
	 */
	static final String left = "left";
	/**
	 * text alignment 
	 */
	static final String center = "center";
	/**
	 * text alignment 
	 */
	static final String right = "right";
	
	/**
	 * color 
	 */
	static final String black = "black";
	/**
	 * color 
	 */
	static final String white = "white";
	/**
	 * color 
	 */
	static final String gray= "gray";
	
	/**
	 * icon image 
	 */
	static final String redIcon = "redicon.jpg";
	/**
	 * icon image 
	 */
	static final String greenIcon = "greenicon.jpg";
	
	/**
	 * simulated video image 
	 */
	static final String simulatedVideoImage = "videoSimulation.jpg";
	
	/**
	 * length of trend plot [s]  
	 */
	static final String timeInterval3600 = "3600";
	
	/**
	 * length of history plot [s] 
	 */
	static final String historyInterval = "120";
	/**
	 * width of history plot [px] 
	 */
	static final String historyWidth = "1000px";
	/**
	 *  height of history plot [px] 
	 */
	static final String historyHeight = "800px";
	
	/**
	 *  yes 
	 */
	static final String yes = "yes";
	/**
	 *  no 
	 */
	static final String no = "no";
	
	/**
	 *  data mode  
	 */
	static final boolean simulated = true;
	/**
	 *  data mode 
	 */
	static final boolean real = false;
		
	/**
	 *  plot type 
	 */
	static final String linePlot = "linePlot";	
	/**
	 *  plot type 
	 */
	static final String areaPlot = "areaPlot";
	/**
	 *  plot type
	 */
	static final String barPlot = "barPlot";	
	/**
	 *  plot type
	 */
	static final String lineTimePlot = "lineTimePlot";	
	/**
	 *  plot type
	 */
	static final String areaTimePlot = "areaTimePlot";	
	/**
	 *  plot type
	 */
	static final String barTimePlot = "barTimePlot";	
	/**
	 *  plot style 
	 */
	static final String single = "single";
	/**
	 *  plot style 
	 */
	static final String multi = "multi";
	/**
	 *  order of time axis 
	 */
	static final String invert = "invert";	
	/**
	 *  transparency
	 */
	static final String transparent = "transparent";
	/**
	 *  color red
	 */
	static final Color red = new Color(255,0,0);
	/**
	 *  color blue
	 */
	static final Color blue = Color.blue;
	/**
	 *  color magenta
	 */
	static final Color magenta = Color.magenta;
	/**
	 *  color green
	 */
	static final Color green = Color.green;
	/**
	 *  color dark red
	 */
	static final Color darkRed = new Color(158,0,0);
	/**
	 *  color yellow
	 */
	static final Color yellow = Color.yellow;
	/**
	 *  color orange
	 */
	static final Color orange = Color.orange;
	/**
	 *  color cyan
	 */
	static final Color cyan = Color.cyan;
	/**
	 *  color dark gray
	 */
	static final Color darkGray = Color.darkGray;
	/**
	 *  color dark black 
	 */
	static final Color darkBlack = Color.black;
	/**
	 *  maximum number of plot traces
	 */
	static final int maxTraces = 10;
	/**
	 *  plot colors 
	 */
	static final Color[] traceColor = {red, blue, magenta, green, darkRed, yellow, orange, cyan, darkGray, darkBlack}; 

	/**
	 *  visibility of component 
	 */
	static final String visible = "visible";	
	/**
	 *  visibility of component 
	 */
	static final String hidden = "hidden";
	
	/**
	 *  width of archive plot [px] 
	 */
	static final String horizontalArchivePlotSize = "970px";
	/**
	 *  height of archive plot [px] 
	 */
	static final String verticalArchivePlotSize = "295px";	
	/**
	 *  fixed axis (left) in archive plot (Java units)   
	 */
	static final int axisSpaceLeft = 200;	
	/**
	 *  fixed axis (right) in archive plot (Java units)    
	 */
	static final int axisSpaceRight = 10;
	/**
	 *  maximum number of data points in archive plot 
	 */
	static final int numberofArchivePlotDataPoints = 300;

	/**
	 *  maximum number of chat entries per channel
	 */
	static final int maxChatDepth = 20;
	
	/**
	 *  maximum number of input characters (Web2cInput, Web2cChat)
	 */
	static final int maxInputLength = 1000;
	

}
