package de.desy.web2cToolkit.web2cUtil.web2cInterfaces;

/**
 * @author R.Bacher, DESY
 */
public interface IWeb2cDeviceAccess {
	
	/**
	 * Registers monitor-type link
	 * @param property property of data channel
	 * @return status
	 */
	public int registerMonitor(String property);
	
	/**
	 * Closes the monitor link specified
	 */
	public void closeMonitorLink(String property);
	
	/**
	 * Returns data from a registered monitor-type link
	 * @param property property of data channel
	 * @return data (Format: &#060;value&#062;data[startIndex],..,data[stopIndex]&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp&#060;&#047;timeStamp&#062;)
	 */
	public String getMonitorData(String property);
	
	/**
	 * Executes an execute-type link
	 * @param property
	 * @param messageEnabled
	 */
	public void execute(String property, boolean messageEnabled);
	
	/**
	 * Returns array of history data
	 * @param property property of data channel
	 * @param messageEnabled flag indicating whether action logger is enabled
	 * @return data (Format: &#060;value&#062;data1&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp1&#060;&#047;timeStamp&#062;&#060;value&#062;data2&#060;&#047;value&#062;&#060;timeStamp&#062;timeStamp2&#060;&#047;timeStamp&#062;.....)
	 */
	public String getHistory(String property);
	
	/**
	 * Returns the error code of the class
	 * @return error
	 */
	public String getError();
	
	/**
	 * Returns the activity counter value
	 * @return counter value
	 */
	public String getActivityCounter();
	
	/**
	 * Returns the property description
	 * @param property
	 * @return description
	 */
	public String getPropertyDescription(String property);
}
