package de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl;

import java.io.File;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;

import de.desy.web2cToolkit.web2cMessenger.Web2cSMTPInterface;
import de.desy.web2cToolkit.web2cMessenger.Web2cTwitterInterface;
import de.desy.web2cToolkit.web2cServlets.Web2c;
import de.desy.web2cToolkit.web2cServlets.Web2cArchiveViewer;
import de.desy.web2cToolkit.web2cServlets.Web2cEditor;
import de.desy.web2cToolkit.web2cServlets.Web2cGateway;
import de.desy.web2cToolkit.web2cServlets.Web2cLogbook;
import de.desy.web2cToolkit.web2cServlets.Web2cManager;
import de.desy.web2cToolkit.web2cServlets.Web2cMessenger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cInitWriterAndReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cLogbookInitFileWriterAndReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cRegistryWriterAndReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cToolkitInitFilesWriterAndReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;

/**
 * @author  R.Bacher, DESY
 */
public class Web2cContextHandler implements IWeb2cConstants{
	
	private static Web2cContextHandler web2cToolkitContextHandler = null;
	private static Web2cToolkitInitFilesWriterAndReader web2cToolkitInitFilesWriterAndReader = null;
	private ArrayList<HttpServlet> servletList = new ArrayList<HttpServlet>();
	private static String homeDir = null;
	private static String webDir = null;
	private static String loggingDir = null;
	private static String logbookDir = null;
	private static ServletContext servletContext = null;
	private boolean garbageCollectionIsEnabled = garbageCollectionEnabled;
	private boolean sessionWatcherIsEnabled = sessionWatcherEnabled;
	
	/**
	 * Singleton Constructor; 
	 * Stores and provides context specific informations
	 */
	private Web2cContextHandler() {}
	
	/**
	 * Returns the only existing instance of the class Web2cContextHandler
	 * @param context servlet context
	 * @return instance
	 */
	public static synchronized Web2cContextHandler getInstance(ServletContext context) {

		if (web2cToolkitContextHandler == null) {
			
			web2cToolkitContextHandler = new Web2cContextHandler();
			web2cToolkitInitFilesWriterAndReader = new Web2cToolkitInitFilesWriterAndReader();	

			servletContext = context;
			
			if (homeDir == null) {
				homeDir = servletContext.getRealPath(userDir);
			}
			if (webDir == null) {
				webDir = servletContext.getRealPath(web2cDir);
			}
			if (loggingDir == null) {
				loggingDir = servletContext.getRealPath(logDir);
			}
			if (logbookDir == null) {
				logbookDir = servletContext.getRealPath(logbooksDir);
			}
		}
		
		return web2cToolkitContextHandler;
	}
	
	/**
	 * Returns the servlet context
	 * @return context
	 */
	public ServletContext getServletContext() {
		
		return servletContext;
	}
	
	/**
	 * Returns the tookit init files writer and reader
	 * @return writer and reader
	 */
	public Web2cToolkitInitFilesWriterAndReader getWeb2ToolkitInitFilesWriterAndReader() {
		
		return web2cToolkitInitFilesWriterAndReader;
	}
	
	/**
	 * Returns user dir
	 * @return user dir
	 */
	public String getUserDir() {
		
		return homeDir;
	}
	
	/**
	 * Returns web dir
	 * @return web dir
	 */
	public String getWebDir() {
		
		return webDir;
	}
	
	/**
	 * Returns logging dir
	 * @return logging dir
	 */
	public String getLoggingDir() {
		
		return loggingDir;
	}
	
	/**
	 * Returns logbook dir
	 * @return logbook dir
	 */
	public String getLogbookDir() {
		
		return logbookDir;
	}
	
	/**
	 * Adds a servlet instance to the context handler's servlet list
	 * @param servlet servlet to add
	 */
	public void addToServletList(HttpServlet servlet) {
		
		servletList.add(servlet);
	}
	
	/**
	 * Returns the context handler's servlet list
	 * @return servlet list
	 */
	public ArrayList<HttpServlet> getServletList() {
		
		return servletList;
	}
	
	/**
	 * Returns the Web2c servlet
	 * @return servlet 
	 */
	public Web2c getWeb2cServlet() {
		
		Web2c servlet = null;

		for (Iterator<HttpServlet> it = servletList.iterator(); it.hasNext();) {
			HttpServlet genericServlet = it.next();
			if ((genericServlet.getServletConfig().getServletName()).equals("Web2c")) {
				servlet = (Web2c)genericServlet;
			}
		}
		
		return servlet;
	}
	
	/**
	 * Returns the Web2cArchiveViewer servlet
	 * @return servlet 
	 */
	public Web2cArchiveViewer getWeb2cArchiveViewerServlet() {
		
		Web2cArchiveViewer servlet = null;

		for (Iterator<HttpServlet> it = servletList.iterator(); it.hasNext();) {
			HttpServlet genericServlet = it.next();
			if ((genericServlet.getServletConfig().getServletName()).equals("Web2cArchiveViewer")) {
				servlet = (Web2cArchiveViewer)genericServlet;
			}
		}
		
		return servlet;
	}
	
	/**
	 * Returns the Web2cEditor servlet
	 * @return servlet 
	 */
	public Web2cEditor getWeb2cEditorServlet() {
		
		Web2cEditor servlet = null;

		for (Iterator<HttpServlet> it = servletList.iterator(); it.hasNext();) {
			HttpServlet genericServlet = it.next();
			if ((genericServlet.getServletConfig().getServletName()).equals("Editor")) {
				servlet = (Web2cEditor)genericServlet;
			}
		}
		
		return servlet;
	}
	
	/**
	 * Returns the Web2cGateway servlet
	 * @return servlet 
	 */
	public Web2cGateway getWeb2cGatewayServlet() {
		
		Web2cGateway servlet = null;

		for (Iterator<HttpServlet> it = servletList.iterator(); it.hasNext();) {
			HttpServlet genericServlet = it.next();
			if ((genericServlet.getServletConfig().getServletName()).equals("Web2cGateway")) {
				servlet = (Web2cGateway)genericServlet;
			}
		}
		
		return servlet;
	}
	
	/**
	 * Returns the Web2cMessenger servlet
	 * @return servlet 
	 */
	public Web2cMessenger getWeb2cMessengerServlet() {
		
		Web2cMessenger servlet = null;

		for (Iterator<HttpServlet> it = servletList.iterator(); it.hasNext();) {
			HttpServlet genericServlet = it.next();
			if ((genericServlet.getServletConfig().getServletName()).equals("Web2cMessenger")) {
				servlet = (Web2cMessenger)genericServlet;
			}
		}
		
		return servlet;
	}
	
	/**
	 * Returns the Web2cManager servlet
	 * @return servlet 
	 */
	public Web2cManager getWeb2cManagerServlet() {
		
		Web2cManager servlet = null;

		for (Iterator<HttpServlet> it = servletList.iterator(); it.hasNext();) {
			HttpServlet genericServlet = it.next();
			if ((genericServlet.getServletConfig().getServletName()).equals("Web2cManager")) {
				servlet = (Web2cManager)genericServlet;
			}
		}
		
		return servlet;
	}
	
	/**
	 * Returns the Web2cLogbook servlet
	 * @return servlet 
	 */
	public Web2cLogbook getWeb2cLogbookServlet() {
		
		Web2cLogbook servlet = null;

		for (Iterator<HttpServlet> it = servletList.iterator(); it.hasNext();) {
			HttpServlet genericServlet = it.next();
			if ((genericServlet.getServletConfig().getServletName()).equals("Web2cLogbook")) {
				servlet = (Web2cLogbook)genericServlet;
			}
		}
		
		return servlet;
	}
	
	/**
	 * Returns the servlet names
	 * @return servlet names
	 */
	public String[] getServletNames() {
	
		String[] servletNames = new String[servletList.size()];
		int i = 0;
		for (Iterator<HttpServlet> it = servletList.iterator(); it.hasNext();) {
			HttpServlet servlet = it.next();
			servletNames[i] = servlet.getServletConfig().getServletName();
			i = i + 1;
		}
		
		return servletNames;
	}
	
	/**
	 * Returns the corresponding servlet handler
	 * @param servletName servlet name
	 * @return servlet handler
	 */
	public Web2cServletHandler getServletHandler(String servletName) {
		
		Web2cServletHandler servletHandler = null;

		for (Iterator<HttpServlet> it = servletList.iterator(); it.hasNext();) {
			HttpServlet servlet = it.next();
			if ((servlet.getServletConfig().getServletName()).equals(servletName)) {
				try {	
					Class<?>  servletClass =  Class.forName("de.desy.web2cToolkit.web2cServlets." + servletName);
					Method getServletHandlerMethod = servletClass.cast(servlet).getClass().getMethod("getServletHandler", (Class<?>[])null);
					servletHandler = (Web2cServletHandler)(getServletHandlerMethod.invoke((Object)(servletClass.cast(servlet)), (Object[])null));
				}
				catch (Exception e) {			
					System.err.println(e.toString());
					e.printStackTrace();
				} 
			}
		}
		
		return servletHandler;
	}
	
	
	/**
	 * Returns the Web2c init file name
	 * @return file name
	 */
	public String getWeb2cInitFileName() {
		
		return web2cToolkitInitFilesWriterAndReader.getWeb2cInitFileName();
	}
	
	/**
	 * Returns the Web2cMessenger (Twitter) init file name
	 * @return file name
	 */
	public String getWeb2cTwitterInitFileName() {
		
		return web2cToolkitInitFilesWriterAndReader.getWeb2cTwitterInitFileName();
	}
	
	/**
	 * Returns the Web2cMessenger (SMTP) init file name
	 * @return file name
	 */
	public String getWeb2cSMTPInitFileName() {
		
		return web2cToolkitInitFilesWriterAndReader.getWeb2cSMTPInitFileName();
	}
	
	/**
	 * Returns the Web2cToolkit logbook init file name
	 * @return file name
	 */
	public String getWeb2cLogbookInitFileName() {
		
		return web2cToolkitInitFilesWriterAndReader.getWeb2cLogbookInitFileName();
	}
	
	/**
	 * Returns the Web2cToolkit user registry file name
	 * @return file name
	 */
	public String getWeb2cUserRegistryFileName() {
		
		return web2cToolkitInitFilesWriterAndReader.getWeb2cUserRegistryFileName();
	}
	
	/**
	 * Returns a list of files located in the directory specified (and subdirectories, if any)
	 * @param dir directory to be browsed
	 * @param externsion name extension of files to be listed
	 * @return file list
	 */
	public ArrayList<String> getListOfAllFiles(File dir, String extension) {
		
		ArrayList<String> fileList = new ArrayList<String>();

		String[] dirListing = dir.list();
		for (int i = 0; i < dirListing.length; i++) {
			File file = new File(dir.getAbsolutePath() + File.separator + dirListing[i]);		
			if (file.isDirectory()) {
				ArrayList<String> list = getListOfAllFiles(file, extension);
				for (Iterator<String> it = list.iterator(); it.hasNext();) {
					fileList.add(file.getName() +  "/" + (String)(it.next()));
				}
			}
			else if (dirListing[i].contains(extension)) {
				fileList.add(dirListing[i]);
			}
		}
		
		return fileList;
	}

	/**
	 * Returns the file logger for the log file specified
	 * @param name log file name
	 * @return logger
	 */
	public Web2cFileLogger getFileLogger(String name) {
		
		Web2cFileLogger logger = null;
		
		if (name.equals(synopticDisplayLogFile)) logger = getServletHandler("Web2c").getDebugLogger();
		if (name.equals(archiveViewerLogFile)) logger = getServletHandler("Web2cArchiveViewer").getDebugLogger();
		if (name.equals(editorLogFile)) logger = getServletHandler("Web2cEditor").getDebugLogger();
		if (name.equals(messengerLogFile)) logger = getServletHandler("Web2cMessenger").getDebugLogger();
		if (name.equals(managerLogFile)) logger = getServletHandler("Web2cManager").getDebugLogger();
		if (name.equals(gatewayLogFile)) logger = getServletHandler("Web2cGateway").getDebugLogger();
		if (name.equals(logbookLogFile)) logger = getServletHandler("Web2cLogbbok").getDebugLogger();
		
		if (name.equals(synopticDisplaySessionLogFile)) logger = getServletHandler("Web2c").getSessionLogger();		
		if (name.equals(archiveViewerSessionLogFile)) logger = getServletHandler("Web2cArchiveViewer").getSessionLogger();
		if (name.equals(editorSessionLogFile)) logger = getServletHandler("Web2cEditor").getSessionLogger();
		if (name.equals(messengerSessionLogFile)) logger = getServletHandler("Web2cMessenger").getSessionLogger();
		if (name.equals(managerSessionLogFile)) logger = getServletHandler("Web2cManager").getSessionLogger();
		if (name.equals(gatewaySessionLogFile)) logger = getServletHandler("Web2cGateway").getSessionLogger();
		if (name.equals(logbookSessionLogFile)) logger = getServletHandler("Web2cLogbook").getSessionLogger();
		
		if (name.equals(synopticDisplayCdaActionLogFile)) logger = getServletHandler("Web2c").getActionLogger();
		if (name.equals(messengerCdaActionLogFile)) logger = getServletHandler("Web2cMessenger").getActionLogger();
		if (name.equals(gatewayCdaActionLogFile)) logger = getServletHandler("Web2cGateway").getActionLogger();
		if (name.equals(logbookCdaActionLogFile)) logger = getServletHandler("Web2cLogbook").getActionLogger();
		
		return logger;
	}
	
	/**
	 * Returns the keyboard listener
	 * @return listener
	 */
	public Web2cKeyBoardListener getWeb2cKeyBoardListener() {
		
		return getServletHandler("Web2cManager").getWeb2cKeyBoardListener();
	}
	
	/**
	 * Returns the registry writer and reader
	 * @return writer/reader
	 */
	public Web2cRegistryWriterAndReader getWeb2cRegistryWriterAndReader() {
		
		return getServletHandler("Web2cManager").getWeb2cRegistryWriterAndReader();
	}
	
	/**
	 * Returns the init writer and reader
	 * @return writer/reader
	 */
	public Web2cInitWriterAndReader getWeb2cInitWriterAndReader() {
		
		return getServletHandler("Web2c").getWeb2cInitWriterAndReader();
	}
	
	/**
	 * Returns the logbook init writer and reader
	 * @return writer/reader
	 */
	public Web2cLogbookInitFileWriterAndReader getWeb2cLogbookInitFileWriterAndReader() {
		
		return getServletHandler("Web2cLogbook").getWeb2cLogbookInitFileWriterAndReader();
	}
	
	/**
	 * Returns the SMTP interface
	 * @return interface
	 */
	public Web2cSMTPInterface getWeb2cSMTPInterface() {
		
		return getServletHandler("Web2cMessenger").getWeb2cSMTPInterface();
	}
	
	/**
	 * Returns the Twitter interface
	 * @return interface
	 */
	public Web2cTwitterInterface getWeb2cTwitterInterface() {
		
		return getServletHandler("Web2cMessenger").getWeb2cTwitterInterface();
	}
	
	/**
	 * Returns a session handler 
	 * @param servlet servlet name
	 * @param sessionId session id 
	 * @return handler
	 */
	public Web2cSessionHandler getWeb2cSessionHandler(String servletName, String sessionId) {
		
		return getServletHandler(servletName).getSessionHandler(sessionId);
	}
	
	/**
	 * Sets the Java garbage collector enabled/disabled flag
	 * @param flag
	 */
	public void setJavaGarbageCollection(boolean flag) {
		
		garbageCollectionIsEnabled = flag;
		String[] servletNames = getServletNames();
		for (int i = 0; i < servletNames.length; i++) {
			Web2cServletHandler servletHandler = getServletHandler(servletNames[i]);
			ArrayList<Web2cSessionHandler> sessionHandlerList = servletHandler.getWeb2cSessionHandlerList();
			synchronized (sessionHandlerList) {
				servletHandler.getWeb2cGarbageCollector().setJavaGarbageCollection(flag);	
				if (servletNames[i].equals("Web2c")) servletHandler.getWeb2cProbe().setJavaGarbageCollection(flag);
				if (servletNames[i].equals("Web2cArchiveViewer")) servletHandler.getWeb2cArchiveViewerProbe().setJavaGarbageCollection(flag);
				if (servletNames[i].equals("Web2cEditor")) servletHandler.getWeb2cEditorProbe().setJavaGarbageCollection(flag);
				if (servletNames[i].equals("Web2cGateway")) servletHandler.getWeb2cGatewayProbe().setJavaGarbageCollection(flag); 
				if (servletNames[i].equals("Web2cMessenger")) servletHandler.getWeb2cMessengerProbe().setJavaGarbageCollection(flag); 
				if (servletNames[i].equals("Web2cManager")) servletHandler.getWeb2cManagerProbe().setJavaGarbageCollection(flag); 
				if (servletNames[i].equals("Web2cLogbook")) servletHandler.getWeb2cLogbookProbe().setJavaGarbageCollection(flag); 
			}
		}
	}
	
	/**
	 * Returns the Java garbage collector enabled/disabled flag
	 * @return flag
	 */
	public boolean getJavaGarbageCollection() {
		
		return garbageCollectionIsEnabled;
	}
	
	/**
	 * Perform Java garbage collection
	 */
	public void doJavaGarbageCollection() {
		
		System.gc();
	}
	
	/**
	 * Sets the Web2cToolkit garbage collector enabled/disabled flag
	 * @param flag
	 */
	public void setWeb2cToolkitGarbageCollection(boolean flag) {
		
		sessionWatcherIsEnabled = flag;	
		String[] servletNames = getServletNames();
		for (int i = 0; i < servletNames.length; i++) {
			Web2cServletHandler servletHandler = getServletHandler(servletNames[i]);
			ArrayList<Web2cSessionHandler> sessionHandlerList = servletHandler.getWeb2cSessionHandlerList();
			synchronized (sessionHandlerList) {
				servletHandler.getWeb2cGarbageCollector().setWeb2cToolkitGarbageCollection(flag);		
				if (servletNames[i].equals("Web2c")) servletHandler.getWeb2cProbe().setWeb2cToolkitGarbageCollection(flag);
				if (servletNames[i].equals("Web2cArchiveViewer")) servletHandler.getWeb2cArchiveViewerProbe().setWeb2cToolkitGarbageCollection(flag);
				if (servletNames[i].equals("Web2cEditor")) servletHandler.getWeb2cEditorProbe().setWeb2cToolkitGarbageCollection(flag);
				if (servletNames[i].equals("Web2cGateway")) servletHandler.getWeb2cGatewayProbe().setWeb2cToolkitGarbageCollection(flag); 
				if (servletNames[i].equals("Web2cMessenger")) servletHandler.getWeb2cMessengerProbe().setWeb2cToolkitGarbageCollection(flag); 
				if (servletNames[i].equals("Web2cManager")) servletHandler.getWeb2cManagerProbe().setWeb2cToolkitGarbageCollection(flag); 
				if (servletNames[i].equals("Web2cLogbook")) servletHandler.getWeb2cLogbookProbe().setWeb2cToolkitGarbageCollection(flag); 
			}
		}
	}
	
	/**
	 * Returns the Web2cToolkit garbage collector enabled/disabled flag
	 * @return flag
	 */
	public boolean getWeb2cToolkitGarbageCollection() {
		
		return sessionWatcherIsEnabled;
	}
}
