package de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import org.jfree.chart.ChartUtilities;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cPage;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;

/**
 * @author R.Bacher, DESY
 */
public class Web2cImageCache implements IWeb2cConstants {

	private ArrayList<ImageCache> chartCacheList = new ArrayList<ImageCache>();	
	private ArrayList<ImageCache> videoCacheList = new ArrayList<ImageCache>();
	private long timeout = cacheTimeoutInterval * 1000;
	private boolean cacheIsEnabled = cacheEnabled;

	/**
	 * Cache for charts and video images
	 */
	public Web2cImageCache() {
		
		super();
	}
	
	/**
	 * Returns the chart cache list
	 * @return chart cache list
	 */
	public ArrayList<ImageCache> getChartCacheList() {
		
		return chartCacheList;
	}
	
	/**
	 * Clears the chart cache by removing entries if updating has been stopped
	 */
	public synchronized void clearChartCache() {
		
		synchronized(chartCacheList) {
			if (cacheIsEnabled == true) {	
				ArrayList<ImageCache> chartCacheToDeleteList = new ArrayList<ImageCache>();
				for (Iterator<ImageCache>it = chartCacheList.iterator(); it.hasNext();) {
					ImageCache cache = (ImageCache)it.next();
					if ((System.currentTimeMillis() - cache.getMostRecentUpdate()) > timeout) chartCacheToDeleteList.add(cache);
				}
				for (Iterator<ImageCache>it = chartCacheToDeleteList.iterator(); it.hasNext();) {
					ImageCache chartCacheToDelete = (ImageCache)it.next();
					chartCacheToDelete.clear();
					chartCacheList.remove(chartCacheToDelete);
				}
			}
		}
		
	}
	
	/**
	 * Returns the video cache list
	 * @return video cache list
	 */
	public ArrayList<ImageCache> getVideoCacheList() {
		
		return videoCacheList;
	}
	
	/**
	 * Clears the video cache by removing entries if updating has been stopped
	 */
	public synchronized void clearVideoCache() {
		
		synchronized(videoCacheList) {
			if (cacheIsEnabled == true) {	
				ArrayList<ImageCache> videoCacheToDeleteList = new ArrayList<ImageCache>();
				for (Iterator<ImageCache>it = videoCacheList.iterator(); it.hasNext();) {
					ImageCache cache = (ImageCache)it.next();
					if ((System.currentTimeMillis() - cache.getMostRecentUpdate()) > timeout) videoCacheToDeleteList.add(cache);
				}
				for (Iterator<ImageCache>it = videoCacheToDeleteList.iterator(); it.hasNext();) {
					ImageCache videoCacheToDelete = (ImageCache)it.next();
					videoCacheToDelete.clear();
					videoCacheList.remove(videoCacheToDelete);
				}
			}
		}
		
	}
	
	/**
	 * Returns chart; updates chart entry in cache if update interval exceeded; adds chart entry to cache if missing; deletes content of chart cache if timeout interval exceeded
	 * @param sessionHandler session handler
	 * @param id component id
	 * @return chart as stream
	 */
	public synchronized ByteArrayOutputStream getChartFromCache(Web2cSessionHandler sessionHandler, String id) {

		String file = "";
		int language = 0;
		String dataMode = "";
		int interval = 0;
		Web2cPage page = null;
		ImageCache chartCache = null;
		ByteArrayOutputStream chartStream = null;
		int index = 0;
		int i = 0;

		if (cacheIsEnabled == true) {			
			file = sessionHandler.getAttribute("loadedConfigFileName");
			language = Integer.valueOf((String)sessionHandler.getAttribute("language"));
			dataMode = sessionHandler.getAttribute("dataSource");
			page = sessionHandler.getWeb2cPage();	
			interval = page.getUpdateTime();
			
			synchronized(chartCacheList) {
				index = -1;
				i = -1;
				for (Iterator<ImageCache>it = chartCacheList.iterator(); it.hasNext();) {
					ImageCache item = (ImageCache)it.next();
					i = i + 1;
					if (file.equals(item.getConfigFileName()) && (language == item.getPageLanguage()) && dataMode.equals(item.getDataSource())) index = i;
				}
				if (index == -1) {
					chartCache = new ImageCache("chart", file, language, dataMode, interval);
					chartStream = chartCache.addToCache(page, id);
					chartCacheList.add(chartCache);
				}
				else {
					chartCache = chartCacheList.get(index);
					if (chartCache.exists(id) == false) chartStream = chartCache.addToCache(page, id);
					if (chartCache.exists(id) == true) chartStream = chartCache.getFromCache(page, id);
				}
			}
		}
		else {		
			try {
				ChartUtilities.writeChartAsPNG(chartStream, sessionHandler.getWeb2cPage().getChart(id), sessionHandler.getWeb2cPage().getChartWidth(id), sessionHandler.getWeb2cPage().getChartHeight(id));
			} 
			catch (IOException e) {
//				System.out.println("Error converting chart to byte stream !");
			}
		}
		
		return chartStream;
	}
	
	/**
	 * Returns video; updates video entry in cache if update interval exceeded; adds video entry to cache if missing; deletes content of cache if timeout interval exceeded
	 * @param sessionHandler session handler
	 * @param id component id
	 * @return video as stream
	 */
	public synchronized ByteArrayOutputStream getVideoFromCache(Web2cSessionHandler sessionHandler, String id) {

		String file = "";
		int language = 0;
		String dataMode = "";
		int interval = 0;
		Web2cPage page = null;
		ImageCache videoCache = null;
		ByteArrayOutputStream videoStream = null;
		int index = 0;
		int i = 0;

		if (cacheIsEnabled == true) {			
			file = sessionHandler.getAttribute("loadedConfigFileName");
			language = Integer.valueOf((String)sessionHandler.getAttribute("language"));
			dataMode = sessionHandler.getAttribute("dataSource");
			interval = sessionHandler.getWeb2cPage().getUpdateTime();
			page = sessionHandler.getWeb2cPage();	

			synchronized(videoCacheList) {
				index = -1;
				i = -1;	
				for (Iterator<ImageCache>it = videoCacheList.iterator(); it.hasNext();) {
					ImageCache item = (ImageCache)it.next();
					i = i + 1;
					if (file.equals(item.getConfigFileName()) && (language == item.getPageLanguage()) && dataMode.equals(item.getDataSource())) index = i;
				}
	
				if (index == -1) {
					videoCache = new ImageCache("video", file, language, dataMode, interval);
					videoStream = videoCache.addToCache(page, id);
					videoCacheList.add(videoCache);
				}
				else {
					videoCache = videoCacheList.get(index);
					if (videoCache.exists(id) == false) videoStream = videoCache.addToCache(page, id);
					if (videoCache.exists(id) == true) videoStream = videoCache.getFromCache(page, id);
				}
			}
		}
		else {	
			sessionHandler.getWeb2cPage().getVideoImage(id, videoStream);
		}
		
		return videoStream;
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class ImageCache implements Observer{

		private String cacheType = "";
		private String configFileName = "";
		private String dataSource = "";
		private int pageLanguage = 0;
		private int updateTime = 0;
		private long mostRecentUpdate = 0;
		private ArrayList<Long> lastUpdate = new ArrayList<Long>();
		private ArrayList<ByteArrayOutputStream> cacheData = new ArrayList<ByteArrayOutputStream>();
		private ArrayList<String> componentId = new ArrayList<String>(); 
		private ArrayList<ImageCacheUpdater> cacheUpdater = new ArrayList<ImageCacheUpdater>();
		
		/**
		 * @param type cache type ("chart", "video")
		 * @param file configuration file name
		 * @param language page language
		 * @param dataMode data source
		 * @param interval update time interval
		 */
		public ImageCache(String type, String file, int language, String dataMode, int interval) {
			
			super();	
			cacheType = type;			
			configFileName = file;
			pageLanguage = language;
			dataSource = dataMode;
			updateTime = interval;	
		}	
		
		/**
		 * Returns the configuration file name 
		 * @return  configuration file name
		 */
		public String getConfigFileName() {
			
			return configFileName;
		}
		
		/**
		 * Returns the page language 
		 * @return  page language
		 */
		public int getPageLanguage() {
			
			return pageLanguage;
		}
		
		/**
		 * Returns the cache type 
		 * @return cache type
		 */
		public String getCacheType() {
			
			return cacheType;
		}
		
		/**
		 * Returns the data source 
		 * @return data source
		 */
		public String getDataSource() {
			
			return dataSource;
		}
		
		/**
		 * Returns the list of component identifiers. 
		 * @return list of component identifiers
		 */
		public ArrayList<String> getComponentId() {
			
			return componentId;
		}
		
		/**
		 * Returns the list of most recent component's update times. 
		 * @return list of most recent component's update times.
		 */
		public ArrayList<Long> getLastUpdate() {
			
			return lastUpdate;
		}
		
		/**
		 * Returns the most recent update time
		 * @return most recent update time
		 */
		public long getMostRecentUpdate() {
			
			return mostRecentUpdate;
		}
		
		/**
		 * Adds initial data with component identifier id to the cache and writes the data to the output stream; starts the data updater
		 * @param id identifier
		 * @return data
		 */
		public ByteArrayOutputStream addToCache(Web2cPage page, String id) {
			
			ByteArrayOutputStream cacheDataToAdd = null;
			long now = 0;
			
			if (exists(id) == false) {
				now = System.currentTimeMillis();
				cacheDataToAdd = new ByteArrayOutputStream();
				cacheData.add(cacheDataToAdd);
				lastUpdate.add(now);	
				mostRecentUpdate = now;
				componentId.add(id);
				ImageCacheUpdater updater = new ImageCacheUpdater(page, id, updateTime, cacheType);
				cacheUpdater.add(updater);
				updater.addObserver(this);
//				System.out.println("New entry added to cache and cache updater started at " + Calendar.getInstance().getTime().toString() + ": configFile = " + configFileName  + ", componentId = " + id);
			}
			
			return cacheDataToAdd;
		}
		
		/**
		 * WGets from the cache the data with component identifier id
		 * @param id identifier
		 * @return data
		 */
		public ByteArrayOutputStream getFromCache(Web2cPage page, String id) {
			
			ByteArrayOutputStream cacheDataToGet = null;
			int index = -1;
			long now = 0;

			if (exists(id) == true) {
				index = componentId.indexOf(id);
				now = System.currentTimeMillis();
				cacheDataToGet = cacheData.get(index);
				lastUpdate.set(index, now);		
				mostRecentUpdate = now;
//				System.out.println("Get from cache at " + Calendar.getInstance().getTime().toString() + ": configFile = " + configFileName  + ", componentId = " + id);
				ImageCacheUpdater oldCacheUpdater = cacheUpdater.get(index);
				if (oldCacheUpdater.getUpdaterPage().getUpdaterStatus() == false) {
					oldCacheUpdater.stopUpdate();
					oldCacheUpdater.deleteObserver(this);
					ImageCacheUpdater updater = new ImageCacheUpdater(page, id, updateTime, cacheType);
					cacheUpdater.set(index, updater);
					updater.addObserver(this);
//					System.out.println("Cache updater switched at " + Calendar.getInstance().getTime().toString() + ": configFile = " + configFileName  + ", componentId = " + id);
				}
			}
			
			return cacheDataToGet;
		}
		
		/**
		 * Updates the cache if update time interval is exceeded
		 * @param updater updater
		 * @param dataToAdd updated data
		 */
		public void update(Observable updater, Object dataToAdd) {

			int index = -1;
			String id = "";
			
			index = cacheUpdater.indexOf(updater);
			id = componentId.get(index);
			cacheData.set(index, (ByteArrayOutputStream)dataToAdd);
//			System.out.println("Cache updated at " + Calendar.getInstance().getTime().toString() + ": configFile = " + configFileName  + ", componentId = " + id);
		}
		
		/**
		 * Checks whether data with component identifier id already exists in the cache
		 * @return exist flag
		 */
		public boolean exists(String id) {
			
			boolean existFlag = false;

			existFlag = componentId.contains(id);
			
			return existFlag;
		}
		
		/**
		 * Clears the cache instance; does garbage collection
		 */
		public void clear() {

//			System.out.println("Cache updater stopped and cache including all entries deleted (cache timeout interval exceeded) at " + Calendar.getInstance().getTime().toString() + ": configFile = " + configFileName);
			cacheData.clear();
			lastUpdate.clear();
			componentId.clear();
			for (Iterator<ImageCacheUpdater>it = cacheUpdater.iterator(); it.hasNext();) {
				ImageCacheUpdater updater = (ImageCacheUpdater)it.next();
				updater.deleteObserver(this);
				updater.stopUpdate();
			}
			cacheUpdater.clear();
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class ImageCacheUpdater extends Observable implements Runnable {

		private Thread updater = null;
		boolean updaterStarted = false;
		private String cacheType = "";
		private int updateInterval = 0;
		private String componentId = "";
		private Web2cPage updatePage = null;
		
		/**	
		 * @param page update page
		 * @param id component id
		 * @param interval update time interval
		 * @param type cache type ("chart", "video")
		 */
		public ImageCacheUpdater (Web2cPage page, String id, int interval, String type) {
			
			super();
			
			updatePage = page;
			componentId = id;
			updateInterval = interval;
			cacheType = type;
		
			if (updater == null) {
				updater = new Thread(this);
				updaterStarted = true;
				updater.start();
			}
		}
		
		/**
		 * Updates periodically the cache data for the component with identifier id
		 */
		public void run() {

			while (updaterStarted == true) {
				
				ByteArrayOutputStream cacheDataToAdd = new ByteArrayOutputStream();
				try {
					if (cacheType.equals("chart")) ChartUtilities.writeChartAsPNG(cacheDataToAdd, updatePage.getChart(componentId), updatePage.getChartWidth(componentId), updatePage.getChartHeight(componentId));
					if (cacheType.equals("video")) updatePage.getVideoImage(componentId, cacheDataToAdd);
				} 
				catch (IOException e) {
//					System.out.println("Error converting data to byte stream !");
				}
				
				setChanged();
				notifyObservers(cacheDataToAdd);

				try {Thread.sleep(updateInterval);}
				catch (InterruptedException e) {}
			}
			
			updater.interrupt();
		}
		
		/**
		 * Terminates updating cache data
		 */
		public void stopUpdate() {
			
			updaterStarted = false;
		}
		
		/**
		 * Returns the page which provides update data
		 */
		public Web2cPage getUpdaterPage() {
			
			return updatePage;
		}
	}
}
