package de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Random;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import de.desy.web2cToolkit.web2c.web2cChatUtilities.Web2cChatHandler;
import de.desy.web2cToolkit.web2c.web2cComponents.Web2cPage;
import de.desy.web2cToolkit.web2c.web2cDeviceAccess.Web2cCommonDeviceAccess;
import de.desy.web2cToolkit.web2c.web2cVideoUtilities.Web2cCommonVideoConnector;
import de.desy.web2cToolkit.web2cArchive.web2cArchiveViewerComponents.Web2cArchiveViewerPage;
import de.desy.web2cToolkit.web2cGateway.Web2cGatewayPage;
import de.desy.web2cToolkit.web2cLogbook.Web2cLogbookAutoReporter;
import de.desy.web2cToolkit.web2cLogbook.Web2cLogbookPage;
import de.desy.web2cToolkit.web2cMessenger.Web2cSMTPInterface;
import de.desy.web2cToolkit.web2cMessenger.Web2cTwitterInterface;
import de.desy.web2cToolkit.web2cServlets.Web2cManager;
import de.desy.web2cToolkit.web2cServlets.Web2cManager.Web2cPerformanceMonitor;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cFileLogger;
import de.desy.web2cToolkit.web2cUtil.web2cDebug.Web2cKeyBoardListener;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cArchiveViewerProbe;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cEditorProbe;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cGatewayProbe;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cLogbookProbe;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cManagerProbe;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cMessengerProbe;
import de.desy.web2cToolkit.web2cUtil.web2cDiagnostics.Web2cProbe;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cGatewayConfigReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cInitWriterAndReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cJavascriptWriter;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cLogbookInitFileWriterAndReader;
import de.desy.web2cToolkit.web2cUtil.web2cFileIO.Web2cRegistryWriterAndReader;
import de.desy.web2cToolkit.web2cUtil.web2cInterfaces.IWeb2cConstants;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cGarbageCollector;
import de.desy.web2cToolkit.web2cUtil.web2cLifeCycleControl.Web2cSessionHandler;
import de.desy.web2cToolkit.web2cUtil.web2cSecurity.Web2cMD5HashGenerator;

/**
 * @author  R.Bacher, DESY
 */
public class Web2cServletHandler implements IWeb2cConstants, Observer {
	
	private ServletContext servletContext = null;
	private Web2cContextHandler servletContextHandler = null;
	private String servletName = null;	
	private HttpServlet servletInstance = null;	
	private Class<?> servletClass = null;
	private int inactivityTimeout = 0;   // HTTP session inactivity timeout [s]
	private String userDir = null;
	private String webDir = null;
	private String logbookDir = null;
	
	private ArrayList<Web2cSessionHandler> web2cSessionHandlerList = null;
	private boolean sessionWatcherIsEnabled = sessionWatcherEnabled;	
	private int maxSessions = 0; // maximum number of Web2c sessions allowed
	
	private Web2cRegistryWriterAndReader registryWriterAndReader = null;
	private Web2cInitWriterAndReader initWriterAndReader = null;
	
	private Web2cFileLogger sessionLogger = null;
	private Web2cFileLogger debugLogger = null;
	private Web2cKeyBoardListener debugListener = null;
	private Web2cFileLogger actionLogger = null;
	
	private Object probe = null;

	private Web2cImageCache imageCache = null;
	private boolean cacheIsEnabled = cacheEnabled;
	private Web2cChatHandler chatHandler = null;

	private Web2cGatewayConfigReader gatewayConfigReader = null;
	
	private Web2cTwitterInterface twitterInterface = null;
	private Web2cSMTPInterface smtpInterface = null;
	
	private Web2cLogbookAutoReporter logbookAutoReporter = null;
	private Web2cLogbookInitFileWriterAndReader logbookInitFileWriterAndReader = null;
	
	private Web2cPerformanceMonitor performanceMonitor = null;

	private Web2cCommonVideoConnector commonVideoConnector = null;
	private Web2cCommonDeviceAccess commonDeviceAccess = null;
	
	private Web2cGarbageCollector garbageCollector = null;
	private boolean garbageCollectionIsEnabled = garbageCollectionEnabled;

	private Double[] constructionProgress = null;

	/**
	 * Constructor; 
	 * - instantiates all servlet specific fields; 
	 * - performs user login procedure
	 * @param context servlet context
	 * @param name servlet name
	 * @param servlet servlet instance
	 */
	public Web2cServletHandler(ServletConfig config, HttpServlet servlet) {
		
		servletContext = config.getServletContext();
		servletContextHandler = Web2cContextHandler.getInstance(servletContext);
		servletName = config.getServletName();
		servletInstance = servlet;		
		try {
			servletClass = Class.forName("de.desy.web2cToolkit.web2cServlets." + servletName);
		} 
		catch (ClassNotFoundException e) {e.printStackTrace();}
		garbageCollectionIsEnabled = servletContextHandler.getJavaGarbageCollection();
		sessionWatcherIsEnabled = servletContextHandler.getWeb2cToolkitGarbageCollection();
		
		System.out.println("Initializing servlet: " + servletName);
		
		if (garbageCollectionIsEnabled == true) System.out.println("	Java garbage collection enabled!");	
		if (garbageCollectionIsEnabled == false) System.out.println("	Java garbage collection disabled!");
		
		if (sessionWatcherIsEnabled == true) System.out.println("	Web2cToolkit garbage collection enabled!");	
		if (sessionWatcherIsEnabled == false) System.out.println("	Web2ctoolkit garbage collection disabled!");

		if (web2cSessionHandlerList == null) {
			web2cSessionHandlerList = new ArrayList<Web2cSessionHandler>();
		}
		if (garbageCollector == null) {
			garbageCollector = new Web2cGarbageCollector(servletName);
		}
		if (debugListener == null) {
			debugListener = new Web2cKeyBoardListener();
		}
		if (userDir == null) {
			userDir = servletContextHandler.getUserDir();
		}
		if (webDir == null) {
			webDir = servletContextHandler.getWebDir();
		}
		if (logbookDir == null) {
			logbookDir = servletContextHandler.getLogbookDir();
		}
		if (registryWriterAndReader == null) {
			String fileName = "";
			if ((servletContextHandler.getWeb2cUserRegistryFileName()).equals("")) fileName = web2cUserRegistryFile;
			else fileName = servletContextHandler.getWeb2cUserRegistryFileName();
			registryWriterAndReader = new Web2cRegistryWriterAndReader(new File(registryDir + File.separator + fileName));
		}		
		if (initWriterAndReader == null) {
			String fileName = "";
			if ((servletContextHandler.getWeb2cInitFileName()).equals("")) fileName = web2cInitFile;
			else fileName = servletContextHandler.getWeb2cInitFileName();
			initWriterAndReader = new Web2cInitWriterAndReader(new File(registryDir + File.separator + fileName), userDir);
		}

		if (servletName.equals("Web2c")) {
			inactivityTimeout = sessionTimeoutInterval;
			maxSessions = maxSessionNumber;
			if (cacheIsEnabled == true) System.out.println("	Web2c cache enabled!");	
			if (cacheIsEnabled == false) System.out.println("	Web2c cache disabled!");
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + synopticDisplaySessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + synopticDisplayLogFile), false);
			}
			if (imageCache == null) {
				imageCache = new Web2cImageCache();
			}		
			if (chatHandler == null) {
				chatHandler = new Web2cChatHandler();
			}
			if (commonDeviceAccess == null) {
				actionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + synopticDisplayCdaActionLogFile), true);
				commonDeviceAccess = new Web2cCommonDeviceAccess(debugListener, debugLogger, actionLogger);
			}
			if (commonVideoConnector == null) {
				commonVideoConnector = new Web2cCommonVideoConnector();
			}
			if (probe == null) {
				probe = new Web2cProbe(web2cSessionHandlerList, imageCache, commonDeviceAccess, garbageCollector);
			}
		}
		
		if (servletName.equals("Web2cArchiveViewer")) {
			inactivityTimeout = archiveViewerSessionTimeoutInterval;
			maxSessions = maxArchiveViewerSessionNumber;
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + archiveViewerSessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + archiveViewerLogFile), false);
			}
			if (probe == null) {
				probe = new Web2cArchiveViewerProbe(web2cSessionHandlerList, garbageCollector);
			}
		}	
		
		if (servletName.equals("Web2cEditor")) {
			inactivityTimeout = editorSessionTimeoutInterval;
			maxSessions = maxEditorSessionNumber;
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + editorSessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + editorLogFile), false);
			}
			if (probe == null) {
				probe = new Web2cEditorProbe(web2cSessionHandlerList, garbageCollector);
			}
		}
		
		if (servletName.equals("Web2cGateway")) {	
			inactivityTimeout = gatewaySessionTimeoutInterval;
			maxSessions = maxGatewaySessionNumber;		
			if (gatewayConfigReader == null) {
				gatewayConfigReader = new Web2cGatewayConfigReader(new File(userDir + File.separator + gatewayConfigFile));
			}	
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + gatewaySessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + gatewayLogFile), false);
			}
			if (commonDeviceAccess == null) {
				actionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + gatewayCdaActionLogFile), true);
				commonDeviceAccess = new Web2cCommonDeviceAccess(debugListener, debugLogger, actionLogger);
			}
			if (probe == null) {
				probe = new Web2cGatewayProbe(web2cSessionHandlerList, commonDeviceAccess, garbageCollector);
			}
		}
		
		if (servletName.equals("Web2cMessenger")) {
			inactivityTimeout = messengerSessionTimeoutInterval;
			maxSessions = maxMessengerSessionNumber;
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + messengerSessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + messengerLogFile), false);
			}
			if (commonDeviceAccess == null) {
				actionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + messengerCdaActionLogFile), true);
				commonDeviceAccess = new Web2cCommonDeviceAccess(debugListener, debugLogger, actionLogger);
			}
			if (twitterInterface == null) {
				String initFileName = "";
				if (servletContextHandler.getWeb2cTwitterInitFileName().equals("")) initFileName = web2cTwitterInitFile;
				else initFileName = servletContextHandler.getWeb2cTwitterInitFileName();
				twitterInterface = new Web2cTwitterInterface(initFileName, commonDeviceAccess, userDir, debugLogger);
			}
			if (smtpInterface == null) {
				String initFileName = "";
				if (servletContextHandler.getWeb2cSMTPInitFileName().equals("")) initFileName = web2cSMTPInitFile;
				else initFileName = servletContextHandler.getWeb2cSMTPInitFileName();
				smtpInterface = new Web2cSMTPInterface(initFileName, debugLogger);
				smtpInterface.armService();
			}
			if (probe == null) {
				probe = new Web2cMessengerProbe(web2cSessionHandlerList, twitterInterface, commonDeviceAccess, garbageCollector);
			}
		}		
		
		if (servletName.equals("Web2cManager")) {
			inactivityTimeout = managerSessionTimeoutInterval;
			maxSessions = maxManagerSessionNumber;
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + managerSessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + managerLogFile), false);
			}
			if (probe == null) {
				probe = new Web2cManagerProbe(web2cSessionHandlerList, garbageCollector);
			}
			if (performanceMonitor == null) {
				performanceMonitor = ((Web2cManager)servletInstance).new Web2cPerformanceMonitor();
				performanceMonitor.startMonitor();
			}
		}
		
		if (servletName.equals("Web2cLogbook")) {	
			inactivityTimeout = logbookSessionTimeoutInterval;
			maxSessions = maxLogbookSessionNumber;	
			if (sessionLogger == null) {
				sessionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + logbookSessionLogFile), true);
			}
			if (debugLogger == null) {
				debugLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + logbookLogFile), false);
			}
			if (commonDeviceAccess == null) {
				actionLogger = new Web2cFileLogger(new File(servletContext.getRealPath(logDir) + File.separator + logbookCdaActionLogFile), true);
				commonDeviceAccess = new Web2cCommonDeviceAccess(debugListener, debugLogger, actionLogger);
			}
			if (smtpInterface == null) {
				smtpInterface = servletContextHandler.getWeb2cSMTPInterface();
			}
			if (logbookAutoReporter == null) {
				String initFileName = "";
				if (servletContextHandler.getWeb2cLogbookInitFileName().equals("")) initFileName = web2cLogbookInitFile;
				else initFileName = servletContextHandler.getWeb2cLogbookInitFileName();
				logbookInitFileWriterAndReader = new Web2cLogbookInitFileWriterAndReader(initFileName);
				logbookAutoReporter = new Web2cLogbookAutoReporter(userDir, logbookDir, commonDeviceAccess, smtpInterface, logbookInitFileWriterAndReader);
				logbookAutoReporter.armService();
			}
			if (probe == null) {
				probe = new Web2cLogbookProbe(web2cSessionHandlerList, commonDeviceAccess, garbageCollector);
			}
		}
		
		constructionProgress = new Double[1];
	}
	
	/**
	 * Returns servlet instance
	 * @return servlet instance
	 */
	public HttpServlet getServlet() {
		
		return servletInstance;
	}
	
	/**
	 * Returns servlet class
	 * @return servlet class
	 */
	public Class<?> getServletClass() {
		
		return servletClass;
	}
	
	/**
	 * Returns user dir
	 * @return user dir
	 */
	public String getUserDir() {
		
		return userDir;
	}
	
	/**
	 * Returns web dir
	 * @return web dir
	 */
	public String getWebDir() {
		
		return webDir;
	}
	
	/**
	 * Returns logbook dir
	 * @return logbook dir
	 */
	public String getLogbookDir() {
		
		return logbookDir;
	}
	
	/**
	 * Returns session handler list
	 * @return  session handler list
	 */
	public ArrayList<Web2cSessionHandler>getWeb2cSessionHandlerList() {
		
		return web2cSessionHandlerList;
	}
	
	/**
	 * Returns probe
	 * @return probe
	 */
	public Web2cProbe getWeb2cProbe() {
		
		return (Web2cProbe)probe;
	}
	
	/**
	 * Returns archive viewer probe
	 * @return probe
	 */
	public Web2cArchiveViewerProbe getWeb2cArchiveViewerProbe() {
		
		return (Web2cArchiveViewerProbe)probe;
	}
	
	/**
	 * Returns editor probe
	 * @return probe
	 */
	public Web2cEditorProbe getWeb2cEditorProbe() {
		
		return (Web2cEditorProbe)probe;
	}
	
	/**
	 * Returns gateway probe
	 * @return probe
	 */
	public Web2cGatewayProbe getWeb2cGatewayProbe() {
		
		return (Web2cGatewayProbe)probe;
	}
	
	/**
	 * Returns messenger probe
	 * @return probe
	 */
	public Web2cMessengerProbe getWeb2cMessengerProbe() {
		
		return (Web2cMessengerProbe)probe;
	}
	
	/**
	 * Returns manager probe
	 * @return probe
	 */
	public Web2cManagerProbe getWeb2cManagerProbe() {
		
		return (Web2cManagerProbe)probe;
	}
	
	/**
	 * Returns logbook probe
	 * @return probe
	 */
	public Web2cLogbookProbe getWeb2cLogbookProbe() {
		
		return (Web2cLogbookProbe)probe;
	}
	
	/**
	 * Returns image cache
	 * @return image cache
	 */
	public Web2cImageCache getWeb2cImageCache() {
		
		return imageCache;
	}
	
	/**
	 * Returns video connector
	 * @return video connector
	 */
	public Web2cCommonVideoConnector getWeb2cCommonVideoConnector() {
		
		return commonVideoConnector;
	}	
	
	/**
	 * Returns chat handler
	 * @return chat handler
	 */
	public Web2cChatHandler getWeb2cChatHandler() {
		
		return chatHandler;
	}	
	
	/**
	 * Returns common device access interface
	 * @return common device access interface
	 */
	public Web2cCommonDeviceAccess getWeb2cCommonDeviceAccess() {
		
		return commonDeviceAccess;
	}
	
	/**
	 * Returns twitter interface
	 * @return twitter interface
	 */
	public Web2cTwitterInterface getWeb2cTwitterInterface() {
		
		return twitterInterface;
	}
	
	/**
	 * Returns SMTP interface
	 * @return SMTP interface
	 */
	public Web2cSMTPInterface getWeb2cSMTPInterface() {
		
		return smtpInterface;
	}
	
	/**
	 * Returns logbook auto-reporter
	 * @return reporter
	 */
	public Web2cLogbookAutoReporter getWeb2cLogbookAutoReporter() {
		
		return logbookAutoReporter;
	}
	
	/**
	 * Returns logbook init file reader/writer
	 * @return reader/writer
	 */
	public Web2cLogbookInitFileWriterAndReader getWeb2cLogbookInitFileWriterAndReader() {
		
		return logbookInitFileWriterAndReader;
	}
	
	/**
	 * Returns performance monitor
	 * @return monitor
	 */
	public Web2cPerformanceMonitor getWeb2cPerformanceMonitor() {
		
		return performanceMonitor;
	}
	
	/**
	 * Returns garbage collector
	 * @return garbage collector
	 */
	public Web2cGarbageCollector getWeb2cGarbageCollector() {
		
		return garbageCollector;
	}
	
	public Web2cKeyBoardListener getWeb2cKeyBoardListener() {
		
		return debugListener;
	}
	
	/**
	 * Returns registry writer and reader
	 * @return registry writer and reader
	 */
	public Web2cRegistryWriterAndReader getWeb2cRegistryWriterAndReader() {
		
		return registryWriterAndReader;
	}
	
	/**
	 * Returns init writer and reader
	 * @return init writer and reader
	 */
	public Web2cInitWriterAndReader getWeb2cInitWriterAndReader() {
		
		return initWriterAndReader;
	}
	
	/**
	 * Returns gateway config reader
	 * @return gateway config reader
	 */
	public Web2cGatewayConfigReader getWeb2cGatewayConfigReader() {
		
		return gatewayConfigReader;
	}
	
	/**
	 * Returns session logger
	 * @return  session logger
	 */
	public Web2cFileLogger getSessionLogger() {
		
		return sessionLogger;
	}
	
	/**
	 * Returns debug logger
	 * @return  debug logger
	 */
	public Web2cFileLogger getDebugLogger() {
		
		return debugLogger;
	}
	
	/**
	 * Returns action logger
	 * @return  action logger
	 */
	public Web2cFileLogger getActionLogger() {
		
		return actionLogger;
	}
	
	/**
	 * Returns http session inactivity timeout
	 * @return  time out interval
	 */
	public int getHTTPSessionTimeout() {
		
		return inactivityTimeout;
	}
	
	/**
	 * Returns maximum session number
	 * @return number
	 */
	public int getMaxSessionNumber() {
		
		return maxSessions;
	}
	
	/**
	 * Returns the session handler associated to this session id
	 * @param sessionId session id
	 * @return handler
	 */
	public Web2cSessionHandler getSessionHandler (String sessionId) {
		
		Web2cSessionHandler associatedSessionHandler = null;
		
		synchronized (getWeb2cSessionHandlerList()) {
			for (Iterator<Web2cSessionHandler> it = getWeb2cSessionHandlerList().iterator(); it.hasNext();) {
				Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
				if (((String)sessionHandler.getAttribute("id")).equals(sessionId)) {
					associatedSessionHandler = sessionHandler;
				}
			}
		}
		
		return associatedSessionHandler;
	}
	
	/**
	 * Creates and returns the session handler for this session id
	 * @param sessionId session id
	 * @return handler
	 */
	public Web2cSessionHandler createSessionHandler (String sessionId) {
		
		Web2cSessionHandler associatedSessionHandler = null;

		synchronized (getWeb2cSessionHandlerList()) {
			if (getWeb2cSessionHandlerList().toArray().length < maxSessions) {
				associatedSessionHandler = new Web2cSessionHandler(sessionId);
				getWeb2cSessionHandlerList().add(associatedSessionHandler);
			}
		}
		
		return associatedSessionHandler;
	}
	
	/**
	 * Returns login page
	 * @param randomNumber random number for password encryption
	 * @param timeStamp timeStamp to make login file name unique
	 * @return HTML page
	 */
	public ArrayList<String> getLoginPage(String randomNumber, String timeStamp) {

		ArrayList<String> out = null;
		String enterKeyCode = "13";
		
		out = new ArrayList<String>();
		
		out.add("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		out.add("<html>");
		out.add("   <head>");
		out.add("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		out.add("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		out.add("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		out.add("       <meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
		out.add("       <title>" + servletName + " Login</title>");
		out.add("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
		out.add("       <script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + servletName + "Login_" + timeStamp + ".js\"></script>");
		out.add("       <script type=\"text/javascript\" src=\"" + web2cdocs + directorySeparator + "md5.js\"></script>");
		out.add("   </head>");
		out.add("	<body onload=\"initPage()\">");
		out.add("		<input type=\"hidden\" id=\"random\" name=\"random\" value=\"" + randomNumber + "\">");
		if (initWriterAndReader != null) {
			out.add("		<div align=\"center\"><b><h1>" + initWriterAndReader.getSalutation() + "</h1></b></div>");
		}
		else {
			out.add("		<div align=\"center\"><b><h1>" + web2cSalutation + "</h1></b></div>");
		}
		out.add("		<div align=\"center\"><b><h2>Please enter your username and password to log in.</h2></b></div>");
		out.add("		<div align=\"center\">");
		out.add("			<table cellspacing=\"0\" class=\"lined\">");
		out.add("				<tr><td align=\"left\"><b>User name: </b></td><td class=\"control\" height=\"50\"><input type=\"text\" name=\"username\" id=\"username\" size=\"20\" onKeyPress=\"if (event.keyCode == " + enterKeyCode + ") {login(); return false;}\"></td></tr>");
		out.add("				<tr><td align=\"left\"><b>Password: </b></td><td class=\"control\" height=\"50\"><input type=\"password\" id=\"password\" name=\"password\" size=\"20\" onKeyPress=\"if (event.keyCode == " + enterKeyCode + ") {login(); return false;}\"></td></tr>");
		out.add("				<tr><td> </td><td class=\"control\" height=\"25\"></td></tr>");
		out.add("				<tr><td colspan=\"3\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"login\" id=\"login\" value=\"Login\" onclick=\"login()\"></center></td></tr>");
		out.add("				<tr><td colspan=\"3\" class=\"control\" height=\"50\"><center><input type=\"button\" name=\"close\" id=\"close\" value=\"Close Login-Window\" onclick=\"closeLoginWindow()\"></center></td></tr>");
		out.add("			</table>");
		out.add("		</div>");
		out.add("		<p></p><p></p><p align=\"center\"><b>N.B.</b> Web2c pages are best displayed with the following browser's default font settings: encoding = Western, variable-font size: 16px, fixed-font size: 13px, minimum font size: 6px or smaller</p>");
		out.add("	</body>");
		out.add("</html>");
		
		return out;
	}
	
	/**
	 * Returns Javascript code
	 * @param sessionHandler session handler
	 * @return Javascript code as text
	 */
	public ArrayList<String> getJavascript(Web2cSessionHandler sessionHandler) {
		
		ArrayList<String> javascript = new ArrayList<String>();

		String sessionId = (String)(sessionHandler.getAttribute("id"));

		javascript.add("		var xhr;");
		javascript.add("		function initPage() {");
		javascript.add("			initXhr();");
		javascript.add("			setTimeout(closeLoginWindow," + loginTimeOut + ");");
		javascript.add("		}");
		javascript.add("		function encryptPassword() {");
		javascript.add("			var data = document.getElementById(\"password\").value;");
		javascript.add("			var random = document.getElementById(\"random\").value;");
		javascript.add("			var string = data + random;");
		javascript.add("			return hex_md5(string);");
		javascript.add("		}");
		javascript.add("		function initXhr() {"); 
		javascript.add("			try {");                                                  
		javascript.add("				xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		javascript.add("			} catch (event1) {");                             
		javascript.add("				try {");
		javascript.add("					xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		javascript.add("				} catch (event2) {");
		javascript.add("					xhr = false;");
		javascript.add("				}");
		javascript.add("			}");
		javascript.add("			if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		javascript.add("				xhr = new XMLHttpRequest();");                 
		javascript.add("			}"); 																	
		javascript.add("		}");
		javascript.add("		function login() {");
		javascript.add("			var now = new Date();");
		javascript.add("			var password = encryptPassword();");
		javascript.add("			var username = document.getElementById(\"username\").value;");
		javascript.add("			xhr.open(\"post\", \"/web2cToolkit/" + servletName + "?param=<elogin><sessionId>" + sessionId + "</sessionId><userName>\" + username + \"</userName><password>\" + password + \"</password><timeStamp>\" + String(now.getTime()) + \"</timeStamp></elogin>\", true);"); 
		javascript.add("			xhr.onreadystatechange = function() {");
		javascript.add("				if (xhr.readyState == 4) {");
		javascript.add("					window.location.href = xhr.responseText;");
		javascript.add("           		};");   
		javascript.add("            };"); 
		javascript.add("			xhr.send(null);");  
		javascript.add("		}");
		javascript.add("		function closeLoginWindow() {");
		javascript.add("			window.close();");  
		javascript.add("		}");
		
		return javascript;
	}
	
	/**
	 * Returns error page if page creation has failed
	 * @param configFileName name of configuration file
	 * @return page
	 */
	public String  getPageCreationErrorPage(String configFileName) {
		
		StringBuffer strBfr = new StringBuffer();
		strBfr.append("<html>");
		strBfr.append("	<body>");
		strBfr.append("		<div>Page \"" + configFileName + "\" could not be created!</div>");
		strBfr.append("		<div>Reasons might be:</div>");
		strBfr.append("		<div> - Configuration file does not exist</div>");
		strBfr.append("		<div> - Wrong configuration file name</div>");
		strBfr.append("		<div> - Syntax error(s) in configuration file</div>");
		strBfr.append("		<div> - Invalid control system address(es) specified in configuration file</div>");
		strBfr.append("	</body>");
		strBfr.append("</html>");
		
		return strBfr.toString();
	}
	
	/**
	 * Returns error page if session creation has failed
	 * @param configFileName name of configuration file
	 * @return page
	 */
	public String  getSessionCreationErrorPage(String configFileName) {

		StringBuffer strBfr = new StringBuffer();
		strBfr.append("<html>");
		strBfr.append("<body>");
		strBfr.append("<div>Page \"" + configFileName + "\" could not be created!</div>");
		strBfr.append("<div>Resources exhausted. Please try again later.</div>");
		strBfr.append("</body>");
		strBfr.append("</html>");
		
		return strBfr.toString();
	}
	
	/**
	 * Returns loading progress page
	 * @param sessionId session id
	 * @return page
	 */
	public String getLoadingProgressPage(String sessionId) {

		StringBuffer strBfr = new StringBuffer();
		strBfr.append("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		strBfr.append("<html>");
		strBfr.append("   <head>");
		strBfr.append("		<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">");
		strBfr.append("		<meta http-equiv=\"pragma\" content=\"no-cache\">");
		strBfr.append("		<meta http-equiv=\"cache-control\" content=\"no-cache\">");
		strBfr.append("		<title>Web2cSynopticDisplayViewer loading...</title>");
		strBfr.append("		<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">");
		strBfr.append("     <script type=\"text/javascript\">");
		strBfr.append("			var xhr;");
		strBfr.append("			var sessionId = " + sessionId + ";");
		strBfr.append("			function initPage() {");	
		strBfr.append("				setInterval(\"getPageLoadingStatus()\", 1000);");
		strBfr.append("			}");		
		strBfr.append("			function getPageLoadingStatus() {"); 
		strBfr.append("				try {");                                                  
		strBfr.append("					xhr = new ActiveXObject(\"Msxml2.XMLHTTP\");");                     
		strBfr.append("				} catch (event1) {");                             
		strBfr.append("					try {");
		strBfr.append("						xhr = new ActiveXObject(\"Microsoft.XMLHTTP\");");
		strBfr.append("					} catch (event2) {");
		strBfr.append("						xhr = false;");
		strBfr.append("					}");
		strBfr.append("				}");
		strBfr.append("				if (!xhr && typeof XMLHttpRequest !=\"undefined\") {");
		strBfr.append("					xhr = new XMLHttpRequest();");                 
		strBfr.append("				}"); 	
		strBfr.append("				var now = new Date();");
		strBfr.append("				xhr.open(\"post\", \"/web2cToolkit/" + servletName + "?param=<ldchck><sessionId>\" + sessionId + \"</sessionId><timeStamp>\" + String(now.getTime()) + \"</timeStamp></ldchck>\", true);"); 
		strBfr.append("				xhr.onreadystatechange = function() {");       
		strBfr.append("					if (xhr.readyState == 4) {"); 
		strBfr.append("						if ((xhr.responseText.indexOf(\"loading\") == -1) && (xhr.responseText != \"\")) {"); 
		strBfr.append("							window.location.href = location.href.split(\"=\")[0] + \"=\" + xhr.responseText;");
		strBfr.append("						}");      
		strBfr.append("						else {");  
		strBfr.append("							ref = document.getElementById(\"loadingText\");");
		strBfr.append("							ref.innerHTML = xhr.responseText;");      
		strBfr.append("						}");          
		strBfr.append("					}");          
		strBfr.append("				};");   
		strBfr.append("				xhr.send(null);");  
		strBfr.append("			}");  
		strBfr.append("       </script>");
		strBfr.append("	</head>");
		strBfr.append("	<body onload=\"initPage()\">");
		strBfr.append("		<h1><b><div id=\"loadingText\">Page loading: 0% ...</div></b></h1>");
		strBfr.append("	</body>");
		strBfr.append("</html>");
		
		return strBfr.toString();
	}
	
	/**
	 * Handles client request to get loading progress page
	 * @param parameter request parameter
	 * @param remoteAddress client address
	 * @param requestedURL by client requested URL
	 * @return return data (either loading progress page html code or null)
	 * @throws InvocationTargetException 
	 * @throws IllegalAccessException 
	 * @throws IllegalArgumentException 
	 */
	public String doGetLoadingProgressPage(String parameter, String remoteAddress, String requestedURL) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException {

		String sessionId = parseGetRequestParameter(parameter, "sessionId");
		String configFileName = parseGetRequestParameter(parameter, "fileName");
	    String language = parseGetRequestParameter(parameter, "language");
	    if (language == null) language = "0";
		Web2cSessionHandler associatedSessionHandler = getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			if (associatedSessionHandler.getAttribute("loginStatus").equals("in")) {
				associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
				if (configFileName.equals("DiagnosticsPage") || (!servletName.equals("Web2c") && !servletName.equals("Web2cArchiveViewer") && !servletName.equals("Web2cLogbook"))) {					
					Object[] parameterArray = new Object[3];
					parameterArray[0] = parameter;
					parameterArray[1] = remoteAddress;
					parameterArray[2] = requestedURL;
					Method[] methods = (servletClass.cast((Object)servletInstance)).getClass().getMethods();
					String str = "";
					for (int i = 0; i < methods.length; i++) {
						if (methods[i].getName().equals("doGetPage")) {
							Method doGetPage = methods[i];
							str = (String)doGetPage.invoke((Object)(servletClass.cast((Object)servletInstance)), parameterArray);
						}
					}
					return str;
				}
				else {
					associatedSessionHandler.setAttribute("requestedConfigFileName", configFileName);
					associatedSessionHandler.setAttribute("language", language);
					associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));	
					if (servletName.equals("Web2c")) {
						Boolean dataMode = real;
						if (((String)associatedSessionHandler.getAttribute("dataSource")).equals("simulation")) {
							dataMode = simulated;
						}
						else {
							dataMode = real;
						}
						String accessMode = "file";
						Web2cPageGenerator generator = new Web2cPageGenerator(sessionId, accessMode, dataMode, remoteAddress, configFileName, Integer.valueOf((String)associatedSessionHandler.getAttribute("language")), (String)associatedSessionHandler.getAttribute("userName"), requestedURL);
						associatedSessionHandler.setWeb2cPageGenerator(generator);
						generator.addObserver(this);		
						generator.startGenerator();	
					}
					else if (servletName.equals("Web2cArchiveViewer")) {
						Web2cArchiveViewerPageGenerator generator = new Web2cArchiveViewerPageGenerator(sessionId, remoteAddress, configFileName, (String)associatedSessionHandler.getAttribute("fromTime"), (String)associatedSessionHandler.getAttribute("toTime"), (String)associatedSessionHandler.getAttribute("userName"), requestedURL);
						associatedSessionHandler.setWeb2cArchiveViewerPageGenerator(generator);
						generator.addObserver(this);		
						generator.startGenerator();	
					}		
					else if (servletName.equals("Web2cLogbook")) {
						Web2cLogbookPageGenerator generator = new Web2cLogbookPageGenerator(sessionId, remoteAddress, configFileName, (String)associatedSessionHandler.getAttribute("userName"));
						generator.addObserver(this);	
						associatedSessionHandler.setWeb2cLogbookPageGenerator(generator);	
						generator.startGenerator();	
					}	
					return getLoadingProgressPage(sessionId);
				}	
			}
			else {
				return null;
			}
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to initiate loading a page with login procedure
	 * @param parameter request parameter
	 * @param remoteAddress client address
	 * @param requestedURL by client requested URL
	 * @param httpSessionId id of associated HTTP session
	 * @param httpSessionCreationTime creation time of associated HTTP session
	 * @param dataSource real or simulated data
	 * @param verifyUserCredentials if true user credentials will be verified else guest-login with restricted privileges only
	 * @return page html code
	 */
	public String doGetLogin(String parameter, String remoteAddress, String requestedURL, String httpSessionId, long httpSessionCreationTime, String dataSource, boolean verifyUserCredentials) {

		String configFileName = parseGetRequestParameter(parameter, "fileName");
	    String language = parseGetRequestParameter(parameter, "language");
	    if (language == null) language = "0";
		Random randomGenerator = new Random();
		int randomNumber = randomGenerator.nextInt(100000);
		String sessionId = String.valueOf(Calendar.getInstance().getTimeInMillis());
		Web2cSessionHandler associatedSessionHandler = createSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			associatedSessionHandler.setAttribute("requestedConfigFileName", configFileName);
			associatedSessionHandler.setAttribute("loadedConfigFileName", "");
			associatedSessionHandler.setAttribute("dataSource", dataSource);
			associatedSessionHandler.setAttribute("userName", "guest");
			associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
			associatedSessionHandler.setAttribute("language", language);
			Calendar calendar = Calendar.getInstance();
			calendar.setTimeInMillis(httpSessionCreationTime);
			associatedSessionHandler.setAttribute("clientInfo", "Client IP address: " + remoteAddress);
			associatedSessionHandler.setAttribute("httpSessionInfo", "HTTPSession id: " + httpSessionId + ", HTTPSession creation time: " + (DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, localSettings)).format(calendar.getTime()));
			if (verifyUserCredentials == true) {
				associatedSessionHandler.setAttribute("loginStatus", "out");
				associatedSessionHandler.setAttribute("encryptionKey", String.valueOf(randomNumber));
				String key = (String)(associatedSessionHandler.getAttribute("encryptionKey"));
				File jScriptDir = new File(webDir);
				String[] fileList = jScriptDir.list();
				for (int i = 0; i < fileList.length; i++) {
					if (fileList[i].contains(servletName + "Login_")) {
						File jScriptFile = new File(webDir + File.separator + fileList[i]);
						if ((System.currentTimeMillis() - jScriptFile.lastModified() > (deleteFileTimeout * 1000))) jScriptFile.delete();
					}
				}
				String timeStamp = String.valueOf(System.currentTimeMillis());
				Web2cJavascriptWriter javascriptWriter = new Web2cJavascriptWriter(new File(webDir + File.separator + servletName + "Login_" + timeStamp + ".js"), getDebugLogger(), getWeb2cKeyBoardListener());
				StringBuffer strBfr = new StringBuffer();
				javascriptWriter.setJavascript(getJavascript(associatedSessionHandler));
				for (Iterator<String> it= getLoginPage(key, timeStamp).iterator(); it.hasNext();) {
					String out = (String)it.next();
					if (getWeb2cKeyBoardListener().getDebugWebCommunication()) getDebugLogger().appendLog(out);
					strBfr.append(out);
				}		
				return strBfr.toString();
			}
			else {
				associatedSessionHandler.setAttribute("loginStatus", "in");
				associatedSessionHandler.setAttribute("encryptionKey", "0");
				if (servletName.equals("Web2c")) {
					Boolean dataMode = real;
					String accessMode = "open";					
					Web2cPageGenerator generator = new Web2cPageGenerator(sessionId, accessMode, dataMode, remoteAddress, configFileName, Integer.valueOf((String)associatedSessionHandler.getAttribute("language")), (String)associatedSessionHandler.getAttribute("userName"), requestedURL.toString());
					generator.addObserver(this);
					associatedSessionHandler.setWeb2cPageGenerator(generator);
					generator.startGenerator();
				}
				else if (servletName.equals("Web2cArchiveViewer")) {
					Web2cArchiveViewerPageGenerator generator = new Web2cArchiveViewerPageGenerator(sessionId, remoteAddress, configFileName, (String)associatedSessionHandler.getAttribute("fromTime"), (String)associatedSessionHandler.getAttribute("toTime"), (String)associatedSessionHandler.getAttribute("userName"), requestedURL);
					generator.addObserver(this);	
					associatedSessionHandler.setWeb2cArchiveViewerPageGenerator(generator);	
					generator.startGenerator();	
				}	
				else if (servletName.equals("Web2cLogbook")) {
					Web2cLogbookPageGenerator generator = new Web2cLogbookPageGenerator(sessionId, remoteAddress, configFileName, (String)associatedSessionHandler.getAttribute("userName"));
					generator.addObserver(this);	
					associatedSessionHandler.setWeb2cLogbookPageGenerator(generator);	
					generator.startGenerator();	
				}	
				return getLoadingProgressPage(sessionId);
			}
		}
		else {
			return getSessionCreationErrorPage(configFileName);
		}
	}
	
	/**
	 * Handles client request to perform check if server alive
	 * @return alive status
	 */
	public String doPostCheckAlive () {

		return "<return>alive</return>";
	}
	
	/**
	 * Handles client request to get login encrytion key
	 * @param remoteAddress client address
	 * @param httpSessionId id of associated HTTP session
	 * @param httpSessionCreationTime creation time of associated HTTP session
	 * @param verifyUserCredentials if true user credentials will be verified else guest-login with restricted privileges only
	 * @return return data
	 */
	public String doPostLogin(String remoteAddress, String httpSessionId, long httpSessionCreationTime, boolean verifyUserCredentials) {

		String sessionId = String.valueOf(Calendar.getInstance().getTimeInMillis());
		Web2cSessionHandler associatedSessionHandler = createSessionHandler(sessionId);
		if (associatedSessionHandler != null) {	
			Random randomGenerator = new Random();
			int randomNumber = randomGenerator.nextInt(100000);
			associatedSessionHandler.setAttribute("userName", "guest");
			associatedSessionHandler.setAttribute("language", "0");
			associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
			Calendar calendar = Calendar.getInstance();
			calendar.setTimeInMillis(httpSessionCreationTime);
			associatedSessionHandler.setAttribute("clientInfo", "Client IP address: " + remoteAddress);
			associatedSessionHandler.setAttribute("httpSessionInfo", "HTTPSession id: " + httpSessionId + ", HTTPSession creation time: " + (DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, localSettings)).format(calendar.getTime()));
			String str = "";
			if (verifyUserCredentials == true) {
				associatedSessionHandler.setAttribute("loginStatus", "out");
				associatedSessionHandler.setAttribute("encryptionKey", String.valueOf(randomNumber));
				if (servletName.equals("Web2cGateway")) {
					associatedSessionHandler.setAttribute("requestedConfigFileName", gatewayConfigFile);
					str = "<return><key>" + String.valueOf(randomNumber) + "</key><sessionId>" + sessionId + "</sessionId><loginURL><elogin><sessionId>" + sessionId + "</sessionId><userName>\"username\"</userName><password>\"encryptedPassword\"</password><timeStamp>\"timeStamp\"</timeStamp></elogin></loginURL></return>";		
				}
				else {
					str = "<return><key>" + String.valueOf(randomNumber) + "</key><sessionId>" + sessionId + "</sessionId></return>";
				}
			}
			else {
				if (servletName.equals("Web2cGateway")) {
					associatedSessionHandler.setAttribute("loginStatus", "in");
					associatedSessionHandler.setAttribute("encryptionKey", "0");
					associatedSessionHandler.setAttribute("requestedConfigFileName", gatewayConfigFile);
					associatedSessionHandler.setAttribute("loadedConfigFileName", gatewayConfigFile);
					Web2cGatewayPage web2cGatewayPage = new Web2cGatewayPage(sessionId, userDir, gatewayConfigFile, (String)(associatedSessionHandler.getAttribute("userName")), remoteAddress, commonDeviceAccess);
					associatedSessionHandler.setWeb2cGatewayPage(web2cGatewayPage);
					str = "<return>";
    				str = str + "<updateURL><action><sessionId>" + sessionId + "</sessionId><value><do>update</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></updateURL>";
    				str = str + "<registerURL><action><sessionId>" + sessionId + "</sessionId><value><do>register</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></registerURL>";
    				str = str + "<executeURL><action><sessionId>" + sessionId + "</sessionId><value><do>execute</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></executeURL>";
    				str = str + "<logoutURL><action><sessionId>" + sessionId + "</sessionId><value>logout</value><timeStamp>\"timeStampInMillis\"</timeStamp></action></logoutURL>";
    				str = str + "</return>";
				}
			}
			return str;
		}
		else {
			return null;
		}
	}
	
	/**
	 * Handles client request to perform user authentication and authorization verification
	 * @param parameter request parameter
	 * @param remoteAddress client address
	 * @return return data
	 */
	public String doPostElogin(String parameter, String remoteAddress) {

		String sessionId = parsePostRequestParameter(parameter, "sessionId")[0];
		Web2cSessionHandler associatedSessionHandler = getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {			
			associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
			String userName = parsePostRequestParameter(parameter, "userName")[0];
			String sentPassword = parsePostRequestParameter(parameter, "password")[0];	 	    	
			String randomNumber = (String)(associatedSessionHandler.getAttribute("encryptionKey"));	 
			String page = parsePostRequestParameter(parameter, "page")[0]; 
			if (page.equals("")) page = (String)associatedSessionHandler.getAttribute("requestedConfigFileName"); 
			String userPassword = registryWriterAndReader.getPassword(userName);			    	
			Web2cMD5HashGenerator md5 = new Web2cMD5HashGenerator(userPassword + randomNumber);	
			String str = "";
			if ((md5.getMD5()).equals(sentPassword)) {
				associatedSessionHandler.setAttribute("userName", userName);
				associatedSessionHandler.setAttribute("loginStatus", "in");		
				if (servletName.equals("Web2cGateway")) {
					if (page.equals("DiagnosticsPage")) {
						associatedSessionHandler.setAttribute("requestedConfigFileName", page);
						associatedSessionHandler.setAttribute("loadedConfigFileName", page);	
						str = "/web2cToolkit/" + servletName + "?param=(load)" + "[" + sessionId + "]" + page + "," + (String)associatedSessionHandler.getAttribute("language");		
					}
					else {
						associatedSessionHandler.setAttribute("requestedConfigFileName", gatewayConfigFile);
						associatedSessionHandler.setAttribute("loadedConfigFileName", gatewayConfigFile);
						Web2cGatewayPage web2cGatewayPage = new Web2cGatewayPage(sessionId, userDir, gatewayConfigFile, (String)(associatedSessionHandler.getAttribute("userName")), remoteAddress, commonDeviceAccess);
						associatedSessionHandler.setWeb2cGatewayPage(web2cGatewayPage);
						str = "<return>";
						str = str + "<updateURL><action><sessionId>" + sessionId + "</sessionId><value><do>update</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></updateURL>";
    					str = str + "<registerURL><action><sessionId>" + sessionId + "</sessionId><value><do>register</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></registerURL>";
    					str = str + "<executeURL><action><sessionId>" + sessionId + "</sessionId><value><do>execute</do><parameter><uri>\"uri\"</uri></parameter></value><timeStamp>\"timeStampInMillis\"</timeStamp></action></executeURL>";
    					str = str + "<logoutURL><action><sessionId>" + sessionId + "</sessionId><value>logout</value><timeStamp>\"timeStampInMillis\"</timeStamp></action></logoutURL>";
    					str = str + "</return>";
					}
				}
				else {
					associatedSessionHandler.setAttribute("requestedConfigFileName", page);
					if (page.equals("DiagnosticsPage")) associatedSessionHandler.setAttribute("loadedConfigFileName", page);	
					str = "/web2cToolkit/" + servletName + "?param=(load)" + "[" + sessionId + "]" + page + "," + (String)associatedSessionHandler.getAttribute("language");
				}
				sessionLogger.appendLog("IP Address: " + remoteAddress + " User Name: " + (String)associatedSessionHandler.getAttribute("userName") + " Page loaded: " + (String)associatedSessionHandler.getAttribute("requestedConfigFileName"));
			}
			else {
				associatedSessionHandler.setAttribute("userName", "guest");
				associatedSessionHandler.setAttribute("loginStatus", "out");	
				if (servletName.equals("Web2cGateway")) {
					str = "<return><key>" + String.valueOf(randomNumber) + "</key><loginURL><elogin><sessionId>" + sessionId + "</sessionId><username>\"username\"</username><password>\"encryptedPassword\"</password><timeStamp>\"timeStamp\"</timeStamp></elogin></loginURL></return>";		
				}
				else {
					if (associatedSessionHandler.getAttribute("dataSource").equals("real")) str = "/web2cToolkit/" + servletName + "?param=(file)" + page + "," + (String)associatedSessionHandler.getAttribute("language");
					if (associatedSessionHandler.getAttribute("dataSource").equals("simulated")) str = "/web2cToolkit/" + servletName + "?param=(simu)" + page + "," + (String)associatedSessionHandler.getAttribute("language");
				}
			}
			return str;
		}
		else {
			return null;
		}
	}	
	
	/**
	 * Does single-sign-on login
	 * @param loginMasterSessionHandler session handler of login-master session
	 * @param page page requested by this login
	 * @return login return data
	 */
	public String doSingleSignOnLogin(Web2cSessionHandler loginMasterSessionHandler, String page) {
		
		String ret = "";

		if (loginMasterSessionHandler != null) {	
			String loginString = doPostLogin(loginMasterSessionHandler.getClientIPAddress(), loginMasterSessionHandler.getClientHTTPSessionId(), loginMasterSessionHandler.getClientHTTPSessionCreationTime(), true);						    			    	
			String sessionId = parsePostRequestParameter(loginString, "sessionId")[0];
			Web2cMD5HashGenerator md5 = new Web2cMD5HashGenerator(registryWriterAndReader.getPassword(loginMasterSessionHandler.getAttribute("userName")) + parsePostRequestParameter(loginString, "key")[0]);
			String encryptedPassWord = md5.getMD5();
			String parameter = "<sessionId>" + sessionId + "</sessionId><userName>" + loginMasterSessionHandler.getAttribute("userName") + "</userName><password>" + encryptedPassWord + "</password><page>" + page + "</page>"; 	
			ret = doPostElogin(parameter, loginMasterSessionHandler.getClientIPAddress());
		}
		
		return ret;
	}
	
	/**
	 * Checks page loading status
	 * @param parameter request parameter
	 * @return return data
	 */
	public String doPostCheckPageLoadingStatus(String parameter) {

		String sessionId = parsePostRequestParameter(parameter, "sessionId")[0];
		Web2cSessionHandler associatedSessionHandler = getSessionHandler(sessionId);
		if (associatedSessionHandler != null) {
			String str = "";		
			associatedSessionHandler.setAttribute("lastPoll", String.valueOf(Calendar.getInstance().getTimeInMillis()));
			if (associatedSessionHandler.getAttribute("requestedConfigFileName").equals(associatedSessionHandler.getAttribute("loadedConfigFileName"))) {
				str = "(page)" + "[" + sessionId + "]" + (String)(associatedSessionHandler.getAttribute("loadedConfigFileName") + "," + (String)associatedSessionHandler.getAttribute("language"));
			}
			else {
				DecimalFormat formatter = new DecimalFormat("##0", new DecimalFormatSymbols(localSettings));
				str = "Page loading: " + formatter.format(constructionProgress[0]) + "% ...";
			}
			return str;
		}
		else {
			return null;
		}
	}
	
	/**
	 * Performs early monitor/receiver link registering
	 */
	public void doEarlyLinkRegistering() {

		ArrayList<String> monitorToRegisterList = initWriterAndReader.getMonitorList();
		if (!monitorToRegisterList.isEmpty()) {
			for (Iterator<String> it = monitorToRegisterList.iterator(); it.hasNext();) {
				String monitor = (String)it.next();
				commonDeviceAccess.registerMonitor(monitor);
			}
		}		
		ArrayList<String> receiverToRegisterList = initWriterAndReader.getReceiverList();
		if (!receiverToRegisterList.isEmpty()) {
			for (Iterator<String> it = receiverToRegisterList.iterator(); it.hasNext();) {
				String receiver = (String)it.next();
				commonVideoConnector.connectToVideoSource(receiver);
			}
		}
	}
	
	/**
	 * Parses the client HTTP-POST request parameter and finds values associated with the key specified; In case of multiple occurrence the values are returned as String array in the order as found
	 * @param parameter request parameter string
	 * @parameter key key string
	 */
	public String[] parsePostRequestParameter(String parameter, String key) {
		
		if (parameter.contains(key)) {
			int first = parameter.indexOf("<" + key + ">");
			int last = parameter.lastIndexOf("</" + key + ">");
			String[] params = parameter.substring(first, last).replaceAll("<" + key + ">", "").split("</" + key + ">");		
			return params;
		}
		else {
			String[] params = {""};
			return params;
		}
	}
	
	/**
	 * Parses the client HTTP-GET request parameter and finds values associated with the key specified; In case of multiple occurrence the values are returned as String array in the order as found
	 * @param parameter request parameter string
	 * @parameter key key string
	 */
	public String parseGetRequestParameter(String parameter, String key) {
		
		String value = null;
		String sessionId = null;
		String fileName = null;
		String language = null;
		
		if (parameter.contains("]")) {
			sessionId = parameter.split("]")[0].replace("[", "");
			if (parameter.split("]")[1].contains(",")) {
				fileName = parameter.split("]")[1].replace("/", File.separator).split(",")[0];
				language = parameter.split("]")[1].split(",")[1];
			}
			else {
				fileName = parameter.split("]")[1];
			}
		}
		else {
			if (parameter.contains(",")) {
				fileName = parameter.split(",")[0].replace("/", File.separator);
				language = parameter.split(",")[1];
			}
			else {
				fileName = parameter.replace("/", File.separator);
			}
		}
		
		if (key.equals("sessionId")) value = sessionId;
		if (key.equals("fileName")) value = fileName;
		if (key.equals("language")) value = language;
		
		return value;
	}
	
	/**
	 * Returns the finally generated page from the page generator
	 * @param generator page generator
	 * @param page generated page
	 */
	public void update(Observable generator, Object page) {

		synchronized (getWeb2cSessionHandlerList()) {
			for (Iterator<Web2cSessionHandler> it = getWeb2cSessionHandlerList().iterator(); it.hasNext();) {
				Web2cSessionHandler sessionHandler = (Web2cSessionHandler)it.next();
				if (servletName.equals("Web2c")) {
					if ((Web2cPageGenerator)sessionHandler.getWeb2cPageGenerator() != null) {
						if (((Web2cPageGenerator)sessionHandler.getWeb2cPageGenerator()).equals(generator)) {
							sessionHandler.setWeb2cPage((Web2cPage)page);	
							sessionHandler.setAttribute("loadedConfigFileName", (String)sessionHandler.getAttribute("requestedConfigFileName"));
						}
					}
				}
				else if (servletName.equals("Web2cArchiveViewer")) {
					if (((Web2cArchiveViewerPageGenerator)sessionHandler.getWeb2cArchiveViewerPageGenerator()).equals(generator)) {
						sessionHandler.setWeb2cArchiveViewerPage((Web2cArchiveViewerPage)page);	
						sessionHandler.setAttribute("loadedConfigFileName", (String)sessionHandler.getAttribute("requestedConfigFileName"));
					}
				}
				else if (servletName.equals("Web2cLogbook")) {
					if (((Web2cLogbookPageGenerator)sessionHandler.getWeb2cLogbookPageGenerator()).equals(generator)) {
						sessionHandler.setWeb2cLogbookPage((Web2cLogbookPage)page);	
						sessionHandler.setAttribute("loadedConfigFileName", (String)sessionHandler.getAttribute("requestedConfigFileName"));
					}
				}
			}
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class Web2cPageGenerator extends Observable implements Runnable {

		private Thread generator = null;
		private String sessionId = "";
		private String accessMode = "";
		private boolean dataMode = simulated; 
		private String remoteAddress = "";
		private String fileName = "";
		private int pageLanguage = 0;
		private String userName = "";
		private String servletURL = "";
		
		/**	
		 * @param id session id
		 * @param access access mode 
		 * @param data data mode 
		 * @param address remote ip address
		 * @param file config file name
		 * @param language page language
		 * @param user user name
		 * @param url servlet URL
		 */
		public Web2cPageGenerator (String id, String access, boolean data, String address, String file, int language, String user, String url) {
			
			super();
		
			sessionId = id;
			accessMode = access;
			dataMode = data;
			remoteAddress = address;
			fileName = file;
			pageLanguage = language;
			userName = user;
			servletURL = url;
		
			if (generator == null) {
				generator = new Thread(this);
			}
		}
		
		/**
		 * Generates page
		 */
		public void run() {	

			Web2cPage web2cPage = new Web2cPage(servletURL, sessionId, accessMode, userDir, webDir, fileName, pageLanguage, userName, remoteAddress, dataMode, getWeb2cCommonDeviceAccess(), getWeb2cCommonVideoConnector(), getWeb2cChatHandler(), getDebugLogger(), getWeb2cKeyBoardListener(), constructionProgress);	
			
			setChanged();
			notifyObservers(web2cPage);
			
			generator.interrupt();
		}
		
		/**
		 * Starts generator
		 */
		public void startGenerator() {	
			
			generator.start();
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class Web2cArchiveViewerPageGenerator extends Observable implements Runnable {

		private Thread generator = null;
		private String sessionId = "";
		private String remoteAddress = "";
		private String fileName = "";
		private String fromTime = "";
		private String toTime = "";
		private String userName = "";
		private String servletURL = "";
		
		/**	
		 * @param id session id
		 * @param address remote ip address
		 * @param file config file name
		 * @param from start time of requested interval
		 * @param to end time of requested interval
		 * @param user user name
		 * @param url servlet URL
		 */
		public Web2cArchiveViewerPageGenerator (String id, String address, String file, String from, String to, String user, String url) {
			
			super();
		
			sessionId = id;
			remoteAddress = address;
			fileName = file;
			fromTime = from;
			toTime = to;
			userName = user;
			servletURL = url;
		
			if (generator == null) {
				generator = new Thread(this);
			}
		}
		
		/**
		 * Generates page
		 */
		public void run() {	

			Web2cArchiveViewerPage web2cArchiveViewerPage = new Web2cArchiveViewerPage(servletURL, sessionId, userDir, webDir, fileName, fromTime, toTime, userName, remoteAddress, debugLogger, debugListener);
			
			setChanged();
			notifyObservers(web2cArchiveViewerPage);
			
			generator.interrupt();
		}
		
		/**
		 * Starts generator
		 */
		public void startGenerator() {	
			
			generator.start();
		}
	}
	
	/**
	 * @author  R.Bacher, DESY
	 */
	public class Web2cLogbookPageGenerator extends Observable implements Runnable {

		private Thread generator = null;
		private String sessionId = "";
		private String remoteAddress = "";
		private String fileName = "";
		private String userName = "";
		
		/**	
		 * @param id session id
		 * @param address remote ip address
		 * @param file config file name
		 * @param user user name
		 */
		public Web2cLogbookPageGenerator (String id, String address, String file, String user) {
			
			super();
		
			sessionId = id;
			remoteAddress = address;
			fileName = file;
			userName = user;
		
			if (generator == null) {
				generator = new Thread(this);
			}
		}
		
		/**
		 * Generates page
		 */
		public void run() {	

			Web2cLogbookPage web2cLogbookPage = new Web2cLogbookPage(sessionId, userDir, logbookDir, fileName, userName, remoteAddress, getWeb2cCommonDeviceAccess(), getWeb2cSMTPInterface(), getWeb2cLogbookInitFileWriterAndReader(), constructionProgress);
			
			setChanged();
			notifyObservers(web2cLogbookPage);
			
			generator.interrupt();
		}
		
		/**
		 * Starts generator
		 */
		public void startGenerator() {	
			
			generator.start();
		}
	}
}
