/*
 * Copyright (c) 2006 Stiftung Deutsches Elektronen-Synchroton,
 * Member of the Helmholtz Association, (DESY), HAMBURG, GERMANY.
 *
 * THIS SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS" BASIS.
 * WITHOUT WARRANTY OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE AND
 * NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE. SHOULD THE SOFTWARE PROVE DEFECTIVE
 * IN ANY RESPECT, THE USER ASSUMES THE COST OF ANY NECESSARY SERVICING, REPAIR OR
 * CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE.
 * NO USE OF ANY SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
 * DESY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS,
 * OR MODIFICATIONS.
 * THE FULL LICENSE SPECIFYING FOR THE SOFTWARE THE REDISTRIBUTION, MODIFICATION,
 * USAGE AND OTHER RIGHTS AND OBLIGATIONS IS INCLUDED WITH THE DISTRIBUTION OF THIS
 * PROJECT IN THE FILE LICENSE.HTML. IF THE LICENSE IS NOT INCLUDED YOU MAY FIND A COPY
 * AT HTTP://WWW.DESY.DE/LEGAL/LICENSE.HTM
 */

package de.desy.acop.transport.adapters;

/*
 * Copyright (c) 2006 by Cosylab d.o.o.
 *
 * The full license specifying the redistribution, modification, usage and other
 * rights and obligations is included with the distribution of this project in
 * the file license.html. If the license is not included you may find a copy at
 * http://www.cosylab.com/legal/abeans_license.htm or may write to Cosylab, d.o.o.
 *
 * THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND, NOT EVEN THE
 * IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE, ASSUMES
 * _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE RESULTING FROM THE USE, MODIFICATION,
 * OR REDISTRIBUTION OF THIS SOFTWARE.
 */

import java.util.Map;

import com.cosylab.gui.displayers.DataConsumer;
import com.cosylab.gui.displayers.DataState;
import com.cosylab.gui.displayers.NonblockingNumberConsumer;
import com.cosylab.gui.displayers.SynchronousNumberConsumer;
import com.cosylab.util.CommonException;

import de.desy.acop.transport.AccessMode;
import de.desy.acop.transport.AcopTransport;
import de.desy.acop.transport.ConnectionParameters;
import de.desy.tine.client.TCallback;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TMode;


/**
 * <code>ReverseConsumer</code> is an implementation consumer that is attached
 * to those consumers which also act as data sources. This consumers receive
 * data from those sources and forward it to the TINE channel.
 * 
 * @author Jaka Bobnar, Cosylab
 *
 */
public class ReverseConsumer implements SynchronousNumberConsumer, NonblockingNumberConsumer
	
{
	/** Supporter consumer types */
	public static final Class<?>[] SUPPORTED_CONSUMER_TYPES = {
		SynchronousNumberConsumer.class, NonblockingNumberConsumer.class};
	
	private AcopTransport transport;
	private ConnectionParameters parameters;
	private int timeout = 30000;
	private double[] data;
	private TLink asyncLink;
	private TDataType asyncDataIn;
	private TCallback asyncCallback;
	
	public ReverseConsumer(ConnectionParameters param, AcopTransport tran, int size)
	{
		transport= tran;
        parameters = param.deriveWith(AccessMode.WRITE,timeout);
        transport.setConnectionParameters(parameters);
        data= new double[size];
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDataConsumer(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
    public DataConsumer getDataConsumer(Class type)
	{
		if (type.isAssignableFrom(this.getClass())) {
			return this;
		}

		return null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getDefaultDataConsumer()
	 */
	public DataConsumer getDefaultDataConsumer()
	{
		return this;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#updateDataState(com.cosylab.gui.displayers.DataState)
	 */
	public void updateDataState(DataState state)
	{
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#setCharacteristics(java.util.Map)
	 */
	public void setCharacteristics(Map attributes)
	{
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getName()
	 */
	public String getName()
	{
		return "WriteAdapter";
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedCharacteristics()
	 */
	public String[] getSupportedCharacteristics()
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see com.cosylab.gui.displayers.DataConsumer#getSupportedConsumerTypes()
	 */
	@SuppressWarnings("unchecked")
	public Class[] getSupportedConsumerTypes()
	{
		return SUPPORTED_CONSUMER_TYPES;
	}

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.displayers.SynchronousNumberConsumer#updateSynchronously(long, java.lang.Number)
	 */
	public void updateSynchronously(long timestamp, Number value) throws CommonException {
		data[0] = value.doubleValue();
	    transport.execute(null,0,data,1);
	    
    }

	/*
	 * (non-Javadoc)
	 * @see com.cosylab.gui.displayers.NonblockingNumberConsumer#updateNonblocking(java.lang.Number)
	 */
	public void updateNonblocking(Number value) {
		data[0] = value.doubleValue();
		
		if (asyncLink == null) {
			String deviceName = "/" + parameters.getDeviceContext() +"/" + parameters.getDeviceGroup() +"/" + parameters.getDeviceName();
			asyncDataIn = new TDataType(data);
			asyncLink = new TLink(deviceName, parameters.getDeviceProperty(), new TDataType(), asyncDataIn, TAccess.CA_WRITE);
			asyncCallback = new TCallback(){
//				int count = 0;
				public void callback(int LinkIndex, int LinkStatus) {
//					System.out.println(count++);
				}
			};
		}
		asyncDataIn.putData(data);		
		asyncLink.attach(TMode.CM_SINGLE, asyncCallback, 1000);
    }
	
	
	/**
	 * Releases this consumer.
	 */
	public void release() {
		transport=null;
		parameters=null;
		data=null;
	}
}
/* __oOo__ */
