/**
 * 
 */
package de.desy.acop.video;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.DecimalFormat;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.border.LineBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

/**
 * </code>NumericDataDisplayer</code> displays numerical image analysis data in form of
 * a table.
 * 
 * @author Tilen Kusterle, Cosylab
 *
 */
public class NumericDataDisplayer extends JPanel {

	private static final long serialVersionUID = 1L;
	private static final DecimalFormat decimalFormat = new DecimalFormat("#0.0");
	
	private JTable table3C;
	private JTable table2C;
	private DefaultTableModel tm3C;
	private DefaultTableModel tm2C;
	
	private class CellRenderer extends DefaultTableCellRenderer {

		private static final long serialVersionUID = 1L;
		
		private int[] headerRows;
		
		public CellRenderer(int[] headerRows) {
			this.headerRows = headerRows;
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
		 */
		@Override
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			JLabel label =  (JLabel) super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
					row, column);
			for (int i = 0; i < headerRows.length; i++) {
				if (headerRows[i] == row) {
					label.setHorizontalAlignment(SwingConstants.CENTER);
					label.setFont(label.getFont().deriveFont(Font.BOLD));
					return label;
				}
			}
			if (column == 0) label.setFont(label.getFont().deriveFont(Font.ITALIC));
			label.setHorizontalAlignment(SwingConstants.LEADING);
			return label;
		}
		
	}
	
	/**
	 * Constructs this </code>NumericDataDisplayer</code>..
	 */
	public NumericDataDisplayer() {
		super(new BorderLayout());
		
		JPanel tablePanel = new JPanel(new GridBagLayout());
		
		tm3C = new DefaultTableModel(6, 3) {

			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};
		tm3C.setValueAt("XY PROJECTION", 0, 0);
		tm3C.setValueAt("X VALUE (stat./fit)", 0, 1);
		tm3C.setValueAt("Y VALUE (stat./fit)", 0, 2);
		tm3C.setValueAt("mean [pixels]", 1, 0);
		tm3C.setValueAt("STD [pixels]", 2, 0);
		tm3C.setValueAt("amplitude", 3, 0);
		tm3C.setValueAt("constant", 4, 0);
		tm3C.setValueAt("slope", 5, 0);
		table3C = new JTable(tm3C);
//		table3C.setBorder(new LineBorder(Color.BLACK));
		table3C.setDefaultRenderer(Object.class, new CellRenderer(new int[]{0}));
		
		GridBagConstraints gbc = new GridBagConstraints(0,0,1,1,1,0,GridBagConstraints.CENTER,GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0);
		tablePanel.add(table3C, gbc);
		
		tm2C = new DefaultTableModel(14, 2) {

			private static final long serialVersionUID = 1L;

			@Override
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};
		tm2C.setValueAt("2D DATA", 0, 0);
		tm2C.setValueAt("VALUE", 0, 1);
		tm2C.setValueAt("STD A [pixels]", 1, 0);
		tm2C.setValueAt("STD B [pixels]", 2, 0);
		tm2C.setValueAt("rotation [degrees]", 3, 0);
		tm2C.setValueAt("amplitude", 4, 0);
		tm2C.setValueAt("minimum", 5, 0);
		tm2C.setValueAt("IMAGE DATA", 6, 0);
		tm2C.setValueAt("VALUE", 6, 1);
		tm2C.setValueAt("width [pixels]", 7, 0);
		tm2C.setValueAt("height [pixels]", 8, 0);
		tm2C.setValueAt("ROI X [pixels]", 9, 0);
		tm2C.setValueAt("ROI Y [pixels]", 10, 0);
		tm2C.setValueAt("ROI width [pixels]", 11, 0);
		tm2C.setValueAt("ROI height [pixels]", 12, 0);
		tm2C.setValueAt("threshold", 13, 0);
		table2C = new JTable(tm2C);
//		table2C.setBorder(new LineBorder(Color.BLACK));
		table2C.setDefaultRenderer(Object.class, new CellRenderer(new int[]{0, 6}));
		
		gbc.gridy++;
		tablePanel.add(table2C, gbc);
		
		tablePanel.setBorder(new LineBorder(Color.BLACK));
		
		add(tablePanel, BorderLayout.NORTH);
	}
	
	/**
	 * Sets table data.
	 * @param imageW the width of image
	 * @param imageH the width of image
	 * @param roiX the start X coordinate of region of interest
	 * @param roiY the start Y coordinate of region of interest
	 * @param roiW the width of region of interest
	 * @param roiH the height of region of interest
	 * @param threshold the threshold used in calculation
	 * @param meanX the X mean
	 * @param stdX the X STD
	 * @param sideAmplitudeX the X amplitude
	 * @param sideConstX the X constant
	 * @param meanXFit the X mean
	 * @param stdXFit the X STD
	 * @param sideAmplitudeXFit the X amplitude
	 * @param sideConstXFit the X constant
	 * @param meanY the Y mean
	 * @param stdY the Y STD
	 * @param sideAmplitudeY the Y amplitude
	 * @param sideConstY the Y constant
	 * @param meanYFit the Y mean
	 * @param stdYFit the Y STD
	 * @param sideAmplitudeYFit the Y amplitude
	 * @param sideConstYFit the Y constant
	 * @param stdA the A STD
	 * @param stdB the B STD
	 * @param angle2D the rotation of 2D profile 
	 * @param amplitude2D the amplitude of 2D profile
	 * @param const2D the constant of 2D profile
	 */
	public void setData(int imageW, int imageH, int roiX, int roiY, int roiW, int roiH, double threshold,
			double meanX, double stdX, double sideAmplitudeX, double sideConstX,
			double meanXFit, double stdXFit, double sideAmplitudeXFit, double sideConstXFit, double sideSlopeXFit,
			double meanY, double stdY, double sideAmplitudeY, double sideConstY,
			double meanYFit, double stdYFit, double sideAmplitudeYFit, double sideConstYFit, double sideSlopeYFit,
			double stdA, double stdB, double angle2D, double amplitude2D, double const2D) {
		tm3C.setValueAt(decimalFormat.format(meanX) + " / " + decimalFormat.format(meanXFit), 1, 1);
		tm3C.setValueAt(decimalFormat.format(meanY) + " / " + decimalFormat.format(meanYFit), 1, 2);
		tm3C.setValueAt(decimalFormat.format(stdX) + " / " + decimalFormat.format(stdXFit), 2, 1);
		tm3C.setValueAt(decimalFormat.format(stdY) + " / " + decimalFormat.format(stdYFit), 2, 2);
		tm3C.setValueAt(decimalFormat.format(sideAmplitudeX) + " / " + decimalFormat.format(sideAmplitudeXFit), 3, 1);
		tm3C.setValueAt(decimalFormat.format(sideAmplitudeY) + " / " + decimalFormat.format(sideAmplitudeYFit), 3, 2);
		tm3C.setValueAt(decimalFormat.format(sideConstX) + " / " + decimalFormat.format(sideConstXFit), 4, 1);
		tm3C.setValueAt(decimalFormat.format(sideConstY) + " / " + decimalFormat.format(sideConstYFit), 4, 2);
		tm3C.setValueAt(decimalFormat.format(0) + " / " + decimalFormat.format(sideSlopeXFit), 5, 1);
		tm3C.setValueAt(decimalFormat.format(0) + " / " + decimalFormat.format(sideSlopeYFit), 5, 2);

		
		tm2C.setValueAt(decimalFormat.format(stdA), 1, 1);
		tm2C.setValueAt(decimalFormat.format(stdB), 2, 1);
		tm2C.setValueAt(decimalFormat.format(angle2D), 3, 1);
		tm2C.setValueAt(decimalFormat.format(amplitude2D), 4, 1);
		tm2C.setValueAt(decimalFormat.format(const2D), 5, 1);
		tm2C.setValueAt(imageW, 7, 1);
		tm2C.setValueAt(imageH, 8, 1);
		tm2C.setValueAt(roiX, 9, 1);
		tm2C.setValueAt(roiY, 10, 1);
		tm2C.setValueAt(roiW, 11, 1);
		tm2C.setValueAt(roiH, 12, 1);
		tm2C.setValueAt(decimalFormat.format(threshold), 13, 1);
	}
	
	/**
	 * Clears table contents.
	 */
	public void clearData() {
		tm3C.setValueAt(null, 1, 1);
		tm3C.setValueAt(null, 1, 2);
		tm3C.setValueAt(null, 2, 1);
		tm3C.setValueAt(null, 2, 2);
		tm3C.setValueAt(null, 3, 1);
		tm3C.setValueAt(null, 3, 2);
		tm3C.setValueAt(null, 4, 1);
		tm3C.setValueAt(null, 4, 2);
		tm3C.setValueAt(null, 5, 1);
		tm3C.setValueAt(null, 5, 2);
		
		tm2C.setValueAt(null, 1, 1);
		tm2C.setValueAt(null, 2, 1);
		tm2C.setValueAt(null, 3, 1);
		tm2C.setValueAt(null, 4, 1);
		tm2C.setValueAt(null, 5, 1);
		tm2C.setValueAt(null, 7, 1);
		tm2C.setValueAt(null, 8, 1);
		tm2C.setValueAt(null, 9, 1);
		tm2C.setValueAt(null, 10, 1);
		tm2C.setValueAt(null, 11, 1);
		tm2C.setValueAt(null, 12, 1);
		tm2C.setValueAt(null, 13, 1);
	}
	
	/**
	 * Test method.
	 * @param args
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setTitle("Test");
		frame.setSize(700, 700);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		NumericDataDisplayer ndd = new NumericDataDisplayer();
		frame.setContentPane(ndd);
		frame.setVisible(true);
		
		while(true) {
			if (Math.random() > 0.1) {
				int imageW = (int) Math.round(1000*Math.random());
				int imageH = (int) Math.round(1000*Math.random());
				int roiX = (int) Math.round(1000*Math.random());
				int roiY = (int) Math.round(1000*Math.random());
				int roiW = (int) Math.round(1000*Math.random());
				int roiH = (int) Math.round(1000*Math.random());
				double threshold = Math.random();
				double meanX = Math.random();
				double stdX = Math.random();
				double sideAmplitudeX = Math.random();
				double sideConstX = Math.random();
				double meanY = Math.random();
				double stdY = Math.random();
				double sideAmplitudeY = Math.random();
				double sideConstY = Math.random();			
				double stdA = Math.random();
				double stdB = Math.random();
				double angle2D = Math.random();
				double amplitude2D = Math.random();
				double const2D = Math.random();
				ndd.setData(imageW, imageH, roiX, roiY, roiW, roiH, threshold, meanX, stdX, sideAmplitudeX, sideConstX,
						Math.random(),Math.random(),Math.random(),Math.random(),Math.random(),
						meanY, stdY, sideAmplitudeY, sideConstY, 
						Math.random(),Math.random(),Math.random(),Math.random(), Math.random(),
						stdA, stdB, angle2D, amplitude2D, const2D);
			} else {
				ndd.clearData();
			}
			
			try {
				Thread.sleep(2000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

}
