package de.desy.tine.console;

import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import de.desy.tine.addrUtils.TSrvEntry;
import de.desy.tine.client.TLinkFactory;
import de.desy.tine.dataUtils.TDataTime;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.server.logger.DbgLog;
import de.desy.tine.server.logger.MsgLog;
import de.desy.tine.startup.TInitializerFactory;

public class TCommandList
{
  private static boolean initialized = false;
  private static void initialize()
  {
    if (initialized) return;
    try
    {
      addCommandItem(new TCommandItem("help","available debug commands",TCommandList.class.getMethod("listItems", (Class<?>[])null),(int)TAccess.CA_NULL));
      addCommandItem(new TCommandItem("connections","current connection list",TLinkFactory.class.getMethod("dumpLinkTable", (Class<?>[])null)));
      addCommandItem(new TCommandItem("clients","current client list",TLinkFactory.class.getMethod("dumpClientTable", (Class<?>[])null)));
      addCommandItem(new TCommandItem("contracts","current contract list",TLinkFactory.class.getMethod("dumpContractTable", (Class<?>[])null)));
      addCommandItem(new TCommandItem("messages","message table entries",MsgLog.class.getMethod("dumpMessages", (Class<?>[])null)));
      addCommandItem(new TCommandItem("stats","relevant system statistics",TLinkFactory.class.getMethod("dumpStats", (Class<?>[])null)));
      addCommandItem(new TCommandItem("users","relevant system statistics",TLinkFactory.class.getMethod("dumpUserLists", (Class<?>[])null)));
      addCommandItem(new TCommandItem("nets","relevant system statistics",TLinkFactory.class.getMethod("dumpNetsLists", (Class<?>[])null)));
      addCommandItem(new TCommandItem("settings","current system settings",TLinkFactory.class.getMethod("dumpSettings", (Class<?>[])null)));
      addCommandItem(new TCommandItem("debug","debug level",TLinkFactory.class.getMethod("setOutputDebugLevel", new Class[]{int.class}),TAccess.CA_READ|TAccess.CA_WRITE));
      addCommandItem(new TCommandItem("filter","debug filter",DbgLog.class.getMethod("setFilter", new Class[]{String.class}),TAccess.CA_READ|TAccess.CA_WRITE));
      addCommandItem(new TCommandItem("which","address check",TSrvEntry.class.getMethod("which", new Class[]{String.class})));
      addCommandItem(new TCommandItem("modules","registered equipment modules",TLinkFactory.class.getMethod("dumpModules", (Class<?>[])null)));
      addCommandItem(new TCommandItem("properties","registered properties for equipment module given",TLinkFactory.class.getMethod("dumpProperties", new Class[]{String.class}),TAccess.CA_READ));
      addCommandItem(new TCommandItem("devices","registered devices for equipment module given",TLinkFactory.class.getMethod("dumpDevices", new Class[]{String.class}),TAccess.CA_READ));
      addCommandItem(new TCommandItem("version","current system version",TCommandList.class.getMethod("dumpVersion", (Class<?>[])null)));
      addCommandItem(new TCommandItem("time","current system time",TDataTime.class.getMethod("dumpTime", (Class<?>[])null)));
    }
    catch (Exception e)
    {
      MsgLog.log("TCommandList.initialize", "could not add command : "+e.getMessage(),TErrorList.code_failure,e,1);
    }
    initialized = true;
  }
  TCommandList()
  {
    initialize();
  }
  private static HashMap<String,TCommandItem> cmdLst = new HashMap<String,TCommandItem>();
  public static void addCommandItem(TCommandItem item)
  {
    if (item == null) return;
    if (cmdLst.containsKey(item.getCommandString())) return;
    cmdLst.put(item.getCommandString(), item);
  }
  public static void dumpVersion()
  {
    String v = TInitializerFactory.getInstance().getInitializer().getVersion();
    TLinkFactory.dbgPrint("System version: "+v);
  }
  public static void listItems()
  {
    if (!initialized) initialize();
    Iterator<TCommandItem> it = cmdLst.values().iterator();
    TCommandItem cmd;
    String prefix;
    while (it.hasNext())
    {
      cmd = it.next();
      if (cmd.canSet()) prefix = "set ";
      else if (cmd.canGet()) prefix = "get ";
      else prefix = "";
      String cmdstr = prefix + cmd.getCommandString();
      //if (cmdstr.length() < 8) cmdstr += "\t";
      TLinkFactory.dbgPrint(cmdstr+"\t->\t"+cmd.getCommandDescription());
    }
  }
  public static void parseCommand(String cmdstr)
  {
    String cmd;
    StringTokenizer st = new StringTokenizer(cmdstr," \t\n\r\f=()");
    if (!st.hasMoreTokens()) return;
    cmd = st.nextToken();
    if (cmd.compareToIgnoreCase("get") == 0)
    {
      if (!st.hasMoreTokens()) 
      {
        TLinkFactory.dbgPrint("command incomplete!");
        return;
      }
      cmd = st.nextToken();
      if (!st.hasMoreTokens())
      {
        executeCommand(cmd);
        return;        
      }
      String strval = st.nextToken();
      try 
      {
        int ival = Integer.parseInt(strval);
        executeCommand(cmd,ival);
      }
      catch (Exception e)
      {
        executeCommand(cmd,strval);
      }
      return;
    }
    if (cmd.compareToIgnoreCase("set") == 0)
    {
      if (!st.hasMoreTokens()) 
      {
        TLinkFactory.dbgPrint("command incomplete!");
        return;
      }
      cmd = st.nextToken();
      String strval = "";
      if (st.hasMoreTokens()) 
      {
        strval = st.nextToken();
      }
      try 
      {
        int ival = Integer.parseInt(strval);
        executeCommand(cmd,ival);
      }
      catch (Exception e)
      {
        executeCommand(cmd,strval);
      }
      return;
    }
    if (cmd.compareToIgnoreCase("which") == 0)
    {
      if (!st.hasMoreTokens()) 
      {
        TLinkFactory.dbgPrint("command incomplete!");
        return;
      }
      String strval = st.nextToken();
      try 
      {
        executeCommand(cmd,strval);
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
      return;
    }
    executeCommand(cmd);
  }
  private static boolean isExecuting = false;
  public static boolean isExecuting() { return isExecuting; }
  public static void executeCommand(String cmd)
  {
    if (!initialized) initialize();
    TCommandItem ci = cmdLst.get(cmd);
    if (ci == null)
    {
      TLinkFactory.dbgPrint("command "+cmd+" not found in command list");
      return;
    }
    isExecuting = true;
    ci.invoke();
    isExecuting = false;
  }
  public static void executeCommand(String cmd,int value)
  {
    if (!initialized) initialize();
    TCommandItem ci = cmdLst.get(cmd);
    if (ci == null)
    {
      System.out.println("command "+cmd+" not found in command list");
      return;
    }
    isExecuting = true;
    ci.invoke(value);
    isExecuting = false;
  }
  public static void executeCommand(String cmd,String value)
  {
    if (!initialized) initialize();
    TCommandItem ci = cmdLst.get(cmd);
    if (ci == null)
    {
      System.out.println("command "+cmd+" not found in command list");
      return;
    }
    isExecuting = true;
    ci.invoke(value);
    isExecuting = false;
  }
}
