/*
 * Created on Dec 15, 2004
 * CHANGES 
 * 2005-02-15 JW Made non-static
 */
package de.desy.tine.server.properties;
import java.util.*;

import de.desy.tine.definitions.TFormat;

// TODO Is ID needed at all?
// TODO If not, can this class be replaced by a plain HashSet?

/**
 * \internal
 * 
 * @author duval
 *
 */
public final class TMetaProperties
{
  public final class TMetaProperty
  {
    private String tag;
    private int id;
    private boolean isEmbedded;
    public String getTag() { return tag; }
    public int getId() { return id; }
    public TMetaProperty(String extension,int index)
    {
      tag = new String(extension);
      id = index;
      isEmbedded = false;
    }
    public TMetaProperty(String extension,int index,boolean embedded)
    {
      tag = new String(extension);
      id = index;
      isEmbedded = embedded;
    }
  }
  private  static LinkedList<TMetaProperty> mp = new LinkedList<TMetaProperty>();
  
  public static final int EGU_ID = 1;
  public static final int HST_ID = 2;
  public static final int NAM_ID = 3;
  public static final int DSC_ID = 4;
  public static final int BIT_ID = 5;
  public static final int MSK_ID = 6;
  public static final int GAT_ID = 7;
  public static final int XEGU_ID = 8;

  public static TMetaProperties getInstance()
  {
    if (instance == null) instance = new TMetaProperties();
    return instance;
  }
  private static TMetaProperties instance = new TMetaProperties();

  private TMetaProperties()
  {
    mp.add(new TMetaProperty(".AR",HST_ID));
    mp.add(new TMetaProperty(".ARC",HST_ID));
    mp.add(new TMetaProperty(".ARCH",HST_ID));
    mp.add(new TMetaProperty(".EGU",EGU_ID));
    mp.add(new TMetaProperty(".HST",HST_ID));
    mp.add(new TMetaProperty(".HIST",HST_ID));
    mp.add(new TMetaProperty(".HST@",HST_ID));
    mp.add(new TMetaProperty(".HIST@",HST_ID));
    mp.add(new TMetaProperty(".NAM",NAM_ID));
    mp.add(new TMetaProperty(".DSC",DSC_ID));
    mp.add(new TMetaProperty(".DESC",DSC_ID));
    mp.add(new TMetaProperty(".BIT.",BIT_ID,true));
    mp.add(new TMetaProperty(".MASK.",MSK_ID,true));
    mp.add(new TMetaProperty(".GATE.",GAT_ID,true));
    mp.add(new TMetaProperty(".XEGU",XEGU_ID));
  }
  public String[] getMetaPropertyDecorations()
  {
    if (instance == null) instance = new TMetaProperties();
    int len = mp.size();
    if (len == 0) return null;
    String[] lst = new String[len];
    for (int i=0; i<len; i++)
    {
      lst[i] = mp.get(i).getTag();
    }
    return lst;    
  }
  public int getGate(String property)
  {
    int idx, bit, gate = 0;
    try
    {
      if ((idx=property.indexOf(".BIT.")) > 0)
      {
        bit = Integer.parseInt(property.substring(idx+5));
        gate = 0x01 << bit;
      }
      else if ((idx=property.indexOf(".MASK.")) > 0 ||
               (idx=property.indexOf(".GATE.")) > 0)
      {
        String istr = property.substring(idx+6);
        if (istr.startsWith("0x") || istr.startsWith("0X"))
          gate = Integer.parseInt(istr,16);
        else
          gate = Integer.parseInt(istr);
      }
    }
    catch (Exception ignore) {}
    return gate;
  }
  public static String getTargetProperty(String property)
  {
    String ext;
    int idx = 0;
    TMetaProperty tmp;
    Iterator<TMetaProperty> it = mp.iterator();
    while (it.hasNext())
    {
      tmp = it.next();
      ext = tmp.getTag();
      if ((idx=property.indexOf(ext)) > 0)
      {
        if (property.endsWith(ext) || tmp.isEmbedded) 
          return property.substring(0,idx);
      }
    }
    return property;
  }
  public static int getIndex(String property)
  {
    String ext;
    TMetaProperty tmp;
    Iterator<TMetaProperty> it = mp.iterator();
    while (it.hasNext())
    {
      tmp = it.next();
      ext = tmp.getTag();
      if (property.indexOf(ext) > 0)
      {
        if (property.endsWith(ext) || tmp.isEmbedded) return tmp.getId();
      }
    }
    return -1;
  }
  public static boolean isMetaProperty(String property)
  {
    if (getIndex(property) < 0) return false;
    return true; 
  }
  public static String getMetaExtension(String property)
  {
    String ext;
    TMetaProperty tmp;
    Iterator<TMetaProperty> it = mp.iterator();
    while (it.hasNext())
    {
      tmp = it.next();
      ext = tmp.getTag();
      if (property.indexOf(ext) > 0)
      {
        if (property.endsWith(ext) || tmp.isEmbedded) return tmp.getTag();
      }
    }
    return null;
  }
  public static boolean isStatic(String property)
  {
    if (property == null) return false;
    if (property.endsWith(".EGU")) return true;
    if (property.endsWith(".DESC")) return true;
    if (property.endsWith(".DSC")) return true;
    return false;
  }
  public static String getSuggestedDescription(String property)
  {
    if (property.endsWith(".AR") || 
        property.endsWith(".ARC") ||
        property.endsWith(".ARCH")) return "central archived data for targeted device over time range";
    if (property.endsWith(".AR@") || 
        property.endsWith(".ARC@") || 
        property.endsWith(".ARCH@")) return "central archived data snapshot at given time";
    if (property.endsWith(".HIST") || 
        property.endsWith(".HST")) return "local archived data for targeted device over time range";;
    if (property.endsWith(".HIST@") ||
        property.endsWith(".HST@")) return "local archived data snapshot at given time";
    if (property.endsWith(".EGU")) return "units and max/min settings";
    if (property.endsWith(".XEGU")) return "x-axis units and max/min settings (spectrum arrays)";
    if (property.endsWith(".NAM")) return "device/keyword names associated with property";
    if (property.endsWith(".DESC") || 
        property.endsWith(".DSC")) return "description of property";
    if (property.contains(".BIT.")) return "value of specified bit (integer types only)";
    if (property.contains(".MASK.")) return "boolean result of applied mask (integers only)";
    if (property.contains(".GATE.")) return "gated data value (integers only)";
    return "unknown meta property";
  }
  public static short getSuggestedFormat(String property)
  {
    if (property.endsWith(".AR")) return TFormat.CF_DBLDBL;
    if (property.endsWith(".ARC")) return TFormat.CF_DBLDBL;
    if (property.endsWith(".ARCH")) return TFormat.CF_DBLDBL;
    if (property.endsWith(".AR@")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".ARC@")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".ARCH@")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".HIST")) return TFormat.CF_DBLDBL;
    if (property.endsWith(".HST")) return TFormat.CF_DBLDBL;
    if (property.endsWith(".HIST@")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".HST@")) return TFormat.CF_DOUBLE;
    if (property.endsWith(".EGU")) return TFormat.CF_USTRING;
    if (property.endsWith(".XEGU")) return TFormat.CF_USTRING;
    if (property.endsWith(".NAM")) return TFormat.CF_NAME64;
    if (property.endsWith(".DESC")) return TFormat.CF_NAME64;
    if (property.endsWith(".DSC")) return TFormat.CF_NAME64;
    if (property.contains(".BIT.")) return TFormat.CF_INT32;
    if (property.contains(".MASK.")) return TFormat.CF_INT32;
    if (property.contains(".GATE.")) return TFormat.CF_INT32;
    return TFormat.CF_NULL;
  }
  public static int getSuggestedDataSize(String property)
  {
    if (property.endsWith(".AR")) return 1000;
    if (property.endsWith(".ARC")) return 1000;
    if (property.endsWith(".ARCH")) return 1000;
    if (property.endsWith(".HIST")) return 1000;
    if (property.endsWith(".HST")) return 1000;
    if (property.endsWith(".NAM")) return 1000;
    if (property.endsWith(".AR@")) return 1000;
    if (property.endsWith(".ARC@")) return 0;
    if (property.endsWith(".ARCH@")) return 0;
    if (property.endsWith(".HIST@")) return 0;
    if (property.endsWith(".HST@")) return 0;
    return 1;
  }
  public static boolean isHistorySnapshot(String property)
  {
    return (property.endsWith(".HIST@") || property.endsWith(".HST@"));
  }
  public static boolean isCentralArchive(String property)
  {
    return (property.endsWith(".ARCH") || 
            property.endsWith(".ARC") || 
            property.endsWith(".AR"));
  }
}
